<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions to help with displaying metric data.
 */

/**
 *  Implements hook_block_info()
 *  XXX: This should probably be deleted or filled in.
 */
function famc_analytics_block_info($delta = '') {
  return array();
}

function famc_analytics_get_view_content($view_name, $display_name) {

  return views_embed_view($view_name, $display_name);
}

/**
 * Returns the view content for ajax refresh
 */
function famc_analytics_get_metrics_content($type, $arg) {

  if ($type != 'host_vm') {
    // Fetch the latest metrics, before sending
    // famc_analytics_fetch_vm_metric($arg);
  }

  $view_settings = famc_analytics_metric_views();

  $view_data = $view_settings[$type];

  $return = array(
    views_embed_view($view_data['name'], $view_data['display'], $arg),
  );

  // In case of VM metric we also return vDisk metric view
  if ($type == 'vm') {
    $view_data = $view_settings['vm_vdisk'];
    $return[] = views_embed_view($view_data['name'], $view_data['display'], $arg);
  }

  header('Content-type: text/json');
  print drupal_json_encode($return);
  exit();
}

/**
 * Convenience method for mapping the metric view names with entities
 */
function famc_analytics_metric_views() {

  return array(
    'host_vm' => array('name' => FAMC_ANALYTICS_VM_METRIC_HOST_VIEW_NAME, 'display' => FAMC_ANALYTICS_VM_METRIC_HOST_VIEW_DISPLAY_NAME),
    'vm' => array('name' => FAMC_ANALYTICS_VM_METRIC_VIEW_NAME, 'display' => FAMC_ANALYTICS_VM_METRIC_VIEW_DISPLAY_NAME),
    'vm_vdisk' => array('name' => FAMC_ANALYTICS_VDISK_METRIC_VIEW_NAME, 'display' => FAMC_ANALYTICS_VDISK_METRIC_VIEW_DISPLAY_NAME),
  );
}

function famc_analytics_render_hist_perf_graph($entity_type, $entity_id) {

  famc_analytics_load_jqplot();

  // Defaults?
  $from = FAMC_ANALYTICS_METRIC_DEFAULT_PERIOD;
  $to = 15;
  $metric = FAMC_ANALYTICS_METRIC_DEFAULT;
  $is_metric_group = FALSE;
  $matches = array();

  if (isset($_SESSION['vm_hist_perf'][$entity_id])) {
    $values = $_SESSION['vm_hist_perf'][$entity_id];
    $from = $values['from'];
    $to = $values['to'];
    $metric = $values['metric'];
  }

  if (preg_match('/^group-(.*)$/', $metric, $matches)) {
    $is_metric_group = TRUE;
    $group = $matches[1];
    $metrics = famc_analytics_metrics();
    $display_metrics = array_filter($metrics, function($value) use ($group) {
        return ($value['group'] == $group);
      }
    );
  }

  drupal_get_messages();
  if ($to > $from) {
    drupal_set_message(t('Selected duration is invalid.'), 'warning');
  }
  $time_points = famc_analytics_period_dist($from, $to);

  $data = array();
  $ticks = array_fill(0, sizeof($time_points) - 1, '');
  $ticks[0] = date('M-j G:i', $time_points[0]);
  $ticks[] = date('M-j G:i', $time_points[sizeof($time_points) - 1]);

  if ($is_metric_group) {
    $series = array();

    foreach ($display_metrics as $metric => $display_metric) {
      $metric_values = array();
      for ($i = 0; $i < sizeof($time_points); $i++) {

        if ($entity_type == 'vm') {

          $metrics = famc_analytics_get_vm_metrics($entity_id, $time_points[$i]);
          // $data[] = array(date('M-j G:i', $time_points[$i]), (int) $metrics[$metric]);
          $metric_values[] = round((double) $metrics[$metric], 2);
        }
      }
      $data[] = $metric_values;
      $series[] = array('label' => $display_metric['label']);
    }

    $metrics = famc_analytics_metrics();
    $label = $metrics[$metric]['grplabel'] . (empty($metrics[$metric]['unit']) ? '' : ' (' . $metrics[$metric]['unit'] . ')');

    drupal_add_js(
      array('famc_analytics_hist_data' => array(
        'data' => $data,
        'series' => $series,
        'elem' => 'famc-vm-hist-graph',
        'yaxis' => $label,
        'ticks' => $ticks,
      )
      ), 'setting'
    );
  } else {
    for ($i = 0; $i < sizeof($time_points); $i++) {

      if ($entity_type == 'vm') {

        $metrics = famc_analytics_get_vm_metrics($entity_id, $time_points[$i]);
        // $data[] = array(date('M-j G:i', $time_points[$i]), (int) $metrics[$metric]);
        $data[] = round((double) $metrics[$metric], 2);
      }
    }

    $metrics = famc_analytics_metrics();
    $label = $metrics[$metric]['label'] . (empty($metrics[$metric]['unit']) ? '' : ' (' . $metrics[$metric]['unit'] . ')');

    drupal_add_js(
      array('famc_analytics_hist_data' => array(
        'data' => array($data),
        'elem' => 'famc-vm-hist-graph',
        'yaxis' => $label,
        'ticks' => $ticks)
      ), 'setting'
    );
  }
}

/**
 *  Render the analytics graph
 *  THIS FUNCTION NEEDS TO BE MERGED WITH THE FUNCTION ABOVE
 */
function famc_analytics_render_analytics_graph($entity_type = FAMC_ANALYTICS_VM_NODE_TYPE, $ids = array()) {

  famc_analytics_load_jqplot();

  $from = FAMC_ANALYTICS_METRIC_DEFAULT_PERIOD;
  $to = 0;
  $metric = FAMC_ANALYTICS_METRIC_DEFAULT;
  $is_metric_group = FALSE;
  $matches = array();

  if (isset($_SESSION['analytics'])) {
    $values = $_SESSION['analytics'];
    $from = $values['from'];
    $to = $values['to'];
    $metric = $values['metric'];
  }

  if (preg_match('/^group-(.*)$/', $metric, $matches)) {
    $is_metric_group = TRUE;
    $group = $matches[1];
    $metrics = famc_analytics_metrics();
    $display_metrics = array_filter($metrics, function($value) use ($group) {
        return ($value['group'] == $group);
      }
    );
  }

  drupal_get_messages();
  if ($to > $from) {
    drupal_set_message(t('Selected duration is invalid.'), 'warning');
  }
  $time_points = famc_analytics_period_dist($from, $to);

  if ($is_metric_group) {
    $ids = array($ids[0]); // Consider only the first VM if we need to display multiple metrics
  }

  $nodes = famc_analytics_get_entities($entity_type, $ids);
  $nodes = node_load_multiple(array_keys($nodes));

  $data = array();
  $series = array();
  $ticks = array_fill(0, sizeof($time_points) - 1, '');
  $ticks[0] = date('M-j G:i', $time_points[0]);
  $ticks[] = date('M-j G:i', $time_points[sizeof($time_points) - 1]);

  if ($is_metric_group) {

    $node = array_pop($nodes);
    foreach ($display_metrics as $metric => $display_metric) {
      $metric_values = array();
      for ($i = 0; $i < sizeof($time_points); $i++) {

        if ($entity_type == FAMC_ANALYTICS_VM_NODE_TYPE) {

          $metrics = famc_analytics_get_vm_metrics($node->nid, $time_points[$i]);
          // $data[] = array(date('M-j G:H', $time_points[$i]), (int) $metrics[$metric]);
          // $metric_values[] = array(date('M-j G:i', $time_points[$i]), (int) $metrics[$metric]);
          $metric_values[] = round((double) $metrics[$metric], 2);
        }
      }
      $data[] = $metric_values;
      $series[] = array('label' => $display_metric['label']);
      $metrics = famc_analytics_metrics();
      $label = $metrics[$metric]['grplabel'] . (empty($metrics[$metric]['unit']) ? '' : ' (' . $metrics[$metric]['unit'] . ')');
    }
  } else {

    foreach ($nodes as $node) {
      $metric_values = array();
      for ($i = 0; $i < sizeof($time_points); $i++) {

        if ($entity_type == FAMC_ANALYTICS_VM_NODE_TYPE) {

          $metrics = famc_analytics_get_vm_metrics($node->nid, $time_points[$i]);
          // $data[] = array(date('M-j G:H', $time_points[$i]), (int) $metrics[$metric]);
          // $metric_values[] = array(date('M-j G:i', $time_points[$i]), (int) $metrics[$metric]);
          $metric_values[] = round((double) $metrics[$metric], 2);
        }
      }
      $data[] = $metric_values;
      $series[] = array('label' => $node->title);
    }

    $metrics = famc_analytics_metrics();
    $label = $metrics[$metric]['label'] . (empty($metrics[$metric]['unit']) ? '' : ' (' . $metrics[$metric]['unit'] . ')');
  }

  drupal_add_js(array('famc_analytics_hist_data' => array('data' => $data, 'series' => $series, 'elem' => 'famc-analytics-graph', 'yaxis' => $label, 'xaxis' => 'Time', 'ticks' => $ticks)), 'setting');
}

/**
 * Distribution of periods
 * @param int $from {In minutes}
 * @param int $to {In minutes. This used to be the To time. But now changed to "range".}
 */
function famc_analytics_period_dist($from, $to) {

  $num_points = 15;

  /* if ($to >= 50) {
    $num_points = 60;
    }
    elseif ($to >= 30) {
    $num_points = 30;
    } */

  if ($to > $from) {
    $to = $from;
  }

  if ($to > 15) {
    $num_points = $to;
  }
  if ($num_points > 120) {
    $num_points = 120;
  }

  $now = time();
  $from = $now - ($from * 60);
  $to = $from + ($to * 60);

  $points = array();
  $diff = round(($to - $from) / $num_points);
  for ($i = 0; $i < $num_points; $i++) {
    $value = round($from + $i * $diff);
    $points[] = $value;
  }

  return $points;
}

/**
 * Overrides some strings
 */
function famc_analytics_string_overrides($str) {

  $overrides = array(
    'ADMIN_ENABLED' => t('Enabled'),
    'ADMIN_DISABLED' => t('Disabled'),
    'OP_ENABLED' => t('Enabled'),
    'OP_DISABLED' => t('Disabled'),
    'OP_NORMAL' => t('Normal'),
    'OP_PENDING' => t('Pending'),
    'OP_FAILED' => t('Failed'),
    'OP_DEGRADED' => t('Degraded'),
  );

  return array_key_exists($str, $overrides) ? $overrides[$str] : $str;
}

/**
 *  Displays popup for VM and Host selection
 */
function famc_analytics_ctools_popup($js, $entity_type) {

  if (!$js) {
    return drupal_get_form('famc_analytics_get_entity_select_form', $entity_type);
  }

  ctools_include('modal');
  ctools_include('ajax');

  $form_state = array(
    'title' => '',
    'ajax' => TRUE,
    'entity_type' => $entity_type,
  );

  $output = ctools_modal_form_wrapper('famc_analytics_get_entity_select_form', $form_state);

  if (!empty($form_state['executed'])) {
    $checked_values = famc_analytics_get_checked_values($form_state['values']['entities']);
    $nodes = node_load_multiple($checked_values);
    $titles = array_map(function($value) {
        if (is_object($value)) {
          return $value->title;
        }
      }, $nodes);
    $output = array();
    $output[] = ajax_command_html('#famc-analytics-vm-selections span', implode(', ', $titles));
    $output[] = ctools_modal_command_dismiss();
  }
  print ajax_render($output);
  exit;
}

/**
 *  Generates the links for modal popups
 */
function famc_analytics_ctools_popup_link($entity_type, $text, $alt = "", $class = "") {

  ctools_include('ajax');
  ctools_include('modal');
  ctools_modal_add_js();

  return ctools_modal_text_button($text, 'analytics/nojs/entity/' . $entity_type, $alt, $class);
}

/**
 * Returns the metrics and their display names
 */
function famc_analytics_metrics() {

  return array(
    'read_latency' => array(
      'label' => t('Read latency'),
      'unit' => 'ms',
      'grplabel' => 'Latencies',
      'group' => 'latencies',
    ),
    'write_latency' => array(
      'label' => t('Write Latency'),
      'unit' => 'ms',
      'grplabel' => 'Latencies',
      'group' => 'latencies',
    ),
    'cache_latency' => array(
      'label' => t('Cache Latency'),
      'unit' => 'ms',
      'grplabel' => 'Latencies',
      'group' => 'latencies',
    ),
    'read_ops' => array(
      'label' => t('Read IOPS'),
      'unit' => '',
      'grplabel' => 'IOPS',
      'group' => 'ops',
    ),
    'write_ops' => array(
      'label' => t('Write IOPS'),
      'unit' => '',
      'grplabel' => 'IOPS',
      'group' => 'ops',
    ),
    'hit_rate_pc' => array(
      'label' => t('Hit Rate %'),
      'unit' => '',
      'grplabel' => 'Rates',
      'group' => 'rates',
    ),
    'population_pc' => array(
      'label' => t('Population %'),
      'unit' => '',
      'grplabel' => 'Rates',
      'group' => 'rates',
    ),
    'insertion_rate_pc' => array(
      'label' => t('Insertion rate %'),
      'unit' => '',
      'grplabel' => 'Rates',
      'group' => 'rates',
    ),
  );
}

/**
 *  Format the metrics for displaying in select boxes
 */
function famc_analytics_format_metric_list() {

  $metrics = famc_analytics_metrics();

  $latencies = array_filter($metrics, function($value) {
      return ($value['group'] == 'latencies');
    });
  $latencies = array_map(function($value) {
      return ' - ' . $value['label'];
    }, $latencies
  );
  $latencies = array_merge(array('group-latencies' => t('All Latencies')), $latencies);

  $ops = array_filter($metrics, function($value) {
      return ($value['group'] == 'ops');
    });
  $ops = array_map(function($value) {
      return ' - ' . $value['label'];
    }, $ops
  );
  $ops = array_merge(array('group-ops' => t('All IOPS')), $ops);

  $rates = array_filter($metrics, function($value) {
      return ($value['group'] == 'rates');
    });
  $rates = array_map(function($value) {
      return ' - ' . $value['label'];
    }, $rates
  );
  $rates = array_merge(array('group-rates' => t('All Rates')), $rates);

  return array_merge($latencies, $ops, $rates);
}

/**
 *  Add the jQPlot library files
 *  @param Array $plugins The plugins to load, for future use
 */
function famc_analytics_load_jqplot($plugins = array()) {

  $lib_path = drupal_get_path('module', 'famc_analytics') . '/js/jqplot/';

  // IE 8 and lower needs excanvas
  $markup = array(
    '#type' => 'markup',
    '#markup' => '<!--[if lt IE 9]><script type="text/javascript" src="' . base_path() . $lib_path . 'excanvas.min.js"></script><![endif]-->',
  );

  drupal_add_html_head($markup, 'jqplot');

  drupal_add_js($lib_path . 'jquery.jqplot.min.js');
  drupal_add_js($lib_path . 'jqplot.categoryAxisRenderer.min.js');
  drupal_add_js($lib_path . 'jqplot.highlighter.min.js');
  drupal_add_js($lib_path . 'jqplot.canvasTextRenderer.min.js');
  drupal_add_js($lib_path . 'jqplot.canvasAxisLabelRenderer.min.js');
  drupal_add_js($lib_path . '/plugins/jqplot.enhancedLegendRenderer.js');

  drupal_add_css($lib_path . 'jquery.jqplot.min.css');
}
