<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions to help with generating the metrics data.
 */

/**
 *  Processes the web-service response and returns the proper fields
 *  @param $response object {The response object}
 */
function famc_analytics_process_vm_matrix_data($response, $op_response) {

  // The processed data to return
  $processed = array('attributes' => array(), 'metrics' => array(), 'disk_values' => array(), 'options' => array());

  try {

    // Get the cache options
    $options_info = $op_response->HgGetCacheOptionsResult->HgCacheOptions;
    // Get the global cache info
    $vm_info = $response->HgGetCacheInfoResult->GlobalCacheInfo;
    // Get the Disk metrics
    $disk_info = $response->HgGetCacheInfoResult->DiskCacheInfoList->DiskCacheInfo;

    // Find the VM metrics values by averaging the disk metrics
    $vm_read_latency = 0;
    $vm_write_latency = 0;
    $vm_read_ops = 0;
    $vm_write_ops = 0;
    $vm_hits = 0;
    $vm_read_blocks = 0;
    $vm_write_blocks = 0;
    $vm_usable_blocks = $vm_info->FormatInfo->UsableBlocks;
    $vm_cache_latency = 0;
    $vm_population = 0;
    $vm_insertions = 0;
    $vm_hit_ops = 0;

    $vdisk_count = 0;
    foreach ($disk_info as $disk_metric) {
      // We need to consider only the enabled devices where performance monitoring is enabled
      if ($disk_metric->AdminState == FAMC_ANALYTICS_ENABLED && $disk_metric->PerfState == FAMC_ANALYTICS_ENABLED) {
        $metric_object = $disk_metric->Metrics;

        // Convert all Ops to be Ops/Sec
        $metric_object->ReadOps /= $options_info->MetricsResetInterval;
        $metric_object->WriteOps /= $options_info->MetricsResetInterval;
        $metric_object->HitOps /= $options_info->MetricsResetInterval;

        // calculate VM wide read latency
        $total_read_time = ($vm_read_latency * $vm_read_ops) + (round($metric_object->ReadLatencyPerOp / 1000, 2) * $metric_object->ReadOps);
        $vm_read_ops += $metric_object->ReadOps;
        if ($vm_read_ops > 0) {
          $vm_read_latency = $total_read_time / $vm_read_ops;
        }

        // calculate VM wide write latency
        $total_write_time = ($vm_write_latency * $vm_write_ops) + (round($metric_object->WriteLatencyPerOp / 1000, 2) * $metric_object->WriteOps);
        $vm_write_ops += $metric_object->WriteOps;
        if ($vm_write_ops > 0) {
          $vm_write_latency = $total_write_time / $vm_write_ops;
        }

        $total_hit_time = ($vm_cache_latency * $vm_hit_ops) + (round($metric_object->AvgHitLatency / 1000, 2) * $metric_object->HitOps);
        $vm_hit_ops += $metric_object->HitOps;
        if ($vm_hit_ops > 0) {
          $vm_cache_latency = $total_hit_time / $vm_hit_ops;
        }

        # Pursuant to barium burt 720667 (FMC: change hit rate computation
        # to use OPs instead of Blocks for accuracy): we will keep track of
        # HitOps instead of Hits in the VM metrics node to avoid a database
        # change.  In copper+ we should add a field and track both,
        # regardless of how we calculate hit rate.
        # $vm_hits          += $metric_object->Hits;
        $vm_hits = $vm_hit_ops;

        $vm_read_blocks += $metric_object->ReadBlocks;
        $vm_write_blocks += $metric_object->WriteBlocks;

        $vm_population += $metric_object->Population;

        $vm_insertions += $metric_object->Insertions;

        // Increment the vDisk count
        $vdisk_count++;

        // Store the vDisk metrics & attributes
        $processed['disk_values'][] = array(
          'read_latency' => round($metric_object->ReadLatencyPerOp / 1000, 2),
          'write_latency' => round($metric_object->WriteLatencyPerOp / 1000, 2),
          'read_ops' => $metric_object->ReadOps,
          'write_ops' => $metric_object->WriteOps,
          'hits' => $metric_object->Hits,
          'hit_ops' => $metric_object->HitOps,
          'read_blocks' => $metric_object->ReadBlocks,
          'write_blocks' => $metric_object->WriteBlocks,
          'cache_latency' => round($metric_object->AvgHitLatency / 1000, 2),
          'population' => $metric_object->Population,
          'insertions' => $metric_object->Insertions,
          'identifier' => $disk_metric->Identifier,
          'admin_state' => $disk_metric->AdminState,
          'cache_mode' => $disk_metric->CacheMode,
          'disk_id' => $disk_metric->DiskID,
          'drive_letter' => $disk_metric->DriveLetters,
          'op_state_message' => $disk_metric->OpStateMsg,
          'op_state' => $disk_metric->OperationalState,
          'perf_state' => $disk_metric->PerfState,
          'usable_blocks' => $vm_usable_blocks,
        );
      }
    }

    $processed['attributes']['build_version'] = $vm_info->BuildVersion;
    $processed['attributes']['cache_mode'] = $vm_info->CacheMode;
    $processed['attributes']['admin_state'] = $vm_info->AdminState;
    $processed['attributes']['cache_size'] = $vm_info->Sectors * $vm_info->SectorSize;
    $processed['attributes']['op_state'] = $vm_info->OperationalState;
    $processed['attributes']['op_message'] = $vm_info->OpStateMsg;
    $processed['attributes']['perf_state'] = $vm_info->PerfState;

    $processed['metrics']['read_latency'] = $vm_read_latency;
    $processed['metrics']['write_latency'] = $vm_write_latency;
    $processed['metrics']['read_ops'] = $vm_read_ops;
    $processed['metrics']['write_ops'] = $vm_write_ops;
    $processed['metrics']['hits'] = $vm_hits;
    $processed['metrics']['read_blocks'] = $vm_read_blocks;
    $processed['metrics']['write_blocks'] = $vm_write_blocks;
    $processed['metrics']['usable_blocks'] = $vm_usable_blocks;
    $processed['metrics']['cache_latency'] = $vm_cache_latency;
    $processed['metrics']['population'] = $vm_population;
    $processed['metrics']['insertions'] = $vm_insertions;
  } catch (Exception $e) {
    mercury_utilities_fmc_log_message(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, 'VM metrics processing error - @error', array('@error' => $e->getMessage()), WATCHDOG_ERROR);
    $processed = FALSE;
  }

  return $processed;
}

function famc_analytics_get_disk_usage_level() {
  $percentage_usage = exec("/bin/df -k /opt/flashaccel/mysql | /bin/grep mysql | /usr/bin/awk '{print $5}' | /bin/sed -e s/\%//g", $df_output);

  if ($percentage_usage > 85) {
    $warning_level = "alert";
  } else if ($percentage_usage > 70) {
    $warning_level = "high";
  } else {
    $warning_level = "low";
  }

  return $warning_level;
}

/**
 *  Fetches metric data for VMs and stores in DB
 *  Fetches metric for all VMs if $etids is NULL
 *  @param array etids {The entity ids of the VMs for which the data are being fetched}
 */
function famc_fetch_vm_metric_multiple($etids = NULL) {

  // Fetch metrics for the VMs that have cache enabled and performance monitoring enabled
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', FAMC_ANALYTICS_VM_NODE_TYPE)
    ->propertyCondition('status', 1)
    ->fieldCondition('field_fa_administration_state', 'value', FAMC_ANALYTICS_ENABLED)
    ->fieldCondition('field_fa_performance_state', 'value', FAMC_ANALYTICS_ENABLED);

  if (is_array($etids) && !empty($etids)) {
    // Select only the specified nodes
    $query->entityCondition('entity_id', $etids, 'IN');
  }
  $result = $query->execute();
  $success = TRUE;

  // Fetch the records for each VM and store as nodes
  if (is_array($result) && isset($result['node'])) {

    module_load_include('inc', 'mercury_management');

    foreach ($result['node'] as $nid => $node) {
      try {
        // Call web-service to fetch data
        // Refer the Mercury Management module

        $node = node_load($node->nid);
        if (!$node) {
          continue;
        }
        if (!field_get_items('node', $node, 'field_vm_address')) {
          continue;
        }

        $hostname = array_pop(array_pop(field_get_items('node', $node, 'field_vm_address')));
        $credentials = mercury_management_get_credentials($node);
        $username = $credentials['username'];
        $password = $credentials['password'];
        if (empty($username)) {
          continue;
        }

        if (!empty($node->field_install_port)) {
          $port = $node->field_install_port[LANGUAGE_NONE][0]['value'];
        } else {
          $port = FAMC_ANALYTICS_WS_DEFAULT_PORT;
        }

        // Get the cache options
        $client = mercury_management_wsdl_construct($hostname, $username, $password, $port);
        $param = array();
        $method = 'HgGetCacheOptions';
        $op_error_message = '';
        $op_result = mercury_management_consume_webservice($client, $param, $method);
        if (array_key_exists('error', $op_result)) {
// This error generates too many logs (metrics collected regularly). Commenting
// out for barium.
//	    foreach ($op_result as $get_result) {
//	  		$op_error_message = $op_error_message . ' ' . $get_result;
//	    }
//	    mercury_utilities_fmc_log_message(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, t('Failed to obtain cache options from VM @nid. @error', array('@nid' => $node->title, '@error' => $op_error_message)), WATCHDOG_ERROR);
          continue;
        }

        // Get the cache information and it includes the metrics
        $client = mercury_management_wsdl_construct($hostname, $username, $password, $port);
        $param = array();
        $method = 'HgGetCacheInfo';
        $ws_error_message = '';
        $ws_result = mercury_management_consume_webservice($client, $param, $method);
        if (array_key_exists('error', $ws_result)) {
// This error generates too many logs (metrics collected regularly). Commenting
// out for barium.
//	  	foreach ($ws_result as $get_result) {
//	  	  $ws_error_message = $ws_error_message . ' ' . $get_result;
//	    }
//	    mercury_utilities_fmc_log_message(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, t('Failed to obtain cache options from VM @nid. @error', array('@nid' => $node->title, '@error' => $ws_error_message)), WATCHDOG_ERROR);
          continue;
        }

        // If the webservice returns valid data, save as node
        if ($op_result && isset($op_result['response']) && $ws_result && isset($ws_result['response'])) {

          // Check to see if the reset interval was changed, if so update it
          if ($node->field_fa_metrics_reset_interval) {
            $current_resetinterval = $node->field_fa_metrics_reset_interval['und'][0]['value'];
          } else {
            $current_resetinterval = 0;
          }

          $options_info = $op_result['response']->HgGetCacheOptionsResult->HgCacheOptions;
          if ($current_resetinterval != $options_info->MetricsResetInterval) {
            $node->field_fa_metrics_reset_interval['und'][0]['value'] = $options_info->MetricsResetInterval;
            node_save($node);
          }
          // If metrics interval is 0, do nothing
          if ($options_info->MetricsResetInterval == 0) {
            continue;
          }

          // Process the perf data
          $processed_response = famc_analytics_process_vm_matrix_data($ws_result['response'], $op_result['response']);
          if ($processed_response) {
            $processed_response['nid'] = $node->nid;
            $saved = famc_analytics_save_vm_matrix_data($processed_response);
          }
        }
      } catch (Exception $e) {
        $success = FALSE;
// This error generates too many logs (metrics collected regularly). Commenting
// out for barium.
//          mercury_utilities_fmc_log_message(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, 'Failed to obtain cache metrics from VM @nid. @error', array('@nid' => $node->title, '@error' => $e->getMessage()), WATCHDOG_ERROR);
      }
    }
  }

  return $success;
}

/**
 * Create a empty VM metrics node with zero values. This function is called when FAMC 
 * agent in a VM is uninstalled. This results in an empty node in the database that
 * is then used to display zeroed perf information on the VM node page.
 */
function famc_save_empty_vm_metric($vm_nid) {

  // VM info
  $vmInfo = node_load($vm_nid);

  // The processed data to return
  $processed = array('metrics' => array(), 'disk_values' => array());
  $processed['nid'] = $vm_nid;

  // create emty metrics data
  $processed['metrics']['read_latency'] = 0;
  $processed['metrics']['write_latency'] = 0;
  $processed['metrics']['read_ops'] = 0;
  $processed['metrics']['write_ops'] = 0;
  $processed['metrics']['hits'] = 0;
  $processed['metrics']['read_blocks'] = 0;
  $processed['metrics']['write_blocks'] = 0;
  $processed['metrics']['usable_blocks'] = 0;
  $processed['metrics']['cache_latency'] = 0;
  $processed['metrics']['population'] = 0;
  $processed['metrics']['insertions'] = 0;

  // dummy disk data
  $processed['disk_values'][] = array();

  // write the database node
  famc_analytics_save_vm_matrix_data($processed);

  mercury_utilities_fmc_log_message(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, t('Metrics cleared for VM @vm.', array('@vm' => $vmInfo->title)), WATCHDOG_INFO);
}
