<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions to help with getting data from
 * or modifying nodes and other database entries.
 */

/**
 *  Little admin facility to delete all metrics
 */
function famc_analytics_clear_metrics() {

  famc_analytics_delete_metrics_by_number(FAMC_ANALYTICS_METRIC_ALL_CLEAR_COUNT);

  return t('All metrics are gone!');
}

function famc_analytics_delete_nodes($query_result) {

  if (is_array($query_result) && isset($query_result['node'])) {
    $nids = array();
    foreach ($query_result['node'] as $nid => $node) {
      $nids[] = $nid;
    }
    if (!empty($nids)) {
      node_delete_multiple($nids);
    }
  }
}

function famc_analytics_delete_metrics_by_number($max_records) {
  $query = new EntityFieldQuery();

  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', FAMC_ANALYTICS_VDISK_METRIC_TYPE)
    ->fieldOrderBy('field_timestamp', 'value', 'ASC')
    ->range(0, $max_records)
    ->propertyCondition('status', 1);

  $result = $query->execute();

  if (!is_array($result) || !isset($result['node'])) {
    return;
  }
  $node_array = $result['node'];
  $nid_last_record = end($node_array);
  $last_record = node_load($nid_last_record->nid);

  $time_stamp = field_get_items('node', $last_record, 'field_timestamp');
  $query->fieldCondition('field_timestamp', 'value', $time_stamp[0]['value'], '<=');

  $result = $query->execute();

  famc_analytics_delete_nodes($result);

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', FAMC_ANALYTICS_VM_METRIC_TYPE)
    ->fieldOrderBy('field_timestamp', 'value', 'ASC')
    ->propertyCondition('status', 1)
    ->fieldCondition('field_timestamp', 'value', $time_stamp[0]['value'], '<=');

  $result = $query->execute();
  famc_analytics_delete_nodes($result);
}

/**
 *  Clean up old records.
 */
function famc_analytics_clean_data($warning_level, $evict_records = FAMC_ANALYTICS_METRIC_CLEAR_COUNT) {
  $entities = array(FAMC_ANALYTICS_VM_METRIC_TYPE, FAMC_ANALYTICS_VDISK_METRIC_TYPE);
  if ($warning_level == "low") {
    foreach ($entities as $entity) {
      famc_analytics_delete_metrics_by_time($entity);
    }
  } else {
    famc_analytics_delete_metrics_by_number($evict_records);
  }
}

/**
 *  Deletes metric nodes
 *  @param String $type The type of metric to delete
 *  @param Integer $max_records The maximum number of records to delete. Passing 0 will delete all records older than $older_than
 *  @param Integer $older_than How old the metric should be to be deleted. In hours.
 *  @param array $entity_ids The VMs or vDisks whose metrics to delete
 */
function famc_analytics_delete_metrics_by_time($type, $older_than = FAMC_ANALYTICS_METRIC_RETAIN_PERIOD, $entity_ids = array()) {

  // We delete maximum 100 records per run by default
  $query = new EntityFieldQuery();

  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', $type)
    ->fieldOrderBy('field_timestamp', 'value', 'ASC')
    ->propertyCondition('status', 1);

  if (!empty($entity_ids)) {
    if ($type == FAMC_ANALYTICS_VDISK_METRIC_TYPE) {
      $field = 'field_vdisk';
    } else {
      $field = 'field_vm';
    }
    $query->fieldCondition($field, 'target_id', $entity_ids, 'IN');
  }

  /* Evict max_records older than 48 hours */
  $older_than = $older_than * 3600;
  $now = time();
  $delete_before = $now - $older_than;
  $query->fieldCondition('field_timestamp', 'value', $delete_before, '<=');

  $result = $query->execute();

  famc_analytics_delete_nodes($result);
}

/**
 *  Saves VM and vDisk metrics data to DB
 *  @param array $data {The processed data as array}
 */
function famc_analytics_save_vm_matrix_data($data) {

  $return = array('vm' => FALSE, 'disk' => FALSE);

  // Save VM metrics
  $vm_metric_fields = array(
    'read_latency' => 'vm_read_latency',
    'write_latency' => 'vm_write_latency',
    'read_ops' => 'vm_read_ops',
    'write_ops' => 'vm_write_ops',
    'hits' => 'vm_hits',
    'read_blocks' => 'vm_read_blocks',
    'write_blocks' => 'vm_write_blocks',
    'usable_blocks' => 'vm_usable_blocks',
    'cache_latency' => 'vm_cache_latency',
    'population' => 'vm_population',
    'insertions' => 'vm_insertions',
  );
  $node_language = LANGUAGE_NONE;
  $timestamp = time();
  $node = new stdClass();
  $node->title = $timestamp;
  $node->type = FAMC_ANALYTICS_VM_METRIC_TYPE;
  $node->language = $node_language;
  $node->created = $timestamp;
  $node->changed = $timestamp;
  $node->status = 1;
  $node->promote = 0;
  $node->sticky = 0;
  $node->format = 1;
  $node->revision = 0;
  $node->uid = 1;
  foreach ($vm_metric_fields as $metric => $vm_metric_field) {
    $node->{'field_' . $vm_metric_field}[$node_language][0]['value'] = $data['metrics'][$metric];
  }
  // Entity reference field
  $node->field_vm[$node_language][0]['target_id'] = $data['nid'];
  // Date field
  $node->field_timestamp[$node_language][0] = array(
    'value' => $timestamp,
    'show_todate' => 1,
    'timezone' => variable_get('date_default_timezone', 'UTC'),
    'value2' => $timestamp,
    'timezone_db' => variable_get('date_default_timezone', 'UTC'),
    'date_type' => 'datestamp',
  );
  node_submit($node);
  node_save($node);
  if (isset($node->nid) && $node->nid) {
    $return['vm'] = $node->nid;
  }

  // Save the disk metrics
  $disk_values = $data['disk_values'];
  if (!empty($disk_values)) {
    $return['disk'] = famc_analytics_save_disk_metrics_data($disk_values, $data['nid'], $timestamp);
  }

  return $return;
}

/**
 *  Saves the vDisk metrics as nodes
 */
function famc_analytics_save_disk_metrics_data($data, $vm_nid, $timestamp) {

  $return = TRUE;

  $vdisk_metric_fields = array(
    'read_latency' => 'vdisk_read_latency',
    'write_latency' => 'vdisk_write_latency',
    'read_ops' => 'vdisk_read_ops',
    'write_ops' => 'vdisk_write_ops',
    'hits' => 'vdisk_hits',
    'hit_ops' => 'hit_ops',
    'read_blocks' => 'vdisk_read_blocks',
    'write_blocks' => 'vdisk_write_blocks',
    'usable_blocks' => 'vdisk_usable_blocks',
    'cache_latency' => 'vdisk_cache_latency',
    'population' => 'vdisk_population',
    'insertions' => 'vdisk_insertions',
  );

  foreach ($data as $disk_index => $metrics) {
    $identifier = $metrics['identifier'];

    if (!empty($identifier)) {
      $disk = famc_analytics_get_vdisk_nodes($vm_nid, array('field_vdisk_identifier' => $identifier));

      if (!empty($disk)) {
        $disk = array_pop($disk); // The disk node

        $node_language = LANGUAGE_NONE;
        $node = new stdClass();
        $node->title = $timestamp;
        $node->type = FAMC_ANALYTICS_VDISK_METRIC_TYPE;
        $node->language = $node_language;
        $node->created = $timestamp;
        $node->changed = $timestamp;
        $node->status = 1;
        $node->promote = 0;
        $node->sticky = 0;
        $node->format = 1;
        $node->revision = 0;
        $node->uid = 1;
        foreach ($vdisk_metric_fields as $metric => $vdisk_metric_field) {
          $node->{'field_' . $vdisk_metric_field}[$node_language][0]['value'] = $metrics[$metric];
        }
        // Entity reference field
        $node->field_vdisk[$node_language][0]['target_id'] = $disk->nid;
        // Date field
        $node->field_timestamp[$node_language][0] = array(
          'value' => $timestamp,
          'show_todate' => 1,
          'timezone' => variable_get('date_default_timezone', 'UTC'),
          'value2' => $timestamp,
          'timezone_db' => variable_get('date_default_timezone', 'UTC'),
          'date_type' => 'datestamp',
        );

        node_submit($node);
        node_save($node);

        if (!$node) {
          mercury_utilities_fmc_log_message(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, 'Error saving metrics for disk [@identifier] of vm [Id - @vm_nid]', array('@identifier' => $identifier, '@vm_nid' => $vm_nid), WATCHDOG_ERROR);
          $return = FALSE;
        }
      }
    }
  }

  return $return;
}

/**
 *  Gets the vDisk node from the VM nid and other filters (mainly Disk Id or the Identifier)
 *  @param Integer $vm_nid {The nid of the VM}
 *  @param Array $filters {Array of fields to filter. Fields as indexes and field values as values}
 */
function famc_analytics_get_vdisk_nodes($vm_nid, $filters = array()) {

  $return = FALSE;

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', FAMC_ANALYTICS_DISK_NODE_TYPE)
    ->propertyCondition('status', 1)
    ->fieldCondition('field_virtual_machine', 'target_id', $vm_nid);

  if (!empty($filters)) {
    foreach ($filters as $field => $value) {
      $query->fieldCondition($field, 'value', $value);
    }
  }

  $result = $query->execute();

  if (is_array($result) && isset($result['node']) && sizeof($result['node'])) {
    $return = $result['node'];
  }

  return $return;
}

/**
 *  Returns the disk attributes and disk metrics for a specific VM
 */
function famc_analytics_vm_disk_info($vm_nid, $disk_nid = NULL, $enabled_only = TRUE) {

  $disk_nodes = array();

  if (!empty($disk_nid)) {
    $disk_node = new stdClass();
    $disk_node->nid = $disk_nid;
    $disk_nodes[] = $disk_node;
  } else {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', FAMC_ANALYTICS_DISK_NODE_TYPE)
      ->propertyCondition('status', 1)
      ->fieldCondition('field_virtual_machine', 'target_id', $vm_nid);
    $result = $query->execute();
    if (is_array($result) && isset($result['node'])) {
      $disk_nodes = $result['node'];
    }
  }

  $info = array();
  foreach ($disk_nodes as $disk_node) {
    $disk_node = node_load($disk_node->nid);
    if (isset($disk_node->nid)) {

      $info[$disk_node->nid] = array(
        'admin_state' => famc_analytics_string_overrides(array_pop(array_pop(field_get_items('node', $disk_node, 'field_fa_administration_state')))),
        'op_state' => famc_analytics_string_overrides(array_pop(array_pop(field_get_items('node', $disk_node, 'field_fa_operational_state')))),
        'cache' => t('N/A'),
        'read_latency' => t('N/A'),
        'write_latency' => t('N/A'),
        'hit_rate' => t('N/A'),
      );

      if (famc_analytics_metric_enabled($disk_node)) {
        $info[$disk_node->nid] = array(
          'admin_state' => famc_analytics_string_overrides(array_pop(array_pop(field_get_items('node', $disk_node, 'field_fa_administration_state')))),
          'op_state' => famc_analytics_string_overrides(array_pop(array_pop(field_get_items('node', $disk_node, 'field_fa_operational_state')))),
          'cache' => 0,
          'read_latency' => 0,
          'write_latency' => 0,
          'hit_rate' => 0,
        );
        // Get the latest metric
        $query = new EntityFieldQuery();
        $query->entityCondition('entity_type', 'node')
          ->entityCondition('bundle', FAMC_ANALYTICS_VDISK_METRIC_TYPE)
          ->propertyCondition('status', 1)
          ->fieldCondition('field_vdisk', 'target_id', $disk_node->nid)
          ->fieldOrderBy('field_timestamp', 'value', 'DESC')
          ->range(0, 1);
        $result = $query->execute();
        if (is_array($result) && isset($result['node']) && sizeof($result['node'])) {
          $item = array_pop($result['node']);
          $metric_node = node_load($item->nid);
          $info[$disk_node->nid]['read_latency'] = array_pop(array_pop(field_get_items('node', $metric_node, 'field_vdisk_read_latency')));
          $info[$disk_node->nid]['write_latency'] = array_pop(array_pop(field_get_items('node', $metric_node, 'field_vdisk_write_latency')));
          $info[$disk_node->nid]['hit_rate'] = array_pop(array_pop(field_get_items('node', $metric_node, 'field_vdisk_hit_rate_pc')));
        }
      }
    }
  }

  return $info;
}

/**
 * Get VM, Host nodes
 */
function famc_analytics_get_entities($entity_type = FAMC_ANALYTICS_VM_NODE_TYPE, $etids = array()) {

  $return = array();

  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', $entity_type)
    ->propertyCondition('status', 1);

  if (!empty($etids)) {
    $query->entityCondition('entity_id', $etids, 'IN');
  }
  $result = $query->execute();

  if (is_array($result) && isset($result['node']) && is_array($result['node'])) {
    $return = $result['node'];
  }

  return $return;
}

/**
 *  Checks whether metrics is enabled on a VM or vDisk (or other stuff in future, probably!)
 *  If cache is enabled and metric is enabled then returns TRUE
 *  @param int/object $entity {The nid of the VM or vDisk or the fully loaded node object}
 */
function famc_analytics_metric_enabled($entity) {

  $return = TRUE;

  if (is_numeric($entity)) {
    $entity = node_load($entity);
  }

  switch ($entity->type) {

    case FAMC_ANALYTICS_VM_NODE_TYPE:
    case FAMC_ANALYTICS_DISK_NODE_TYPE:

      $admin_state = field_get_items('node', $entity, 'field_fa_administration_state');
      if ($admin_state) {
        $admin_state = array_pop(array_pop($admin_state));
        if ($admin_state != FAMC_ANALYTICS_ENABLED) {
          $return = FALSE;
        } else {
          $perf_state = field_get_items('node', $entity, 'field_fa_performance_state');
          if ($perf_state) {
            $perf_state = array_pop(array_pop($perf_state));
            if ($perf_state != FAMC_ANALYTICS_ENABLED) {
              $return = FALSE;
            }
          } else {
            $return = FALSE;
          }
        }
      } else {
        $return = FALSE;
      }
  }

  return $return;
}

/**
 *  Retrieves the most recent metrics for a VM
 */
function famc_analytics_get_current_vm_metrics($vm_nid) {

  $return = array();

  if (is_numeric($vm_nid)) {
    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', FAMC_ANALYTICS_VM_METRIC_TYPE)
      ->propertyCondition('status', 1)
      ->fieldCondition('field_vm', 'target_id', $vm_nid)
      ->fieldOrderBy('field_timestamp', 'value', 'DESC')
      ->range(0, 1);

    $result = $query->execute();

    if (is_array($result) && isset($result['node']) && sizeof($result['node'])) {
      $item = array_pop($result['node']);
      $node = node_load($item->nid);

      if ($node) {
        $return = famc_analytics_extract_vm_metrics_from_node($node);
      }
    }
  }

  return $return;
}

/**
 *  Get VM metrics for a specific time
 */
function famc_analytics_get_vm_metrics($vm_nid, $timestamp) {

  // Cache metrics on the same page
  $metrics = &drupal_static(__FUNCTION__ . '--' . $vm_nid . '--' . $timestamp);

  if (!$metrics) {

    $metrics = array(
      'read_latency' => 0,
      'write_latency' => 0,
      'read_ops' => 0,
      'write_ops' => 0,
      'hit_rate_pc' => 0,
      'cache_latency' => 0,
      'population_pc' => 0,
      'insertion_rate_pc' => 0,
    );

    $query = new EntityFieldQuery();
    $query->entityCondition('entity_type', 'node')
      ->entityCondition('bundle', FAMC_ANALYTICS_VM_METRIC_TYPE)
      ->propertyCondition('status', 1)
      ->fieldCondition('field_vm', 'target_id', $vm_nid)
      ->fieldCondition('field_timestamp', 'value', $timestamp, '<=')
      ->fieldOrderBy('field_timestamp', 'value', 'DESC')
      ->range(0, 1);
    $result = $query->execute();

    if (is_array($result) && isset($result['node']) && sizeof($result['node'])) {
      $item = array_pop($result['node']);
      $node = node_load($item->nid);

      $metrics = famc_analytics_extract_vm_metrics_from_node($node);
    }
  }

  return $metrics;
}

/**
 *  Alias for famc_fetch_vm_metric_multiple, for fetching metric for single VM
 *  @param int etid {The entity id of the VM}
 */
function famc_analytics_fetch_vm_metric($etid) {

  return famc_fetch_vm_metric_multiple(array($etid));
}

/**
 * Extract the metrics array from VM metric node
 */
function famc_analytics_extract_vm_metrics_from_node($node) {

  $read_latency = array_pop(array_pop(field_get_items('node', $node, 'field_vm_read_latency')));
  $write_latency = array_pop(array_pop(field_get_items('node', $node, 'field_vm_write_latency')));
  $read_ops = array_pop(array_pop(field_get_items('node', $node, 'field_vm_read_ops')));
  $write_ops = array_pop(array_pop(field_get_items('node', $node, 'field_vm_write_ops')));
  $hit_rate_pc = array_pop(array_pop(field_get_items('node', $node, 'field_vm_hit_rate_pc')));
  $cache_latency = array_pop(array_pop(field_get_items('node', $node, 'field_vm_cache_latency')));
  $population_pc = array_pop(array_pop(field_get_items('node', $node, 'field_vm_population_pc')));
  $insertion_rate_pc = array_pop(array_pop(field_get_items('node', $node, 'field_vm_insertion_rate_pc')));


  return array(
    'read_latency' => $read_latency,
    'write_latency' => $write_latency,
    'read_ops' => $read_ops,
    'write_ops' => $write_ops,
    'hit_rate_pc' => $hit_rate_pc,
    'cache_latency' => $cache_latency,
    'population_pc' => $population_pc,
    'insertion_rate_pc' => $insertion_rate_pc,
  );
}
