<?php

define('WIN2K8R2', 'windows7server64guest');
define('WIN2K12', 'windows8server64guest');
define('RHEL6', 'rhel6_64guest');
define('SUSE11', 'sles11_64guest');

/**
 *  generates the fa-config.xml
 *  this file will contain the deployment and configuration information of FA 
 *
 *  this function will return the config in XML format
*/
function gen_fa_config()
{
  // for each vcenter in FMC database
  // for each datacenter in vcenter
  // 	collect each esx property

  $hosts = array();
  $host_names = array();

  // enumerate all vcenters from FMC database
  $query_str = "SELECT * FROM {node} WHERE type='vcenter'";
  $vcenter_query = db_query($query_str);
  foreach ($vcenter_query as $vcenter) {
    $vcenter_info = node_load($vcenter->nid);
    $vc_username = fmc_cipher::decrypt($vcenter_info->field_username['und'][0]['value']);
    $vc_password = fmc_cipher::decrypt($vcenter_info->field_password['und'][0]['value']);
    $vc_hostname = $vcenter_info->field_hostname['und'][0]['value'];

    // enumerate all DCs from the current vcenter object
    $dclist = array();
    $result = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --operation dclist", $dclist);

    foreach ($dclist as $dc) {
      $hostlist = array();

      // enumerate all ESX hosts from current DC
      $result = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --datacenter '" . $dc . "' --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --operation hostlist", $hostlist);

      foreach ($hostlist as $value) {
	$val_split = explode(':', $value);
	if (count($val_split) <= 1) {
	  continue;
	}
	$host  = trim($val_split[1]);
	array_push($host_names, $host);
	$hosts[$host]['dc'] = $dc;

	// get ESX version
	$esx_version = array();
	$result = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl  --hostname $host --url https://" . $vc_hostname . "/sdk/vimService --username '" . $vc_username . "' --password '" . $vc_password . "' --operation hostinfo | grep 'Software on host' | cut -f2 -d':'", $esx_version);
	$hosts[$host]['host_ver'] = trim(reset($esx_version)); 

	// get VMs belonging to the current ESX host
	$vmlist = array();
	$result = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService --username '" . $vc_username . "' --password '" . $vc_password . "' --operation vmlist | sed -n \"/Host : $host/,/Host :/p\"", $vmlist);
	$hosts[$host]['host_vm_count'] = 0;
	foreach ($vmlist as $value) {
	  $val_split = explode('VM :', $value);
	  if (count($val_split) <= 1) {
	    continue;
	  }
	  $hosts[$host]['host_vm_count'] += 1;
	  // VM name is unused at the moment
	  $currVM = trim($val_split[1]);
	}
      }
    }
  }

  // enumerate all ESX servers from FMC database
  $query_str = "SELECT * FROM {node} WHERE type='host'";
  $host_query = db_query($query_str);
  foreach ($host_query as $host_prop) {
    $host_nid = $host_prop->nid;
    $host_info = node_load($host_nid);
    $host_name = $host_prop->title;
    $bundle_install = field_get_items('node', $host_info, 'field_bundle_status');
    $alloc_segments = $host_info->field_allocated_segments['und'][0]['value'];
    $segment_size = $host_info->field_segment_size['und'][0]['value'];
    $total_segments = $host_info->field_total_segments['und'][0]['value'];
    $host_vmotion_status = $host_info->field_vmotion['und'][0]['value'];

    // get FMC install status
    if ($bundle_install[0]['value'] == "Agent Installed") {
      $hosts[$host_name]['host_fa_installed'] = 'yes';
    } else {
      $hosts[$host_name]['host_fa_installed'] = 'no';
    }

    // get SSD size
    $hosts[$host_name]['host_ssd_size'] = ($total_segments * $segment_size) / (1024 * 1024 * 1024);

    // get allocated cache size
    $hosts[$host_name]['host_cache_size'] = ($alloc_segments * $segment_size) / (1024 * 1024 * 1024);

    // get host vmotion status
    $hosts[$host_name]['host_vmotion_status'] = $host_vmotion_status;
  }
	
  // enumerate all VMs from FMC database
  $query_str = "SELECT * FROM {node} WHERE type='virtual_machine'";
  $vm_query = db_query($query_str);
  foreach ($vm_query as $vm_prop) {
    $vm_id = $vm_prop->nid;
    $vmInfo = node_load($vm_id);
    $host_nid = $vmInfo->field_host['und'][0]['target_id'];
    $host_info = node_load($host_nid);
    if (!$host_info) {
      continue;
    }
    $host_name = $host_info->title;

    // capture the count for each supported OS
    $guest_id = trim($vmInfo->field_guest_id['und'][0]['value']);
    if (!empty($guest_id)) {
      if (stristr($guest_id, WIN2K8R2)) {
	if (isset($hosts[$host_name][WIN2K8R2])) {
	  $hosts[$host_name][WIN2K8R2] += 1;
	} else {
	  $hosts[$host_name][WIN2K8R2] = 1;
	}
      }

      if (stristr($guest_id, WIN2K12)) {
	if (isset($hosts[$host_name][WIN2K12])) {
	  $hosts[$host_name][WIN2K12] += 1;
	} else {
	  $hosts[$host_name][WIN2K12] = 1;
	}
      }
      
      if (stristr($guest_id, RHEL6)) {
	if (isset($hosts[$host_name][RHEL6])) {
	  $hosts[$host_name][RHEL6] += 1;
	} else {
	  $hosts[$host_name][RHEL6] = 1;
	}
      }
 
      if (stristr($guest_id, SUSE11)) {
	if (isset($hosts[$host_name][SUSE11])) {
	  $hosts[$host_name][SUSE11] += 1;
	} else {
	  $hosts[$host_name][SUSE11] = 1;
	}
      }

    }

    // Get the VM cache size
    $block_size = field_get_items('node', $vmInfo, 'field_fa_sector_size');
    $num_blocks = field_get_items('node', $vmInfo, 'field_fa_cache_size');
    $cache_size = ($num_blocks[0]['value'] * $block_size[0]['value']) / (1024 * 1024 * 1024);

    // FA enabled count = agent is installed and cache is allocated
    if ($vmInfo->field_driver_status['und'][0]['value'] == 'Agent Installed' &&
	$cache_size > 0) {
      if (isset($hosts[$host_name]['fa_enabled_vm_count'])) {
	$hosts[$host_name]['fa_enabled_vm_count'] += 1;
      } else {
	$hosts[$host_name]['fa_enabled_vm_count'] = 1;
      }
    }

    // get vmotion enabled VM count
    if ($vmInfo->field_vm_vmotionable['und'][0]['value'] != 0) {
      if (isset($hosts[$host_name]['vmotion_enabled_vm_count'])) {
	$hosts[$host_name]['vmotion_enabled_vm_count'] += 1;
      } else {
	$hosts[$host_name]['vmotion_enabled_vm_count'] = 1;
      }
    }

    // get min, max and median VM cache sizes
    if ($cache_size > 0) {
      if ((!isset($hosts[$host_name]['vm_min_cache_size'])) || 
	  $cache_size < $hosts[$host_name]['vm_min_cache_size']) {
	$hosts[$host_name]['vm_min_cache_size'] = $cache_size;
      }

      // store cache size if larger
      if ((!isset($hosts[$host_name]['vm_max_cache_size'])) || 
	  $cache_size > $hosts[$host_name]['vm_max_cache_size']) {
	$hosts[$host_name]['vm_max_cache_size'] = $cache_size;
      }

      if (empty($hosts[$host_name]['vm_median_cache_size_array'])) {
	$hosts[$host_name]['vm_median_cache_size_array'] = array();
      }
      array_push($hosts[$host_name]['vm_median_cache_size_array'], $cache_size);
    }
  }

  // let's get the cluster name of each ESX host
  $clusterlist = array();
  $result = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService --username '" . $vc_username . "' --password '" . $vc_password . "' --operation clusterlist", $clusterlist);

  foreach ($clusterlist as $cluster) {
    $hostlist = array();
    $result = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService --username '" . $vc_username . "' --password '" . $vc_password . "' --operation hostlist --cluster $cluster", $hostlist);
    
    foreach ($hostlist as $value) {
      $val_split = explode(':', $value);
      if (count($val_split) <= 1) {
	continue;
      }
      $host  = trim($val_split[1]);
      $hosts[$host]['cluster'] = $cluster;
    }
  }

  // calculate VM cache median size	
  foreach ($host_names as $host) {
    if (!isset($hosts[$host]['vm_median_cache_size_array'])) {
      continue;
    }
    sort($hosts[$host]['vm_median_cache_size_array']);
    $len = count($hosts[$host]['vm_median_cache_size_array']);
    $mid = $len / 2;
    if ($len % 2) {
      $median = $hosts[$host]['vm_median_cache_size_array'][$mid];
    }
    else {
      $median = ($hosts[$host]['vm_median_cache_size_array'][$mid-1] + 
		 $hosts[$host]['vm_median_cache_size_array'][$mid]) / 2;
    }
    $hosts[$host]['vm_median_cache_size'] = $median;
  }

  $oldval = ini_set('precision', 20);
  // ASUP time format
  $seconds = microtime(true) * 1000000; 
  ini_set('precision', $oldval);

  // generate XML content for ASUP    
  $hdr = gen_smf_header($seconds);
  $hdr .= gen_fa_content($host_names, $hosts, $seconds);
  $hdr .= "</flashaccel_config>\n";
	
  return  $hdr;
}


function gen_smf_header($seconds) {

	$xml = "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n";
	$xml .= "<!DOCTYPE flashaccel_config [\n";
	$xml .= "<!ELEMENT flashaccel_config (asup:TABLE_INFO,asup:ROW*)>\n";
	$xml .= "<!ATTLIST flashaccel_config xmlns CDATA #FIXED \"http://asup_search.netapp.com/ns/flashaccel_config/1.0\">\n";
	$xml .= "<!ATTLIST flashaccel_config xmlns:asup CDATA #FIXED \"http://asup_search.netapp.com/ns/ASUP/1.0\">\n";
	$xml .= "<!ELEMENT asup:TABLE_INFO (asup:field+)>\n";
	$xml .= "<!ELEMENT asup:field (asup:tag,asup:ui_name,asup:type)>\n";
	$xml .= "<!ELEMENT asup:tag (#PCDATA)>\n";
	$xml .= "<!ELEMENT asup:ui_name (#PCDATA)>\n";
	$xml .= "<!ELEMENT asup:type (#PCDATA)>\n";
	$xml .= "<!ELEMENT asup:ROW (host-version,datacenter,cluster,fa-installed,ssd-size,cache-size,host-vmotion-enabled,host-drs-enabled,host-ha-enabled,faenabled-vm-count,host-vm-count,vmotion-enabled-vm-count,vm-windows2008-count,vm-windows2012-count,vm-rhel6-count,vm-suse11-count,vm-min-cache-size,vm-max-cache-size,vm-median-cache-size)>\n";
	$xml .= "<!ELEMENT host-version (#PCDATA)>\n";
	$xml .= "<!ELEMENT datacenter (#PCDATA)>\n";
	$xml .= "<!ELEMENT cluster (#PCDATA)>\n";
	$xml .= "<!ELEMENT fa-installed (#PCDATA)>\n";
	$xml .= "<!ELEMENT ssd-size (#PCDATA)>\n";
	$xml .= "<!ELEMENT cache-size (#PCDATA)>\n";
	$xml .= "<!ELEMENT host-vmotion-enabled (#PCDATA)>\n";
	$xml .= "<!ELEMENT host-drs-enabled (#PCDATA)>\n";
	$xml .= "<!ELEMENT host-ha-enabled (#PCDATA)>\n";
	$xml .= "<!ELEMENT faenabled-vm-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT host-vm-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT vmotion-enabled-vm-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-windows2008-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-windows2012-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-rhel6-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-suse11-count (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-min-cache-size (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-max-cache-size (#PCDATA)>\n";
	$xml .= "<!ELEMENT vm-median-cache-size (#PCDATA)>]>\n";
	$xml .= "<flashaccel_config xmlns=\"http://asup_search.netapp.com/ns/flashaccel_config/1.0\" xmlns:asup=\"http://asup_search.netapp.com/ns/ASUP/1.0\">\n";
	$xml .= "<asup:TABLE_INFO>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>host-version</asup:tag>\n";
	$xml .= "<asup:ui_name>ESX Host Version</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>datacenter</asup:tag>\n";
	$xml .= "<asup:ui_name>Vmware DataCenter containing the ESX Host</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>cluster</asup:tag>\n";
	$xml .= "<asup:ui_name>Vmware Cluster containing the ESX Host</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>fa-installed</asup:tag>\n";
	$xml .= "<asup:ui_name>FlashAccel Installed Status</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>ssd-size</asup:tag>\n";
	$xml .= "<asup:ui_name>SSD Size</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>cache-size</asup:tag>\n";
	$xml .= "<asup:ui_name>Allocated Cache Size</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>host-vmotion-enabled</asup:tag>\n";
	$xml .= "<asup:ui_name>Host VMotion Enabled Status</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>host-drs-enabled</asup:tag>\n";
	$xml .= "<asup:ui_name>Host DRS Enabled Status</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>host-ha-enabled</asup:tag>\n";
	$xml .= "<asup:ui_name>Host HA Enabled Status</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>faenabled-vm-count</asup:tag>\n";
	$xml .= "<asup:ui_name>FlashAccel Enabled VM Count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>host-vm-count</asup:tag>\n";
	$xml .= "<asup:ui_name>ESX Host total VM count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vmotion-enabled-vm-count</asup:tag>\n";
	$xml .= "<asup:ui_name>VMotion Enabled VM Count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-windows2008-count</asup:tag>\n";
	$xml .= "<asup:ui_name>FlashAccel Enabled Windows 2008 R2 VM count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-windows2012-count</asup:tag>\n";
	$xml .= "<asup:ui_name>FlashAccel Enabled Windows 2012 VM count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-rhel6-count</asup:tag>\n";
	$xml .= "<asup:ui_name>FlashAccel Enabled RHEL6 VM count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-suse11-count</asup:tag>\n";
	$xml .= "<asup:ui_name>FlashAccel Enabled SUSE11 VM count</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-min-cache-size</asup:tag>\n";
	$xml .= "<asup:ui_name>VM Minimum Cache Size</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-max-cache-size</asup:tag>\n";
	$xml .= "<asup:ui_name>VM Maximum Cache Size</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "<asup:field>\n";
	$xml .= "<asup:tag>vm-median-cache-size</asup:tag>\n";
	$xml .= "<asup:ui_name>VM Median Cache Size</asup:ui_name>\n";
	$xml .= "<asup:type>text</asup:type>\n";
	$xml .= "</asup:field>\n";
	$xml .= "</asup:TABLE_INFO>\n";

	return $xml;
}


function gen_fa_content($host_names, $hosts, $seconds) {

	$content = "";
	foreach ($host_names as $host) {
		$content .= "<asup:ROW>\n";
		$content .= "\t<host-version>" . $hosts[$host]['host_ver'] . "</host-version>\n";
		$content .= "\t<datacenter>" . (isset($hosts[$host]['dc']) ? $hosts[$host]['dc'] : "") . "</datacenter>\n";
		$content .= "\t<cluster>" . (isset($hosts[$host]['cluster']) ? $hosts[$host]['cluster'] : "") . "</cluster>\n";
		$content .= "\t<fa-installed>" . (isset($hosts[$host]['host_fa_installed']) ? $hosts[$host]['host_fa_installed'] : "") . "</fa-installed>\n";

		if (isset($hosts[$host]['host_ssd_size'])) {
			$content .= "\t<ssd-size>" . $hosts[$host]['host_ssd_size'] . ($hosts[$host]['host_ssd_size'] == 0 ? "" : " GB") . "</ssd-size>\n";
		} else {
			$content .= "\t<ssd-size>" . "</ssd-size>\n";
		} 

		if (isset($hosts[$host]['host_cache_size'])) {
			$content .= "\t<cache-size>" . $hosts[$host]['host_cache_size'] . ($hosts[$host]['host_cache_size'] == 0 ? "" : " GB") . "</cache-size>\n";
		} else {
			$content .= "\t<cache-size>" . "</cache-size>\n";
		}

		if (isset($hosts[$host]['host_vmotion_status'])) {
			$content .= "\t<host-vmotion-enabled>" . $hosts[$host]['host_vmotion_status'] . "</host-vmotion-enabled>\n";
		}
		else {
			$content .= "\t<host-vmotion-enabled>" .  "</host-vmotion-enabled>\n";
		}
		// TODO : DRS and HA are pending
		$content .= "\t<host-drs-enabled>" . "</host-drs-enabled>\n";
		$content .= "\t<host-ha-enabled>" . "</host-ha-enabled>\n";

		if (isset($hosts[$host]['fa_enabled_vm_count'])) {
			$content .= "\t<faenabled-vm-count>" . $hosts[$host]['fa_enabled_vm_count'] . "</faenabled-vm-count>\n";
		} else {
			$content .= "\t<faenabled-vm-count>" . "</faenabled-vm-count>\n";
		}
		$content .= "\t<host-vm-count>" . $hosts[$host]['host_vm_count'] . "</host-vm-count>\n";
		if (isset($hosts[$host]['vmotion_enabled_vm_count'])) {
			$content .= "\t<vmotion-enabled-vm-count>" . $hosts[$host]['vmotion_enabled_vm_count'] . "</vmotion-enabled-vm-count>\n";
		} else {
			$content .= "\t<vmotion-enabled-vm-count>" . "</vmotion-enabled-vm-count>\n";
		}

		if (isset($hosts[$host][WIN2K8R2])) {
			$content .= "\t<vm-windows2008-count>" . $hosts[$host][WIN2K8R2] . "</vm-windows2008-count>\n";
		} else {
			$content .= "\t<vm-windows2008-count>" . "</vm-windows2008-count>\n";
		}

		if (isset($hosts[$host][WIN2K12])) {
			$content .= "\t<vm-windows2012-count>" . $hosts[$host][WIN2K12] . "</vm-windows2012-count>\n";
		} else {
			$content .= "\t<vm-windows2012-count>" . "</vm-windows2012-count>\n";
		}

		if (isset($hosts[$host][RHEL6])) {
			$content .= "\t<vm-rhel6-count>" . $hosts[$host][RHEL6] . "</vm-rhel6-count>\n";
		} else {
			$content .= "\t<vm-rhel6-count>" . "</vm-rhel6-count>\n";
		}

		if (isset($hosts[$host][SUSE11])) {
			$content .= "\t<vm-suse11-count>" . $hosts[$host][SUSE11] . "</vm-suse11-count>\n";
		} else {
			$content .= "\t<vm-suse11-count>" . "</vm-suse11-count>\n";
		}

		if (isset($hosts[$host]['vm_min_cache_size'])) {
			$content .= "\t<vm-min-cache-size>" . $hosts[$host]['vm_min_cache_size'] . ($hosts[$host]['vm_min_cache_size'] == 0 ? "" : " GB") . "</vm-min-cache-size>\n";
		} else {
			$content .= "\t<vm-min-cache-size>" . "</vm-min-cache-size>\n";
		}

		if (isset($hosts[$host]['vm_max_cache_size'])) {
			$content .= "\t<vm-max-cache-size>" . $hosts[$host]['vm_max_cache_size'] . ($hosts[$host]['vm_max_cache_size'] == 0 ? "" : " GB") . "</vm-max-cache-size>\n";
		} else {
			$content .= "\t<vm-max-cache-size>" . "</vm-max-cache-size>\n";
		}

		if (isset($hosts[$host]['vm_median_cache_size'])) {
			$content .= "\t<vm-median-cache-size>" . $hosts[$host]['vm_median_cache_size'] . ($hosts[$host]['vm_median_cache_size'] == 0 ? "" : " GB") . "</vm-median-cache-size>\n";
		} else {
			$content .= "\t<vm-median-cache-size>" . "</vm-median-cache-size>\n";
		}
		$content .= "</asup:ROW>\n";
	}
	return  $content;
}

