<?php

dirname(__FILE__);
include(dirname(__FILE__)."/../mercury_management/mercury_management.inc");

/**
 *  generates the fa-perf-metrics.txt
 *  this file will contain the cache status of FA VMs
 *
 *  this function will return the cache status in text format
*/
function gen_fa_perf_metrics()
{
	$cache_data = "";

	// enumerate all VMs from FMC database
	$query_str = "SELECT * FROM {node} WHERE type='virtual_machine'";
	$vm_query = db_query($query_str);

	$hostname = "";
	$username = "";
	$password = "";
	$port = '';
	foreach ($vm_query as $vm_prop) {
		$vm_id = $vm_prop->nid;
		$vm_info = node_load($vm_id);

		$hostname = mercury_management_get_vm_address($vm_info);
		$username = mercury_management_get_vm_username($vm_info);
		$password = mercury_management_get_vm_password($vm_info);
		$port     = mercury_management_get_vm_port($vm_info);

		module_load_include('inc', 'mercury_management');

		// Get the cache options
		$client = mercury_management_wsdl_construct($hostname, $username, $password, $port);
		$param = array();
		$method = 'HgGetCacheOptions';
		$op_error_message = '';
		$op_result = mercury_management_consume_webservice($client, $param, $method);	  
		if (array_key_exists('error', $op_result)) {
			foreach ($op_result as $get_result) {
				$op_error_message = $op_error_message . ' ' . $get_result;
			}
			watchdog(FAMC_ASUP_MODULE_DISPLAY_NAME, t('AutoSupport - unable to obtain cache options from VM @nid. @error', 
				array('@nid' => $vm_info->title, '@error' => $op_error_message)));
			continue;
		}

		// Get the cache information and it includes the metrics
		$client = mercury_management_wsdl_construct($hostname, $username, $password, $port);
		$param = array();
		$method = 'HgGetCacheInfo';
		$ws_error_message = '';
		$ws_result = mercury_management_consume_webservice($client, $param, $method);

		if (array_key_exists('error', $ws_result)) {
			foreach ($ws_result as $get_result) {
				$ws_error_message = $ws_error_message . ' ' . $get_result;
			}
			
			watchdog(FAMC_ASUP_MODULE_DISPLAY_NAME, t('AutoSupport - unable to obtain cache options from VM @nid. @error', 
				array('@nid' => $vm_info->title, '@error' => $ws_error_message)));
			continue;
		}

		if ($op_result && isset($op_result['response']) && $ws_result && isset($ws_result['response']) ) {

			// Process the perf data
			$processed_response = famc_asup_process_vm_cache_data($ws_result['response'], $op_result['response']);

			if ($processed_response) {
				$cache_data .= "[" . $vm_info->title . "]\n";
				$cache_data .= "BuildVersion          : " . $processed_response['attributes']['build_version'] . "\n"; 
				$cache_data .= "CacheMode             : " . $processed_response['attributes']['cache_mode'] . "\n";
				$cache_data .= "CacheAdminState       : " . $processed_response['attributes']['admin_state'] . "\n";
				$cache_data .= "CacheSize             : " . $processed_response['attributes']['cache_size'] . "\n";
				$cache_data .= "CacheOperationalState : " . $processed_response['attributes']['op_state'] . "\n"; 
				$cache_data .= "OperStateMessage      : " . $processed_response['attributes']['op_message'] . "\n"; 
				$cache_data .= "PerfState             : " . $processed_response['attributes']['perf_state'] . "\n"; 
          
				$cache_data .= "VMReadLatency         : " . $processed_response['metrics']['read_latency'] . "\n";
				$cache_data .= "VMWriteLatency        : " . $processed_response['metrics']['write_latency'] . "\n"; 
				$cache_data .= "VMReadOps             : " . $processed_response['metrics']['read_ops'] . "\n";
				$cache_data .= "VMWriteOps            : " . $processed_response['metrics']['write_ops'] . "\n";
				$cache_data .= "VMHits                : " . $processed_response['metrics']['hits'] . "\n"; 
				$cache_data .= "VMReadBlocks          : " . $processed_response['metrics']['read_blocks'] . "\n";
				$cache_data .= "VMWriteBlocks         : " . $processed_response['metrics']['write_blocks'] . "\n";
				$cache_data .= "VMUsableBlocks        : " . $processed_response['metrics']['usable_blocks'] . "\n"; 
				$cache_data .= "VMCacheLatency        : " . $processed_response['metrics']['cache_latency'] . "\n"; 
				$cache_data .= "VMPopulation          : " . $processed_response['metrics']['population'] . "\n"; 
				$cache_data .= "VMInsertions          : " . $processed_response['metrics']['insertions'] . "\n"; 
				$cache_data .= "Metrics Reset Interval: " . $processed_response['metrics']['reset_interval'] . "\n";
    			$cache_data .= "\n"; 
			}
		}
	}
	return $cache_data;
}



  /**
   *  Processes the web-service response and returns the proper fields
   *  @param $response object {The response object}
   *
   *  This is an exact copy of famc_analytics_process_vm_matrix_data in analytics module
   */
function famc_asup_process_vm_cache_data($response, $op_response) {
    
  // The processed data to return
  $processed = array('attributes' => array(), 'metrics' => array(), 'disk_values' => array(), 'options' => array());
    
  try {
 
    // Get the cache options
    $options_info = $op_response->HgGetCacheOptionsResult->HgCacheOptions;
    // Get the global cache info
    $vm_info = $response->HgGetCacheInfoResult->GlobalCacheInfo;
    // Get the Disk metrics
    $disk_info = $response->HgGetCacheInfoResult->DiskCacheInfoList->DiskCacheInfo;
      
    // Find the VM metrics values by averaging the disk metrics
    $vm_read_latency = 0;
    $vm_write_latency = 0;
    $vm_read_ops = 0;
    $vm_write_ops = 0;
    $vm_hits = 0;
    $vm_read_blocks = 0;
    $vm_write_blocks = 0;
    $vm_usable_blocks = $vm_info->FormatInfo->UsableBlocks;
    $vm_cache_latency = 0;
    $vm_population = 0;
    $vm_insertions = 0;
    $vm_hit_ops=0;

    $vdisk_count = 0;
    foreach ($disk_info as $disk_metric) {
        // We need to consider only the enabled devices where performance monitoring is enabled
        if ($disk_metric->AdminState == FAMC_ANALYTICS_ENABLED && $disk_metric->PerfState == FAMC_ANALYTICS_ENABLED) {
          $metric_object = $disk_metric->Metrics;

	  // Convert all Ops to be Ops/Sec
    if ($options_info->MetricsResetInterval <= 0) {
      // Avoid divide by zero.
      $metric_object->ReadOps = 0;
      $metric_object->WriteOps = 0;
      $metric_object->HitOps = 0;
    } else {
      $metric_object->ReadOps /= $options_info->MetricsResetInterval;
      $metric_object->WriteOps /= $options_info->MetricsResetInterval;
      $metric_object->HitOps /= $options_info->MetricsResetInterval;
    }

	  // calculate VM wide read latency
	  $total_read_time   = ($vm_read_latency * $vm_read_ops) + (round($metric_object->ReadLatencyPerOp / 1000, 1) * $metric_object->ReadOps);
	  $vm_read_ops      += $metric_object->ReadOps;
	  if ($vm_read_ops > 0) {
	    $vm_read_latency   = $total_read_time / $vm_read_ops;
	  }

	  // calculate VM wide write latency
	  $total_write_time  = ($vm_write_latency * $vm_write_ops) + (round($metric_object->WriteLatencyPerOp / 1000, 1) * $metric_object->WriteOps);
          $vm_write_ops     += $metric_object->WriteOps;
	  if ($vm_write_ops > 0) {
	    $vm_write_latency = $total_write_time / $vm_write_ops;
	  }

	  $total_hit_time = ($vm_cache_latency * $vm_hit_ops) + (round($metric_object->AvgHitLatency / 1000, 1) * $metric_object->HitOps);
	  $vm_hit_ops += $metric_object->HitOps;
	  if ($vm_hit_ops > 0) {
	    $vm_cache_latency = $total_hit_time / $vm_hit_ops;
	  }

          # Pursuant to barium burt 720667 (FMC: change hit rate computation 
          # to use OPs instead of Blocks for accuracy): we will keep track of
          # HitOps instead of Hits in the VM metrics node to avoid a database
          # change.  In copper+ we should add a field and track both,
          # regardless of how we calculate hit rate.
          # $vm_hits          += $metric_object->Hits;
          $vm_hits           = $vm_hit_ops;

          $vm_read_blocks   += $metric_object->ReadBlocks;
          $vm_write_blocks  += $metric_object->WriteBlocks;

          $vm_population    += $metric_object->Population;

          $vm_insertions    += $metric_object->Insertions;
          
          // Increment the vDisk count
          $vdisk_count++;
          
          // Store the vDisk metrics & attributes
          $processed['disk_values'][] = array(
            'read_latency' => round($metric_object->ReadLatencyPerOp / 1000, 1),
            'write_latency' => round($metric_object->WriteLatencyPerOp / 1000, 1),
            'read_ops' => $metric_object->ReadOps,
            'write_ops' => $metric_object->WriteOps,
            'hits' => $metric_object->Hits,
            'hit_ops' => $metric_object->HitOps,
            'read_blocks' => $metric_object->ReadBlocks,
            'write_blocks' => $metric_object->WriteBlocks,
            'cache_latency' => $metric_object->AvgHitLatency,
            'population' => $metric_object->Population,
            'insertions' => $metric_object->Insertions,
            'identifier' => $disk_metric->Identifier,
            'admin_state' => $disk_metric->AdminState,
            'cache_mode' => $disk_metric->CacheMode,
            'disk_id' => $disk_metric->DiskID,
            'drive_letter' => $disk_metric->DriveLetters,
            'op_state_message' => $disk_metric->OpStateMsg,
            'op_state' => $disk_metric->OperationalState,
            'perf_state' => $disk_metric->PerfState,
            'usable_blocks' => $vm_usable_blocks,
          );
        }
      }
            
      $processed['attributes']['build_version']  = $vm_info->BuildVersion;
      $processed['attributes']['cache_mode']     = $vm_info->CacheMode;
      $processed['attributes']['admin_state']    = $vm_info->AdminState;
      $processed['attributes']['cache_size']     = $vm_info->Sectors * $vm_info->SectorSize;
      $processed['attributes']['op_state']       = $vm_info->OperationalState;
      $processed['attributes']['op_message']     = $vm_info->OpStateMsg;
      $processed['attributes']['perf_state']     = $vm_info->PerfState;
          
      $processed['metrics']['read_latency']   = $vm_read_latency;
      $processed['metrics']['write_latency']  = $vm_write_latency;
      $processed['metrics']['read_ops']       = $vm_read_ops;
      $processed['metrics']['write_ops']      = $vm_write_ops;
      $processed['metrics']['hits']           = $vm_hits;
      $processed['metrics']['read_blocks']    = $vm_read_blocks;
      $processed['metrics']['write_blocks']   = $vm_write_blocks;
      $processed['metrics']['usable_blocks']  = $vm_usable_blocks;
      $processed['metrics']['cache_latency']  = $vm_cache_latency;
      $processed['metrics']['population']     = $vm_population;
      $processed['metrics']['insertions']     = $vm_insertions;
      $processed['metrics']['reset_interval'] = $options_info->MetricsResetInterval; 
    }
    catch (Exception $e) {
      watchdog(FAMC_ANALYTICS_MODULE_DISPLAY_NAME, 'VM metrics processing error - @error', array('@error' => $e->getMessage()), WATCHDOG_ERROR);
      $processed = FALSE;
    }
    
    return $processed;
  }
  
