<?php

/**
 *  Generates the manifest.xml file
 *  The manifest file will contain the summary of files in the ASUP content
 *  including file size, time taken to generate the file etc 
 *
 * @param dir
 *   directory in which asup content is generated. This function will pickup the files from this directory.
 * Return value
 *  This function will return the manifest file contents as xml
*/
function gen_manifest($dir)
{
	ini_set('precision', 20);
	$milliseconds = microtime(true) * 1000000; 
	$xml = '<?xml version="1.0"?>
';
	$xml .= '<!DOCTYPE Manifest [
';
	$xml .= '<!ELEMENT Manifest (asup:TABLE_INFO,asup:ROW*,asup:ABORT?)>
';
	$xml .= '<!ATTLIST Manifest xmlns CDATA #FIXED "http://asup_search.netapp.com/ns/Manifest/1.0">
';
	$xml .= '<!ATTLIST Manifest xmlns:asup CDATA #FIXED "http://asup_search.netapp.com/ns/ASUP/1.0">
';
	$xml .= '<!ELEMENT asup:TABLE_INFO (asup:field+)>
';
	$xml .= '<!ELEMENT asup:field (asup:tag,asup:smf_name,asup:ui_name,asup:type,asup:qualification,asup:remap)>
';
	$xml .= '<!ELEMENT asup:tag (#PCDATA)>
';
	$xml .= '<!ELEMENT asup:ui_name (#PCDATA)>
';
	$xml .= '<!ELEMENT asup:type (#PCDATA)>
';
	$xml .= '<!ELEMENT asup:ABORT (#PCDATA)>
';
	$xml .= '<!ELEMENT asup:ROW (prio-num,body-file,size-collected,time-collected,status,error,content-type)>
';
	$xml .= '<!ELEMENT prio-num (#PCDATA)>
';
	$xml .= '<!ELEMENT body-file (#PCDATA)>
';
	$xml .= '<!ELEMENT size-collected (#PCDATA)>
';
	$xml .= '<!ELEMENT time-collected (#PCDATA)>
';
	$xml .= '<!ELEMENT status (#PCDATA)>
';
	$xml .= '<!ELEMENT error (#PCDATA)>
';
	$xml .= '<!ELEMENT content-type (#PCDATA)>]>
';
	$xml .= "<Manifest xmlns=\"http://asup_search.netapp.com/ns/Manifest/1.0\" xmlns:asup=\"http://asup_search.netapp.com/ns/ASUP/1.0\">
";
	$xml .= '        <asup:TABLE_INFO>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>prio-num</asup:tag>
';
	$xml .= '                        <asup:smf_name>prio-num</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>Collection Order</asup:ui_name>
';
	$xml .= '                        <asup:type>unsigned</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>body-file</asup:tag>
';
	$xml .= '                        <asup:smf_name>body-file</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>The AutoSupport Content filename for this data</asup:ui_name>
';
	$xml .= '                        <asup:type>text</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>size-collected</asup:tag>
';
	$xml .= '                        <asup:smf_name>size-collected</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>Number of bytes collected</asup:ui_name>
';
	$xml .= '                        <asup:type>size</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>time-collected</asup:tag>
';
	$xml .= '                        <asup:smf_name>time-collected</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>Time spent collecting</asup:ui_name>
';
	$xml .= '                        <asup:type>unsigned64</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>status</asup:tag>
';
	$xml .= '                        <asup:smf_name>status</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>The status of this data item</asup:ui_name>
';
	$xml .= '                        <asup:type>asup_manifest_status</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>error</asup:tag>
';
	$xml .= '                        <asup:smf_name>error</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>The textual description of error</asup:ui_name>
';
	$xml .= '                        <asup:type>text</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '                <asup:field>
';
	$xml .= '                        <asup:tag>content-type</asup:tag>
';
	$xml .= '                        <asup:smf_name>content-type</asup:smf_name>
';
	$xml .= '                        <asup:ui_name>The AutoSupport content type for this data</asup:ui_name>
';
	$xml .= '                        <asup:type>asup_content_type</asup:type>
';
	$xml .= '                </asup:field>
';
	$xml .= '        </asup:TABLE_INFO>
';

	$pri = 0;

	// list of files to look for in asup folder
	$file_present = array('fa-config.xml'=>false, 'fmc-messages.txt'=>false, 'fa-perf-metrics.txt'=>false, 'x-header-data.txt'=>false);
	$files = glob($dir . '/*');	
	foreach ($files as $file) {

		// parse the file name to extract the time taken
		$file_parts = explode("__", $file);
		$filename = $file_parts[0];
    if (isset($file_parts[1])) {
      $time = $file_parts[1];
    } else {
      $time = 0;
    }
		$oldname = $file;
		$newname = $filename;
		$filename = basename($newname);
		if (!rename($oldname, $newname)) {

			watchdog(FAMC_ASUP_MODULE_DISPLAY_NAME, 'ASUP renaming of file failed', NULL, WATCHDOG_ERROR);
			return NULL;
		}

		$filesize = filesize($newname);
        	$xml .= "<asup:ROW>\n";
                $xml .= "	<prio-num>$pri</prio-num>\n";
                $xml .= "	<body-file>$filename</body-file>\n";
		$fe = file_exists($newname);
		if ($fe) {
			$file_present[$filename] = true;
                	$xml .= "	<size-collected>$filesize</size-collected>\n";
                	$xml .= "	<time-collected>$time</time-collected>\n";
                	$xml .= "	<status>completed</status>\n";
                	$xml .= "	<error/>\n";
		}
		else
		{
			$file_present[$filename] = false;	
                	$xml .= "	<size-collected>0</size-collected>\n";
                	$xml .= "	<time-collected>0</time-collected>\n";
                	$xml .= "	<status>failed</status>\n";
                	$xml .= "	<error>failed to generate</error>\n";
		}
                $xml .= "	<content-type>basic</content-type>\n";
        	$xml .= "</asup:ROW>\n";

		$pri++;
	}

	// for files which failed to generate, use default values
	$filenames = array('fa-config.xml', 'fmc-messages.txt', 'fa-perf-metrics.txt', 'x-header-data.txt');
	foreach ($filenames as $file) {

		if (!$file_present[$file]) {
        		$xml .= "<asup:ROW>\n";
                	$xml .= "	<prio-num>$pri</prio-num>\n";
                	$xml .= "	<body-file>$file</body-file>\n";
                	$xml .= "	<size-collected>0</size-collected>\n";
                	$xml .= "	<time-collected>0</time-collected>\n";
                	$xml .= "	<status>failed</status>\n";
                	$xml .= "	<error>failed to generate</error>\n";
                	$xml .= "	<content-type>basic</content-type>\n";
        		$xml .= "</asup:ROW>\n";

			$pri++;
		}
	}
        $xml .= "</Manifest>\n";
	return $xml;
}

