<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * Uninstall a vib from an ESX host.
 * @param stdClass $host Host node
 * @param string $vib_name Vib to remove
 * @param boolean $retry
 * @return mixed true on success, error string on failure
 */
function famc_external_api_host_uninstall_vib(stdClass $host, $vib_name, $retry = true) {
  $credentials = mercury_management_get_credentials($host);
  $out = array();

  exec_with_stderr(
    "/usr/bin/esxcli -s '" . $credentials['address'] .
    "' --username '" . $credentials['username'] .
    "' --password '" . $credentials['password'] .
    "' software vib remove --vibname=$vib_name", $out
  );

  mercury_utilities_fmc_log_message('Host Agent', implode("<br />\n", $out), WATCHDOG_DEBUG, NULL, NULL, 'no_footer');

  foreach ($out as $row) {
    if (strpos($row, 'Error') !== false) {
      if ($retry) {
        mercury_utilities_fmc_log_message('Host Agent', t('Host may be busy. Retrying agent uninstall on @host.', array('@host' => $host->title)), WATCHDOG_WARNING);
        sleep(15);
        return famc_external_api_host_uninstall_vib($host, $vib_name, $retry = false);
      }

      // No more retries, fail out.
      $error = t('Cannot uninstall the host agent on @host. Host may be busy. Please try again later.', array('@host' => $host->title));
      mercury_utilities_fmc_log_message('Host Agent', $error, WATCHDOG_ERROR);
      return $error;
    }
  }

  return true;
}

function famc_external_api_host_validate_credentials(stdClass $host = NULL, $address = NULL, $username = NULL, $password = NULL) {
  if ($password) { // pre-save
    $credentials = array();
    $credentials['address'] = trim($address);
    $credentials['username'] = $username;
    $credentials['password'] = $password;
  } else {
    $credentials = mercury_management_get_credentials($host);
  }

  /* Valid outputs
   * 1. Domain Name: ...
   * 2. Connect to <host> failed: Cannot complete login due to an incorrect user name or password
   * 3. Connect to <host> failed: Connection failure
   */
  $host_status = array();
  exec("esxcli -s '" . $credentials['address'] .
    "' --username '" . $credentials['username'] .
    "' --password '" . $credentials['password'] .
    "' system hostname get", $host_status);
  foreach ($host_status as $output) {
    if (!empty($output) && stristr($output, 'failed')) {
      if (stristr($output, 'password')) {
        mercury_utilities_fmc_log_message('Host Credentials', t('Host @address username or password may be incorrect.', array('@address' => $credentials['address'])));
        return "Error";
      } else {
        mercury_utilities_fmc_log_message('Host Credentials', t('Cannot connect to Host @address. Please check if it is powered on and try again.', array('@address' => $credentials['address'])));
        return "CannotConnect";
      }
    }
  }
  return true;
}

function famc_external_api_host_preinstall_validation(stdClass $host) {
  $credentials = mercury_management_get_credentials($host);

  // Check for acceptance level first
  $acceptance_level = exec("esxcli -s '" . $credentials['address'] . "' --username '" . $credentials['username'] . "' --password '" . $credentials['password'] . "' software acceptance get");
  mercury_utilities_fmc_log_message('Host Agent', t('Host Acceptance Level ' . $acceptance_level));
  if (stristr($acceptance_level, 'VMwareCertified') || stristr($acceptance_level, 'VMwareAccepted')) {
    mercury_utilities_fmc_log_message('Host Agent', t('Flash Accel host agent requires that the system be at a PartnerSupported acceptance level or lower. Please change the acceptance level on "@host" appropriately, before installing the Flash Accel host agent.', array('@host' => $host->title)), WATCHDOG_ERROR);
    return false;
  }
  return true;
}

function famc_external_api_host_in_maintenance_mode(stdClass $host) {
  $credentials = mercury_management_get_credentials($host, $include_ancestors = true);

  $get_host_info = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl " .
    "--url https://" . $credentials['vcenter']['address'] .
    "/sdk/vimService  --username '" . $credentials['vcenter']['username'] .
    "' --password '" . $credentials['vcenter']['password'] .
    "' --operation maintmode --hostname " . $credentials['address']);
  $maintain_mode = explode(':', $get_host_info);
  if (trim($maintain_mode[1]) == 0) {
    return false;
  }
  return true;
}

function famc_external_api_host_initialize_device(stdClass $host) {
  $credentials = mercury_management_get_credentials($host);
  $out = array();
  // Invokes FA_System->InitializeCache CIM method.
  exec_with_stderr(
    "/usr/lib/vmware-vcli/apps/host/host_control.py '" .
    $credentials['address'] . "' '" .
    $credentials['username'] . "' '" .
    $credentials['password'] . "' 3", $out
  );

  // This returns either nothing or an error, but this call takes care
  // of the error handling regardless.
  return famc_external_api_process_array_one_item_per_line($out, 'Initialize Cache Device');
}

function  famc_external_api_host_watchdog_status(stdClass $host) {
  $credentials = mercury_management_get_credentials($host, $include_ancestors = true);
  $outputLine = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl " .
    "--server " . $credentials['vcenter']['address'] .
    " --username '" . $credentials['vcenter']['username'] .
    "' --password '" . $credentials['vcenter']['password'] .
    "' --hostname '" . $credentials['address'] .
    "' --operation serviceGetSfcbdWatchdogStatus");
  if (empty($outputLine)) {
    return "failed";
  } else if (stristr($outputLine, "Stopped")) {
    return "Stopped";
  } else if (stristr($outputLine, "Started")){
    return "Started";
  } else {
    return "failed";
  }
}

function  famc_external_api_host_watchdog_restart(stdClass $host) {
  $credentials = mercury_management_get_credentials($host, $include_ancestors = true);
  $sfcbdRestartOutputError = array();
  $sfcbdRestartOutput = execute_command("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl " .
    "--server " . $credentials['vcenter']['address'] .
    " --username '" . $credentials['vcenter']['username'] .
    "' --password '" . $credentials['vcenter']['password'] .
    "' --hostname '" . $credentials['address'] .
    "' --operation restartSfcbdWatchdog", $sfcbdRestartOutputError);
  if (stristr($sfcbdRestartOutput, 'Error')) {
    mercury_utilities_fmc_log_message('Host Agent', implode('<br>', $sfcbdRestartOutputError), WATCHDOG_WARNING, NULL, NULL, 'no_footer');
    return $sfcbdRestartOutput;
  }
  return true;
}

function famc_external_api_host_get_cache_state(stdClass $host) {
  // For now this is trivially different than _properties,
  // but that's specific to ESX.
  return famc_external_api_host_get_properties($host, $cache_state_only = true);
}

function famc_external_api_host_is_reachable(stdClass $host) {
  $result = famc_external_api_host_get_properties($host);
  if (count($result) > 0) {
    return true;
  }
  // TODO: does it make sense to check the credentials here?
  return false;
}

/**
 * Gets a list of host properties.
 * @param stdClass $host
 * @param type $cache_state_only @see famc_external_api_host_get_cache_state
 * @return array of properties or false if the host was unreachable.
 */
function famc_external_api_host_get_properties(stdClass $host, $cache_state_only = false) {
  $grep = '';
  if ($cache_state_only) {
    $grep = " | grep 'CacheState'";
  }

  $credentials = mercury_management_get_credentials($host);
  $result = array();
  exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" .
    $credentials['address'] . "' '" .
    $credentials['username'] . "' '" .
    $credentials['password'] . "' 1{$grep}", $result);

  // TODO: on failures return false.  No code really uses it in this way yet.
  return $result;
}
