<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that are only used during development.
 */

function mercury_management_check_logs_for_passwords() {
  // Password safety check: when not in production mode, frequently look
  // for unescaped passwords in the logs.  This is simply too expensive to do in
  // production and is just meant to catch bugs.
  // This also breaks during unit tests.
  if (mercury_utilities_application_env() != 'production' && !mercury_utilities_unit_tests_running()) {
    // Grab the search password from the vCenter node.  We typically use the
    // same passwords for a whole setup so this covers host and VM too.
    $vCenter = mercury_management_vcenter_details();

    // We're not going to use the username so don't decrypt it.
    $password = fmc_cipher::decrypt(mercury_utilities_get_node_field($vCenter, 'password'));
    if (empty($password)) {
      return; // For cron runs before setup.
    }
    $password = escapeshellarg($password);

    // Check the usual suspects.
    $password_log_check = `grep --no-messages $password /var/log/apache2/error.log /var/log/syslog /var/log/messages /tmp/*.txt`;
    if (!empty($password_log_check)) {
      mercury_utilities_fmc_log_message('Debug', 'Possible password match in logs! Alert a developer!', WATCHDOG_ERROR);
    }
  }
}

/**
 *  Functions to clear database logs
 */
function mercury_management_clear_database_logs($redirect = true) {
  $_SESSION['dblog_overview_filter'] = array();
  db_delete('watchdog')->execute();
  variable_del('fmc-messages');

  if ($redirect) {
    drupal_goto('console-logs-report');
  }
}

/**
 *  Function to clear all the drupal variables
 */
function mercury_management_clear_drupal_variables() {
  global $base_url;
  $variable_arr = array('vm_username', 'vm_password', 'host_username', 'host_password', 'vcenter_username', 'vcenter_password', 'vm_cur_MSI', 'vm_upgraded_MSI', 'vm_package_agent_version', 'host_cur_zip', 'host_upgraded_zip', 'host_package_agent_version', 'time_from_last_discovery', 'rediscover_status', 'rediscover_item_settings', 'discovery_happened', 'fmc-messages', 'migration_policy', 'enable_migration', 'log_level', 'rediscover_total_ticks', 'rediscover_current_ticks');
  foreach ($variable_arr as $var) {
    variable_del($var);
  }
  $vcInfo = mercury_management_vcenter_details();
  $vcInfo->title = "";
  $vcInfo->field_hostname['und'][0]['value'] = "";
  $vcInfo->field_username['und'][0]['value'] = "";
  $vcInfo->field_password['und'][0]['value'] = "";
  $vcInfo->field_fmc_uuid['und'][0]['value'] = "";
  node_save($vcInfo);
  drupal_set_message(t('All custom drupal variables have been deleted.'));
  drupal_goto($base_url);
}

/**
 * Clear out nodes created by discovery, to make the export simpler.
 * All nodes except the vCenter will be deleted.
 * The vCenter will be in mercury_management_clear_drupal_variables.
 */
function mercury_management_clear_discovered_nodes() {
  $del_query = db_query("SELECT nid FROM node WHERE type != 'vcenter'");
  if (!empty($del_query)) {
    foreach ($del_query as $nid) {
      $del_nids[] = $nid->nid;
    }
    if (!empty($del_nids)) {
      node_delete_multiple($del_nids);

      // Flush all caches to avoid any stale data stored there.
      admin_menu_flush_caches();
    }
  }

  return t('All discovered nodes have been deleted.');
}

/**
 * Calls the other "clear" functions and does a little extra, too!
 */
function mercury_management_clear_database_export() {
  // Also remove the agents. Must re-upload anyway to fix the variables.
  // This will log so do it first.
  $nothing = array(); $nothing2 = array();
  mercury_utilities_agents_remove($nothing, $nothing2);

  famc_analytics_clear_metrics();
  mercury_management_clear_discovered_nodes();
  mercury_management_clear_database_logs($redirect = false);

  // Towards the end...
  admin_menu_flush_caches();

  // This has to be last.  It redirects, and also breaks other stuff.
  mercury_management_clear_drupal_variables();
}

/**
 * Helper function to debug large structure, particularly containing nodes.
 */
function mm_debug($var, $brief = true, $return = false) {
  if (is_array($var)) {
    foreach ($var as $key => $value) {
      $var[$key] = mm_debug($value, $brief, true);
    }
  } elseif ($var instanceof stdClass && !empty($var->title)) { // node
    if ($brief) {
      $var = "Node: {$var->type} {$var->nid} {$var->title}";
    }
  }

  if ($return) {
    return $var;
  }

  // So you get an alert if ajax.
  if (!headers_sent()) {
    header('HTTP/1.1 500 Debug Mode Error');
  }

  // Send output.
  var_dump($var);
}
