<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for managing devices.
 */

/**
 *  Function to show all the current devices
 */
function mercury_management_show_devices($host_nid) {
  $output = '';
  if ($host_nid != '') {
    $host_info = node_load($host_nid);

    $host_data = mercury_management_get_host_dc_vc_of_vm($host_nid);
    $vcInfo = node_load($host_data['vcenter']);
    $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
    $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
    $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

    // get the list of paths to construct the list of devices
    $devices = exec("esxcli -s " . $vc_hostname . " --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host_info->title . "' storage core path list | grep Runtime -A 1", $devicesArr);
    if (!empty($devicesArr)) {
      $rows = array();
      $row = array();
      foreach ($devicesArr as $dev) {
        $runStr = explode('Runtime Name:', $dev);
        $deviceStr = explode('Device:', $dev);
        if (count($runStr) > 1) {
          $row[] = trim($runStr[1]);
        }
        if (count($deviceStr) > 1) {
          $row[] = trim($deviceStr[1]);
          $rows[] = $row;
          $row = array();
        }
      }
    }
    $header = array('Runtime Name', 'Device Name');
    $output .= theme('table', array('header' => $header, 'rows' => $rows));
  }
  return $output;
}

/*
 * Function that Attaches a Device to a host
 *
 */

function mercury_management_attachDeviceToHost($host_nid, $device_nid) {
  $host = node_load($host_nid);
  $device = node_load($device_nid);

  //Get the vCenter ID related to the host
  if (!empty($host->field_datacenter)) {
    $data_center = node_load($host->field_datacenter['und'][0]['target_id']);
    if (!empty($data_center->field_related_vcenter)) {
      $vcenter_id = $data_center->field_related_vcenter['und'][0]['target_id'];
    } else {
      mercury_utilities_fmc_log_message('Flash Device', 'Failed to obtain flash device attributes. Please refresh the host and try again.', WATCHDOG_ERROR);
      return 'error';
    }
  }

  //Get the vCenter details
  $vcenter_info = node_load($vcenter_id);
  $vc_username = fmc_cipher::decrypt($vcenter_info->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcenter_info->field_password['und'][0]['value']);
  $vc_hostname = $vcenter_info->field_hostname['und'][0]['value'];

  //Get the host details
  $host_username = mercury_management_get_host_username($host);
  $host_password = mercury_management_get_host_password($host);

  //Check if the Host is in Maintenance mode (Do not Assign Device)
  $get_host_info = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --operation maintmode --hostname '" . trim($host->title) . "'");
  $maintain_mode = explode(':', $get_host_info);
  if (trim($maintain_mode[1]) == 1) {
    mercury_utilities_fmc_log_message('Flash Device', t('Host "@host_data" is in maintenance mode', array('@host_data' => trim($host->title))), WATCHDOG_WARNING);
    return 'maintenance';
  }
  $is_a_vmfs_datastore = exec("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --server " . $vc_hostname . " --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . $host->title . "' --diskCanonicalName '" . $device->field_serial_no['und'][0]['value'] . "' --operation checkIfVMFSDatastore", $is_a_vmfs_datastore_result);
  if (!empty($is_a_vmfs_datastore) && stristr($is_a_vmfs_datastore, "Error")) {
    mercury_utilities_fmc_log_message('Flash Device', $is_a_vmfs_datastore . " Could not claim device.", WATCHDOG_ERROR);
    return 'error';
  }

  $searchRuntime = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core path list -d '" . $device->field_serial_no['und'][0]['value'] . "'", $result);
  $checkResult = implode('<br>', $result);
  if (empty($searchRuntime) || (stristr($checkResult, "UID:") === FALSE)) {
    mercury_utilities_fmc_log_message('Flash Device', t('Failed to claim device "@device_id" on host: "@name". Error: "@error_data"', array('@device_id' => $device->field_serial_no['und'][0]['value'], '@name' => $host->title, '@error_data' => $searchRuntime)), WATCHDOG_ERROR);
    return 'error';
  }

  foreach ($result as $output_line) {
    $var_split = explode(':', $output_line);
    if (!empty($output_line) && (stristr($output_line, "Channel:"))) {
      $device_channel = trim($var_split[1]);
    } else if (!empty($output_line) && (stristr($output_line, "Target:"))) {
      $device_target = trim($var_split[1]);
    } else if (!empty($output_line) && (stristr($output_line, "LUN:"))) {
      $device_lun = trim($var_split[1]);
    } else if (!empty($output_line) && (stristr($output_line, "Transport:"))) {
      $device_transport = trim($var_split[1]);
    } else if (!empty($output_line) && (stristr($output_line, "Adapter:"))) {
      $device_adapter = trim($var_split[1]);
    }
  }

  $addrule = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule add --autoassign -t location -A " . $device_adapter . ' -C ' . $device_channel . ' -T ' . $device_target . ' -L ' . $device_lun . ' -P FlashAccel');
  if ($addrule) {
    mercury_utilities_fmc_log_message('Flash Device', t('Failed to add claimrule on host @name : "@error_data"', array('@name' => $host->title, '@error_data' => $addrule)), WATCHDOG_ERROR);
    return 'error';
  }

  // Claimrule add was successful - now load and run it
  $claimload = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule load");
  $claimrun = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule run");
  $claimdevice = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claiming reclaim -d " . $device->field_serial_no['und'][0]['value'] . '', $result_claim_error);

  if (!empty($claimload) || !empty($claimrun) || !empty($claimdevice)) {
    //error
    mercury_utilities_fmc_log_message('Flash Device', t('Failed to claim device on host: "@name".', array('@name' => $host->title)), WATCHDOG_ERROR);
    if (!empty($claimload)) {
      mercury_utilities_fmc_log_message('Flash Device', 'Could not claim device. Failed to load claimrule. ' . $claimload, WATCHDOG_ERROR);
    } else if (!empty($claimrun)) {
      mercury_utilities_fmc_log_message('Flash Device', 'Could not claim device. Failed to run claimrule. ' . $claimrun, WATCHDOG_ERROR);
    } else {
      mercury_utilities_fmc_log_message('Flash Device', 'Failed to claim device. ' . $claimdevice, WATCHDOG_ERROR);
    }

    error_out:

    $removerule = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule remove -P  FlashAccel");
    if (!empty($removerule)) {
      mercury_utilities_fmc_log_message('Flash Device', t('Could not remove claimrule host: "@name"', array('@name' => $host->title)), WATCHDOG_ERROR);
    }
    $claimload = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule load");
    $claimrun = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule run");
    $claimdevice = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claiming reclaim -d " . $device->field_serial_no['und'][0]['value'] . '');
    return 'error';
  }

  // Verify that the Device is attached with NAAIdentifier
  $get_host_info = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $host_username . "' '" . $host_password . "' 8 | grep NAAIdentifier", $result);
  $get_host_NAA = explode('NAAIdentifier:', $get_host_info);

  if (!empty($get_host_NAA[1])) {
    $dcInfo = node_load($host->field_datacenter['und'][0]['target_id']);
    $result = mercury_management_updatehost($vc_hostname, $vc_username, $vc_password, trim($host->title), $dcInfo->nid, FALSE);
    $return_arr = explode('--', $result);
    if (count($return_arr) == 1) {
      return 'attached_refresh';
    }
  }

  // Check for the FlashAccel plug-in showing up appropriately in storage core path list
  $path = "$device_adapter:C$device_channel:T$device_target:L$device_lun";
  $cmd = "esxcli -s " . $vc_hostname . " --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core path list --path=$path";
  exec($cmd, $path_output);
  if (!empty($path_output)) {
    foreach ($path_output as $line) {
      $var_split = explode(':', $line);
      if (empty($var_split)) {
        continue;
      }
      $key = trim(array_shift($var_split));
      $val = trim(implode(':', $var_split));
      if (($key == 'Plugin') && ($val == 'FlashAccel')) {
        // We found it, so assume everything's okay (possible CIM issue).
        return 'attached';
      }
    }
  }

  // It appears that we didn't really claim the device.  Clean up and return failure.
  mercury_utilities_fmc_log_message('Flash Device', t('Could not find FlashAccel plug-in in storage core path list!'), WATCHDOG_ERROR);
  mercury_utilities_fmc_log_message('Flash Device', t('Flash device could not be claimed on host "@name"', array('@name' => $host->title)), WATCHDOG_ERROR);
  goto error_out;
}

/*
 * Function that Unattaches a Device from a host
 *
 */

function mercury_management_unattachDeviceFromHost($host_nid) {
  $host = node_load($host_nid);
  //Get the vCenter ID related to the host
  if (!empty($host->field_datacenter)) {

    $data_center = node_load($host->field_datacenter['und'][0]['target_id']);
    if (!empty($data_center->field_related_vcenter)) {
      $vcenter_id = $data_center->field_related_vcenter['und'][0]['target_id'];
    }
  }

  //Get the vCenter details
  $vcenter_info = node_load($vcenter_id);
  $vc_username = fmc_cipher::decrypt($vcenter_info->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcenter_info->field_password['und'][0]['value']);
  $vc_hostname = $vcenter_info->field_hostname['und'][0]['value'];

  //List all the Cache Spaces in the Host
  $cache_space_id = '';
  $get_all_cache_spaces = mercury_management_listcachespaces($host_nid);

  //Check if the Host is in Maintenance mode (Do not Assign Device)
  $get_host_info = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --operation maintmode --hostname '" . trim($host->title) . "'");
  $maintain_mode = explode(':', $get_host_info);

  //If there are Cache Spaces allocated to Virtual Machines, do not allow Cache Unallocate
  if (count($get_all_cache_spaces) > 0) {
    mercury_utilities_fmc_log_message('Flash Device', t('Failed to unassign device on host: "@name" as cache space is allocated to the VMs<br/><br/> ', array('@name' => $host->title)), WATCHDOG_ERROR);
    return 'cachespace';
  } else if (trim($maintain_mode[1]) == 1) {
    mercury_utilities_fmc_log_message('Flash Device', t('Failed to unassign device as host "@host_data" is in maintenance mode', array('@host_data' => trim($host->title))), WATCHDOG_ERROR);
    return 'maintenance';
  }

  $searchRule = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule list | grep FlashAccel", $result);
  if (empty($result)) {
    mercury_utilities_fmc_log_message('Flash Device', 'Flash Accel claimrule for device no longer exists. Reclaiming it for default driver.', WATCHDOG_ERROR);
  } else {
    $removerule = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule remove -P  FlashAccel");
  }

  $unclaimdevice = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claiming unclaim -t plugin -P FlashAccel");
  $claimload = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule load");
  $claimrun = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core claimrule run");

  if (!empty($unclaimdevice)) {
    mercury_utilities_fmc_log_message('Flash Device', t('Unclaim device from host "@name" failed. ' . $unclaimdevice, array('@name' => $host->title)), WATCHDOG_ERROR);
  }

  if (!empty($claimload)) {
    mercury_utilities_fmc_log_message('Flash Device', t('Claimrule load failed when unclaiming device from host "@name". ' . $claimload, array('@name' => $host->title)), WATCHDOG_ERROR);
  }

  if (!empty($claimrun)) {
    mercury_utilities_fmc_log_message('Flash Device', t('Claimrule run failed when unclaiming device from host "@name". ' . $claimrun, array('@name' => $host->title)), WATCHDOG_ERROR);
  }

  $adapterArr = explode(':', $host->field_hbtl['und'][0]['value']);
  $adapter = $adapterArr[0];

  $claimdevice = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core adapter rescan --adapter " . $adapter . '');
  if ($claimdevice) {
    //error
    mercury_utilities_fmc_log_message('Flash Device', t('Cannot rescan adapter for flash device on host: "@name"', array('@name' => $host->title)), WATCHDOG_ERROR);
  }

  //verify attachment worked
  $verifyAttach = exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core" . ' device list | grep "' . $host->field_naa_identifier['und'][0]['value'] . '" -B 1', $result1);
  if (!empty($result1)) {
    mercury_utilities_fmc_log_message('Flash Device', t('Successfully unassigned flash device on host: "@name"', array('@name' => $host->title)));
    return 'unattached';
  } else {
    mercury_utilities_fmc_log_message('Flash Device', t('Failed to unassign flash device from host: "@name"', array('@name' => $host->title)), WATCHDOG_ERROR);
    return 'error';
  }
}
