<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 * 
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *  
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *  
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *  
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

define('FAMC_DISCOVERY_STATE_NAME_ONLY',   0); // Know nothing about the node but its name
define('FAMC_DISCOVERY_STATE_VCENTER',     10); // Fields from vCenter are known
define('FAMC_DISCOVERY_STATE_REACHABLE',   20); // Can connect to basic services like esxcli and VMware Tools
define('FAMC_DISCOVERY_STATE_API_READY',   30); // FA-specific APIs are available

/**
 *  Open log file (truncating previous log file if it exists)
 */
function mercury_management_rediscover_open_logfile($type = '') {
  if ($type == '') {
    $type = 'discovery';
  }
  $file = mercury_management_get_log_dir() . "/$type.txt";
  $fh = fopen($file, "w");
  chmod($file, 0777);
  return $fh;
}


/**
 *  Write string to log file
 */
function mercury_management_rediscover_log($fh, $str) {
  if ($fh) {
     fwrite($fh, $str);
     fflush($fh);
  }
}


/**
 *  Close log file
 */
function mercury_management_rediscover_close_logfile($fh) {
   if ($fh) {
      fclose($fh);
   }
}


/**
 *  Check if user has cancelled discovery.
 */
function mercury_management_rediscover_check_cancel() {
  $vcInfo = mercury_management_vcenter_details();
  $rediscover_status = $vcInfo->field_rediscover_status['und'][0]['value'];
  if ($rediscover_status == 'rediscover_stop') {
    mercury_utilities_fmc_log_message('Discovery', 'Discovery terminated');
    $vcInfo->field_rediscover_status['und'][0]['value'] = 'rediscover_terminated';
    node_save($vcInfo);
    return false;
  }
  return true;
}


/**
 *  Delete a list of VM nodes with progress bar updates
 */
function mercury_management_delete_vms_and_update_ticks($log_fh, $vm_nids)
{
   mercury_management_rediscover_log($log_fh, "Removing obsolete vm nodes from the database.\n");
   foreach ($vm_nids as $nid) {
      mercury_management_rediscover_log($log_fh, "Deleting obsolete vm node id $nid and dependent nodes.\n");
      // Null is "don't log"; too verbose the way we're doing it
      mercury_management_delete_vms(null, array($nid));
      mercury_utilities_discovery_progress_register_ticks(1);
      if (false == mercury_management_rediscover_check_cancel()) {
         return false;
      }
   }
   return true;
}


/**
 *  Delete a list of host nodes with progress bar updates
 */
function mercury_management_delete_hosts_and_update_ticks($log_fh, $host_nids)
{
   mercury_management_rediscover_log($log_fh, "Removing obsolete host nodes from the database.\n");
   foreach ($host_nids as $nid) {
      mercury_management_rediscover_log($log_fh, "Deleting obsolete host node id $nid and dependent nodes.\n");
      // Null is "don't log"; too verbose the way we're doing it
      mercury_management_delete_hosts(null, array($nid));
      mercury_utilities_discovery_progress_register_ticks(1);
      if (false == mercury_management_rediscover_check_cancel()) {
         return false;
      }
   }
   return true;
}


/**
 *  UoW starting callback (for updating discovery pop-up textbox)
 */
function mercury_management_rediscover_uow_starting_cb(&$uow)
{
   if (isset($uow['vm'])) {
      mercury_management_rediscover_vm_log($uow, "starting discovery\n");
   } elseif (isset($uow['host_name'])) {
      mercury_management_rediscover_host_log($uow, "starting discovery\n");
   }
}


/**
 *  UoW finished callback (for updating progress bar and discovery pop-up textbox)
 */
function mercury_management_rediscover_uow_finished_cb(&$uow)
{
   if (isset($uow['vm'])) {
      mercury_management_rediscover_vm_log($uow, "completed discovery\n");
   } elseif (isset($uow['host_name'])) {
      mercury_management_rediscover_host_log($uow, "completed discovery\n");
   }
   mercury_utilities_discovery_progress_register_ticks(1);   
}


/**
 * Initializes tick max and actually discovers/creates all hosts and vms.
 * Also deletes nodes that are no longer in the datacenters.
 * @return boolean false if errors, true otherwise.
 */
function mercury_management_rediscover_hosts_vms_inner($log_fh, stdClass $vcInfo, $discovery_scope_level) {

  // Get vCenter credentials
  $vc_creds = mercury_management_get_vcenter_credentials($vcInfo);
  $vc_hostname = $vc_creds['address'];
  $vc_username = $vc_creds['username'];
  $vc_password = $vc_creds['password'];

  # Using 10 workers for now; semi-arbitrary, but 20 doesn't do much 
  # better than 10 (whereas 10 does a lot better than 1).
  $num_worker_tasks = 10;
  $discovery_error = 0;
  
  mercury_management_rediscover_log($log_fh, "Starting discovery\n");
  mercury_management_rediscover_log($log_fh, "Scope level: $discovery_scope_level\n");

  # Create worker(s) to do the heavy lifting
  $wrkr_array = mercury_management_create_worker_array(
    $num_worker_tasks, 
    $vc_hostname,
    $vc_username,
    $vc_password
  );
  if ($wrkr_array === false) {
    mercury_management_rediscover_log($log_fh, "Terminating discovery: could not create worker array\n");
    mercury_utilities_fmc_log_message('Discovery', t('Could not connect to vCenter server.  Possible causes include invalid credentials, network problem, or server is down.'),  WATCHDOG_ERROR);
    $discovery_error++;
    goto done;
  }

  # Verify that this vCenter is running a supported software version
  $vcv = mercury_management_wa_get_vcversion($wrkr_array);
  if (FALSE == mercury_utilities_vcenter_version_is_supported($vcv)) {
    mercury_management_rediscover_log($log_fh, "Terminating discovery: vCenter $vcv not supported\n");
    mercury_utilities_fmc_log_message('Discovery', t("vCenter version $vcv is not supported by this version of FAMC."),  WATCHDOG_ERROR);
    $discovery_error++;
    goto done;
  }

  # Obtain array of hosts and VMs to be discovered
  # Note that if we eventually want to restrict this to selected DCs,
  # we will need to first get just a list of DCs, filter to selected,
  # and then finally fetch only the hosts and vms for those here.
  # @see mercury_management_gethosts_virtualmachines
  $vCenterArr = mercury_management_gethosts_virtualmachines($vcInfo, $vc_hostname, $vc_username, $vc_password);
  $all_datacenters = array_keys($vCenterArr);

  # Ensure datacenter nodes are synchronized with vSphere
  $datacenter_nids = mercury_management_synchronize_datacenter_nodes($all_datacenters, $vcInfo, $log_fh);

  # Extract the subset of the datacenters that have checkmarks (by reference).
  mercury_management_extract_selected_datacenters($vCenterArr);
  $selected_datacenters = array_keys($vCenterArr);

  # For several sections below, it's more convenient to have a list of host
  # names and a list of VM names instead of the nested structure.
  # Also need to have reverse parent to child map, do that now too.
  $valid_hosts = array();
  $valid_vms = array();
  $host_datacenters = array();
  $vm_hosts = array();
  mercury_management_synchronize_split_hosts_vms($vCenterArr, $valid_hosts, $valid_vms, $host_datacenters, $vm_hosts);

  # Calculate hosts and VMs that no longer exist BEFORE handling "new" vs "all"
  $obsolete_host_nids = mercury_management_find_obsolete_hosts($log_fh, $valid_hosts);
  $obsolete_vm_nids = mercury_management_find_obsolete_vms($log_fh, $valid_vms);
  
  # Count how many host and VM nodes need to be deleted
  $hosts_and_vms_to_delete = count($obsolete_host_nids) + count($obsolete_vm_nids);
  
  # If only discovering new hosts and vms, remove existing targets.
  # This is the reason we need to operate on lists of VMs and Hosts separately:
  # When a host is already found, still need to discover new VMs on it!
  if ($discovery_scope_level == 'new') {
    mercury_management_rediscover_log($log_fh, "Removing existing nodes from target list.\n");
    mercury_management_rediscover_log($log_fh, "Was " . count($valid_hosts) . " hosts and " . count($valid_vms) . " vms\n");
    mercury_management_discover_target_remove_existing($valid_hosts, $valid_vms);
    mercury_management_rediscover_log($log_fh, "Is now " . count($valid_hosts) . " hosts and " . count($valid_vms) . " vms\n\n");
  }
  
  # Count how many hosts and VMs need to be discovered
  $hosts_and_vms_to_discover = count($valid_hosts) + count($valid_vms);

  # This bit is just to make the progress bar move a bit while the annotations 
  # are read, since that takes a while.  Numbers were chosen empirically.
  $ndc = count($selected_datacenters);
  if ($ndc) {
     $ticks_for_annotation = intval(($hosts_and_vms_to_discover)/10);
     $ticks_for_annotation_per_dc = intval($ticks_for_annotation/$ndc);
  } else {
     $ticks_for_annotation = 0;
     $ticks_for_annotation_per_dc = 0;  
  }
  # Initialize the information needed for the progress bar to work
  mercury_management_rediscover_log($log_fh, "hosts and vms to delete   : $hosts_and_vms_to_delete\n");
  mercury_management_rediscover_log($log_fh, "hosts and vms to discover : $hosts_and_vms_to_discover\n");
  $total_ticks = $hosts_and_vms_to_delete + $ticks_for_annotation + $hosts_and_vms_to_discover;

  // Account for garbage collection.
  $total_ticks += mercury_management_gc_get_tick_count();

  // Account for importable data.
  module_load_include('inc', 'mercury_utilities', 'includes/export_import');
  $total_ticks += MercuryImportExport::import_apply_all_saved_credentials_get_ticks();

  // Add one extra tick for cleanup logic, so we are < 100% until really done.
  $total_ticks++;

  // Store the total, reset current state to zero.
  mercury_utilities_discovery_progress_init_ticks($total_ticks);

  # Perform host and VM deletions
  if (false == mercury_management_delete_vms_and_update_ticks($log_fh, $obsolete_vm_nids)) {
    mercury_management_rediscover_log($log_fh, "vm deletion terminated prematurely!\n");
    $discovery_error++;
    goto done;
  }
  if (false == mercury_management_delete_hosts_and_update_ticks($log_fh, $obsolete_host_nids)) {
    mercury_management_rediscover_log($log_fh, "host deletion terminated prematurely!\n");
    $discovery_error++;
    goto done; 
  }
  
  # If nothing new to discover, we're done
  if (empty($valid_hosts) && empty($valid_vms)) {
    mercury_management_rediscover_log($log_fh, "Terminating discovery: no items to discover\n");
    goto done;
  }
  
  # Assemble lists of annotated hosts and VMs in selected datacenters.
  $annotatedVMs = array();
  $annotatedHosts = array();
  foreach ($selected_datacenters as $dataCenter) {
    mercury_utilities_discovery_progress_register_ticks($ticks_for_annotation_per_dc);
    $ticks_for_annotation -= $ticks_for_annotation_per_dc;
    $annotatedVMList = array();
    $getAnnotatedListResult =  mercury_management_list_annotated_components('vm', $dataCenter, $annotatedVMList);
    if ($getAnnotatedListResult === false) {
      mercury_management_rediscover_log($log_fh, "Annotations VM error ". $annotatedVMList . "\n");
      break;
    }

    $annotatedHostList = array();
    $getAnnotatedListResult = mercury_management_list_annotated_components('host', $dataCenter, $annotatedHostList);
    if ($getAnnotatedListResult === false) {
       mercury_management_rediscover_log($log_fh, "Annotations Host error ". $annotatedHostList . "\n");
      break;
    }

    foreach ($annotatedVMList as $annotatedVM) {
      mercury_management_rediscover_log($log_fh, "Annotated VM ". $annotatedVM . "\n");
      $annotatedVMs[$annotatedVM] = true;
    }

    foreach ($annotatedHostList as $annotatedHost) {
      mercury_management_rediscover_log($log_fh, "Annotated Host ". $annotatedHost . "\n");
      $annotatedHosts[$annotatedHost] = true;
    }
  }
  if ($ticks_for_annotation > 0) {
    mercury_utilities_discovery_progress_register_ticks($ticks_for_annotation);
  }
  
  mercury_management_rediscover_log($log_fh, "Resetting query cache.\n");
  db_query("RESET QUERY CACHE");

  # Assemble the host UoW queue
  mercury_management_rediscover_log($log_fh, "Assembling host UoW queue.\n");
  $host_uow_queue = array();
  foreach ($valid_hosts as $host_name) {
    if (!isset($annotatedHosts[$host_name])) {
      $annotatedHosts[$host_name] = false;
    }

    $host_uow = mercury_management_assemble_host_uow(
      $log_fh,
      $vc_hostname,
      $vc_username,
      $vc_password,
      $host_name,
      $datacenter_nids[$host_datacenters[$host_name]],
      $annotatedHosts[$host_name],
      FALSE
    );
    $host_uow['starting_cbk'] = 'mercury_management_rediscover_uow_starting_cb';
    $host_uow['finished_cbk'] = 'mercury_management_rediscover_uow_finished_cb';
    $host_uow['discovery'] = true;
    $host_uow_queue[] = $host_uow;
  }

  # Discover the hosts
  mercury_management_rediscover_log($log_fh, "Initiating host discovery.\n");
  $continue_cb = 'mercury_management_rediscover_check_cancel';
  if (mercury_management_process_uow_queue($host_uow_queue, $wrkr_array, $continue_cb)) {
    mercury_management_rediscover_log($log_fh, "Completed host discovery.\n");
  } else {
    mercury_management_rediscover_log($log_fh, "Host discovery terminated prematurely!\n");
    $discovery_error++;
    goto done;
  }
  
  # Look for any discovery errors.
  foreach ($host_uow_queue as &$uow) {
    if (isset($uow['error'])) {
      $discovery_error++;
    }
  }
  
  # Free the host UoW queue
  unset($host_uow_queue);

  # We also need the host nids for any VMs we're about to discover.
  # Even though $host_uow_queue did contain some host nids, we can't trust them
  # (mainly applies to discover new, but maybe also errors).
  # It's also really fast and entirely accurate to get them from MySQL,
  # so just fetch the whole list and call it a day.
  $query = db_query("SELECT title, nid FROM {node} WHERE type = 'host'");
  $host_nids = $query->fetchAllKeyed();
  
  # Assemble the VM UoW queue
  $vm_uow_queue = array();
  foreach ($valid_vms as $vmName) {
    if (!isset($annotatedVMs[$vmName])) {
      $annotatedVMs[$vmName] = false;
    }

    $vm_uow = mercury_management_assemble_vm_uow(
      $log_fh,
      $vc_hostname,
      $vc_username,
      $vc_password,
      $vmName,
      $annotatedVMs[$vmName],
      $host_nids[$vm_hosts[$vmName]]
    );
    $vm_uow['starting_cbk'] = 'mercury_management_rediscover_uow_starting_cb';
    $vm_uow['finished_cbk'] = 'mercury_management_rediscover_uow_finished_cb';
    $vm_uow['discovery'] = true;
    $vm_uow_queue[] = $vm_uow;
  }

  # Discover the VMs
  mercury_management_rediscover_log($log_fh, "Initiating VM discovery.\n");
  $continue_cb = 'mercury_management_rediscover_check_cancel';
  if (mercury_management_process_uow_queue($vm_uow_queue, $wrkr_array, $continue_cb)) {
    mercury_management_rediscover_log($log_fh, "Completed VM discovery.\n");
  } else {
    mercury_management_rediscover_log($log_fh, "VM discovery terminated prematurely!\n");
    $discovery_error++;
    goto done;
  }
  
  # Look for any discovery errors
  foreach ($vm_uow_queue as &$uow) {
    if (isset($uow['error'])) {
      $discovery_error++;
    }
  }

  mercury_utilities_fmc_log_message('Discovery', t('Finished discovering all entities.'));

done:

  // This will refresh the vCenter node from the database
  // in case any other threads have modified it.
  $vcInfo = mercury_management_vcenter_details();
  if ($vcInfo->field_rediscover_status['und'][0]['value'] == 'rediscover_terminated') {
    mercury_management_rediscover_log($log_fh, "Discovery was cancelled before completion.\n");
  }
  else {
    mercury_management_rediscover_log($log_fh, "Completed discovery with $discovery_error errors.\n");
  }

  # Make sure to clean up the worker pool
  mercury_management_destroy_worker_array($wrkr_array);

  // Also check for password logging at the end of discovery, since this
  // results in good coverage automatically.  This is pretty fast.
  mercury_management_check_logs_for_passwords();

  return empty($discovery_error); // Returns true if no errors.
}

/**
 * Wraps the real work of discovering hosts and vms and handles logging.
 * @return boolean false if errors, true otherwise.
 */
function mercury_management_rediscover_hosts_vms(stdClass $vcInfo, $discovery_scope_level) {

  # Open log file
  $log_fh = mercury_management_rediscover_open_logfile('discovery');

  # Execute discovery in a try block
  try {
     $success = mercury_management_rediscover_hosts_vms_inner($log_fh, $vcInfo, $discovery_scope_level);
  }
  catch (Exception $e) {
     $success = false;
     $msg = $e->getMessage();
     mercury_management_rediscover_log($log_fh, "Discovery terminated by exception: $msg\n");
     mercury_utilities_fmc_log_message('Discovery', t("Discovery terminated by exception: $msg"),  WATCHDOG_ERROR);
     mercury_utilities_log_exception($e);
  }

  # Close the log file
  mercury_management_rediscover_close_logfile($log_fh);
  
  return $success;
}

/**
 * Call this if doing "new" discovery to remove any VMs and Hosts that are
 * already discovered.
 * @param array $hosts list of host names
 * @param array $vms list of VM names
 */
function mercury_management_discover_target_remove_existing(&$hosts, &$vms) {
  // Get a list of all known hosts, remove all entries from the discovery target
  // list that are also in the query result.
  $query = db_query("SELECT title FROM node WHERE type = 'host'");
  $result = $query->fetchAllAssoc('title');
  if (!empty($result)) {
    foreach ($hosts as $key => $name) {
      if (isset($result[$name])) {
        unset($hosts[$key]);
      }
    }
  }

  // Repeat with the VM targets.
  $query = db_query("SELECT title FROM node WHERE type = 'virtual_machine'");
  $result = $query->fetchAllAssoc('title');
  if (!empty($result)) {
    foreach ($vms as $key => $name) {
      if (isset($result[$name])) {
        unset($vms[$key]);
      }
    }
  }
}

function mercury_management_list_annotated_components($componentType, $dataCenter, &$annotatedComponentList) {

  $vcInfo = mercury_management_vcenter_details();
  $fmcUuid = $vcInfo->field_fmc_uuid['und'][0]['value'];
  $vcHostname = $vcInfo->field_hostname['und'][0]['value'];
  $vcUsername = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vcPassword = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $componentList = array();

  if ($fmcUuid == '') {
    return 'Error: fmc_uuid not set';
  }

  switch ($componentType) {
  case 'host':
    $componentTypeField = 'Host';
    $command = 'getAnnotatedHosts';
    break;
  case 'vm':
    $componentTypeField = 'VM';
    $command = 'getAnnotatedVMs';
    break;
  default:
    return 'Error: Invalid component';
  }

  $executionOutputLine = execute_command("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server '". $vcHostname ."' --username '". $vcUsername. "' --password '". $vcPassword ."' --operation ". $command . " --datacenter ". $dataCenter . " --annotation '". $fmcUuid ."'", $componentListOutput);
  if (stristr($executionOutputLine, 'Error')) {
      $annotatedComponentList = $executionOutputLine;
      return false;
  }

  foreach ($componentListOutput as $componentLine) {
    $component = explode(":", $componentLine);
    if ((count($component) < 2) || (strpos(trim($component[0]), $componentTypeField) === false)) {
      continue;
    }
    array_push($componentList, trim($component[1]));
  }
  $annotatedComponentList = $componentList;
  return true;
}



/**
 *  Function to created & update a virtual machine
 * @TODO passing in the host nid is dumb.  We don't use it anyway.
 * Rather, we just pull in the real current host from vminfo.
 * But requiring it to be passed in results in extra work for the caller.
 */
function mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vm, $host_nid) {

  # Initialize return value
  $rv = '';
  
  # Open log file
  $log_fh = mercury_management_rediscover_open_logfile('refresh');

  mercury_management_rediscover_log($log_fh, "Starting refresh of VM $vm (host nid $host_nid)\n");

  # Create worker to do the heavy lifting (primary advantage: persistent 
  # connection to vCenter server).  No need for more than 1 given how
  # parallelism is currently accomplished.
  $num_worker_tasks = 1;
  $wrkr_array = mercury_management_create_worker_array(
    $num_worker_tasks, 
    $vc_hostname,
    $vc_username,
    $vc_password
  );
  if ($wrkr_array === false) {
    mercury_management_rediscover_log($log_fh, "VM $vm refresh: Could not create worker array!\n");
    mercury_utilities_fmc_log_message('VM Refresh', t('Could not connect to vCenter server.  Possible causes include invalid credentials, network problem, or server is down.'),  WATCHDOG_ERROR);
    mercury_management_rediscover_close_logfile($log_fh);
    return 'error--';
  }

  # Assemble the VM UoW queue (just one UoW in it)
  $uow = mercury_management_assemble_vm_uow(
    $log_fh,
    $vc_hostname, 
    $vc_username, 
    $vc_password, 
    $vm, 
    false, # Always redo annotations on a refresh
    $host_nid
  );
  $vm_uow_queue = array();
  $vm_uow_queue[0] = &$uow;
  
  # Refresh the VM
  if (mercury_management_process_uow_queue($vm_uow_queue, $wrkr_array)) {
    if (isset($uow['error'])) {
      $err = $uow['error'];
      mercury_management_rediscover_log($log_fh, "VM $vm refresh error: $err\n");
      $rv .= 'error--';
    } else {
      mercury_management_rediscover_log($log_fh, "Completed VM $vm refresh.\n");
    }
  } else {
    mercury_management_rediscover_log($log_fh, "VM $vm refresh terminated prematurely!\n");
    $rv .= 'error--';
  }
    
  # Make sure to clean up the worker pool
  mercury_management_destroy_worker_array($wrkr_array);
  
  # Append VM nid to our result (this is the only result in the success case)
  if (isset($uow['vm_info'])) {
    $vm_info = $uow['vm_info'];
    $rv .= $vm_info->nid;
  }
  
  # Close the log file
  mercury_management_rediscover_close_logfile($log_fh);

  return $rv;
}


/**
 * Refresh a Host that already exists in the database.
 * Brand new hosts must call mercury_management_assemble_host_uow directly.
 * @TODO Add an optional $retries parameter, see BURT 721683
 * @see mercury_management_updatehost
 * @see mercury_management_assemble_host_uow
 * @param stdClass $host
 * @return boolean true on success, false on failure
 */
function mercury_management_refresh_host(stdClass &$host) {

  // Pull out the data that the old version expects.
  $ancestors = mercury_management_get_ancestors($host);
  $vc_creds = mercury_management_get_vcenter_credentials($ancestors['vcenter']);

  // And pass it along.
  $result = mercury_management_updatehost(
    $vc_creds['address'], $vc_creds['username'], $vc_creds['password'],
    $host->title, $ancestors['datacenter']->nid
  );

  // For safety, object is passed by reference(!) and forced to refresh after
  // the database is updated.  Otherwise you can get mismatches between value
  // and safe value, etc. or simply stale data if you are not careful.
  // Confirmed experimentally that clearing the node cache by passing in true
  // is not needed (cache is already clear after last node_save anyway).
  // Note that PHP5+ objects are passed by "identifier" by default, but refresh
  // involves some clone operations so this has to be a real pass by reference.
  $host = node_load($host->nid);

  // The old return values are pretty useless.
  // Just return true or false (failed) instead.
  if (strpos($result, 'error--') !== false) {
    return false;
  }
  return true;
}

/**
 * Old function for refreshing a host. Should be replaced by
 * mercury_management_refresh_host over time.
 * @TODO Also need a multiple host version for bulk refresh, parallelize.
 * @see mercury_management_refresh_host
 * @deprecated
 */
function mercury_management_updatehost($vc_hostname, $vc_username, $vc_password, $hostname, $datacenter_nid, $is_installing = false) {

  # Initialize return value
  $rv = '';
  
  # Open log file
  $log_fh = mercury_management_rediscover_open_logfile('refresh');

  mercury_management_rediscover_log($log_fh, "Starting refresh of host $hostname (d/c nid $datacenter_nid)\n");

  # Create worker to do the heavy lifting (primary advantage: persistent 
  # connection to vCenter server).  No need for more than 1 given how
  # parallelism is currently accomplished.
  $num_worker_tasks = 1;
  $wrkr_array = mercury_management_create_worker_array(
    $num_worker_tasks, 
    $vc_hostname,
    $vc_username,
    $vc_password
  );
  if ($wrkr_array === false) {
    mercury_management_rediscover_log($log_fh, "Host $hostname refresh: Could not create worker array!\n");
    mercury_utilities_fmc_log_message('Host Refresh', t('Could not connect to vCenter server.  Possible causes include invalid credentials, network problem, or server is down.'),  WATCHDOG_ERROR);
    mercury_management_rediscover_close_logfile($log_fh);
    return 'error--';
  }

  # Assemble the host UoW queue (just one UoW in it)
  $uow = mercury_management_assemble_host_uow(
    $log_fh,
    $vc_hostname, 
    $vc_username, 
    $vc_password, 
    $hostname, 
    $datacenter_nid,
    FALSE, // Force a check of the install status (ignore annotation)
    $is_installing
  );
  $host_uow_queue = array();
  $host_uow_queue[0] = &$uow;
  
  # Refresh the host
  if (mercury_management_process_uow_queue($host_uow_queue, $wrkr_array)) {
    if (isset($uow['error'])) {
      $err = $uow['error'];
      mercury_management_rediscover_log($log_fh, "Host $hostname refresh error: $err\n");
      $rv .= 'error--';
    } else {
      mercury_management_rediscover_log($log_fh, "Completed host $hostname refresh.\n");
    }
  } else {
    mercury_management_rediscover_log($log_fh, "Host $hostname refresh terminated prematurely!\n");
    $rv .= 'error--';
  }
    
  # Make sure to clean up the worker pool
  mercury_management_destroy_worker_array($wrkr_array);
  
  # Append host nid to our result (this is the only result in the success case)
  if (isset($uow['host_info'])) {
    $host_info = $uow['host_info'];
    $rv .= $host_info->nid;
  }
  
  # Close the log file
  mercury_management_rediscover_close_logfile($log_fh);

  return $rv;
}


/**
 *  Function for the redicovery confirm form and submit.
 */
function mercury_management_rediscover_confirm($form, &$form_state) {

  // We need to know whether we're discovering "all" or "new" and pass it to
  // the new form, so that on submit it does the right thing.
  // Globals is not an optimal way to do it, but without more refactoring
  // that is the simplest way to do it.
  // @see mercury_utilities_discover_*_submit
  global $discovery_scope_level;
  if (!empty($form_state['input']['discovery_scope_level'])) {
    // This is a byproduct of some dumb things that the popup system does.
    $discovery_scope_level = $form_state['input']['discovery_scope_level'];
  }
  if (empty($discovery_scope_level)) {
    throw new \Exception('Something went wrong fetching $discovery_scope_level!');
  }
  $form['discovery_scope_level'] = array(
    '#type' => 'hidden', // Cannot use 'value' here, popup system breaks it.
    '#value' => $discovery_scope_level,
  );

  $form['title_message'] = array(
    '#type' => 'markup',
    '#markup' => '<div id="redisc_mesg"></div>',
  );

  $form['title'] = array(
    '#type' => 'markup',
    '#markup' => '<div class = "rediscover-warn">' . t('Discovery is a long running operation. Execution time will vary depending upon the number of Hosts and VMs present in each of the selected Datacenters. <br>') . '</div>',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Start Discovery',
  );

   //Redirecting the Ajax form Submit
  $form['#action'] = url('ctools-popups/nojs/mmc/rediscover');

  // XXX: Move all of these to a js file.
  // Note that multi-submit forms like import cannot let the fields be disabled.
  $form['cancel'] = array(
    '#type' => 'button',
    '#value' => 'Cancel',
    '#suffix' => '<script>jQuery("#edit-cancel").click(function () { jQuery("#edit-cancel").hide()});</script>'
  );

  return $form;
}

function mercury_management_rediscover_confirm_submit($form, &$form_state) {
  if (empty($form_state['input']['discovery_scope_level'])) {
    throw new \Exception('Something went wrong recovering $discovery_scope_level!');
  }
  mercury_management_rediscover($auto_discovery = false, $form_state['input']['discovery_scope_level']);
}

function mercury_management_rediscover_cancel() {
  $vcInfo = mercury_management_vcenter_details();
  $rediscover = $vcInfo->field_rediscover_status['und'][0]['value'];
  $vcInfo->field_rediscover_status['und'][0]['value'] = "rediscover_stop";
  node_save($vcInfo);

  mercury_utilities_fmc_log_message('Discovery', t('Cancelling rediscovery. Status = @status -- @time', array('@status' => $rediscover, '@time' => date('H:i:s'))));

  # Wait a bit for the discovery thread to see the stop and terminate
  $vcInfo = mercury_management_vcenter_details();
  $rediscover = $vcInfo->field_rediscover_status['und'][0]['value'];
  $cancel_wait_secs  = 0;
  $cancel_sleep_secs = 1;
  while ($rediscover != 'rediscover_terminated' && $cancel_wait_secs < 30) {
    sleep($cancel_sleep_secs);
    $cancel_wait_secs += $cancel_sleep_secs;
    $vcInfo = mercury_management_vcenter_details();
    $rediscover = $vcInfo->field_rediscover_status['und'][0]['value'];
  }

  # If we timed out, log it and continue; discovery thread is probably gone.
  if ($rediscover != 'rediscover_terminated') {
    mercury_utilities_fmc_log_message('Discovery', t('Cancel timed out! Status = @status -- @time', array('@status' => $rediscover, '@time' => date('H:i:s'))),  WATCHDOG_WARNING);
  }

  $vcInfo->field_rediscover_status['und'][0]['value'] = '';
  node_save($vcInfo);
  # Ensure node cache is updated
  mercury_management_vcenter_details();

  mercury_utilities_fmc_log_message('Discovery', t('Discovery was cancelled. Information presented might be inaccurate or incomplete.'), WATCHDOG_WARNING, 'show');
  if ($rediscover != 'rediscover_terminated') {
     mercury_utilities_fmc_log_message('Discovery', t('Discovery cancellation timed out. This is not expected and may indicate a software problem.'), WATCHDOG_ERROR, 'show');
  }
}


/**
 * Possible annotation commands and associated outputs (given that username, hostname and password are correct):
 * set/getDataCenterAnnotation
 * set/getHostAnnotation
 * set/getVMAnnotation
 *
 * Error: No custom field named '<field_name>' found. (<field_name> - 'FA', 'FA.HostAgent', 'FA.GoSAgent')
 * Error: Could not get <component> entity ('VirtualMachine', 'HostSystem', 'DataCenter')
 * Error: DataCenter <data_center_name> ownership changed
 * Error: Host <host_name> ownership changed
 * Error: VM <vm_name> ownership changed
 */
function mercury_management_access_annotations($component_type, $component_name, $operation, $fieldvalue='') {

  $vcInfo = mercury_management_vcenter_details();
  if (!$fieldvalue) {
    $fmc_uuid = $vcInfo->field_fmc_uuid['und'][0]['value'];
  } else {
    $fmc_uuid = $fieldvalue;
  }
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  if ($fmc_uuid == '') {
    return 'Error: annotation value not set';
  }

  switch ($component_type) {
  case 'host':
    $component = 'hostname';
    $command = 'HostAnnotation';
    break;
  case 'vm':
    $component = 'vmname';
    $command = 'VMAnnotation';
    break;
  default:
    return 'Error: Invalid component';
  }

  if ($operation != 'set' && $operation != 'get' && $operation != 'clear') {
    return 'Error: Invalid operation';
  }

  $annotation_status_line = execute_command("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server '". $vc_hostname ."' --username '". $vc_username. "' --password '". $vc_password ."' --operation get".   $command . " --". $component. " '".  $component_name ."' --annotation '". $fmc_uuid ."'", $annotation_status_output_array);

  $annotation_status = explode(":", $annotation_status_line);
  if ((count($annotation_status) > 1)) {
    $annotation_result = $annotation_status[1];
  } else {
    $annotation_result = '';
  }
  
  switch ($operation) {
  case 'set':
    /* The third condition is to overlook change in ownership for now. When ownership of FMC is considered, this condition should be removed */
    if (stristr($annotation_result, 'Not Annotated') || stristr($annotation_result, 'No custom field') || stristr($annotation_result, 'ownership changed')) {
      $annotation_result_line = execute_command("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server '". $vc_hostname ."' --username '". $vc_username. "' --password '". $vc_password ."' --operation set". $command . " --". $component. " '".  $component_name ."' --annotation '". $fmc_uuid ."'", $annotation_status_output_array);
      if (stristr($annotation_result_line, 'Error')) {
        return 'Error: Annotation failure '.$annotation_result_line;
      }
    } else if (stristr($annotation_result, 'Annotated')) {
        return 'Success';
    } else {
      return 'Error: Annotation failure '. $annotation_result_line;
    }
    break;
  case 'get':
    return $annotation_status_line;
  case 'clear':
    if (stristr($annotation_result, 'Not Annotated') || stristr($annotation_result, 'No custom field') || stristr($annotation_result, 'Annotated') || stristr($annotation_result, 'ownership changed')) {
      $annotation_result_line = execute_command("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server '". $vc_hostname ."' --username '". $vc_username. "' --password '". $vc_password ."' --operation set". $command . " --". $component. " '".  $component_name ."' --annotation ''", $annotation_status_output_array);
      if (stristr($annotation_result_line, 'Error')) {
        return 'Error: Annotation failure '.$annotation_result_line;
      }
    } else {
      return 'Error: Annotation failure '.$annotation_status_line;
    }

    break;
  default:
    return 'Error: Invalid operation';
  }
}

/**
 * Orchestrates most of the discovery process.
 * @param boolean $auto_discovery If true, skips extra logging and redirects.
 * @param string $discovery_scope_level 'new' or 'all'
 * @return boolean false if errors, true otherwise (only if $auto_discovery)
 */
function mercury_management_rediscover($auto_discovery = false, $discovery_scope_level = 'all') {
  //Get the vCenter Object value
  $vcInfo = mercury_management_vcenter_details();
  if (empty($vcInfo)) {
    mercury_utilities_fmc_log_message('Discovery', 'Failed to locate vCenters for doing discovery',  WATCHDOG_ERROR);
    return false;
  }

  // Mark we are starting rediscovery
  mercury_utilities_discovery_progress_init_ticks(0);
  $vcInfo->field_rediscover_status['und'][0]['value'] = 'rediscover_start';
  node_save($vcInfo);

  $success = mercury_management_rediscover_hosts_vms($vcInfo, $discovery_scope_level);

  // Check discovery status, end early if currently being cancelled.
  // XXX: This needs to be a stronger check, and exit is not great.
  $vcInfo = mercury_management_vcenter_details();
  $rediscover_status = $vcInfo->field_rediscover_status['und'][0]['value'];
  if ($rediscover_status == 'rediscover_terminated') {
    exit;
  }

  // Record last time of discovery
  $time = strtotime('now');
  variable_set('time_from_last_discovery', $time);

  # Reclaim all the cache spaces
  mercury_management_reclaim_cachespaces();

  /*
   * If we have any saved credentials from an import, try to apply them now.
   * In theory we could do this with a node_presave hook, but we save and
   * improperly load hosts and vms several times during discovery
   * and it just gets blown out!
   */
  module_load_include('inc', 'mercury_utilities', 'includes/export_import');
  MercuryImportExport::import_apply_all_saved_credentials();

  // Now that we're actually done, indicate as such in the vc node.
  $vcInfo = node_load($vcInfo->nid, NULL, TRUE); // Reload just in case.
  $vcInfo->field_rediscover_status['und'][0]['value'] = '';
  node_save($vcInfo);

  // Mark ticks as really complete, too.
  // This leaves us at < 100% until now, when it's really done.
  mercury_utilities_discovery_progress_complete_ticks();

  // If it was not auto discovery - display result on screen
  if ($auto_discovery === false) {

    // Check if the rediscovery encountered errors
    if (empty($success)) {
      mercury_utilities_fmc_log_message('Discovery', t('There were failures during discovery.  Preceding messages and Console Logs view may provide more details.'), WATCHDOG_WARNING);
    }

    // Success!
    else {
      mercury_utilities_fmc_log_message('Discovery', 'Rediscovery completed successfully.');
    }

    // Note: the following function calls exit.
    ctools_popups_ajax_redirect();
  }

  // Auto-discovery, need to return something.
  return true;
}
