<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for the discovery cron.
 */

/**
 * Function to execute CRON JOB for Mercury Management Console
 * using hook_cron()
 */
function mercury_management_cron() {

  // Near the start of every cron run (once per minute currently), check the
  // logs for saved passwords.  This offers an extra layer of safety.
  // This only adds about 0.003 seconds per cron run when it finds nothing
  // and doesn't run at all in promoted builds.
  mercury_management_check_logs_for_passwords();

  //Rediscover Hour settings value
  $target_hour = variable_get('redesc_cron_hour', '0');

  // skip rediscovery
  if ($target_hour == 0) {
    return;
  }

  // Reduce by one - accounting for array position
  $target_hour--;

  //Calculate the Hour difference between two cron job executions
  $diff_value = abs(strtotime('now') - variable_get('time_from_last_discovery'));

  //Check of the settings value is greater than last cron call
  $this_hour = date('H');
  if ($this_hour == $target_hour && $diff_value > 3600) {

    // Check to see if a discovery is in progress
    $vcInfo = mercury_management_vcenter_details();
    $rediscover = $vcInfo->field_rediscover_status['und'][0]['value'];
    if ($rediscover == 'rediscover_stop' || $rediscover == 'rediscover_start') {
      mercury_utilities_fmc_log_message('Discovery', t('Rediscover already in progress. Status = @status', array('@status' => $rediscover)));
      return;
    }

    // Do the discovery. Cron discovery should be "all" not just "new"
    $success = mercury_management_rediscover($auto_discovery = true, 'all');

    // Check the outcome of discovery
    $vcInfo = mercury_management_vcenter_details();
    $rediscover = $vcInfo->field_rediscover_status['und'][0]['value'];
    if ($rediscover != 'rediscover_terminated') {
      if (empty($success)) {
        mercury_utilities_fmc_log_message('Discovery', t('Auto Discovery failed on some hosts/VMs.'), WATCHDOG_WARNING);
      } else if ($rediscover != '') {
        mercury_utilities_fmc_log_message('Discovery', t("Auto Discovery BUG: $rediscover!"), WATCHDOG_ERROR);
      } else {
        mercury_utilities_fmc_log_message('Discovery', t('Auto Discovery completed.'));
      }

      // clear rediscovery variable
      $vcInfo->field_rediscover_status['und'][0]['value'] = "";
      node_save($vcInfo);
    }
  }
}
