<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 * 
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *  
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *  
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *  
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 *  Delete multiple given nids.  I'd like to use node_delete_multiple, but it 
 *  can take a really long time; suspect it is memory bound.
 */
function mercury_management_node_delete_multiple($log_fh, $nids, $str_type) 
{
   foreach ($nids as $nid) {
      mercury_management_rediscover_log($log_fh, "deleting $str_type node $nid...\n");     
      node_delete($nid);
   }
}

/**
 *  Cause the FMC database to have datacenter nodes for all and only those 
 *  datacenters whose names are in $datacenter_names.
 */
function mercury_management_synchronize_datacenter_nodes($datacenter_names, stdClass $vcInfo, $log_fh = NULL)
{
   global $user;

   // Since we're already processing all of the nodes, return name => nid.
   $datacenter_nids = array();

   $vcenter_nid = $vcInfo->nid;
   mercury_management_rediscover_log($log_fh, "Synchronizing datacenters with vcenter server (vcenter_nid $vcenter_nid).\n");

   # Query existing datacenters   
   $query = new EntityFieldQuery();
   $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', 'data_center');
   $result = $query->execute();
   if (isset($result['node'])) {
      $datacenter_nids = array_keys($result['node']);
   }

   # Optimize the case where we're deleting all datacenters
   if (empty($datacenter_names)) {
      if (isset($datacenter_nids)) {
         $nvn = count($datacenter_nids);
         mercury_management_rediscover_log($log_fh, "Deleting $nvn datacenter nodes.\n");
         mercury_management_node_delete_multiple($log_fh, $datacenter_nids, 'datacenter');
      } else {
         mercury_management_rediscover_log($log_fh, "No datacenter nodes to delete.\n");
      }
      return array();
   }
   
   # We have some datacenters, some of which may already have nodes.

   # Load all existing nodes; we're going to delete the ones that don't get
   # updated.    
   if (isset($datacenter_nids)) {
      $datacenter_nodes = entity_load('node', $datacenter_nids);
   } else {
      $datacenter_nodes = array();
   }

   # Search for each of the passed-in datacenters in the existing datacenter nodes.
   foreach ($datacenter_names as $datacenter_name) {
      $action = "creating";
      foreach ($datacenter_nodes as $dn_key => $datacenter_node) {
         if ($datacenter_node->title === $datacenter_name) {
             $action = "updating";
             break;
         }
      }
      if ($action == "creating") {
         $datacenter_node = new stdClass();
         $datacenter_node->type = 'data_center';
         $datacenter_node->author = $user->name;
         $datacenter_node->status = 1;
         $datacenter_node->uid = $user->uid;
         $datacenter_node->comment = 0;
         $datacenter_node->language = 'und';
         $datacenter_node->format = 2;
         $datacenter_node->title = trim($datacenter_name);
      } else { # updating
         # Later we're going to delete everything left in datacenter_nodes
         unset($datacenter_nodes[$dn_key]);
      }
      $datacenter_node->field_related_vcenter['und'][0]['target_id'] = $vcenter_nid;

      node_save($datacenter_node);

      
      $nid = $datacenter_node->nid;
      $id = $datacenter_node->title;
      $datacenter_nids[$datacenter_name] = $nid;
      mercury_management_rediscover_log($log_fh, "$action datacenter nid $nid id $id\n");
   }

   # Any datacenter still "set" in datacenter_nodes needs to be deleted   
   $datacenter_nids_to_delete = array();
   foreach ($datacenter_nodes as $datacenter_node) {
      $nid = $datacenter_node->nid;
      $id = $datacenter_node->title;
      mercury_management_rediscover_log($log_fh, "deleting datacenter nid $nid id $id\n");
      $datacenter_nids_to_delete[] = $datacenter_node->nid;
   }
   if (!empty($datacenter_nids_to_delete)) {
      mercury_management_node_delete_multiple($log_fh, $datacenter_nids_to_delete, 'datacenter');
   }

   return $datacenter_nids;
}


/**
 *  Finds a list of node ids for nodes of the given type whose titles do not 
 *  appear in the $non_obsolete_titles list.
 */
function mercury_managment_find_obsolete_nodes($type, $non_obsolete_titles) 
{
   # Obtain nids for all $type nodes with titles NOT in the given list   
   $query = new EntityFieldQuery();
   $query->entityCondition('entity_type', 'node')
      ->propertyCondition('type', $type);
   if (!empty($non_obsolete_titles)) {
      $query->propertyCondition('title', $non_obsolete_titles, 'NOT IN');
   }
   $result = $query->execute();

   # Extract and return the list of nids
   if (isset($result['node'])) {
      $nids = array_keys($result['node']);
   } else {
      $nids = array();
   }
   
   return $nids;
}


/**
 *  Finds a list of node ids for nodes of a given type tha are dependent on 
 *  nodes in a given list of "parent" nids, in the sense that their target_id 
 *  fields point at one of the parent nodes.
 */
function mercury_management_find_dependent_nodes($parent_nids, $dn_type, $target_id_field, $log_fh = NULL, $pfx = '') 
{  
   if (empty($parent_nids)) {
      return array();
   }
   
   # Find the dependent nodes using the provided node type and "link" fields
   $query = new EntityFieldQuery();
   $query->entityCondition('entity_type', 'node')
     ->entityCondition('bundle', $dn_type)
     ->propertyCondition('status', 1)
     ->fieldCondition($target_id_field, 'target_id', $parent_nids, 'IN');

   $result = $query->execute();
   
   # Extract and return the list of nids
   if (isset($result['node'])) {
      $dependent_nids = array_keys($result['node']);
   } else {
      $dependent_nids = array(); 
   }
   
   if ($log_fh) {
      $npn = count($parent_nids);
      $ndn = count($dependent_nids);
      mercury_management_rediscover_log($log_fh, "$pfx$ndn $dn_type nodes linked to $npn parent nodes\n");     
   }
   
   return $dependent_nids;
}


/**
 *  Finds and deletes dependent nodes (see mercury_management_find_dependent_nodes)
 */
function mercury_management_delete_dependent_nodes($log_fh, $parent_nids, $dn_type, $target_id_field) 
{
   # Find the dependent nodes using the provided node type and "link" fields
   $nids = mercury_management_find_dependent_nodes($parent_nids, $dn_type, $target_id_field, $log_fh, 'deleting ');
   
   # Delete them
   mercury_management_node_delete_multiple($log_fh, $nids, $dn_type);
}


/**
 *  Deletes all host nodes in the list, including subtending artifacts.
 */
function mercury_management_delete_hosts($log_fh, $host_nids) 
{
   # If there are some...
   $nhn = count($host_nids);
   if ($nhn) {

      # Delete any subtending device nodes
      mercury_management_delete_dependent_nodes($log_fh, $host_nids, 'device', 'field_related_host');

      # Delete the host nodes
      mercury_management_node_delete_multiple($log_fh, $host_nids, 'host');
   }

   # Log status
   mercury_management_rediscover_log($log_fh, "deleted $nhn obsolete hosts\n");
}


/**
 *  Deletes all VM nodes in the list, including subtending artifacts.
 */
function mercury_management_delete_vms($log_fh, $vm_nids) 
{
   # If there are some...
   $nvn = count($vm_nids);
   $total_nvdn = 0;
   if ($nvn) {

      # Delete any subtending VM metrics nodes
      mercury_management_delete_dependent_nodes($log_fh, $vm_nids, FAMC_ANALYTICS_VM_METRIC_TYPE, 'field_vm');

      # Obtain nids for all vdisk nodes subtending these VM nodes
      $vdisk_nids = mercury_management_find_dependent_nodes($vm_nids, 'vdisk', 'field_virtual_machine', $log_fh, 'deleting ');
      $nvdn = count($vdisk_nids);
      
      # If there are some...
      if ($nvdn) {
         $total_nvdn += $nvdn;
         
         # Delete any subtending vdisk metrics nodes
         mercury_management_delete_dependent_nodes($log_fh, $vdisk_nids, FAMC_ANALYTICS_VDISK_METRIC_TYPE, 'field_vdisk');
         
         # Delete the vdisk nodes
         mercury_management_node_delete_multiple($log_fh, $vdisk_nids, 'vdisk');
      }
      
      # Delete the VM nodes
      mercury_management_node_delete_multiple($log_fh, $vm_nids, 'virtual_machine');
   }
   
   # Log status
   mercury_management_rediscover_log($log_fh, "deleted $nvn obsolete VMs with $total_nvdn total disks\n");
}


/**
 *  Deletes all vdisk nodes in the list, including subtending artifacts.
 */
function mercury_management_delete_vdisks($log_fh, $vdisk_nids) 
{
   # If there are some...
   $nvdn = count($vdisk_nids);
   if ($nvdn) {
      
      # Delete any subtending vdisk metrics nodes
      mercury_management_delete_dependent_nodes($log_fh, $vdisk_nids, FAMC_ANALYTICS_VDISK_METRIC_TYPE, 'field_vdisk');
      
      # Delete the vdisk nodes
      mercury_management_node_delete_multiple($log_fh, $vdisk_nids, 'vdisk');
   }
   
   # Log status
   mercury_management_rediscover_log($log_fh, "deleted $nvdn vdisk nodes\n");
}


/**
 *  Finds all host nodes whose names are not in $non_obsolete_host_names.
 */
function mercury_management_find_obsolete_hosts($log_fh, $non_obsolete_host_names)
{
   # Obtain nids for all host nodes with names NOT in the list
   $host_nids = mercury_managment_find_obsolete_nodes('host', $non_obsolete_host_names);
   $nhn = count($host_nids);

   # Log status
   mercury_management_rediscover_log($log_fh, "found $nhn obsolete hosts\n");
   
   # Return nids
   return $host_nids;
}


/**
 *  Finds all VM nodes whose names are not in $non_obsolete_vm_names.
 */
function mercury_management_find_obsolete_vms($log_fh, $non_obsolete_vm_names) 
{
   # Obtain nids for all VM nodes with names NOT in the list
   $vm_nids = mercury_managment_find_obsolete_nodes('virtual_machine', $non_obsolete_vm_names);
   $nvn = count($vm_nids);

   # Log status
   mercury_management_rediscover_log($log_fh, "found $nvn obsolete VMs\n");

   # Return nids
   return $vm_nids;
}


/**
 * Given set of datacenter relationships, remove all datacenters that do not
 * have checkmarks next to them on Console Settings > Discovery Settings.
 * By reference to reduce copying.
 */
function mercury_management_extract_selected_datacenters(&$vCenterArr) {
  $rediscover_item_settings = variable_get('rediscover_item_settings', array());

  $all_dc_names = array_keys($vCenterArr);
  foreach ($all_dc_names as $dc_name) {
    if (!in_array($dc_name, $rediscover_item_settings)) {
      unset($vCenterArr[$dc_name]);
    }
  }
}


/**
 * Given a nested array of datacenter -> host -> vm,
 * compose separate lists of hosts and VMs in those datacenters.
 */
function mercury_management_synchronize_split_hosts_vms(array $vCenterArr, &$hosts, &$vms, &$host_datacenters, &$vm_hosts) {
  $hosts = array();
  $vms = array();
  foreach ($vCenterArr as $dc_name => $hosts_nested) {
    foreach ($hosts_nested as $host_name => $vms_for_host) {
      $hosts[] = $host_name;
      $host_datacenters[$host_name] = $dc_name;
      foreach ($vms_for_host as $vm_name) {
        $vms[] = $vm_name;
        $vm_hosts[$vm_name] = $host_name;
      }
    }
  }
}
