<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 * 
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *  
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *  
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *  
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

define('FAMC_VM_DRIVER_STATUS_INSTALLED',      'Agent Installed');
define('FAMC_VM_DRIVER_STATUS_NOT_INSTALLED',  'Agent Not Installed');
define('FAMC_VM_DRIVER_STATUS_NOT_COMPATIBLE', 'Not Compatible');
define('FAMC_VM_DRIVER_STATUS_NOT_KNOWN',      'Unable to Communicate with VM');
define('FAMC_VM_GUEST_OS_NOT_KNOWN',           'Not Known');
define('FAMC_WEBSERVICE_DEFAULT_PORT',         '7010');
define('FAMC_VM_VMWARE_TOOLS_OK',              'VMware Tools is running and the version is current');
define('FAMC_VM_VMWARE_TOOLS_OLD',             'VMware Tools is running, but the version is not current');
define('FAMC_VM_VMWARE_TOOLS_NOT_RUNNING',     'VMware Tools is not running.');
define('FAMC_VM_VMWARE_TOOLS_NOT_INSTALLED',   'VMware Tools has never been installed or has not run in the virtual machine.');


/**
 *  Create or update the database entry (node) for a given VM.
 */
function mercury_management_create_or_update_vm_node(&$uow, $vm_info_with_key, $host_nid)
{
  global $user;
 
  $action = "<undefined>";
  $nid = db_query("SELECT nid FROM {node} WHERE type = :tp AND title = :ttl", array(':tp' => 'virtual_machine', ':ttl' => trim($vm_info_with_key['Name'])))->fetchField();
  if (!empty($nid)) {
    $vm_info = node_load($nid);
    $uow['vm_info_orig'] = clone $vm_info;
    $action = "updated";
  }
  else {
    $vm_info = new stdClass();
    $vm_info->type = 'virtual_machine';
    $vm_info->author = $user->name;
    $vm_info->status = 1;
    $vm_info->uid = $user->uid;
    $vm_info->comment = 0;
    $vm_info->language = 'und';
    $vm_info->format = 2;
    $vm_info->title = trim($vm_info_with_key['Name']);
    mercury_management_initialize_vm_node($vm_info);
    $action = "created";
  }

  // Clear out any stale error messages.
  $vm_info->field_vm_discovery_error['und'][0]['value'] = '';

  $vm_info->field_vm_ip_address['und'][0]['value'] = $vm_info_with_key['IP Address'];
  $vm_info->field_no_of_cpu['und'][0]['value'] = $vm_info_with_key['No. of CPU(s)'];
  $vm_info->field_memory_size['und'][0]['value'] = $vm_info_with_key['Memory Size'];
  // @deprecated field_virtual_disks is not used.
  $vm_info->field_virtual_disks['und'][0]['value'] = $vm_info_with_key['Virtual Disks'];
  // @deprecated field_template is not used, should delete the field.
  $vm_info->field_template['und'][0]['value'] = $vm_info_with_key['Template'];
  $vm_info->field_vm_path_name['und'][0]['value'] = $vm_info_with_key['vmPathName'];
  $vm_info->field_vm_hostname['und'][0]['value'] = trim($vm_info_with_key['Host name']);
  $vm_info->field_cpu_usage['und'][0]['value'] = $vm_info_with_key['Cpu usage'];
  $vm_info->field_host_memory_usage['und'][0]['value'] = $vm_info_with_key['Host memory usage'];
  $vm_info->field_guest_memory_usage['und'][0]['value'] = $vm_info_with_key['Guest memory usage'];
  $vm_info->field_host['und'][0]['target_id'] = $host_nid;

  // Always update VMware Tools state.
  // It's always definitive (unlike guest OS, install state, etc.) and when it's
  // not running then the stale displayed state is often contradictory with
  // the error messages.
  $vm_info->field_vmware_tools['und'][0]['value'] = $vm_info_with_key['VMware Tools'];

  // Special case: fields that are in vminfo but actually rely on VMware Tools
  // to be running to actually contain anything useful.
  // That's technically a different discovery phase, so we cannot blow out the
  // data if it cannot be definitively determined and sometimes vCenter seems
  // to remember the last known OS when powered off.
  // For performance reasons it's also beneficial to not make a separate request
  // to get this information.
  if (mercury_management_discover_vm_guest_os_valid($vm_info_with_key)) {
    // Is definitive, update.
    $vm_info->field_guest_os['und'][0]['value'] = $vm_info_with_key['Guest OS'];
    $vm_info->field_guest_id['und'][0]['value'] = $vm_info_with_key['guestId'];
  }

  // Phase complete.
  $vm_info->field_vm_discovery_state['und'][0]['value'] = FAMC_DISCOVERY_STATE_VCENTER;

  node_save($vm_info);
  
  $uow['vm_info'] = $vm_info;
  if ($action == 'updated') {
    $uow['vm_info_is_new'] = false;
  } else {
    $uow['vm_info_is_new'] = true; 
  }

  // Phase succeeded, store consistent state temporarily
  // in case of transactional failure rollback.
  $uow['vm_info_consistent'] = clone($vm_info);

  $nid = $vm_info->nid;
  mercury_management_rediscover_vm_log($uow, "$action vm nid $nid (host_nid $host_nid)\n");
  
  return $vm_info;
}

/**
 * Determines whether the OS fields in vminfo are valid, indicating
 * that we can rely on this information returned from VMware Tools.
 * @param array $vm_info_with_key
 * @return boolean
 */
function mercury_management_discover_vm_guest_os_valid($vm_info_with_key) {
  // Note that this intentionally does not check the tools running state
  // because vCenter will sometimes return the correct OS info regardless.
  // This is helpful during discovery of new nodes.
  if (
    !empty($vm_info_with_key['Guest OS']) &&
    !empty($vm_info_with_key['guestId']) &&
    $vm_info_with_key['Guest OS'] != FAMC_VM_GUEST_OS_NOT_KNOWN &&
    $vm_info_with_key['guestId'] != FAMC_VM_GUEST_OS_NOT_KNOWN
  ) {
    return true;
  }
  return false;
}

/**
 * Returns true if VMware Tools is not only running but is actually returning
 * all data.  For the first few seconds of "running" no data will be returned,
 * and then fields will quickly start filling in as the data is collected.
 * @param array $vm_info_with_key
 * @param boolean $strict If true it will do potentially dangerous checks.
 * @return boolean
 */
function mercury_management_discover_vm_vmware_tools_valid($vm_info_with_key, $strict = false) {
  // Most of the time, this condition will result in an empty guest OS.
  if (!mercury_management_discover_vm_guest_os_valid($vm_info_with_key)) {
    return false;
  }

  // That is the end of the safe checks.
  if (!$strict) {
    return true;
  }

  // There is sometimes a brief window where DNS/IP info won't return.
  // This is only OK if we know for sure that a VM is correctly configured.
  // Otherwise a connectivity problem will report as a VMware Tools problem.
  // @see mercury_management_abort_updatevm_due_to_unknown_vm_hostname
  if (
    empty($vm_info_with_key['Host name']) &&
    (empty($vm_info_with_key['IP Addresses']) || $vm_info_with_key['IP Addresses'] == 'Not Known')
  ) {
    return false;
  }

  // TODO: If we can pull something directly from vCenter to reliably determine
  // the tools state then that would be great.  No luck so far.
  // VirtualMachineRuntimeInfo.bootTime?
  // Use guestOperationsReady or interactiveGuestOperationsReady to check the state?
  // If one of those methods turns out to be reliable, it may be best to move
  // the automated vminfo retries into VCD.pm to avoid the overhead of
  // the extra authentications.

  // All tests passed.
  return true;
}

/**
 * Determines whether VMware Tools is installed and running.
 * Note that this does not determine whether VMware Tools is actually returning
 * useful data or whether it is out of date.
 * @param array $vm_info_with_key
 * @return boolean
 */
function mercury_management_discover_vm_vmware_tools_running($vm_info_with_key) {
  if (
    empty($vm_info_with_key['VMware Tools']) ||
    $vm_info_with_key['VMware Tools'] == 'VMware Tools is not running.' ||
    $vm_info_with_key['VMware Tools'] == 'VMware Tools has never been installed or has not run in the virtual machine.'
  ) {
    return false;
  }

  // Either "VMware Tools is running and the version is current" or
  // "VMware Tools is running, but the version is not current".
  // Out of date is generally OK.
  return true;
}

/**
 * Roll back to the last completed phase so we are internally consistent and
 * set the discovery error message field.
 */
function mercury_management_discover_vm_handle_phase_failure(&$uow, $message) {
   $vm_info = $uow['vm_info_consistent'];
   // This could potentially be long, limit to max field length.
   $message = substr($message, 0, 255);
   $vm_info->field_vm_discovery_error['und'][0]['value'] = $message;
   node_save($vm_info);
}


/**
 * Add cache disk to VM database node. 
 */
function mercury_management_add_cache_disk_to_vm_node(&$vm_info, $vDiskArr)
{
  // @see mercury_management_update_vm_vdisk_db_commit() for non-cache disks.

  // If we don't find a cache disk, these need to be blown out.
  $vm_info->field_fa_cache_size['und'][0]['value'] = '';
  $vm_info->field_cachespace['und'][0]['value'] = '';
  $vm_info->field_fa_sector_size['und'][0]['value'] = '';

  $cacheDevice = '';
  if (!empty($vDiskArr)) {
    foreach ($vDiskArr as $vDisk) {
      if ($vDisk['type'] == "CACHE") {
        $vm_info->field_cachespace['und'][0]['value'] = $vDisk['id'];
        $vm_info->field_fa_cache_size['und'][0]['value'] = $vDisk['Blocks'];
        $vm_info->field_fa_sector_size['und'][0]['value'] = $vDisk['Blocksize'];
        $cacheDevice = $vDisk['id'];
        break;
      }
    }
  }
  return $cacheDevice;
}


/**
 *  Create or update the vdisk database entries associated with a VM.
 */
function mercury_management_update_vm_vdisk_db_commit(&$uow, $attrInfo, $vm_nid)
{
   global $user;

   $vdisk_nids = mercury_management_find_dependent_nodes(array($vm_nid), 'vdisk', 'field_virtual_machine');
   $nvn = count($vdisk_nids);

   # Optimize the case where we're deleting all vdisk nodes.
   if (empty($attrInfo) || ($attrInfo['disk_count'] == 0)) {
      if ($nvn) {
         mercury_management_delete_vdisks($uow['log_fh'], $vdisk_nids);
      } else {
         mercury_management_rediscover_vm_log($uow, "couldn't find any vdisk nodes linked to vm_nid $vm_nid\n");
      }
      return true;
   }

   # We have some vdisks, some of which may already have nodes.

   # Load all existing nodes; we're going to delete the ones that don't get
   # updated.      
   if (isset($vdisk_nids)) {
      $vdisk_nodes = entity_load('node', $vdisk_nids);
   } else {
      $vdisk_nodes = array();
   }

   # Search for each of the passed-in vdisks in the existing vdisk nodes.
   $diskCount = $attrInfo['disk_count'];
   $vDiskArr = $uow['vDiskArr'];
   for ($index = 0; $index < $diskCount; $index++) {
      $identifier = "naa." . $attrInfo[$index]['vdisk_identifier'];
      $identifier = strtolower(trim($identifier));
      $action = "creating";
      foreach ($vdisk_nodes as $vi_key => $vdisk_info) {
         if ($vdisk_info->title === $identifier) {
             $action = "updating";
             break;
         }
      }
      if ($action == "creating") {
         $vdisk_info = new stdClass();
         $vdisk_info->type = 'vdisk';
         $vdisk_info->author = $user->name;
         $vdisk_info->status = 1;
         $vdisk_info->uid = $user->uid;
         $vdisk_info->comment = 0;
         $vdisk_info->language = 'und';
         $vdisk_info->format = 2;
         $vdisk_info->title = $identifier;
         mercury_management_initialize_vdisk_node($vdisk_info);
      } else {
         # Later we're going to delete everything left in vdisk_nodes
         unset($vdisk_nodes[$vi_key]);
      }

      $vdisk_info->field_virtual_machine['und'][0]['target_id'] = $vm_nid;
      //Figure out the appropriate type for the identifier
    
      $disk_type='Direct';
      if (!empty($vDiskArr)) {
        foreach ($vDiskArr as $value){
         // watchdog('test', $identifier .'='. $value['id']);
          if (strtolower($identifier)== strtolower($value['id'])) {
            $disk_type=$value['type'];
          }
        }
      }

      // All 4 of these fields are in $vDiskArr but only field_vdisk_type
      // gets saved in the node. Name, capacity, and freespace are @deprecated.
      $vdisk_info->field_vdisk_type['und'][0]['value'] = $disk_type;
      $vdisk_info->field_vdisk_name['und'][0]['value'] = '';
      $vdisk_info->field_capacity['und'][0]['value'] = 0;
      $vdisk_info->field_freespace['und'][0]['value'] = 0;

      # Update the cache information
      $vdisk_info->field_fa_cache_mode['und'][0]['value'] = $attrInfo[$index]['cache_mode'];
      $vdisk_info->field_fa_administration_state['und'][0]['value'] = $attrInfo[$index]['admin_state'];
      $vdisk_info->field_fa_operational_state['und'][0]['value'] = $attrInfo[$index]['op_state'];
      $vdisk_info->field_fa_operational_message['und'][0]['value'] = $attrInfo[$index]['op_message'];
      $vdisk_info->field_fa_performance_state['und'][0]['value'] = $attrInfo[$index]['perf_state'];
      $vdisk_info->field_vdisk_identifier['und'][0]['value'] = $attrInfo[$index]['vdisk_identifier'];
      if ($attrInfo[$index]['path'] != '') {
         $vdisk_info->field_path['und'][0]['value'] = $attrInfo[$index]['path'];
      }

      $vdisk_info->field_disk_index['und'][0]['value'] = $index;
      $vdisk_info->field_vdisk_id['und'][0]['value'] = "naa." . $attrInfo[$index]['vdisk_identifier'];
            
      node_save($vdisk_info);
      
      $nid = $vdisk_info->nid;
      mercury_management_rediscover_vm_log($uow, "$action vdisk nid $nid (vm_nid $vm_nid) id $identifier\n");
   }

   # Any device still "set" in vdisk_nodes needs to be deleted   
   $vdisk_nids_to_delete = array();
   foreach ($vdisk_nodes as $vdisk_info) {
      $nid = $vdisk_info->nid;
      $id = $vdisk_info->title;
      mercury_management_rediscover_vm_log($uow, "deleting vdisk nid $nid (vm_nid $vm_nid) id $id\n");
      $vdisk_nids_to_delete[] = $vdisk_info->nid;
   }
   if (!empty($vdisk_nids_to_delete)) {
      mercury_management_delete_vdisks($uow['log_fh'], $vdisk_nids_to_delete);
   }

   return true;
}


/**
 * Decide whether VM discovery should be aborted due to an unknown VM hostname.
 * Having only IP addresses is OK, too.
 */
function mercury_management_abort_updatevm_due_to_unknown_vm_hostname(&$uow, $vm_info_with_key)
{
  $vm_info = $uow['vm_info'];
  $error = '';
  $max_retries = 0;
  $retry_type = '';

  // If we have neither DNS nor IP address information...
  if (
    empty($vm_info_with_key['Host name']) &&
    (empty($vm_info_with_key['IP Addresses']) || $vm_info_with_key['IP Addresses'] == 'Not Known')
  ) {
    // If the last remaining field is empty, we can't proceed.  Error out.
    if (empty($vm_info_with_key['IP Address'])) {
      $error = t('Cannot connect to VM @vm because hostname or IP address could not be determined.', array('@vm' => $vm_info->title));

    } else {
      // When some fields are blank because VMware Tools is still starting, the
      // 'IP Address' (singular) field will still be populated (presumably from
      // vCenter).  If that's the case then do a retry.  It typically recovers
      // in a few seconds.
      // But that's not an "error" yet because the single IP address we get back
      // could be valid.  The pings later will check that.
      // So retry but do not fail out.
      // @see mercury_management_discover_vm_vmware_tools_valid
      $max_retries = 3; // 15 seconds
      $retry_type = 'tools-address';
    }
  }

  // For likely transient errors, try again on failure.
  // @see mercury_management_abort_updatevm_due_to_no_vmware_tools
  if ($max_retries && mercury_management_uow_retry($uow, $retry_type, $max_retries)) {
    mercury_management_rediscover_vm_log($uow, "Retry due to missing hostname and addresses\n");
    return 'retry';
  }

  if (!empty($error)) {
    # Peg an actual discovery error
    $uow['error'] = $error;
    mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_ERROR);
    mercury_management_discover_vm_handle_phase_failure($uow, $error);
    mercury_management_rediscover_vm_log($uow, "discovery terminated: " . $uow['error'] . "\n");
    return true;
  }

  return false;
}


/**
 *  Decide whether VM discovery should be aborted due to an unresolvable VM hostname.
 *  Returns 'true' if the VM hostname is not resolvable.
 */
function mercury_management_abort_updatevm_due_to_unresolvable_vm_hostname(&$uow, $result)
{
  $vm_info = $uow['vm_info'];
  $ip = "0.0.0.0";

  # get the known IP addresses
  $vm_ip_addresses = array();
  $vm_info_with_key = $uow['vm_info_with_key'];
  if (!empty($vm_info_with_key['IP Addresses']) && $vm_info_with_key['IP Addresses'] != 'Not Known') {
    $vm_addresses = $vm_info_with_key['IP Addresses'];
    $vm_ip_addresses = explode(',', $vm_addresses);
  } else {
    if (!empty($vm_info_with_key['IP Address'])) {
      $vm_ip_addresses[] = $vm_info_with_key['IP Address'];
    }
  }

  $valid_fqdn = mercury_management_parse_wcmd_validate_fqdn($result, $ip);
  $vm_hostname = mercury_management_get_vm_hostname($vm_info);  
  if ($valid_fqdn == true) {
    # We resolved the VM hostname successfully. Ensure that the IP we 
    # resolved it to is one of the valid addresses. It is possible DNS
    # might point to us to a host in a different domain. If it does
    # none of the IP addresses will match.
    foreach ($vm_ip_addresses as $ip_address) {
      mercury_management_rediscover_vm_log($uow, "match IP address " . $ip_address . " with " . $ip . "\n");
      if (trim($ip) == trim($ip_address)) {
	# found match
	$vm_address = $vm_hostname;
	# override the IP we got from vCenter with the IP we got from name resolution.
	# This is so that the VM node page displays IP address information for the interface
	# FAMC is using.
	$vm_info->field_vm_ip_address['und'][0]['value'] = trim($ip);
	goto save_address;
      }
    }
  }

  mercury_management_rediscover_vm_log($uow, "unable to correctly resolve VM hostname " . $vm_hostname . "\n");

  # Check to see if we have refreshed this VM before and have a valid IP address in the vm_address field
  if (!empty($vm_info->field_vm_address['und'][0]['value'])) {
    $vm_address = $vm_info->field_vm_address['und'][0]['value'];
    foreach ($vm_ip_addresses as $ip_address) {
      if ($vm_address == trim($ip_address)) {
	$result = exec("/bin/ping -c 1 -W 3 $vm_address", $output, $status);
	if ($status == 0) {
	  goto save_address;
	}
	break;
      }
    }
  }

  # FQDN error - Either we could not resolve or resolved IP did not match one of the IP addresses
  # Do a ping test to see which of the IP addresses we have works and save it
  foreach ($vm_ip_addresses as $ip_address) {
    $result = exec("/bin/ping -c 1 -W 3 $ip_address", $output, $status);
    if ($status == 0) {
      # found a working address
      $vm_address = trim($ip_address);
      goto save_address;
    }
  }

  # No IP addresses are responding and DNS does not resolve the hostname
  # We have no good way to verify if network is configured properly
  # Warn the user to fix the DNS and specify or a manual IP address
  $error = t('Unable to connect to VM @vm hostname "@vm_hostname" from FAMC. Please try again after ensuring either the VM hostname is resolvable using DNS or the VM IP address can be pinged from FAMC.',  array('@vm' => $vm_info->title, '@vm_hostname' => $vm_hostname));
  mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_WARNING);
  mercury_management_discover_vm_handle_phase_failure($uow, $error);

  # Log additional information.  This is too long to fit into the max
  # discovery error string length.
  $error2 = t('VM @vm may need a firewall reconfiguration to allow ping requests if not configured for DNS. Alternatively, override the VM IP address using the wrench menu Edit option.',  array('@vm' => $vm_info->title));
  mercury_utilities_fmc_log_message('VM Refresh', $error2, WATCHDOG_WARNING);

# Peg an actual discovery error
  $uow['error'] = "couldn't resolve vm hostname or validate its IP address!";
  mercury_management_rediscover_vm_log($uow, "discovery terminated: " . $uow['error'] . "\n");
  return true;

save_address:
  mercury_management_rediscover_vm_log($uow, "using address " . $vm_address . " to communicate with VM\n");
  $vm_info->field_vm_address['und'][0]['value'] = $vm_address;
  node_save($vm_info);
  return false;
}


/**
 *  Decide whether VM discovery should be aborted because the VM is powered off.
 */
function mercury_management_abort_updatevm_due_to_vm_powered_off(&$uow, $vm_info_with_key)
{
  $vm_info = $uow['vm_info'];
  $powerstate = $vm_info_with_key['Power State'];
  if ($powerstate != "poweredOn") {
    $error = t('Cannot connect to VM @vm. VM is powered off.', array('@vm' => $vm_info->title));
    mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_NOTICE);
    mercury_management_discover_vm_handle_phase_failure($uow, $error);
    # Do not peg a discovery error
    mercury_management_rediscover_vm_log($uow, "discovery terminated: vm is powered off\n");
    return true;
  }

  return false;
}


/**
 *  Decide whether a given string corresponds to a supported guest OS.
 */
function mercury_management_guest_os_is_supported(stdClass $node)
{
  $guest_id = '';
  if (!empty($node->field_guest_id['und'][0]['value'])) {
    $guest_id = $node->field_guest_id['und'][0]['value'];
  }

  // Do case-insensitve comparisons. Note that this applies more to the full
  // OS name field than the ID, but for safety lowercase anyway.
  // @see BURT 756869
  $guest_id = strtolower($guest_id);

  // To make the system more forgiving of potential VMWare Tools bugs
  // and bad customer environments, first consider any valid Guest ID
  // to be a supported OS.
  $supported_ids = array(
    'windows7server64guest' => true, // 2008 R2 (non-R2 is something else)
    'windows8server64guest' => true, // 2012 and 2012 R2
  );
  if (isset($supported_ids[$guest_id])) {
    return true;
  }

  return false;
}


/**
 *  Decide whether VM discovery should be aborted due to an unsupported guest OS.
 */
function mercury_management_abort_updatevm_due_to_unsupported_guest_os(&$uow)
{
  // Do not take this shortcut if VMware Tools is not reporting.
  // Later discovery will try to figure out what's wrong.
  // @see mercury_management_abort_updatevm_due_to_no_vmware_tools
  if (!mercury_management_discover_vm_guest_os_valid($uow['vm_info_with_key'])) {
    return false;
  }

  // VM is not compatible but is definitively known.
  // This is not an error, just a shortcut.
  $vm_info = $uow['vm_info'];
  if (true !== mercury_management_guest_os_is_supported($vm_info)) {
    $guest_os = $vm_info->field_guest_os['und'][0]['value'];
    $guest_os_id = $vm_info->field_guest_id['und'][0]['value'];
    mercury_utilities_fmc_log_message('VM Refresh', t('VM @vm is running an unsupported guest OS "@guest"', array('@vm' => $vm_info->title, '@guest' => $guest_os)), WATCHDOG_NOTICE);
    mercury_management_rediscover_vm_log($uow, "discovery terminated: unsupported guest os $guest_os ($guest_os_id)\n");

    // The driver state is typically "Not Known" but that's not accurate.
    // We know that it's not installed because it's not supported.
    // Change the driver status to avoid confusing error messages elsewhere.
    // During refresh this may already be correct, avoid a duplicate save.
    if ($vm_info->field_driver_status['und'][0]['value'] != FAMC_VM_DRIVER_STATUS_NOT_COMPATIBLE) {
      $vm_info->field_driver_status['und'][0]['value'] = FAMC_VM_DRIVER_STATUS_NOT_COMPATIBLE;
      node_save($vm_info);
    }

    return true; 
  }

  // Guest OS is known and supported.
  return false;
}


/**
 *  Decide whether VM discovery should be aborted due to login failure.
 */
function mercury_management_abort_updatevm_due_to_bad_credentials(&$uow, $validate_username_password)
{
  if ($validate_username_password != "Success") {
    $vm_info = $uow['vm_info'];
    $error = '';
    if ($validate_username_password == "Error") {
      $error = t('Username/Password incorrect for VM @vm. Please set the correct username/password and try again.', array('@vm' => $vm_info->title));
    } else if ($validate_username_password == "CannotConnect") {
      $error = t('Cannot connect to VM @vm. Please check if it is powered on and try again.', array('@vm' => $vm_info->title));
    }
    mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_ERROR);

    # Peg an actual discovery error
    $uow['error'] = "invalid vm credentials!";
    mercury_management_rediscover_vm_log($uow, "discovery terminated: " . $uow['error'] . "\n");
    mercury_management_discover_vm_handle_phase_failure($uow, $error);
    return true;
  }
  
  return false;
}


/**
 *  Decide whether VM discovery should be aborted due to missing VMware Tools.
 */
function mercury_management_abort_updatevm_due_to_no_vmware_tools(&$uow, $vm_info_with_key) {
  $error = '';
  $max_retries = 0;
  $retry_type = '';

  // VMware Tools is required to enter the next discovery phase.
  if (!mercury_management_discover_vm_vmware_tools_running($vm_info_with_key)) {
    // Not running could either be not installed or just not started.
    if ($vm_info_with_key['VMware Tools'] == 'VMware Tools is not running.') {
      // This normally means we're in the middle of a boot, but it's also
      // possible that the user stopped the tools or they crashed.
      // The stopped/crashed cases should be very rare.
      // In either case, vCenter thinks VMware Tools is at least installed.
      // Power state failures happen before this, so power is on.
      // OK to retry but it's important to keep it bounded.
      // This window during boot is typically about 60 seconds and
      // then it moves on to the running but returns nothing case below.
      $max_retries = 18; // 90 seconds
      $retry_type = 'tools-not-running';
      $error = t('VMware Tools is not running on @vm. The VM may be restarting.', array('@vm' => $uow['vm_info']->title));
    } else {
      $error = t('VMware Tools is not running on @vm. Please ensure that it is installed, up to date, and running.', array('@vm' => $uow['vm_info']->title));
    }
  } elseif (!mercury_management_discover_vm_vmware_tools_valid($vm_info_with_key)) {
    // Sometimes VMware Tools is "running" but it hasn't fully started and thus
    // doesn't return anything useful.
    // Generally the solution is to refresh again in a moment.
    // This situation is almost always transient, typically recovers within 15s.
    $max_retries = 6; // 30 seconds to be a little more forgiving.
    $retry_type = 'tools-result-invalid';
    $error = t('VMware Tools is running but is not reporting a Guest OS on @vm. The VM may be restarting.', array('@vm' => $uow['vm_info']->title));
  }

  if (!empty($error)) {
    // For likely transient errors, try again on failure.
    // Note that this can't retry for long because that will potentially make
    // a refresh or discovery very slow.
    // Operations that will need long retries such as VM agent install need to
    // use mercury_utilities_wait_for_valid_vminfo() directly or indirectly.
    // This retry is meant to catch transient problems not related to a major
    // FAMC action, such as a manual VM reboot.
    // @see BURT 783278, sometimes allocations can also trigger this.
    if ($max_retries && mercury_management_uow_retry($uow, $retry_type, $max_retries)) {
      mercury_management_rediscover_vm_log($uow, "Retry due to VMware Tools state: '$error'\n");
      return 'retry';
    }

    $uow['error'] = $error;
    mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_ERROR);
    mercury_management_discover_vm_handle_phase_failure($uow, $error);
    mercury_management_rediscover_vm_log($uow, "discovery terminated: no vmware tools\n");
    return true;
  }

  return false;
}


/**
 *  Set driver status to Agent Installed
 */
function mercury_management_updatevm_set_driver_status_installed(&$uow)
{
  $vm_info = $uow['vm_info'];
  $vm_info->field_driver_status['und'][0]['value'] = FAMC_VM_DRIVER_STATUS_INSTALLED;
}


/**
 *  Decide whether VM discovery should be aborted due to bad driver status
 */
function mercury_management_abort_updatevm_due_to_driver_status(&$uow, $driver_status)
{
  $vm_info = $uow['vm_info'];

  if (
    ($driver_status != FAMC_VM_DRIVER_STATUS_INSTALLED) &&
    ($driver_status != FAMC_VM_DRIVER_STATUS_NOT_INSTALLED)
  ) {
    $error = t('Bad driver status on VM @vm: @status', array('@vm' => $vm_info->title, '@status' => $driver_status));
    mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_WARNING);
    mercury_management_discover_vm_handle_phase_failure($uow, $error);
    # Peg a discovery error
    $uow['error'] = "bad driver status!";
    mercury_management_rediscover_vm_log($uow, "discovery terminated: " . $uow['error'] . "\n");
    return true;
  }
  
  return false;
}


/**
 *  Log a string to the log file associated with this UoW, tagged with VM name.
 */
function mercury_management_rediscover_vm_log(&$uow, $str)
{
  mercury_management_rediscover_log($uow['log_fh'], "vm " . $uow['vm'] . ": " . $str);
}


/**
 *  Callback in the event of a select timeout while processing a VM UoW.
 */
function mercury_management_rediscover_vm_timeout_cb(&$uow, $idx)
{
  $uow['error'] = 'command timeout!';
  mercury_management_rediscover_vm_log($uow, "select timeout declared at idx $idx! command follows:\n");
  mercury_management_rediscover_vm_log($uow, mercury_management_sanitize_command_for_log($uow['cmds'][$idx]) . "\n");

  $vm = $uow['vm'];
  $error = t('Timed out accessing VM @vm. Please resolve VM and/or infrastructure issue and try again.', array('@vm' => $vm));
  mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_ERROR);

  // Some command failed, so roll back and save the last good state.
  mercury_management_discover_vm_handle_phase_failure($uow, $error);
  
  # Return true to allow the rest of the discovery to complete.  This UoW has 
  # failed.
  return true;
}


/**
 * Discovery+Refresh (VM) Process
 * 
 * Discovery of an individual VM is functionally equivalent to performing a 
 * refresh operation on that VM. The process proceeds in multiple phases, as
 * described here:
 * https://wikid.netapp.com/w/Mercury/Mercury_Development/FAMC_multiphase_discovery
 */


/**
 *  Assemble the discovery UoW for the given VM.
 */
function mercury_management_assemble_vm_uow($log_fh, $vc_hostname, $vc_username, $vc_password, $vm, $is_annotated, $host_nid) 
{
  $uow['type'] = 'disc_vm';

  $uow['cmds'] = array();
  $uow['cbks'] = array();

  $uow['cmds'][] = mercury_management_wcmd_vminfo($vm);
  $uow['cbks'][] = 'mercury_management_wa_cb_disc_vm_vminfo';

  $uow['vc_hostname'] = $vc_hostname;
  $uow['vc_username'] = $vc_username;
  $uow['vc_password'] = $vc_password;
  $uow['vm']          = $vm;
  $uow['host_nid']    = $host_nid;
  $uow['log_fh']      = $log_fh;
  $uow['is_annotated']= $is_annotated;
  $uow['timeout_cbk'] = 'mercury_management_rediscover_vm_timeout_cb';
  
  mercury_management_rediscover_vm_log($uow, "obtaining vminfo\n");

  return $uow;
}


/**
 *  Discovery succeeded; save and do some cleanup work.
 */
function mercury_management_vm_discovery_successful(&$uow, $idx, $data_level)
{
   mercury_management_discover_vm_db_commit($uow, $data_level);
   mercury_management_rediscover_vm_log($uow, "discovery succeeded; vm updated in database\n");

  /**
   * Do not do this check during discovery. A full GC will happen at the end.
   */
  if (empty($uow['discovery'])) {

    $orig_cachespace = (!empty($uow['vm_info_orig']->field_cachespace['und'][0]['value']))
      ? $uow['vm_info_orig']->field_cachespace['und'][0]['value'] : '';
    $new_cachespace = (!empty($uow['vm_info']->field_cachespace['und'][0]['value']))
      ? $uow['vm_info']->field_cachespace['und'][0]['value'] : '';
    $orig_host = (!empty($uow['vm_info_orig']->field_host['und'][0]['target_id']))
      ? $uow['vm_info_orig']->field_host['und'][0]['target_id'] : '';
    $new_host = (!empty($uow['vm_info']->field_host['und'][0]['target_id']))
      ? $uow['vm_info']->field_host['und'][0]['target_id'] : '';
    mercury_management_rediscover_vm_log($uow, "Snap restore detection: Cachespace was '$orig_cachespace' and is now '$new_cachespace'\n");
    mercury_management_rediscover_vm_log($uow, "Snap restore detection: Host was '$orig_host' and is now '$new_host'\n");

    /**
     * If FMC previously knew of a cache space but it's no longer valid,
     * the environment changed beyond FMC's control and the most common cause
     * is restoring a snapshot from before install or allocate.
     * Trigger a garbage collection so the user will actually be able to allocate.
     * Also don't continue if the host changed. That's just a normal migration.
     */
    if (!empty($orig_cachespace) && $orig_cachespace != $new_cachespace && $orig_host == $new_host) {
      mercury_management_rediscover_vm_log($uow, "Snap restore detection: Cache space changed! Need to garbage collect.\n");

      // Gather the migration Host targets for this VM and run GC on them.
      $migration_hosts = mercury_management_vmotion_vm_migration_hosts($uow['vm_info'], $include_current = true);
      mercury_management_reclaim_cachespaces($migration_hosts);
      mercury_utilities_fmc_log_message('VM Refresh', t(
        'Cache space change detected in @vm. Cache space cleanup process completed.',
        array('@vm' => $uow['vm_info']->title)
      ));
    }
  }

  // If we know that the driver is not installed but cache is allocated,
  // we also need to remove the cache space.
  // This code is never reached if the VM is shut down, etc.
  // Note that this is not the same as orphan removal, which is a host-level
  // action and works on FA disks that are not mapped to a VM.
  // This happens during synthetic upgrade but we're about to reinstall = skip.
  if (
    empty($_REQUEST['synthetic-upgrade']) &&
    $uow['vm_info']->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_INSTALLED &&
    !empty($uow['vm_info']->field_cachespace['und'][0]['value'])
  ) {
    mercury_utilities_fmc_log_message('VM Refresh', t(
      'Unused cache allocation detected for @vm. Removing cache allocation.',
      array('@vm' => $uow['vm_info']->title)
    ));

    // This function name is misleading; it is only for unallocate.
    // Calls mercury_management_unallocateCacheFromVM then cleans up the
    // cache spaces on other hosts and refreshes the nodes.
    // Must be called after the node is saved!
    // Prevent possible infinite refresh loop.
    mercury_utilities_vm_settings_cachespace('uninstall-unallocate', $uow['vm_info']->nid, $refresh = false);
  }

   $uow['cmds'][$idx + 1] = "all_done";
}


/**
 *  Vminfo now available.
 */
function mercury_management_wa_cb_disc_vm_vminfo(&$uow, $idx)
{
  $result = $uow['results'][$idx];

  $vm_info_with_key = mercury_management_parse_vminfo($result);
  if (empty($vm_info_with_key)) {
    $uow['error'] = "couldn't parse vminfo!";
    mercury_management_rediscover_vm_log($uow, $uow['error'] . "\n");
    return false;    
  }
  $uow['vm_info_with_key'] = $vm_info_with_key;
  
  mercury_management_rediscover_vm_log($uow, "creating/updating vm node\n");
  
  // We know the current host node. Query vcenter to see if VM is still running on the same host.
  $prev_host_nid = $uow['host_nid'];

  // Query vcenter for VM to Host mapping
  $vcInfo = mercury_management_vcenter_details();
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  $currently_running_host = $vm_info_with_key['Current Host'];
  if (empty($currently_running_host) || $currently_running_host == 'Not Known') {
    $uow['error'] = "Could not determine current Host!";
    mercury_management_rediscover_vm_log($uow, $uow['error']);
    return false;
  }

  $host_nid = db_query("SELECT nid FROM {node} WHERE type = :tp AND title = :ttl", array(':tp' => 'host', ':ttl' => trim($currently_running_host)))->fetchField();

  // if a host nid is found build and and save VM otherwise do not build the VM
  if ($host_nid) {

    if ($host_nid != $prev_host_nid) {
      $host = node_load($host_nid);
      mercury_utilities_fmc_log_message('VM Refresh', t('VM @vm is running on host @host', array('@vm' => $vm_info_with_key['Name'], '@host' => $host->title)));
    }
    
    $vm_info = mercury_management_create_or_update_vm_node($uow, $vm_info_with_key, $host_nid);
    $uow['vm_info'] = $vm_info;

    // If we definitively know the guest OS at this point and it's not supported
    // then there's no point in continuing.
    // But if the OS cannot be determined then continue on to try and figure
    // out why we were not able to pull that information in (VMware Tools).
    // The OS is usually cached by vCenter when the VM is powered down,
    // and when powered down and not supported it's better to show unsupported.
    if (mercury_management_abort_updatevm_due_to_unsupported_guest_os($uow)) {
      return false;
    }

    // Now fire off any other shortcuts that we already have the information
    // to evaluate: power state, VMware Tools state, etc.
    if (mercury_management_abort_updatevm_due_to_vm_powered_off($uow, $vm_info_with_key)) {
      return false;
    }

    // The VMware Tools check may result in a retry.
    // If retrying then we need to return true so the UOW continues,
    // but need to skip the hostname check below because it's dependent on
    // the VMware Tools result.
    $tools_result = mercury_management_abort_updatevm_due_to_no_vmware_tools($uow, $vm_info_with_key);
    if ($tools_result === true) {
      return false;
    } elseif ($tools_result === 'retry') {
      return true;
    }

    // The hostname check could also result in a retry.
    $hostname_result = mercury_management_abort_updatevm_due_to_unknown_vm_hostname($uow, $vm_info_with_key);
    if ($hostname_result === true) {
      return false;
    } elseif ($hostname_result === 'retry') {
      return true;
    }

    mercury_management_rediscover_vm_log($uow, "obtaining vdiskinfo\n");
    $uow['cmds'][$idx + 1] = mercury_management_wcmd_vdiskinfo($uow['vm']);
    $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_vdiskinfo";
  }
  else {
    mercury_management_rediscover_vm_log($uow,  "Error:  VM doesn't have a valid host_nid...skipping node creation\n");
    return false;
  }

  return true;
}


/**
 *  Next discovery step: vdiskinfo now available.
 */
function mercury_management_wa_cb_disc_vm_vdiskinfo(&$uow, $idx)
{
  $result = $uow['results'][$idx];
  $vm_info = $uow['vm_info'];
  $vDiskArr = mercury_management_parse_vdiskinfo($vm_info->title, $result);
 
  $uow['vDiskArr'] = $vDiskArr;

  $cacheDevice = mercury_management_add_cache_disk_to_vm_node($vm_info, $vDiskArr);
  if (!empty($cacheDevice)) {
    mercury_management_rediscover_vm_log($uow, "updating vm node with cache device $cacheDevice\n");
  }
  $uow['cacheDevice'] = $cacheDevice;

  # Check if the user has specified a custom VM IP address
  if (empty($vm_info->field_vm_address_custom['und'][0]['value'])) {

    $vm_hostname = mercury_management_get_vm_hostname($vm_info);
    mercury_management_rediscover_vm_log($uow, "checking DNS for vm hostname $vm_hostname\n");
    $uow['cmds'][$idx + 1] = mercury_management_wcmd_validate_fqdn($vm_hostname);
    $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_validate_fqdn";

  } else {
    // Skip the DNS check for VM hostname
    // Speculatively invoke the web service.  If it succeeds, FA is installed,
    // and we can skip the vmruns that check for FA installed.
    mercury_management_rediscover_vm_log($uow, "attempting to obtain cache info using user specified address\n");
    $hostname = mercury_management_get_vm_address($vm_info);
    $username = mercury_management_get_vm_username($vm_info);
    $password = mercury_management_get_vm_password($vm_info);
    $method   = 'HgGetCacheOptions';
    $port     = mercury_management_get_vm_port($vm_info);
    $uow['cmds'][$idx + 1] = mercury_management_wcmd_mm_wsdl($hostname, $username, $password, $method, null, $port);
    $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_getcacheoptions_attempt";
  }
  return true;
}


/**
 *  Next discovery step: host output now available.
 */
function mercury_management_wa_cb_disc_vm_validate_fqdn(&$uow, $idx)
{
  $result = $uow['results'][$idx];
  $vm_info = $uow['vm_info'];

  if (mercury_management_abort_updatevm_due_to_unresolvable_vm_hostname($uow, $result)) {
    return false;
  }

  # Speculatively invoke the web service.  If it succeeds, FA is installed,
  # and we can skip the vmruns that check for FA installed.
  $hostname = mercury_management_get_vm_address($vm_info);
  $username = mercury_management_get_vm_username($vm_info);
  $password = mercury_management_get_vm_password($vm_info);
  $method   = 'HgGetCacheOptions';
  $port     = mercury_management_get_vm_port($vm_info);
  mercury_management_rediscover_vm_log($uow, "attempting to obtain cache info from " . $hostname . ":" . $port . "\n");
  $uow['cmds'][$idx + 1] = mercury_management_wcmd_mm_wsdl($hostname, $username, $password, $method, null, $port);
  $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_getcacheoptions_attempt";
  
  return true;
}


/**
 *  Next discovery step: results of speculative web service call now available 
 */
function mercury_management_wa_cb_disc_vm_getcacheoptions_attempt(&$uow, $idx)
{
  $vm_info = $uow['vm_info'];

  // Pull out the result.
  // This was a speculative call, so if it fails then don't log anything
  // until we are sure that FA is actually installed.  If FA is actually
  // installed, we will retry later and that will log.
  // @see mercury_management_wa_cb_disc_vm_vmrun_chk_for_fa
  $result = implode("\n", $uow['results'][$idx]);
  $attrInfo = mercury_management_parse_mm_wsdl($result, $vm_info->title, $suppress_error_responses = true);

  // Note that metrics_reset_interval is in $attrInfo but it's not actually
  // used for anything until mercury_management_wa_cb_disc_vm_getcacheoptions().

  if ($attrInfo == 'error_wsdl_call') {
    # Web service failed but we don't know why; continue with regular flow
    mercury_management_rediscover_vm_log($uow, "speculative web service call failed\n");
    mercury_management_rediscover_vm_log($uow, "checking vm credentials\n");
    $host_and_vc_of_vm_nids = mercury_management_get_host_dc_vc_of_vm($vm_info->nid);
    $vc_of_vm_node = node_load($host_and_vc_of_vm_nids['vcenter']);
    $vc_hostname = $vc_of_vm_node->field_hostname['und'][0]['value'];
    $vc_username = fmc_cipher::decrypt($vc_of_vm_node->field_username['und'][0]['value']);
    $vc_password = fmc_cipher::decrypt($vc_of_vm_node->field_password['und'][0]['value']);
    $username = mercury_management_get_vm_username($vm_info);
    $password = mercury_management_get_vm_password($vm_info);
    $path = $vm_info->field_vm_path_name['und'][0]['value'];
    $uow['cmds'][$idx + 1] = mercury_management_wcmd_vmrun_chk_cred($vc_hostname, $vc_username, $vc_password, $username, $password, $path);
    $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_vmrun_chk_cred";
    return true;
  } else {
    # Web service call succeeded; FA must be installed.  Short-circuit flow.
    mercury_management_updatevm_set_driver_status_installed($uow);    
    return mercury_management_wa_cb_disc_vm_getcacheoptions($uow, $idx);
  }
}


/**
 *  Next discovery step: vmrun output for credential check now available.
 */
function mercury_management_wa_cb_disc_vm_vmrun_chk_cred(&$uow, $idx)
{
  $result = $uow['results'][$idx];
  $vm_info = $uow['vm_info'];
  
  $chk_cred = mercury_management_parse_vmrun_chk_cred($result);
  if (mercury_management_abort_updatevm_due_to_bad_credentials($uow, $chk_cred)) {
    return false;
  }
  
  $host_and_vc_of_vm_nids = mercury_management_get_host_dc_vc_of_vm($vm_info->nid);
  $vc_of_vm_node = node_load($host_and_vc_of_vm_nids['vcenter']);
  $vc_hostname = $vc_of_vm_node->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vc_of_vm_node->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vc_of_vm_node->field_password['und'][0]['value']);
  $username = mercury_management_get_vm_username($vm_info);
  $password = mercury_management_get_vm_password($vm_info);
  $path = $vm_info->field_vm_path_name['und'][0]['value'];

  // This function is only reached if the web service speculative call did not
  // succeed, so in that case always check for the DLL in the guest because
  // we need to differentiate between service unreachable and uninstalled.
  mercury_management_rediscover_vm_log($uow, "checking for FA agent installed\n");
  $uow['cmds'][$idx + 1] = mercury_management_wcmd_vmrun_chk_for_fa($vc_hostname, $vc_username, $vc_password, $username, $password, $path);
  $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_vmrun_chk_for_fa";
  
  return true;
}


/**
 *  Next discovery step: vmrun output FA installed check now available.
 */
function mercury_management_wa_cb_disc_vm_vmrun_chk_for_fa(&$uow, $idx)
{
  $result = $uow['results'][$idx];
  $vm_info = $uow['vm_info'];
  $hostname = mercury_management_get_vm_address($vm_info);
  $username = mercury_management_get_vm_username($vm_info);
  $password = mercury_management_get_vm_password($vm_info);
  
  $driver_status = mercury_management_parse_vmrun_chk_for_fa($result);
  $vm_info->field_driver_status['und'][0]['value'] = $driver_status;

  if (mercury_management_abort_updatevm_due_to_driver_status($uow, $driver_status)) {
    return false;
  }

  // Credential check passed and FA install state is definitively known.
  $vm_info->field_vm_discovery_state['und'][0]['value'] = FAMC_DISCOVERY_STATE_REACHABLE;
  $uow['vm_info_consistent'] = clone($vm_info);

  if ($driver_status != FAMC_VM_DRIVER_STATUS_INSTALLED) {
    // Not installed, but everything we need for now has been discovered.
    // Clean up any data that was previously discovered via the FA API,
    // as it is no longer relevant/accurate.
    // field_cachespace, field_fa_cache_size, field_fa_sector_size are excluded
    // because they are determined from vdiskinfo not from the API,
    // and could potentially exist even after uninstall.
    $vm_info->field_fa_build_version['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_cache_mode['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_administration_state['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_operational_state['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_operational_message['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_performance_state['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_usable_blocks['und'][0]['value'] = '';
    mercury_management_access_annotations('vm', $vm_info->title, 'clear');

    // And save the VM.
    mercury_management_vm_discovery_successful($uow, $idx, 'VD');
    return true;
  }

  // Update annotations to reflect state: FA installed but web service failed.
  // TODO: re-enable this after mercury_management_parse_vmrun_for_fa_version
  // becomes available again.
  // @see mercury_management_wcmd_vmrun_chk_for_fa
  /*if ($uow['is_annotated'] == false) {
    $driver_version = mercury_management_parse_vmrun_for_fa_version($result);
    mercury_management_access_annotations('vm', $vm_info->title, 'set', $driver_version);
  }*/

  // FA is installed, but if we reached this then the speculative web service
  // call failed.  The following code will try again, which may or may not
  // be productive.
  mercury_management_rediscover_vm_log($uow, "obtaining cache options\n");
  $method = 'HgGetCacheOptions';
  $port   = mercury_management_get_vm_port($vm_info);
  $uow['cmds'][$idx + 1] = mercury_management_wcmd_mm_wsdl($hostname, $username, $password, $method, null, $port);
  $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_getcacheoptions";

  return true;
}


/**
 *  Next discovery step: HgGetCacheOptions output now available.
 */
function mercury_management_wa_cb_disc_vm_getcacheoptions(&$uow, $idx)
{
  $vm_info = $uow['vm_info'];

  $result = implode("\n", $uow['results'][$idx]);
  $attrInfo = mercury_management_parse_mm_wsdl($result, $vm_info->title);

  if ($attrInfo != 'error_wsdl_call') {
    $vm_info->field_fa_metrics_reset_interval['und'][0]['value'] = $attrInfo['metrics_reset_interval'];
    $uow['vm_info'] = $vm_info;
  }

  $hostname = mercury_management_get_vm_address($vm_info);
  $username = mercury_management_get_vm_username($vm_info);
  $password = mercury_management_get_vm_password($vm_info);

  mercury_management_rediscover_vm_log($uow, "obtaining cache info\n");

  $method = 'HgGetCacheInfo';
  $port   = mercury_management_get_vm_port($vm_info);
  $uow['cmds'][$idx + 1] = mercury_management_wcmd_mm_wsdl($hostname, $username, $password, $method, null, $port);
  $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_getcacheinfo";

  return true;
}


/**
 *  Next discovery step: HgGetCacheInfo output now available.
 */
function mercury_management_wa_cb_disc_vm_getcacheinfo(&$uow, $idx)
{
  $vm_info = $uow['vm_info'];

  // Evaluate result, preventing errors from empty $result.
  $result = implode("\n", $uow['results'][$idx]);
  $attrInfo = mercury_management_parse_mm_wsdl($result, $vm_info->title);

  if ($attrInfo == 'error_wsdl_call') {
    if ($uow['is_annotated'] == true) {
      # Corner case where the annotation may be falsely present
      mercury_management_rediscover_vm_log($uow, "Annotation check: Annotated but web service not available.\n");
      $uow['is_annotated'] = false; // TODO: May need to differentiate between uninstalled and web service down here.  Installed but down = leave annotation.
      $host_and_vc_of_vm_nids =  mercury_management_get_host_dc_vc_of_vm($vm_info->nid);
      $vc_of_vm_node = node_load($host_and_vc_of_vm_nids['vcenter']);
      $vc_hostname = $vc_of_vm_node->field_hostname['und'][0]['value'];
      $vc_username = fmc_cipher::decrypt($vc_of_vm_node->field_username['und'][0]['value']);
      $vc_password = fmc_cipher::decrypt($vc_of_vm_node->field_password['und'][0]['value']);
      $username = mercury_management_get_vm_username($vm_info);
      $password = mercury_management_get_vm_password($vm_info);
      $path = $vm_info->field_vm_path_name['und'][0]['value'];

      mercury_management_rediscover_vm_log($uow, "checking for FA agent installed\n");

      $uow['cmds'][$idx + 1] = mercury_management_wcmd_vmrun_chk_for_fa($vc_hostname, $vc_username, $vc_password, $username, $password, $path);
      $uow['cbks'][$idx + 1] = "mercury_management_wa_cb_disc_vm_vmrun_chk_for_fa";
      return true;
    } else {
      $uow['error'] = "vm webservice call failed!";
      mercury_management_rediscover_vm_log($uow, $uow['error'] . "\n");
      $error = t('Error obtaining cache information from VM @vm.', array('@vm' => $vm_info->title));
      mercury_utilities_fmc_log_message('VM Refresh', $error, WATCHDOG_ERROR);
      mercury_management_discover_vm_handle_phase_failure($uow, $error);
      return false;
    }
  } 

  $uow['attrInfo'] = $attrInfo;
  mercury_utilities_fmc_log_message('VM Refresh', t('Successfully obtained cache information from VM @vm.', array('@vm' => $vm_info->title)));
  $vm_info->field_fa_build_version['und'][0]['value'] = $attrInfo['build_version'];
  $vm_info->field_fa_cache_mode['und'][0]['value'] = $attrInfo['cache_mode'];
  $vm_info->field_fa_administration_state['und'][0]['value'] = $attrInfo['admin_state'];
  $vm_info->field_fa_operational_state['und'][0]['value'] = $attrInfo['op_state'];
  $vm_info->field_fa_operational_message['und'][0]['value'] = $attrInfo['op_message'];
  $vm_info->field_fa_performance_state['und'][0]['value'] = $attrInfo['perf_state'];
  $vm_info->field_fa_usable_blocks['und'][0]['value'] = $attrInfo['usable_blocks'];
  // @see mercury_management_update_vm_vdisk_db_commit() for non-cache disks.
  // @see mercury_management_add_cache_disk_to_vm_node() for cache disk info.

  // if VM is not annotated - go ahead and store the build version
  if ($uow['is_annotated'] == false) {
    $driver_version = $attrInfo['build_version'];
    $result = mercury_management_access_annotations('vm', $vm_info->title, 'set', $driver_version);
  }

  // Give a warning if the VM has unsupported software.
  mercury_management_vm_agent_compatibility_warning($vm_info);

  /*
   * Validate that the identifier the service thinks is the cache is the one added
   * to the VM as the cache device .. if not we need to notify the VM of the correct
   * cache space ID
   */
  $vm_cachespace_id = '';
  if (!empty($uow['cacheDevice'])) {
    $parts = explode('.', $uow['cacheDevice']);
    $vm_cachespace_id = $parts[1];
  }

  // Compare and make sure VM thinks the same
  $i1 = (!empty($attrInfo['cachespace_id'])) ? trim($attrInfo['cachespace_id']) : '';
  $i2 = trim($vm_cachespace_id);
  if (!empty($i1) && !empty($i2) && ($i1 != $i2)) {
    mercury_management_rediscover_vm_log($uow, "synchronizing cache space ids $i1 vs $i2!\n");
    
    // If necessary, Disable cache before updating the ID
    if ($attrInfo['admin_state'] == "ADMIN_ENABLED")  {
      $param = array('request' => array('CacheState' => 'ADMIN_DISABLED'));
      $result = mercury_management_cache_operation('nid', $vm_info->nid, 'HgSetCacheState', $param);
    }
    $setcache_parameter = array('request' => array('CacheIdentifier' => $vm_cachespace_id));
    mercury_management_cache_operation('nid', $vm_info->nid, 'HgSetCacheIdentifier', $setcache_parameter);
    if ($attrInfo['admin_state'] == "ADMIN_ENABLED")  {
      $param = array('request' => array('CacheState' => 'ADMIN_ENABLED'));
      $result = mercury_management_cache_operation('nid', $vm_info->nid, 'HgSetCacheState', $param);
    }
    mercury_utilities_fmc_log_message('VM Refresh', t('Updated VM @vm cachespace ID to "@id"', array('@vm' => $vm_info->title, '@id' => $vm_cachespace_id)));
  }

  if (!empty($vm_cachespace_id)) {

    /**
     * Set the vmotion settings for the VM.
     * If cache is allocated to the VM, synchronize the host list.
     * We may need to fill in the policy or checkbox state if lost.
     * If no allocation then the host list is largely irrelevant.
     * @see mercury_management_vmotion_vm_migration_hosts
     */
    $hosts = mercury_management_findCacheHosts($vm_info->nid, trim($vm_cachespace_id));
    mercury_management_rediscover_vm_log($uow, "Has cache allocated on " . count($hosts) . " host(s).\n");
    if (count($hosts) > 0) {
      $vm_info->field_vm_vmotion_hostlist['und'][0]['value'] = serialize($hosts);

      /**
       * We need to do some things to recover from lost data.  For example,
       * if the user deletes the FAMC VM and reinstalls, we have to figure out
       * the VM migration settings based on the environment.
       * But if the policy is already set, no data was lost, so do nothing.
       */
      if (empty($vm_info->field_vm_vmotion_policy['und'][0]['value'])) {
        mercury_management_rediscover_vm_log($uow, "Resetting migration policy based on discovered allocations.\n");

        /**
         * If there are multiple hosts with allocations right now, that means
         * that means that the checkbox must have been enabled.
         * Otherwise there is exactly one target and the checkbox doesn't really
         * matter very much, so just set it to the global default.
         * Using the global default means fewer VMs to fix later, on average.
         */
        if (count($hosts) > 1) {
          $vm_info->field_vm_vmotionable['und'][0]['value'] = 1;
        } else {
          $vm_info->field_vm_vmotionable['und'][0]['value'] = variable_get('enable_migration', 1);
        }

        /**
         * Now policy.  In some cases it might be possible to determine whether
         * the policy maps to cluster or datacenter, but for now it makes no
         * difference because we never use that again after first allocation
         * (in Barium and Copper all reconfigurations require manually editing
         * the VM migration target list).
         * So for now any "recovered" data = custom policy.
         * TODO: This may need refinement for US1347.
         */
        $vm_info->field_vm_vmotion_policy['und'][0]['value'] = 'custom';
      }
    }
  }

  // Final phase complete.
  $vm_info->field_vm_discovery_state['und'][0]['value'] = FAMC_DISCOVERY_STATE_API_READY;

  # Per the spec, we skip the vdisk info unless the VM has cache space 
  # allocated.  For now, since we have the vdisk info anyway, go ahead
  # and create the nodes.
  # if ($attrInfo['cache_size']) {
    mercury_management_vm_discovery_successful($uow, $idx, 'VD+CD+DD');
  # } else {
  #   mercury_management_vm_discovery_successful($uow, $idx, 'VD+CD');
  # }
  
  return true;
}


/*
 * Get the list of hosts containing the specified cachespace
 */
function mercury_management_findCacheHosts($vm_nid, $cachespace_id) {

  $hostnid_list = array();

  $host_and_vc_of_vm_nids =  mercury_management_get_host_dc_vc_of_vm($vm_nid);
  $vc_of_vm_node = node_load($host_and_vc_of_vm_nids['vcenter']);
  $vc_hostname = $vc_of_vm_node->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vc_of_vm_node->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vc_of_vm_node->field_password['und'][0]['value']);

  // Check all hosts in this datacenter for the cachespace
  $hostlist = mercury_management_allocationHostList($vm_nid, 'datacenter', $vc_hostname, $vc_username, $vc_password);
  foreach ($hostlist as $hostname) {
    $host_nid = db_query("SELECT nid FROM {node} WHERE type = :tp AND title = :ttl", array(':tp' => 'host', ':ttl' => trim($hostname)))->fetchField();
    if (!empty($host_nid)) {
      $host_info = node_load($host_nid);
      // Make sure the flash device is assigned and initialized before
      // listing cache spaces.
      if (
        !empty($host_info->field_cache_state['und'][0]['value']) &&
        $host_info->field_cache_state['und'][0]['value'] == 3
      ) {
        $cachespaces = mercury_management_listcachespaces($host_nid);
        foreach($cachespaces as $cache) {
          if ($cache['CacheSpaceID'] == "naa." . $cachespace_id) {
            $hostnid_list[] = $host_nid;
	    break;
          }
        }
      }
    }
  }

  return $hostnid_list;
}



/*
 * Get the list of hosts on which cache should be allocated
 * List is filtered by VM's allocation policy. Additional checking is
 * necessary to verify if the hosts on this list have the agent installed and
 * flash device assigned and initalized.
 */
function mercury_management_allocationHostList($vm_nid, $policy, $vc_hostname, $vc_username, $vc_password) {
  $vm = node_load($vm_nid);
  $vmname = $vm->title;
  $hostlist = array();

  if ($policy == "cluster" || $policy == "datacenter") {
    $dataArr = array();
    $vcenterInfo = exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService --username '" . $vc_username . "' --password '" . $vc_password . "' --vmname '" . $vmname . "' --operation vmhostlist --filter '" . $policy . "'", $dataArr);
    if (!empty($dataArr)) {
      $count=0;
      foreach ($dataArr as $item) {
	$host_split = explode('Host :', $item);
	if (count($host_split) > 1) {
	  $hostlist[] = trim($host_split[1]);
	}
      }
    }
  }
  return $hostlist;
}


/**
 * Save the VM to the database and update vDisk nodes as needed.
 */
function mercury_management_discover_vm_db_commit(&$uow, $data_level)
{
   /*
    * Differences between VD and VD+CD are dealt with by 
    * mercury_management_add_host_control_1_data_to_host_node; all we have 
    * to do is create the VM node, and create/update or delete the vdisk
    * nodes.
    */

   mercury_management_rediscover_vm_log($uow, "committing vm to database ($data_level)\n");
   $vm_info = $uow['vm_info'];
   node_save($vm_info);
   if ($data_level == 'VD+CD+DD') {
      mercury_management_rediscover_vm_log($uow, "committing vdisk nodes to database\n");
      $attrInfo = $uow['attrInfo'];
      mercury_management_update_vm_vdisk_db_commit($uow, $attrInfo, $vm_info->nid);
   } else {
      if (false == $uow['vm_info_is_new']) {
         mercury_management_rediscover_vm_log($uow, "removing vdisk nodes from database\n");
         mercury_management_update_vm_vdisk_db_commit($uow, array(), $vm_info->nid);
      }
   }
}


/**
 * Fill in non-empty default values for new nodes as needed.
 */
function mercury_management_initialize_vm_node(&$vm_info) {
    $vm_info->field_guest_os['und'][0]['value'] = 'Not Known';
    $vm_info->field_guest_id['und'][0]['value'] = 'Not Known';
    $vm_info->field_vmware_tools['und'][0]['value'] = 'VMware Tools has never been installed or has not run in the virtual machine.';
    $vm_info->field_driver_status['und'][0]['value'] = FAMC_VM_DRIVER_STATUS_NOT_KNOWN;
    $vm_info->field_fa_build_version['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_cache_mode['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_administration_state['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_operational_state['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_operational_message['und'][0]['value'] = 'Not Known';
    $vm_info->field_fa_performance_state['und'][0]['value'] = 'Not Known';
}
