<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that render or process forms.
 */

function mercury_management_edit_entity($type, $nid) {
  $output = '';
  $output .= drupal_render(drupal_get_form('mercury_management_entity_edit_form', $type, $nid));
  return $output;
}

/**
 *  Function for edit the VM and Host node to set the username and password
 */
function mercury_management_entity_edit_form($form, &$form_state) {
  //$form = array();
  $username = '';
  $password = '';
  $ipaddr = '';
  $vmotion = 0;
  $vm_reboot = 0;

  $type = $form_state['type'];
  $nid = $form_state['nid'];
  if ($type != '' && $nid != '') {
    switch ($type) {
      case 'vm':
        $vm_info = node_load($nid);
        if (!empty($vm_info->field_vm_username)) {
          $username = fmc_cipher::decrypt($vm_info->field_vm_username['und'][0]['value']);
        }
        if (!empty($vm_info->field_vm_password)) {
          $password = fmc_cipher::decrypt($vm_info->field_vm_password['und'][0]['value']);
        }
        if (!empty($vm_info->field_vm_address_custom)) {
          $ipaddr = $vm_info->field_vm_address_custom['und'][0]['value'];
        }
        if (!empty($vm_info->field_install_port)) {
          $port = $vm_info->field_install_port['und'][0]['value'];
        } else {
          $port = '7010';
        }
        if (!empty($vm_info->field_vm_vmotionable)) {
          $vm_reboot = $vm_info->field_vm_vmotionable['und'][0]['value'];
        }
        if (!empty($vm_info->field_vm_reboot)) {
          $vm_reboot = $vm_info->field_vm_reboot['und'][0]['value'];
        }
        break;
      case 'host':
        $host_info = node_load($nid);
        if (!empty($host_info->field_host_username)) {
          $username = fmc_cipher::decrypt($host_info->field_host_username['und'][0]['value']);
        }
        if (!empty($host_info->field_host_password)) {
          $password = fmc_cipher::decrypt($host_info->field_host_password['und'][0]['value']);
        }
        break;
    }
  }
  $form['username'] = array(
    '#type' => 'textfield',
    '#title' => t('Username'),
    '#default_value' => $username,
    '#size' => 25,
    //'#required' => true
  );
  $form['password'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#default_value' => $password,
    '#size' => 25,
    //'#required' => true,
    '#attributes' => array('value' => $password)
  );
  if ($type == 'vm') {
    $form['ipaddr'] = array(
      '#type' => 'textfield',
      '#title' => t('IP Address'),
      '#default_value' => $ipaddr,
      '#size' => 15,
      '#required' => false,
      '#description' => 'Optional. If empty, FAMC will automatically determine the VM hostname or IP address to communicate with the VM.'
    );

    $form['port'] = array(
      '#type' => 'textfield',
      '#title' => t('Install Port'),
      '#default_value' => $port,
      '#size' => 6,
      '#required' => false
    );

    $form['vmotion'] = array(
      '#type' => 'hidden',
      '#value' => 0
    );
  }
  $form['nid'] = array(
    '#type' => 'hidden',
    '#value' => $nid
  );
  $form['type'] = array(
    '#type' => 'hidden',
    '#value' => $type
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );
  return $form;
}

function mercury_management_entity_edit_form_trim(&$values) {
  if (!empty($values['username'])) {
    $values['username'] = trim($values['username']);
  }

  // DO NOT trim the password; leading and trailing whitespace is legal

  if (!empty($values['port'])) {
    $values['port'] = trim($values['port']);
  }

  if (!empty($values['ipaddr'])) {
    $values['ipaddr'] = trim($values['ipaddr']);
  }
}

function mercury_management_entity_edit_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  mercury_management_entity_edit_form_trim($values);
  if (!empty($values['username']) && !empty($values['password'])) {
    $node = node_load($values['nid']);
    if (empty($values['port'])) {
      $values['port'] = ''; // Prevent NOTICE.
    }
    // It's confusing to say the parent is invalid when you are entering
    // the child's credentials, so don't recursively validate here.
    $edit_form_validate_output = mercury_management_validate_credentials($node, $validate_ancestors = false, $node->title, $values['username'], $values['password'], $values['port']);
    if ($edit_form_validate_output !== true) {
      mercury_utilities_fmc_log_message('System Settings', t('Failed to save credential settings.'), WATCHDOG_ERROR);
      form_set_error('password', 'Username or Password may be incorrect');
    }
  } elseif(!empty($values['username']) || !empty($values['password'])) {
    // Did not provide both, but did provide one.  They have to come as a pair.
    mercury_utilities_fmc_log_message('System Settings', t('Failed to save credential settings.'), WATCHDOG_ERROR);
    form_set_error('password', 'Both Username and Password must be provided or empty.');
  }
  if (!empty($values['ipaddr']) && !filter_var($values['ipaddr'], FILTER_VALIDATE_IP)) {
    mercury_utilities_fmc_log_message('System Settings', t('Failed to save IP address settings.'), WATCHDOG_ERROR);
    form_set_error('ipaddr', 'VM IP address format is invalid.');
  }
  if (!empty($values['port']) && !is_numeric($values['port'])) {
    mercury_utilities_fmc_log_message('System Settings', t('Failed to save port settings.'), WATCHDOG_ERROR);
    form_set_error('port', 'Install Port value must be numeric.');
  }
}

function mercury_management_entity_edit_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  mercury_management_entity_edit_form_trim($values);
  if ($values['type'] != '' && $values['nid'] != '') {
    switch ($values['type']) {
      case 'vm':

        $vm_info = node_load($values['nid']);

        if (!empty($vm_info->field_install_port)) {
          $port = $vm_info->field_install_port['und'][0]['value'];
        } else {
          $port = '7010';
        }

        $vm_info->field_vm_username['und'][0]['value'] = fmc_cipher::encrypt($values['username']);
        $vm_info->field_vm_password['und'][0]['value'] = fmc_cipher::encrypt($values['password']);
	$vm_info->field_vm_address_custom['und'][0]['value'] = $values['ipaddr'];
        if ($values['port'] && is_numeric($values['port'])) {
          $vm_info->field_install_port['und'][0]['value'] = $values['port'];
        } else {
          $vm_info->field_install_port['und'][0]['value'] = 7010;
        }

        // XXX: These fields aren't exposed in the interface, so it's
        // a bit of a hack.  Try to come up with something better.
        $vm_info->field_vm_vmotionable['und'][0]['value'] = $values['vmotion'];
        // XXX: This triggers a NOTICE.
        $vm_info->field_vm_reboot['und'][0]['value'] = $values['vm_reboot'];
        node_save($vm_info);
        mercury_utilities_fmc_log_message('System Settings', t('Settings saved successfully for the VM @vm_name.', array('@vm_name' => $vm_info->title)), WATCHDOG_NOTICE, 'show');
        break;
      case 'host':

        $host_info = node_load($values['nid']);
        $host_info->field_host_username['und'][0]['value'] = fmc_cipher::encrypt($values['username']);
        $host_info->field_host_password['und'][0]['value'] = fmc_cipher::encrypt($values['password']);
        node_save($host_info);
        mercury_utilities_fmc_log_message('System Settings', t('Settings saved successfully for host @host_name', array('@host_name' => $host_info->title)), WATCHDOG_NOTICE, 'show');
        break;
    }
  } else {
    mercury_utilities_fmc_log_message('System Settings', t('Failed to save settings.'), WATCHDOG_ERROR);
  }
  $form_state['message'] = theme('status_messages');
}

function mercury_management_form_validate_credentials($form, &$form_state) {
  $values = $form_state['values'];

  $form_id = $form_state['input']['form_id'];

  switch ($form_id) {
    case 'virtual_machine_node_form':
      $vm_node_data = node_load($values['nid']);

      $field_vm_password = fmc_cipher::decrypt($values['field_vm_password']['und'][0]['value']);
      $field_vm_username = fmc_cipher::decrypt($values['field_vm_username']['und'][0]['value']);

      if ($field_vm_password != '' && $field_vm_username != '') {
        $vm_data = mercury_management_get_host_dc_vc_of_vm($values['nid']);
        $vcInfo = node_load($vm_data['vcenter']);
        $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
        $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
        $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

        $get_vm_info = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $field_vm_username . "' -gp '" . $field_vm_password . "' fileExistsInGuest '" . $vm_node_data->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\Windows\explorer.exe"');

        if (stristr($get_vm_info, 'Error')) {
          form_set_error('field_vm_password', $get_vm_info);
        }
      }
      break;
    case 'host_node_form':
      $host_data = $form_state['values'];
      $host_username = fmc_cipher::decrypt($host_data['field_host_username']['und'][0]['value']);
      $host_password = fmc_cipher::decrypt($host_data['field_host_password']['und'][0]['value']);
      if ($host_username != '' && $host_password != '') {
        $host_status = exec("esxcli -s '" . trim($host_data['title']) . "' --username '" . $host_username . "' --password '" . $host_password . "' system hostname get");
        if (stristr($host_status, 'failed')) {
          form_set_error('field_host_password', $host_status);
          mercury_utilities_fmc_log_message('System Settings', 'Failed to change the host username/password.', WATCHDOG_WARNING, 'show');
        }
      }
      break;

    // This is never called.  @see mercury_utilities_vcenter_settings_form_validate
    case 'vcenter_node_form':
      $vCenter_data = $form_state['values'];
      $vCenter_hostname = $vCenter_data['field_hostname']['und'][0]['value'];
      $vCenter_username = fmc_cipher::decrypt($vCenter_data['field_username']['und'][0]['value']);
      $vCenter_password = fmc_cipher::decrypt($vCenter_data['field_password']['und'][0]['value']);

      if ($vCenter_username != '' && $vCenter_username != '') {
        $valid = famc_external_api_manager_validate_credentials(NULL, $vCenter_hostname, $vCenter_username, $vCenter_password);
        if ($valid !== true) {
          form_set_error('vcenter_username', $valid); // Will contain details.
        }
      }
      break;

    //case 'user_pass':
    //  unset($form['links']);
    //  break;
  }
}

/**
 * Implements hook_form_alter()
 * XXX: All to most of this can be handled in better ways.
 */
function mercury_management_form_alter(&$form, &$form_state, $form_id) {

  $get_password = '';

  switch ($form_id) {
    case 'virtual_machine_node_form':

      if (!empty($form['#node']->field_vm_password['und'][0]['value'])) {
        $get_password = fmc_cipher::decrypt($form['#node']->field_vm_password['und'][0]['value']);
      }
      $form['field_vm_password']['#title'] = 'Password';
      //$form['field_vm_password_new']['#type'] = 'password';
      $form['field_vm_password']['#description'] = 'Please provide the Password';
      $form['field_vm_password']['#attributes'] = array('value' => $get_password);
      $form['field_vm_password']['#suffix'] = '<script type="text/javascript">
        jQuery("#edit-field-vm-password-und-0-value").get(0).type = "password";
      </script>';

      $form['#validate'][] = 'mercury_management_form_validate_credentials';

      break;
    case 'host_node_form':
      if (!empty($form['#node']->field_host_password['und'][0]['value'])) {
        $get_password = fmc_cipher::decrypt($form['#node']->field_host_password['und'][0]['value']);
      }
      $form['field_host_password']['#title'] = 'Password';
      $form['field_host_password']['#description'] = 'Please provide the Password';
      $form['field_host_password']['#attributes'] = array('value' => $get_password);
      $form['field_host_password']['#suffix'] = '<script type="text/javascript">
        jQuery("#edit-field-host-password-und-0-value").get(0).type = "password";
      </script>';

      $form['#validate'][] = 'mercury_management_form_validate_credentials';

      break;

    case 'vcenter_node_form':
      if (!empty($form['#node']->field_password['und'][0]['value'])) {
        $get_password = fmc_cipher::decrypt($form['#node']->field_password['und'][0]['value']);
      }
      $form['field_password']['#title'] = 'Password';
      $form['field_password']['#description'] = 'Please provide the Password';
      $form['field_password']['#attributes'] = array('value' => $get_password);
      $form['field_password']['#suffix'] = '<script type="text/javascript">
        jQuery("#edit-field-password-und-0-value").get(0).type = "password";
      </script>';

      $form['#validate'][] = 'mercury_management_form_validate_credentials';

      break;
  }
}

