<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for garbage collection.
 */

function mercury_management_gc_get_tick_count() {
  /*
   * Account for garbage collection.
   * GC time is loosely dependent on the number of host and vm nodes total.
   * However, there are a lot of variables and GC usually doesn't take a long
   * time, so for now just do two ticks (one after vms and one after hosts).
   */
  return 2;
}

/*
 * Function to list unassigned cache spaces on a host
 */
function mercury_management_get_orphan_cachespaces($host_nid) {

  // For bulk refresh, there is some small chance that a host could be processed
  // multiple times if GC is needed on several VMs in a cluster.  Prevent that.
  static $processed_hosts = array();
  if (isset($processed_hosts[$host_nid])) {
    return;
  }
  $processed_hosts[$host_nid] = true;

  $host = node_load($host_nid);
  $orphanedCacheSpaces = array();

  $vcInfo = mercury_management_vcenter_details();
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  // Get all orphaned cachespaces on the host
  exec("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . $host->title . "'" . ' --operation getOrphanedCacheSpaces', $result);
  if (empty($result)) {
    return;
  }
  foreach ($result as $output_line) {
    if (!empty($output_line) && stristr($output_line, "Error")) {
      return;
    }
    if (!empty($output_line) && stristr($output_line, "NAAID:")) {
      $orphanedCacheSpaceIDArray = explode(':', $output_line);
      if (count($orphanedCacheSpaceIDArray) > 1) {
        array_push($orphanedCacheSpaces, trim($orphanedCacheSpaceIDArray[1]));
      }
    }
  }
  return $orphanedCacheSpaces;
}

/**
 * Function to garbage collect the unassigned cache space on each host
 */
function mercury_management_reclaim_cachespaces($hosts = NULL) {

  // Find all the VM cache spaces. Need to do this for safety regardless
  // of $hosts because the node to host map might be stale.
  // XXX: We only use field_cachespace so why not get just VMs with cache
  // spaces or even just get only the field data?
  $vms = mercury_utilities_node_load_all('virtual_machine');
  if (!$vms || empty($vms)) {
    // This probably shouldn't return early, but is unlikely to be hit.
    // Still need to register all ticks.
    mercury_utilities_discovery_progress_register_ticks(2);
    return;
  }
  $cachespaces = array();
  foreach ($vms as $vm) {
    if (!empty($vm->field_cachespace['und'][0]['value'])) {
      $cachespaces[$vm->field_cachespace['und'][0]['value']] = $vm->nid;
    }
  }

  // VMs are done, register a tick.
  mercury_utilities_discovery_progress_register_ticks(1);

  // Continue even if no cache spaces were found in VMs; there still may be
  // orphans in the hosts.

  // If no hosts provided, load all of them.  Mostly for discovery.
  if (empty($hosts)) {
    $hosts = mercury_utilities_node_load_all('host');
  }

  if (empty($hosts)) {
    // Still need to register remaining ticks.
    mercury_utilities_discovery_progress_register_ticks(1);
    return;
  }

  /**
   * TODO: US1391 Make GC faster by making it use the worker threads
   * fyi the easiest and quickest way to do that with a worker process is to
   * use the VCD.pm "exec" command to run rdmDisk.pl in a worker process, as
   * opposed to directly from PHP.  It would be more efficient to incorporate the
   * rdmDisk.pl functions into VCD.pm and skip the extra exec, but not trivial.
   * @see mercury_management_wcmd_get_orphans() in mm_cli.php
   */
  foreach ($hosts as $host) {
    $host_nid = $host->nid;
    $dc_nid = $host->field_datacenter['und'][0]['target_id'];
    $host_reclaimed = false;

    // Get the caches on the host
    $hostcaches = mercury_management_get_orphan_cachespaces($host_nid);
    if (empty($hostcaches) || count($hostcaches) == 0) {
      continue;
    }

    // Loop over all caches on the hosts and reclaim / free the ones that are not
    // being used by any VM. $hostcaches should already be limited to actual
    // orphan cache spaces, but this is a good secondary check.
    foreach ($hostcaches as $cachespace_id) {
      $freecache = false;
      if (empty($cachespaces[$cachespace_id])) {
        $freecache = true;
      } else {
        $vm_nid = $cachespaces[$cachespace_id];
        $vm = node_load($vm_nid);
        $vm_host = node_load($vm->field_host['und'][0]['target_id']);
        $vm_dc_nid = $vm_host->field_datacenter['und'][0]['target_id'];
        if ($vm_dc_nid != $dc_nid) {
          $freecache = true;
        }
      }

      // Free cache space if there is no owner
      if ($freecache == true) {
        $host_reclaimed = true;
        mercury_management_freeCache($host_nid, $cachespace_id);
      }
    }

    // Refresh the host if cache was reclaimed on the host
    if ($host_reclaimed == true) {
      $vcInfo = mercury_management_vcenter_details();
      $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
      $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
      $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
      mercury_management_updatehostConfig($host_nid, $vc_hostname, $vc_username, $vc_password);
    }
  }

  // Hosts are done, register a tick.
  mercury_utilities_discovery_progress_register_ticks(1);
}
