<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that operate on host nodes.
 */

function mercury_management_installhypervisorbundleonhost($nid) {
  $req_type = '';
  $username = '';
  $password = '';
  $host_info = node_load($nid);

  if (!empty($host_info)) {
    //get host username password else load the default username/password
    $username = mercury_management_get_host_username($host_info);
    $password = mercury_management_get_host_password($host_info);

    if (isset($_REQUEST['step'])) {
      $req_type = $_REQUEST['step'];
    }

    $validation_result = famc_external_api_host_preinstall_validation($host_info);
    if ($validation_result !== true) {
      print 'acceptance--' . trim($host_info->title);
      exit;
    }

    $host_data = mercury_management_get_host_data($nid);
    $dcInfo = node_load($host_info->field_datacenter['und'][0]['target_id']);
    $vcInfo = node_load($host_data['vcenter']);
    $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
    $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
    $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

    // Do external requests only once, on "start"
    // This isn't the greatest place to put this, but doing it in a more
    // sensible way requires massively rewriting this function.
    if ($req_type == 'start') {
      mercury_management_refresh_host($host_info);
      $driver_status = $host_info->field_bundle_status['und'][0]['value'];
      $module_status = mercury_management_checkforhvmodule($vc_hostname, $vc_username, $vc_password, $nid);
      if ($module_status == 'Module Loaded' && $driver_status == FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED) {
        mercury_utilities_fmc_log_message('Host Agent', t('Host agent module is currently loaded in memory on "@host". A reboot is required prior to an install.', array('@host' => $host_info->title)), WATCHDOG_ERROR);
        print 'error--' . trim($host_info->title);
        exit;
      }
    }

    $api_error = mercury_management_has_vsphere_api_error($host_info);
    if ($api_error) {
      // Already logged.
      print 'error--' . trim($host_info->title);
      exit;
    }

    // If VIB has been removed and if the plugin is still loaded, fail and notify reboot is required.
    $driver_status = $host_info->field_bundle_status['und'][0]['value'];

    if ($driver_status == FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED) {
      if ($username != '' && $password != '') {
        switch ($req_type) {
          case 'start':
            print '<div class="res-message">Initializing Installation Process on the host "' . $host_info->title . '".</div>';
            exit;
          case 'maintain':
            print '<div class="res-message">Checking host for maintenance mode.</div>';
            exit;
          case 'maintanace':
            if ( ! famc_external_api_host_in_maintenance_mode($host_info) ) {
              print 'Exit--' . trim($host_info->title);
            } else {
              print '<div class="res-message">The host is in maintenance mode. Starting installation process.</div>';
            }
            exit;
          case 'copyZip':
            $copyZip = exec("vifs --server '" . trim($host_info->title) . "' --username '" . $username . "' --password '" . $password . "'" . ' -p "' . FLASH_AGENT_VIB_PATH . variable_get('host_upgraded_zip') . '" "/tmp/' . variable_get('host_upgraded_zip') . '"', $copyZipStatus);
            if (strpos($copyZip, 'successfully')) {
              // mercury_utilities_fmc_log_message('Host Agent', 'Copied Installation ZIP file to the Host.');
              print '<div class="res-message">Copied Installation ZIP file to the Host.</div>';
            } else {
              print '<div class="res-message">Installation failed. Unable to copy ZIP file to the Host.</div>';
              mercury_utilities_fmc_log_message('Host Agent', 'Installation failed. Unable to copy ZIP file to the Host.' . implode('<br>', $copyZipStatus), WATCHDOG_ERROR);
            }
            exit;
          case 'install':
            $install_status = exec("esxcli -s " . trim($host_info->title) . " --username '" . $username . "' --password '" . $password . "' software vib" . ' install -d "/tmp/' . variable_get('host_upgraded_zip') . '"', $vib_install_status_log);
            mercury_utilities_fmc_log_message('Host Agent', implode('<br>', $vib_install_status_log), WATCHDOG_NOTICE, NULL, NULL, 'no_footer');
            if (stristr($vib_install_status_log[1], 'success')) {
              mercury_utilities_fmc_log_message('Host Agent', t('Successfully installed host agent on @host', array('@host' => $host_info->title)));
	      mercury_management_access_annotations('host', trim($host_info->title), 'set', variable_get('host_package_agent_version',''));
              print '<div class="res-message">Host agent has been installed successfully on the host "' . $host_info->title . '".<br /></div>';
            } else {
              mercury_utilities_fmc_log_message('Host Agent', t('Unsuccessful installation of host agent on @host', array('@host' => $host_info->title)), WATCHDOG_ERROR);
              print '<div class="res-message">Flash Accel host agent was not installed on the host "' . $host_info->title . '".<br /></div>';
            }

            // Occasionally the update fails when we do it right away.  For
            // now, blind 30 second delay to work around that.
            sleep(30);

            // Let's also do up to 10 retries; burt 721683, testing on
            // mercuryesx1.nane.netapp.com fails without this.
            $refresh_successful = 0;
            for ($retries = 0; $retries < 10; $retries++) {
              // Update the Host (including obtaining flash devices)
              $result = mercury_management_updatehost($vc_hostname, $vc_username, $vc_password, trim($host_info->title), $dcInfo->nid, $is_installing = true);
              $error_check = explode('--', $result);
              if (count($error_check) == 1) {
                mercury_utilities_fmc_log_message('Host Agent', t('Host @host_name refreshed after install.', array('@host_name' => trim($host_info->title))));
                $refresh_successful = 1;
                break;
              }
              sleep(5);
            }
            if (!$refresh_successful) {
              mercury_utilities_fmc_log_message('Host Agent', t('Host @host_name refresh unsuccessful after install.', array('@host_name' => trim($host_info->title))), WATCHDOG_ERROR);
            }
            exit;
        }
      }
    } elseif ($driver_status == FAMC_HOST_BUNDLE_STATUS_INSTALLED) {
      print 'install--' . trim($host_info->title);
      mercury_utilities_fmc_log_message('Host Agent', t('Installation failed because the host agent is already installed in the Host "@host"', array('@host' => $host_info->title)), WATCHDOG_ERROR);
      exit;
    } elseif ($driver_status == FAMC_HOST_BUNDLE_STATUS_PARTIAL_INSTALL) {
      print 'install--' . trim($host_info->title); // same is fine
      mercury_utilities_fmc_log_message('Host Agent', t('Installation failed because the host agent is partially installed in the Host @host. Please complete the uninstall to continue.', array('@host' => $host_info->title)), WATCHDOG_ERROR);
      exit;
    } else { // Any unknown or unhandled state is also a failure.
      print 'error--' . trim($host_info->title);
      mercury_utilities_fmc_log_message('Host Agent', t('Installation failed because of a communication error with Host "@host"', array('@host' => $host_info->title)), WATCHDOG_ERROR);
      exit;
    }
  }
}

/**
 *  Function to install the Mercury Hypervisor Bundle on EXSI Host (Flash Accel)
 */
function mercury_management_uninstallhypervisorbundlefromhost($nid) {
  $req_type = '';
  $username = '';
  $password = '';
  $host_info = node_load($nid);
  $brand = strtolower(mercury_utilities_get_fmc_brand_string());

  if (!empty($host_info)) {
    //get host username password else load the default username/password
    $username = mercury_management_get_host_username($host_info);
    $password = mercury_management_get_host_password($host_info);

    if (isset($_REQUEST['step'])) {
      $req_type = $_REQUEST['step'];
      // Do external requests only once, on "start"
      // This isn't the greatest place to put this, but doing it in a more
      // sensible way requires massively rewriting this function.
      if ($req_type == 'start') {
        mercury_management_refresh_host($host_info);
      }
    }

    $host_data = mercury_management_get_host_data($nid);
    $dcInfo = node_load($host_info->field_datacenter['und'][0]['target_id']);
    $vcInfo = node_load($host_data['vcenter']);
    $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
    $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
    $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

    $api_error = mercury_management_has_vsphere_api_error($host_info);
    if ($api_error) {
      // Already logged.
      print 'error--' . trim($host_info->title);
      exit;
    }

    if ($host_info->field_bundle_status['und'][0]['value'] == FAMC_HOST_BUNDLE_STATUS_INSTALLED || $host_info->field_bundle_status['und'][0]['value'] == FAMC_HOST_BUNDLE_STATUS_PARTIAL_INSTALL) {

      if (!empty($host_info->field_naa_identifier['und'][0]['value']) && !empty($host_info->field_hbtl['und'][0]['value'])) {
        mercury_utilities_fmc_log_message('Host Agent', t("Uninstall can't continue as a flash device is assigned to host @host. Please unassign the flash device and retry.", array('@host' => $host_info->title)), WATCHDOG_ERROR);
        print 'unassign--' . trim($host_info->title);
        exit;
      }

      switch ($req_type) {
        case 'start':
          print '<div class="res-message">Uninstallation process started on the host "' . $host_info->title . '".</div>';
          exit;
        case 'maintain':
          print '<div class="res-message">Checking host for maintenance mode.</div>';
          exit;
        case 'maintanace':
          if ( ! famc_external_api_host_in_maintenance_mode($host_info) ) {
            print 'Exit--' . trim($host_info->title);
          } else {
            print '<div class="res-message">The host is in maintenance mode. Starting uninstallation process.</div>';
          }
          exit;
        case 'uninstall':
          mercury_utilities_fmc_log_message(
            'Host Agent',
            t('Beginning host agent uninstall on @host. Current agent status: @status',
              array('@host' => $host_info->title, '@status' => $host_info->field_bundle_status['und'][0]['value'])),
            WATCHDOG_NOTICE
          );

          /* Restarting the sfcbd before uninstall just to make sure it succeeds */
          $sfcbdRestartOutput = famc_external_api_host_watchdog_restart($host_info);
          if ($sfcbdRestartOutput !== true) {
            mercury_utilities_fmc_log_message('Host Agent', t($sfcbdRestartOutput . ' Cannot uninstall the host agent on @host. Please try again later.', array('@host' => $host_info->title)), WATCHDOG_WARNING);
            print 'busy--' . trim($host_info->title);
            exit;
          }

          // We blindly uninstall both VIBs before checking for errors to give
          // the user a way to recover from a partial install scenario, where
          // one is installed and the other isn't.
          // Perform all steps regardless of failure, but failure determines
          // what end result is displayed to the user.
          $has_errors = false;

          // CIM module unistall
          $vib_result = famc_external_api_host_uninstall_vib($host_info, "$brand-flashaccel-cim");
          if ($vib_result !== true) {
            // Already logged.
            $has_errors = true;
          }

          // HV Plugin Uninstall
          $vib_result = famc_external_api_host_uninstall_vib($host_info, "$brand-flashaccel");
          if ($vib_result !== true) {
            // Already logged.
            $has_errors = true;
          }

          //Update the Host
          $result = mercury_management_updatehost($vc_hostname, $vc_username, $vc_password, trim($host_info->title), $dcInfo->nid, FALSE);
          $error_check = explode('--', $result);
          if (count($error_check) == 1) {
            mercury_utilities_fmc_log_message('Host Agent', t('Host @host_name refreshed successfully following uninstall.', array('@host_name' => $host_info->title)));
          } else {
            mercury_utilities_fmc_log_message('Host Agent', t('Host @host_name refresh unsuccessful following uninstall.', array('@host_name' => $host_info->title)), WATCHDOG_ERROR);
            $has_errors = true;
          }

          //mercury_management_update_single_host($ip, $vc_username, $vc_password, $nid);
          mercury_utilities_fmc_log_message('Host Agent', t('Uninstallation process completed on the host @host_name.', array('@host_name' => $host_info->title)));
          mercury_management_access_annotations('host', trim($host_info->title), 'clear');

          if ($has_errors) {
            print '<div class="res-message">Uninstallation process completed with errors on host "' . $host_info->title . '". Please check the log for details.<br /></div>';
          } else {
            print '<div class="res-message">Uninstallation process completed successfully on the host "' . $host_info->title . '".<br /></div>';
          }
          exit;
      }
    } elseif ($host_info->field_bundle_status['und'][0]['value'] == FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED) {
      print 'uninstall--' . trim($host_info->title);
      mercury_utilities_fmc_log_message('Host Agent', t('Uninstallation failed because host agent is not installed on @host', array('@host' => $host_info->title)), WATCHDOG_ERROR);
      exit;
    } else { // Any unknown or unhandled state is also a failure.
      print 'error--' . trim($host_info->title);
      mercury_utilities_fmc_log_message('Host Agent', t('Uninstallation failed because of a communication error with host @host', array('@host' => $host_info->title)), WATCHDOG_ERROR);
      exit;
    }
  }
}

function mercury_management_upgradehypervisorbundlefromhost($nid) {
  $req_type = '';
  $username = '';
  $password = '';
  $host_info = node_load($nid);

  //get host username password else load the default username/password
  $username = mercury_management_get_host_username($host_info);
  $password = mercury_management_get_host_password($host_info);

  if (isset($_REQUEST['step'])) {
    $req_type = $_REQUEST['step'];
  }

  if (!empty($host_info)) {

    // Do external requests only once, on "start"
    // This isn't the greatest place to put this, but doing it in a more
    // sensible way requires massively rewriting this function.
    if ($req_type == 'start') {
      mercury_management_refresh_host($host_info);
    }

    $api_error = mercury_management_has_vsphere_api_error($host_info);
    if ($api_error) {
      // Already logged.
      print 'error--' . trim($host_info->title);
      exit;
    }

    if ($host_info->field_bundle_status['und'][0]['value'] == FAMC_HOST_BUNDLE_STATUS_INSTALLED) {
      switch ($req_type) {
        case 'start':
          print '<div class="res-message">Flash Accel package exists in host. Starting update process"' . $host_info->title . '".</div>';
          exit;
        case 'versionCheck':
          $versionCheck = mercury_utilities_check_upgrade_host($host_info->nid);
          if ($versionCheck == 'no-upgrade') {
            print 'version--' . trim($host_info->title);
          }
          exit;
        case 'maintain':
          print '<div class="res-message">Checking host for maintenance mode.</div>';
          exit;
        case 'maintanace':
          if ( ! famc_external_api_host_in_maintenance_mode($host_info) ) {
            print 'exit--' . trim($host_info->title);
          } else {
            print '<div class="res-message">The host is in maintenance mode.</div>';
          }
          exit;
        case 'copyIZip':
          $copyZip = exec("vifs --server '" . trim($host_info->title) . "' --username '" . $username . "' --password '" . $password . "'" . ' -p "' . FLASH_AGENT_VIB_PATH . variable_get('host_upgraded_zip') . '" "/tmp/' . variable_get('host_upgraded_zip') . '"');

          if (strpos($copyZip, 'successfully')) {
            mercury_utilities_fmc_log_message('Host Agent', 'Copied zip file bundle successfully to ESXi host');
            print '<div class="res-message">Copied installation zip file to the host.</div>';
          } else {
            mercury_utilities_fmc_log_message('Host Agent', t('Failed to copy zip file bundle to ESXi host: @result', array('@result' => $copyZip)), WATCHDOG_ERROR);
            print 'zip_copy_failure--' . trim($host_info->title);
          }

          exit;
        case 'update':
          $sfcbdRestartOutput = famc_external_api_host_watchdog_restart($host_info);
          if ($sfcbdRestartOutput !== true) {
            mercury_utilities_fmc_log_message('Host Agent', t($sfcbdRestartOutput . ' Cannot upgrade the host agent on @host. Please try again later.', array('@host' => $host_info->title)), WATCHDOG_WARNING);
            print 'busy--' . trim($host_info->title);
            exit;
          }

          $install_status = exec("esxcli -s '" . trim($host_info->title) . "' --username '" . $username . "' --password '" . $password . "' software vib" . ' update -d "/tmp/' . variable_get('host_upgraded_zip') . '"', $vib_install_status_log);

          mercury_utilities_fmc_log_message('Host Agent', implode('<br>', $vib_install_status_log), WATCHDOG_NOTICE, NULL, NULL, 'no_footer');
          foreach ($vib_install_status_log as $cmd_output) {
            if (!empty($cmd_output) && stristr($cmd_output, 'Host is not changed')) {
              //  print '<div class="res-message">Host agent upgrade failed on host "' . $host_info->title . '". Host is not changed</div>';
              mercury_utilities_fmc_log_message('Host Agent', t('Upgrade failed on host @host. Host is unchanged. Host agent may either not be installed or is already up to date.', array('@host' => $host_info->title)), WATCHDOG_WARNING);
              print 'unchanged--' . trim($host_info->title);
              exit;
            } else if (!empty($cmd_output) && stristr($cmd_output, 'successfully')) {
              print '<div class="res-message">Host agent has been updated successfully on host "' . $host_info->title . '".</div>';
              mercury_utilities_fmc_log_message('Host Agent', t('Host agent upgrade successful on host @host. A reboot of the host is required for changes to take effect.', array('@host' => $host_info->title)));
              variable_set('host_cur_zip', variable_get('host_upgraded_zip'));
              exit;
            }
          }

          /* Means we have met none of the above conditions and an unknown condition has occured */
          // print '<div class="res-message">Host agent upgrade failed on host "' . $host_info->title . '". Please verify that host is in consistent state.</div>';
          mercury_utilities_fmc_log_message('Host Agent', t('Upgrade failed on host @host. Host may be in an inconsistent state.', array('@host' => $host_info->title)), WATCHDOG_ERROR);
          print 'unknown--' . trim($host_info->title);
          exit;

        case 'postupgrade':
          print '<div class="res-message">Upgrade completed successfully on the host "' . $host_info->title . '". Please reboot the host for changes to take effect</div>';
          mercury_utilities_fmc_log_message('Host Agent', t('Upgrade completed successfully on host @host', array('@host' => $host_info->title)));
          exit;
      }
    } else { // Any unknown or unhandled state is also a failure.
      mercury_utilities_fmc_log_message('Host Agent', t('Upgrade failed because the host agent was not installed on host @host or cannot communicate with host.', array('@host' => $host_info->title)), WATCHDOG_WARNING);
      print 'uninstalled--' . trim($host_info->title);
      exit;
    }
  }
}

function mercury_management_checkforhvmodule($vc_hostname, $vc_username, $vc_password, $host_nid) {
  $host_info = node_load($host_nid);
  $dataArr = array();
  $module_status = "Module Not Loaded";

  if ($host_info->field_host_status['und'][0]['value'] == 'The entity definitely has a problem') {
    // XXX: This block is messed up and doesn't do anything.
    // Doesn't appear to be causing a bug, but does present debugging issues.
    $driver_status = FAMC_HOST_BUNDLE_STATUS_NOT_KNOWN;
  } else {
    $status = exec("esxcli -s '" . $vc_hostname . "' -u '" . $vc_username . "' -p '" . $vc_password . "' -h '" . $host_info->title . "' system module" . ' get -m=flashaccel | grep "Module: flashaccel"', $dataArr);

    if (!empty($dataArr)) {
      $module_status = "Module Loaded";
    }
  }

  return $module_status;
}

/**
 * Function that returns array of the cachespaces on a Host (CIM) Operation 2
 */
function mercury_management_listcachespaces($nid, $username = "", $password = "") {
  $host = node_load($nid);

  // If this call could not possibly succeed because the device is not ready,
  // record a non-footer log and return.
  if (empty($host->field_cache_state['und'][0]['value']) || $host->field_cache_state['und'][0]['value'] != FAMC_HOST_CACHE_STATE_INITIALIZED) {
    mercury_utilities_fmc_log_message(
      'Cache Space Mgmt',
      t('Host @host does not have a cache device assigned and initialized, skipping List Cachespaces.',
        array('@host' => $host->title)),
      WATCHDOG_DEBUG, NULL, NULL, $skip_footer_log = true
    );
    return array();
  }

  //get host username password else load the default username/password
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);

  if ($username != '' && $password != '') {
    $get_host_info2 = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $username . "' '" . $password . "' 2", $result);
    $host_info_with_key = array();
    $error_count = 0;
    if (!empty($result)) {
      foreach ($result as $value) {
        if ((stripos($value, 'ERROR CODE:') !== false) ||
          (stripos($value, 'ERROR MSG:') !== false)) {
          mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Failed retrieving cache spaces on host @host: @error', array('@host' => $host->title, '@error' => $value)), WATCHDOG_ERROR);
          $error_count++;
          continue;
        }
        $val_split = explode(':', $value);
        if (count($val_split) > 1) {
          $key = $val_split[0];
          if ($key == "FA_CacheSpace") {
            $count_key = $val_split[1];
          }
          $host_info_with_key[$count_key][$key] = trim($val_split[1]);
        }
      }
    }
    if ($error_count > 0) {
      return array();
    } else {
      return $host_info_with_key;
    }
  } else {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Could not retrieve allocated cache spaces on host @name" because configured username/password are invalid.', array('@name' => $host->title)), WATCHDOG_ERROR);
  }
}

/**
 *  Function to initialize the host ssd cache
 */
function mercury_management_initialize_host_ssd($nid) {
  $host = node_load($nid);
  $req_type = '';
  //get host username password else load the default username/password
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);

  if (isset($_REQUEST['step'])) {
    $req_type = $_REQUEST['step'];
  }

  if ($username != '' && $password != '') {
    switch ($req_type) {
      case 'start':
        print '<div class="res-message">Initialization Process Started.</div>';
        exit;
      case 'initialize':
        //Check for cache spaces allocated to the VMs for the Host
        $get_all_cache_space = mercury_management_listcachespaces($nid, $username, $password);
        if (count($get_all_cache_space) > 0) {
          mercury_utilities_fmc_log_message('Flash Device', t('Cannot initialize flash device as cache space is allocated to the VMs on host "@name"', array('@name' => $host->title)), WATCHDOG_WARNING);
          print '<div class="res-message">Flash device is in use by the VMs attached to the host "' . $host->title . '". Please unallocate the cache space(s) from the VMs before initializing flash device.</div>';
        } else {
          // Make sure the cache device has been claimed.
          $cache_state_check = famc_external_api_host_get_properties($host, $cache_state_only = true);
          if (!empty($cache_state_check) && !empty($cache_state_check[0])) {
            $cache_state_parts = explode(':', $cache_state_check[0]);
            $cache_state_value = -1;
            if (!empty($cache_state_parts[1])) {
              $cache_state_value = intval($cache_state_parts[1]);
            }
            mercury_utilities_fmc_log_message('Flash Device', t('Initializing cache on @host, current state = @state', array('@host' => $host->title, '@state' => $cache_state_value)));

            if ($cache_state_value > FAMC_HOST_CACHE_STATE_NO_DEVICE) { // if claimed or better

              // Time to actually initialize.
              $response = famc_external_api_host_initialize_device($host);
              if (famc_external_api_response_is_error($response)) {
                print '<div class="res-message">Error initializing cache on "' . $host->title . '"</div>';
                mercury_utilities_fmc_log_message(
                  'Flash Device',
                  t('Error initializing flash device on "@name". @message',
                    array('@name' => $host->title, '@message' => famc_external_api_response_get_error_message($response))
                  ),
                  WATCHDOG_ERROR
                );
              } else { // Success!

                // Log this first, so the footer is in the right order.
                mercury_utilities_fmc_log_message('Flash Device', t('Flash device initialized on "@name"', array('@name' => $host->title)));

                // Update the Host so that the yellow LED goes away.
                if (mercury_management_refresh_host($host)) {
                  mercury_utilities_fmc_log_message('Host Agent', t('Host @host refreshed successfully following initialize.', array('@host' => $host->title)));
                } else {
                  // This is unlikely to fail, so this is sufficient for now.
                  // A full solution likely requires a major progress indicator
                  // refactoring, but even a partial solution requires big
                  // changes to allow one request to return multiple messages
                  // or to split this into multiple steps.
                  mercury_utilities_fmc_log_message('Host Agent', t('Host @host refresh unsuccessful following initialize.', array('@host' => $host->title)), WATCHDOG_ERROR);
                }

                // Refresh complete too, output success.
                print '<div class="res-message">Flash device initialized on "' . $host->title . '"</div>';
              }
              exit;
            } else {
              mercury_utilities_fmc_log_message('Flash Device', t('No device assigned on "@name"', array('@name' => $host->title)), WATCHDOG_WARNING);
              print '<div class="res-message">No device assigned on "' . $host->title . '"</div>';
            }
          } else {
            mercury_utilities_fmc_log_message('Flash Device', t('Cannot obtain device assignment info on "@name"', array('@name' => $host->title)), WATCHDOG_ERROR);
            print '<div class="res-message">Cannot obtain device assignment info on "' . $host->title . '"</div>';
          }
        }
    }
  } else {
    print '<div class="res-message">Could not initialize flash device on "' . $host->title . '" because no username/password was defined</div>';
  }
}

/*
 * Function that Allocates a cachespaces on a Host (CIM) Operation 4
 * will occur when a vm is being allocated space and will return a cachespace id
 *
 */

function mercury_management_allocateCache($nid, $num_segments, $cachespace_id = "", $username = "", $password = "") {

  $host = node_load($nid);

  //get host username password else load the default username/password
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);

  if ($username != '' && $password != '') {
    $get_host_info2 = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $username . "' '" . $password . "' 4 " . $num_segments . " " . $cachespace_id, $result);
    if (!empty($result)) {
      $errorCheck = explode(':', $result[0]);
      if ($errorCheck[0] == "ERROR CODE:") {
        mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Error allocating cache on host @name with error: @msg', array('@name' => $host->title, '@msg' => $result[1])), WATCHDOG_ERROR);
        return $get_host_info2;
      } else {
        watchdog('Cache Space Mgmt', t('Cache space allocated/reserved on host @name.', array('@name' => $host->title)));
        return $get_host_info2;
      }
    }
  } else {
    // TODO: This should return something containing ERROR.
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Could not allocate cache on host "@name" because no username/password was defined', array('@name' => $host->title)), WATCHDOG_ERROR);
  }
}

/*
 * Function that Frees a cachespaces on a Host (CIM) Operation 5
 * will occur when a vm space is being unallocated
 *
 */

function mercury_management_freeCache_hostinfo($host, $cachespace, $username = "", $password = "") {

  //get host username password else load the default username/password
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);

  if ($username != '' && $password != '') {
    $get_host_info2 = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $username . "' '" . $password . "' 5 " . $cachespace, $result);
    if (!empty($result)) {
      $errorCheck = explode(':', $result[0]);
      if ($errorCheck[0] == "ERROR CODE:") {
        mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Error unallocating cache on host @name with error: @msg', array('@name' => $host->title, '@msg' => $result[1])), WATCHDOG_ERROR);
        return $get_host_info2;
      } else {
        return $get_host_info2;
      }
    }
  } else {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Cannot free cache on host @name as no username/password was defined', array('@name' => $host->title)), WATCHDOG_ERROR);
    return "ERROR: No host username/password was defined";
  }
}

/*
 * Function that Frees a cachespaces on a Host (CIM) Operation 5
 * will occur when a vm space is being unallocated
 *
 */

function mercury_management_freeCache($nid, $cachespace, $username = "", $password = "") {
  return mercury_management_freeCache_hostinfo(node_load($nid), $cachespace, $username, $password);
}

/*
 * Function that Get Largest Available Space on a Host (CIM) Operation 6
 * will occur when a Allocating VM space will return available segments
 *
 */

function mercury_management_largestAvailableCache($nid, $username = "", $password = "") {
  $host = node_load($nid);
  //get host username password else load the default username/password
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);

  if ($username != '' && $password != '') {
    $get_host_info2 = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $username . "' '" . $password . "' 7", $result);
    if (!empty($result)) {
      $errorCheck = explode(':', $result[0]);
      if ($errorCheck[0] == "ERROR CODE:") {
        mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Error retrieving largest available cache space on host @name', array('@name' => $host->title, '@msg' => $result[1])), WATCHDOG_ERROR);
        return $get_host_info2;
      } else {
        // mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Available Cache (@space) Retrieved on Host @name', array('@name' => $host->title, '@space' => $get_host_info2 )));
        return $get_host_info2;
      }
    }
  } else {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Could not retrieve largest available cache on host "@name" because no username/password was defined', array('@name' => $host->title)), WATCHDOG_ERROR);
  }
}

/**
 * Checks a Host agent for compatibility with this FAMC install.
 * @param stdClass $host The Host to check
 * @param boolean $check_package_version If true, checks the uploaded agent
 *   package as well.  This is mainly for agent upgrade and install.
 * @return string Warning text if a warning is needed, empty string otherwise
 */
function mercury_management_host_agent_compatibility_warning(stdClass $host, $check_package_version = false) {

  // Maximum supported is always the FAMC major/minor.
  // Not supporting future versions, only backward.
  $famc_version = mercury_utilities_get_fmc_version_string();

  // Check package version first if checking it at all.
  // Simplifies the logic and gives an actionable result.
  // Also show no warning if no agents have been uploaded.
  if ($check_package_version) {
    $host_package_agent_version = variable_get('host_package_agent_version', '');
    if (
      floatval($host_package_agent_version) > 0.0 &&
      !mercury_utilities_version_compare($host_package_agent_version, FLASH_AGENT_MINIMUM_SUPPORTED_VERSION, $famc_version, $major_minor = true, $allow_equal = true)
    ) {
      $warning = t(
        'Uploaded agent package version @version is not supported by FAMC version @famc.',
        array('@version' => $host_package_agent_version, '@famc' => $famc_version)
      );
      mercury_utilities_fmc_log_message('Host Agent', $warning, WATCHDOG_WARNING);
      return $warning;
    }
  }

  // Pull agent version.
  // If agent version is not known, do not show the warning.
  // Calling code must provide a proper error message.
  if (empty($host->field_vib_version['und'][0]['value']) || floatval($host->field_vib_version['und'][0]['value']) < 1.0) {
    return '';
  }

  // Check the agent against the supported range.
  if (
    !mercury_utilities_version_compare($host->field_vib_version['und'][0]['value'], FLASH_AGENT_MINIMUM_SUPPORTED_VERSION, $famc_version, $major_minor = true, $allow_equal = true)
  ) {
    $warning = t(
      'Host @host agent version @version is not supported by FAMC version @famc.',
      array('@host' => $host->title, '@version' => $host->field_vib_version['und'][0]['value'], '@famc' => $famc_version)
    );
    mercury_utilities_fmc_log_message('Host Agent', $warning, WATCHDOG_WARNING);
    return $warning;
  }

  // Success, return empty string for consistent type.
  return '';
}

/*
 * Function to check if specific host is ready for cache allocation.
 * This may fail if the host already has an allocation for the VM, so don't
 * call it in that situation! It makes assumptions that may fail if the device
 * is nearly full, max number of VMs is reached, etc.
 * @return mixed true on success, string message on failure
 */
function mercury_management_host_can_allocate($vm, $host, $vmotion_required = false) {

  // TODO: In US1347 this may need to also check for current allocations, and
  // return true.  Or otherwise do that check in the calling functions.
  // Currently the only place that is handled is in
  // mercury_utilities_vm_settings_advance_migration_form_submit
  // which removes the pre-allocated hosts before calling
  // mercury_management_addCacheHosts

  // check if agent is installed
  if ($host->field_bundle_status['und'][0]['value'] != FAMC_HOST_BUNDLE_STATUS_INSTALLED) {
    $errmsg = t('No host agent installed on host @name.', array('@name' => $host->title));
    mercury_utilities_fmc_log_message('Host Agent', $errmsg, WATCHDOG_WARNING);
    return $errmsg;
  }

  // Agent is installed.  Check to make sure we can connect to the API.
  $api_error = mercury_management_has_fa_api_error($host);
  if ($api_error) {
    // Already logged.
    return $api_error;
  }

  // TODO: May need to do a host agent compatibility check here.
  // Fail if Argon etc.

  // check if host supports vmotion
  if ($vmotion_required) {
    if ($host->field_vmotion['und'][0]['value'] != 1) {
      $errmsg = t('Host @name cannot participate in cache migration group. Please ensure vMotion support is licensed on the host and Flash Accel host agent version supports migration.', array('@name' => $host->title));
      mercury_utilities_fmc_log_message('Host Agent', $errmsg, WATCHDOG_WARNING);
      return $errmsg;
    }
  }

  // Get host username and password
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);
  if ($username == '' || $password == '') {
    $errmsg = t('Cannot connect to host @host. Please update host username and password before cache allocation', array('@name' => $host->title));
    mercury_utilities_fmc_log_message('Host Refresh', $errmsg, WATCHDOG_WARNING);
    return $errmsg;
  }

  // Fetch some details about the host.
  $maxVM = false;
  $allocVM = false;
  $cache_state = false;
  $available_segments = false;
  $errmsg = false;
  $host_status = famc_external_api_host_get_properties($host);
  if (!empty($host_status)) {
    foreach ($host_status as $value) {
      if ((stripos($value, 'ERROR CODE:') !== false) ||
        (stripos($value, 'ERROR MSG:') !== false)) {
        $errmsg = t('Failed to retrieve cache info on host @name. @details', array('@name' => $host->title, '@details' => $value));
        mercury_utilities_fmc_log_message('Host Refresh', $errmsg, WATCHDOG_WARNING);
        return $errmsg;
      }
      $val_split = explode(':', $value);
      if (count($val_split) > 1) {
        $key = $val_split[0];
        switch ($key) {
          case 'MaxVirtualMachines':
            $maxVM = $val_split[1];
            break;
          case 'AllocatedVirtualMachines':
            $allocVM = $val_split[1];
            break;
          case 'CacheState':
            $cache_state = $val_split[1];
            break;
          case 'NumMaxAllocatableSegments':
            $available_segments = $val_split[1];
            break;
        }
      }
    }
  } else {
    $errmsg = t('Failed to retrieve cache info on host @name (empty result).', array('@name' => $host->title));
    mercury_utilities_fmc_log_message('Host Refresh', $errmsg, WATCHDOG_WARNING);
    return $errmsg;
  }

  // Check if the host cache state is ready.
  if ($cache_state != 3) {
    $errmsg = t('Flash device has not been assigned/initialized on host @name.', array('@name' => $host->title));
    mercury_utilities_fmc_log_message('Flash Device', $errmsg, WATCHDOG_WARNING);
    return $errmsg;
  }

  // Check host has at least the minimum allocation size available.
  // If it's completely full then don't consider it a valid target.
  $cache_space_settings_value = variable_get('minimum_cache_space_vm', 1024);
  $segment_size_mb = $host->field_segment_size['und'][0]['value'] / (1024 * 1024);
  if (($available_segments * $segment_size_mb) - ($cache_space_settings_value * 1024) < 0) {
    $errmsg = t('There is not enough cache space available on host @name.', array('@name' => $host->title));
    mercury_utilities_fmc_log_message('Cache Space Mgmt', $errmsg, WATCHDOG_WARNING);
    return $errmsg;
  }

  // Check if max vLUNs has been allocated
  if ($allocVM >= $maxVM) {
    $errmsg = t('Host @name has reached the limit for maximum supported VMs.', array('@name' => $host->title));
    mercury_utilities_fmc_log_message('Cache Space Mgmt', $errmsg, WATCHDOG_WARNING);
    return $errmsg;
  }

  return true;
}

/**
 * Function to return all the hosts on which cache should be allocated
 */
function mercury_management_get_all_host_data($node_param, $vmotion = "") {

  $query = db_select('node', 'node');
  $query->condition('node.status', 1)
    ->condition('node.type', 'host');
  $result = $query->fields('node', array('nid', 'title'))
    ->orderBy('node.title', 'DESC')
    ->execute();

  $cache_space = array();
  $node_values = array();

  while ($host_row = $result->fetch()) {
    $node_values[] = $host_row->nid;
    $node_data = node_load($host_row->nid);

    if ($vmotion == 'vmotion') {
      $host_vmotion = $node_data->field_vmotion['und'][0]['value'];
      if ($host_vmotion == 1) {
        $cache_space[] = mercury_management_largestAvailableCache($host_row->nid);
      }
    } else {
      $cache_space[] = mercury_management_largestAvailableCache($host_row->nid);
    }
  }

  if ($node_param == 'cache_space') {
    //Sort the Cache Space value from low to high
    sort($cache_space);
    //Get the minimum Cache Space value
    $minimum_cache_space = $cache_space[0];

    return $minimum_cache_space;
  }

  if ($node_param == 'node_data') {
    return $node_values;
  }

  if ($node_param == "host_cache") {
    $cache_space = mercury_management_largestAvailableCache($vmotion);
    return $cache_space;
  }
}

/**
 * @deprecated Use mercury_management_get_credentials instead.
 */
function mercury_management_get_host_username(stdClass $node) {
  if (!empty($node->field_host_username) && $node->field_host_username['und'][0]['value'] != '') {
    $username = fmc_cipher::decrypt($node->field_host_username['und'][0]['value']);
  } else {
    $username = fmc_cipher::decrypt(variable_get('host_username'));
  }
  return $username;
}

/**
 * @deprecated Use mercury_management_get_credentials instead.
 */
function mercury_management_get_host_password(stdClass $node) {
  if (!empty($node->field_host_password) && $node->field_host_password['und'][0]['value'] != '') {
    $password = fmc_cipher::decrypt($node->field_host_password['und'][0]['value']);
  } else {
    $password = fmc_cipher::decrypt(variable_get('host_password'));
  }
  return $password;
}

/**
 * @see mercury_management_get_credentials
 */
function mercury_management_get_host_credentials(stdClass $node) {
  return array(
    'address' => trim($node->title),
    'username' => mercury_management_get_host_username($node),
    'password' => mercury_management_get_host_password($node),
  );
}

/**
 * @see mercury_management_get_ancestors
 */
function mercury_management_get_host_ancestors(stdClass $node, &$info = array()) {
  $info['host'] = $node;
  if (!empty($node->field_datacenter)) {
    mercury_management_get_datacenter_ancestors(node_load($node->field_datacenter['und'][0]['target_id']), $info);
  }
  return $info;
}

function mercury_management_get_host_status(stdClass $node) {
  $result = array();
  $cache_state = '';
  if (isset($node->field_cache_state['und'][0]['value'])) {
    $cache_state = $node->field_cache_state['und'][0]['value'];
  }
  $bundle_status = FAMC_HOST_BUNDLE_STATUS_NOT_KNOWN;
  if (isset($node->field_bundle_status['und'][0]['value'])) {
    $bundle_status = $node->field_bundle_status['und'][0]['value'];
  }

  $status_message = 'Unknown Error';
  $status_color = 'red';
  $installed = 'N/A';
  // For the LED display, discovery failures trump all others.
  // Make sure the user is aware of it!
  // In some contexts this is also the only place where the error text is shown.
  if (!empty($node->field_host_discovery_error['und'][0]['value'])) {
    $status_message = 'Communication error with Host: ' .
      $node->field_host_discovery_error['und'][0]['value'];
  } elseif ($bundle_status == FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED) {
    $status_message = 'Agent Not Installed';
    $status_color = 'black';
    $installed = 'No';
  } elseif ($bundle_status  == FAMC_HOST_BUNDLE_STATUS_INSTALLED) {
    $installed = 'Yes';
    if ($cache_state == 1 || $cache_state == 0) {
      $status_message = 'Cache Device Not Assigned';
      $status_color = 'yellow';
    } else if ($cache_state == 2){
      $status_message = 'Cache Device Not Initialized';
      $status_color = 'yellow';
    } else if ($cache_state == 3){
      $status_message = 'Device Initialized and Operational';
      $status_color = 'green';
    } else if ($cache_state == 4){
      $status_message = 'Device Failed';
    } else {
      // This should never happen, but output a unique message regardless.
      // in any error message that it might solve.
      $status_message = 'Software installed but cannot determine device state. Check for communication problems and try refreshing the Host.';
    }
  } elseif ($bundle_status  == FAMC_HOST_BUNDLE_STATUS_PARTIAL_INSTALL) {
    $status_message = 'Host agent partially installed.  Please fully uninstall the host agent software to continue.';
  } else { // FAMC_HOST_BUNDLE_STATUS_NOT_KNOWN = Unable to Communicate
    // This should no longer be reached anymore because to the discovery error
    // field, but provide a unique error message to make it easier to debug.
    $status_message = 'Cannot Communicate with Host. Check for communication problems and try refreshing the Host.';
  }

  $status_image = '<img src="' . base_path() . drupal_get_path('module', 'mercury_utilities') .
    '/images/' . $status_color . '.png" class="' . $status_color .
    '" alt="' . $status_message .'" title="' . $status_message .'"/>';
  $span_str = '<span class = "columns ' . $status_color . '" id = "column-4">' . $status_image . '</span>';

  $result['status_color'] = $status_color;
  $result['status_message'] = $status_message;
  $result['installed'] = $installed;
  $result['icon'] = $status_image;
  $result['wrapped_icon'] = $span_str;
  return $result;
}

function mercury_management_get_host_count() {
  return db_query("SELECT count(*) from {node} where `type` = 'host'")->fetchField();
}

/**
 * Return the largest possible allocation supported by the given VMFS version.
 * Note that the VMFS limits are of the form X TB - 512 bytes, but the
 * allocation granularity is 256MB so this must return a multiple of 256MB.
 * @param string $vmfs_version
 * @return int max number of megabytes allowed in a single allocation
 */
function mercury_management_host_get_largest_supported_allocation_size_mb(stdClass $host, $vmfs_version) {
  $tb_in_mb = 1048576; // 1024 * 1024 MB = 1 TB
  $segment_size_mb = $host->field_segment_size['und'][0]['value'] / (1024 * 1024);

  // VMFS3 - Raw Device Mapping size(physical compatibility) - 2TB minus 512 bytes
  // VMFS5 - Raw Device Mapping size(physical compatibility) - 64TB
  if ($vmfs_version < 5) {
    // 512 bytes is smaller than our minimum allocation segment size, so round
    // up to subtracting one full segment.
    return (2 * $tb_in_mb) - $segment_size_mb;
  }

  // The exact VMFS-5 limits are unknown, but 256MB is sufficiently accurate
  // for now, and we're nowhere close to supporting 64TB devices anyway.
  return $vmfs_cap = (64 * $tb_in_mb) - $segment_size_mb;
}
