<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that should probably
 * go somewhere in mercury_utilities instead...
 */

/**
 *  Return directory to use for temporary files (no trailing separator).
 */
function mercury_management_get_tmp_dir() {
  return '/tmp';
}

/**
 *  Return directory to use for log files (no trailing separator).
 */
function mercury_management_get_log_dir() {
  return mercury_management_get_tmp_dir();
}

/**
 * Strips identifiable passwords out of a log string.
 * Note that passwords not contained in command-line arguments will not
 * be sanitized! You must avoid putting the password in there to start with.
 * @param string $s source
 * @return string sanitized result
 */
function mercury_management_sanitize_command_for_log($s) {
  // Prevent logging of passwords by replacing the password with stars.
  // Note that this only handles a specific set of patterns, so if we
  // implement some very different worker commands this may need an update.
  $s = preg_replace('/-(password|pass|p|gp) (["\']).*?(["\'])/i', '-$1 $2******$3', $s);

  // Special case: host_control.py puts the password in arg 3.
  if (stripos($s, 'host_control.py') !== false) {
    $s = preg_replace('/(host_control\.py ["\'].*?["\'] ["\'].*?["\']) (["\']).*?(["\'])/i', '$1 $2******$3', $s);
  }

  return $s;
}

/**
 * XXX: NOT USED
 */
function mercury_management_validate_fqdn($hostname) {

  if ($hostname == '') {
    return false;
  }

  $hostparts = explode('.', $hostname);
  exec("/usr/bin/host " . $hostparts[0], $result);
  if (empty($result)) {
    return false;
  }

  foreach ($result as $output) {
    if (stristr($output, 'has address')) {
      $parts = explode(' ', $output);
      $ip = $parts[3];
      mercury_utilities_fmc_log_message('VM Refresh', t('VM @vm resolves to @ip', array('@vm' => $hostparts[0], '@ip' => $ip)));
      return true;
    } else {
      return false;
    }
  }
}

/**
 * Execute an external command.  Basically a wrapper for the proc_open way
 * instead of using the simpler (but limited) exec function.
 * @todo replace implementation with exec_with_stderr or something better
 */
function execute_command($cmd, &$outputArr = array()) {
  $descspec = array(
    0 => array("pipe", "r"), # stdin is a pipe that the child will read from
    1 => array("pipe", "w"), # stdout is a pipe that the child will write to
    2 => array("pipe", "w")  # stderr is a pipe that the child will write to
  );

  $Pipes = array();
  $process = proc_open($cmd, $descspec, $Pipes, NULL, NULL);
  $StdOut = array();
  $StdErr = array();

  while (!feof($Pipes[1])) {
    array_push($StdOut, fgets($Pipes[1]));
  }
  fclose($Pipes[1]);

  while (!feof($Pipes[2])) {
    array_push($StdErr, fgets($Pipes[2]));
  }
  fclose($Pipes[2]);

  // No more input, close command pipe.
  fclose($Pipes[0]);

  // Now that the pipes are closed, we also have to close the process.
  // Must happen after all pipes are closed.
  proc_close($process);

  if ($StdErr[0]) {
    $outputArr = $StdErr;
    return $StdErr[0];
  } else {
    $outputArr = $StdOut;
    return $StdOut[0];
  }
}

/**
 *  Another exec wrapper, because it's too late in the development cycle
 *  to fix execute_command.  Ultimately we should remove all of these;
 *  we need to execute all subprocesses that interact with the outside
 *  world at the end of non-blocking pipes that we wait on for X seconds
 *  maximum, rather than forever.  In the shorter term, I think
 *  execute_command should just be a wrapper for this one; it ensures
 *  that if standard error output is generated, the last line is prefixed
 *  with Error:, which some code relies one, and it returns the last
 *  line of output (matching the behavior of exec) rather than the first
 *  one.
 *
 * Some thoughts regarding the timeout problem: the simplest implementation
 * may be to just use the "timeout" Linux command.  If that doesn't work out,
 * the most likely implementation is the fork/exec/wait/kill functions provided
 * by the pcntl library.
 * @see http://us1.php.net/manual/en/function.pcntl-fork.php
 */
function exec_with_stderr($cmd, &$outputArr = array()) {
  $descspec = array(
    0 => array("pipe", "r"), # stdin is a pipe that the child will read from
    1 => array("pipe", "w"), # stdout is a pipe that the child will write to
    2 => array("pipe", "w")  # stderr is a pipe that the child will write to
  );

  $Pipes = array();
  $process = proc_open($cmd, $descspec, $Pipes, NULL, NULL);
  $StdOut = array();
  $StdErr = array();

  while (!feof($Pipes[1])) {
    $out = fgets($Pipes[1]);
    if (!empty($out)) {
      // Calling fgets() results in trailing newlines that
      // calls to exec() do not get for the same result.
      // All other whitespace may be significant, so only trim newlines.
      // This covers Win/Unix/Mac formats.
      array_push($StdOut, trim($out, "\r\n"));
    }
  }
  fclose($Pipes[1]);

  while (!feof($Pipes[2])) {
    $out = fgets($Pipes[2]);
    if (!empty($out)) {
      array_push($StdErr, trim($out, "\r\n"));
    }
  }
  fclose($Pipes[2]);

  // No more input, close command pipe.
  fclose($Pipes[0]);

  // Now that the pipes are closed, we also have to close the process.
  // Must happen after all pipes are closed.
  proc_close($process);

  if (!empty($StdErr)) {
    $last_line = array_pop($StdErr);
    if (stripos($last_line, 'Error:') === false) {
      $last_line = 'Error: ' . $last_line;
    }
    array_push($StdErr, $last_line);
    $outputArr = $StdErr;
    return end($StdErr);
  } else {
    $outputArr = $StdOut;
    return end($StdOut);
  }
}

function mercury_management_state2string($str) {

  switch ($str) {
    case 'Not Known':
      $str = 'Unable to communicate with VM agent';
      break;
    case 'OP_ENABLED':
      $str = 'Cache Enabled';
      break;
    case 'OP_DISABLED':
      $str = 'Cache Disabled';
      break;
    case 'OP_NORMAL':
      $str = 'Cache Normal';
      break;
    case 'OP_PENDING':
      $str = 'Cache Pending';
      break;
    case 'OP_FAILED':
      $str = 'Cache Failed';
      break;
    case 'OP_DEGRADED':
      $str = 'Cache Degraded';
      break;
  }
  return $str;
}

/**
 * Function to convert Bytes to KB, MB, GB and TB
 * XXX: NOT USED
 */
function mercury_management_bytesToSize($bytes, $precision = 2) {
  $kilobyte = 1024;
  $megabyte = $kilobyte * 1024;
  $gigabyte = $megabyte * 1024;
  $terabyte = $gigabyte * 1024;

  if (($bytes >= 0) && ($bytes < $kilobyte)) {
    return $bytes . ' Bytes';
  } elseif (($bytes >= $kilobyte) && ($bytes < $megabyte)) {
    return round($bytes / $kilobyte, $precision) . ' KB';
  } elseif (($bytes >= $megabyte) && ($bytes < $gigabyte)) {
    return round($bytes / $megabyte, $precision) . ' MB';
  } elseif (($bytes >= $gigabyte) && ($bytes < $terabyte)) {
    return round($bytes / $gigabyte, $precision) . ' GB';
  } elseif ($bytes >= $terabyte) {
    return round($bytes / $terabyte, $precision) . ' TB';
  } else {
    return $bytes . ' Bytes';
  }
}

/**
 *  Function to redirect user to home page if no VM or Host selected.
 */
function mercury_management_no_operation_redirect($type, $op) {
  global $base_url;
  switch ($type) {
    case 'host':
      if ($op == 'install') {
        mercury_utilities_fmc_log_message('Host Agent', 'No host was selected for installing the host agent.', WATCHDOG_WARNING);
      } else if ($op == 'uninstall') {
        mercury_utilities_fmc_log_message('Host Agent', 'No host was selected for uninstalling the host agent.', WATCHDOG_WARNING);
      } else {
        mercury_utilities_fmc_log_message('Host Agent', 'No host was selected for upgrading the host agent.', WATCHDOG_WARNING);
      }
      break;
    case 'vm':
      if ($op == 'install') {
        mercury_utilities_fmc_log_message('VM Agent', 'No VM was selected for installing the VM agent.', WATCHDOG_WARNING);
      } else if ($op == 'uninstall') {
        mercury_utilities_fmc_log_message('VM Agent', 'No VM was selected for uninstalling the VM agent.', WATCHDOG_WARNING);
      } else {
        mercury_utilities_fmc_log_message('VM Agent', 'No VM was selected for upgrading the VM agent.', WATCHDOG_WARNING);
      }
      break;
  }
  drupal_goto($base_url);
}
