<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that operate on vCenter nodes.
 */

/**
 * Function to get all the details of a vCenter
 */
function mercury_management_vcenter_details() {
  $vcInfo = array();
  db_query("SET GLOBAL query_cache_size=0");
  $vcenters = db_query("SELECT nid FROM {node} WHERE type = :tp AND status = :st", array(':tp' => 'vcenter', ':st' => 1));
  if (!empty($vcenters)) {
    foreach ($vcenters as $vc) {
      $vcInfo = node_load($vc->nid, NULL, true);
    }
    return $vcInfo;
  }
}

/**
 * Function to return array of hierarchy of datacenter objects (datacenters, hosts, vms)
 * @TODO: For performance reasons, split this into two underlying
 * implementations. The discovery tab under settings need the VM *count* for
 * every DC, but the actual discovery process needs the name of all hosts and
 * VMs in every datacenter *that the user selected*. So that way both contexts
 * can load only the data they need and thus do less work.
 */
function mercury_management_gethosts_virtualmachines(stdClass $vCenter, $vc_hostname, $vc_username, $vc_password) {

  $response = famc_external_api_datacenter_list($vCenter);
  if (famc_external_api_response_is_error($response)) {
    mercury_utilities_fmc_log_message(
      'Discovery',
      t('Could not get a list of datacenters for @vCenter: @message',
        array('@vCenter' => $vCenter->title, '@message' => famc_external_api_response_get_error_message($response))
      ),
      WATCHDOG_ERROR
    );

    return array();
  }

  $dataCenters = famc_external_api_response_get_data($response);
  if (!empty($dataCenters)) {
    $vCenterArr = array();
    foreach ($dataCenters as $dc) {
      $dataArr = array();
      exec("perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --url https://" . $vc_hostname . "/sdk/vimService --username '" . $vc_username . "' --password '" . $vc_password . "' --operation vmlist --datacenter '" . $dc . "'", $dataArr);
      if (!empty($dataArr)) {
        foreach ($dataArr as $item) {
          $host_split = explode('Host :', $item);
          if (count($host_split) > 1) {
            $currHost = trim($host_split[1]);
            $vCenterArr[$dc][$currHost] = array();
          }
          $vm_split = explode('VM :', $item);
          if (count($vm_split) > 1) {
            $vCenterArr[$dc][$currHost][] = trim($vm_split[1]);
          }
        }
      }
    }
  } else {
    mercury_utilities_fmc_log_message('Discovery', t('Discovery failed. There is no datacenter configured on vCenter "@vcenter"', array('@vcenter' => $vCenter->title)), WATCHDOG_ERROR, 'show');
  }
  return $vCenterArr;
}

/**
 * @see mercury_management_get_credentials
 */
function mercury_management_get_vcenter_credentials(stdClass $node) {
  return array(
    'address' => mercury_utilities_get_node_field($node, 'hostname'),
    'username' => fmc_cipher::decrypt(mercury_utilities_get_node_field($node, 'username')),
    'password' => fmc_cipher::decrypt(mercury_utilities_get_node_field($node, 'password')),
  );
}

/**
 * @see mercury_management_get_ancestors
 */
function mercury_management_get_vcenter_ancestors(stdClass $node, &$info = array()) {
  // Even though vCenter nodes currently have no ancestors, this is useful
  // anyway for generalizing code across multiple types of nodes.
  $info['vcenter'] = $node;
  return $info;
}
