<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that operate on VM nodes.
 */

/**
 *  Function to update single VM after install and uninstallation of mercury driver
 * @deprecated Not referenced!
 */
function mercury_management_update_single_vm($vc_hostname, $vc_username, $vc_password, $vm, $host_nid) {
  if (is_numeric($vm)) {
    $vm_info = node_load($vm);
    $vm_info->field_host['und'][0]['target_id'] = $host_nid;
    $driver_status = mercury_management_checkformercuryinstalled($vc_hostname, $vc_username, $vc_password, $vm);
    $vm_info->field_driver_status['und'][0]['value'] = $driver_status;

    node_save($vm_info);
  }
}

/**
 *  get the vCenter nid and the host nid associate with the VM
 * @deprecated use mercury_management_get_vm_ancestors() instead
 */
function mercury_management_get_host_dc_vc_of_vm($vm_nid) {
  $vm_node = node_load($vm_nid);
  $info = array();
  if (!empty($vm_node)) {
    $info['host'] = $vm_node->field_host['und'][0]['target_id'];
    $host_info = node_load($vm_node->field_host['und'][0]['target_id']);
    if (!empty($host_info->field_datacenter)) {
      $info['datacenter'] = $host_info->field_datacenter['und'][0]['target_id'];
      $data_center = node_load($host_info->field_datacenter['und'][0]['target_id']);
      if (!empty($data_center->field_related_vcenter)) {
        $info['vcenter'] = $data_center->field_related_vcenter['und'][0]['target_id'];
      }
    }
  }
  return $info;
}

/**
 *  get the vCenter nid and the host nid associate with the VM
 */
function mercury_management_get_host_data($vm) {
  $host_info = node_load($vm);
  $info = array();
  if (!empty($host_info)) {
    if (!empty($host_info->field_datacenter)) {
      $data_center = node_load($host_info->field_datacenter['und'][0]['target_id']);
      if (!empty($data_center->field_related_vcenter)) {
        $info['vcenter'] = $data_center->field_related_vcenter['und'][0]['target_id'];
      }
    }
  }
  return $info;
}

/**
 *  Function to create the batch file for the installation
 */
function mercury_management_create_uninstall_batch($vmInfo) {

  # In Argon the product name is 'Flash Accel'; in Barium+ it is
  # 'NetApp Flash Accel'  Check for field_fa_build_version
  # < 1.2.anything; if so, use the Argon name, otherwise
  # use the Barium+ name.
  $vers = explode('.', $vmInfo->field_fa_build_version['und'][0]['value']);
  if (intval($vers[0]) == 1 && intval($vers[1]) < 2) {
    $product_name = 'Flash Accel';
  } else {
    // Note that empty or 'Not Known' version results should use this version,
    // which is far more likely than trying to uninstall Argon.
    $product_name = mercury_utilities_get_fmc_brand_string() . ' Flash Accel';
  }

  // TODO: This mkdir is redundant.
  // @see 787658
  $filestr = "mkdir %SystemDrive%\\temp\\FlashAccelInstallationFiles\r\n";
  // TODO: Add privileges logic here too? @see BURT 787658
  $filestr .= "wmic /node:localhost /output:'%SystemDrive%\\temp\\FlashAccelInstallationFiles\\fa_uninstall.log' product where name='$product_name' call uninstall\r\n";
  $file_name = "sites/default/files/vmware_packages/uninstall.bat";
  $fileHandler = fopen($file_name, 'w');
  fwrite($fileHandler, $filestr);
  fclose($fileHandler);
}

/**
 *  Function to invoke the vmrun that executes the uninstall batch file in a
 *  subprocess that can be killed if the vmrun hangs (burt 718641)
 */
function mercury_management_run_uninstall_with_timeout($vc_hostname, $vc_username, $vc_password, $vmInfo) {

  # Create a worker array
  $wa = mercury_management_create_worker_array(1, $vc_hostname, $vc_username, $vc_password);
  if ($wa === false) {
    mercury_utilities_fmc_log_message('VM Agent', t('Could not connect to vCenter server.  Possible causes include invalid credentials, network problem, or server is down.'), WATCHDOG_ERROR);
    return 'Could not connect to vCenter server!';
  }

  # Set the select timeout to 180 seconds; if we time out the post-uninstall
  # status might be wrong because the reboot poller assumes the reboot has
  # actually been initiated, so best not to timeout.  However the vmrun
  # appears to hang for ESX5.1 (rather than returning on the reboot), so I
  # really don't want to wait longer than 3 minutes.
  $wa['select_timeout'] = 180;

  # Assemble the command line
  $username = mercury_management_get_vm_username($vmInfo);
  $password = mercury_management_get_vm_password($vmInfo);
  $cli = "vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' runProgramInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "c:\temp\FlashAccelInstallationFiles\fa_uninstall.bat"';

  # Execute the command line
  $res = mercury_management_wa_exec($wa, $cli);

  # Free the worker array
  mercury_management_destroy_worker_array($wa);

  # Interpret the result
  if ($res === false) {
    # Assume a select timeout occurred because the uninstall worked and
    # rebooted the VM, so actually this is a success.  We could be more
    # rigorous by providing a select() timeout handler and only declare
    # success if it gets called, but the cost of being wrong isn't
    # high; refreshing the VM will show the user that the FA agent is
    # still installed, if it is.
    return '';
  } else {
    # Return the last line of output from the vmrun like exec() does
    return (empty($res) ? '' : end($res));
  }
}

/**
 *  Function to uninstall the mercury driver (Flash Accel) from a virtual machine (VM)
 */
function mercury_management_uninstallmercurydriveronvm($nid) {
  $vmInfo = node_load($nid);
  $credentials = mercury_management_get_credentials($vmInfo, $include_ancestors = true);

  // For the old code...
  $vc_hostname = $credentials['vcenter']['address'];
  $vc_username = $credentials['vcenter']['username'];
  $vc_password = $credentials['vcenter']['password'];
  $username = $credentials['username'];
  $password = $credentials['password'];

  $req_type = '';
  if (isset($_REQUEST['step'])) {
    $req_type = $_REQUEST['step'];
  }

  $reboot = 0;
  if (isset($_REQUEST['reboot'])) {
    if ($_REQUEST['reboot'] == 'yes') {
      $reboot = 1;
    }
  }

  if (!is_dir('sites/default/files/vmware_packages')) {
    mkdir('sites/default/files/vmware_packages/', '0777');
  }

  $api_error = mercury_management_has_vsphere_api_error($vmInfo);
  if ($api_error) {
    // Already logged.
    print 'error--' . trim($vmInfo->title);
    exit;
  }

  if ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_INSTALLED) {
    if (!empty($vmInfo->field_cachespace['und'][0]['value']) && empty($_REQUEST['synthetic-upgrade'])) {
      // This does not apply to the synthetic upgrade steps because we want to
      // keep the cache and transfer it to the new agent.
      print 'cache_exists--' . $vmInfo->title;
      exit;
    } elseif ($vmInfo->field_fa_administration_state['und'][0]['value'] == "ADMIN_ENABLED") {
      print 'admin_enabled--' . $vmInfo->title;
      exit;
    } else {
      if ($username != '' && $password != '') {
        switch ($req_type) {
          case 'start':
            print '<div class="res-message">Uninstallation Process Started for the VM "' . $vmInfo->title . '".</div>';
            exit;
          case 'createBatch':
            mercury_management_create_uninstall_batch($vmInfo);
            //mercury_utilities_fmc_log_message('VM Agent', 'Uninstall Batch File Created.');
            print '<div class="res-message">Uninstall batch file created.</div>';
            exit;
          case 'copyBat':
            exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' createDirectoryInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles"');
            $copyBAT = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromHostToGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "sites/default/files/vmware_packages/uninstall.bat" "C:\temp\FlashAccelInstallationFiles\fa_uninstall.bat"');
            if ($copyBAT == '') {
              //mercury_utilities_fmc_log_message('VM Agent', 'Uninstall Batch File Copied to Guest.');
              print '<div class="res-message">Uninstall Batch File Copied to Guest.<br>Uninstalling Agent on the Guest. Please Wait ...</div>';
            } else {
              mercury_utilities_fmc_log_message('VM Agent', t('VM @vm, Could not copy uninstall batch file to C:\temp\FlashAccelInstallationFiles. @status', array('@vm' => $vmInfo->title, '@status' => $copyBAT)), WATCHDOG_ERROR);
              print 'copyFailed--' . $vmInfo->title;
            }
            exit;
          case 'uninstall':
            $batch_start = time();
            $uninstall = mercury_management_run_uninstall_with_timeout($vc_hostname, $vc_username, $vc_password, $vmInfo);
            $batch_time = time() - $batch_start;
            if (!empty($uninstall)) {
              //uninstall failed
              mercury_utilities_fmc_log_message('VM Agent', t('VM @vm, uninstall failed. @status', array('@vm' => $vmInfo->title, '@status' => $uninstall)), WATCHDOG_ERROR);

              // Losing the connection with the VM results in a terrible error
              // message.  Append something actionable.  @see BURT 788306
              // "Error: VMware Tools are not running in the guest"
              // Note that we don't know whether the batch file is done yet,
              // so it's unsafe to take any further action at this point.
              if (stristr($uninstall, 'VMware Tools are not running')) {
                mercury_utilities_fmc_log_message(
                  'VM Agent',
                  t('Lost connection with VM @vm during uninstall after @sec seconds. Please refresh the VM using the wrench menu.',
                    array('@vm' => $vmInfo->title, '@sec' => $batch_time)),
                  WATCHDOG_ERROR
                );
              }

              if (stristr($uninstall, '3010')) {
                // XXX - This scenario is defunct as we are now using wmic instead of msiexec.
                // However, we should still check fa_uninstall.log and log the
                // contents on failures like install and upgrade do.
                mercury_utilities_fmc_log_message('VM Agent', t('Uninstall failed on VM @vm. The VM needs to be rebooted before uninstall can be done.', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
                print 'rebootRequired--' . $vmInfo->title;
              } else {
                print 'vmrunBatError--' . $vmInfo->title;
              }
              exit;
            }
            $reboot_completed = false;
            if ($uninstall == '') {
              if ($reboot == 1) {
                # The uninstaller always reboots the VM, so we don't have to.
                # $reboot_status = exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' esxadmin --operation reboot --vmname " . trim($vmInfo->title));
                print '<div class="res-message">Uninstall Process completed successfully on the VM  "' . $vmInfo->title . '". Initiated VM reboot.<br /></div>';

                # We wait up to 900 seconds for the VM to come up, and up to 300
                # seconds after that for vCenter to re-populate some key VM data
                # (IP address and guest OS).
                $reboot_completed = mercury_utilities_poll_for_vm_reboot_complete($vmInfo, 900, 300);
                if (false == $reboot_completed) {
                  mercury_utilities_fmc_log_message('VM Agent', t('Timed out waiting for VM @vm to finish rebooting.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                }
              } else {
                $reboot_status = exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' esxadmin --operation shutdown --vmname '" . trim($vmInfo->title) . "'");
                print '<div class="res-message">Uninstall Process completed successfully on the VM ' . $vmInfo->title . '. Initiated VM shutdown. <br /></div>';
              }
            }

            $result = mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vmInfo->title, $credentials['host']['node']->nid);
            mercury_management_access_annotations('vm', $vmInfo->title, 'clear');

	    # save an empty metrics node. This is so that FAMC VM node page does not display stale 
	    # values when software was uninstalled.
	    famc_save_empty_vm_metric($vmInfo->nid);

            if (true == $reboot_completed) {
              # Load the updated node (invalidate node cache); probably not
              # necessary but I want to make sure I've got an up-to-date
              # driver status, and it was potentially altered in the node
              # by the call to mercury_management_updatevm above.
              $vmInfo = node_load($vmInfo->nid, NULL, TRUE);
              if ($vmInfo->field_driver_status['und'][0]['value'] != FAMC_VM_DRIVER_STATUS_NOT_INSTALLED) {
                $reboot_completed = false;
              }
            }
            if (false == $reboot_completed) {
              mercury_utilities_fmc_log_message('VM Agent', t('Uninstallation completed on VM @vm. Please refresh the VM after it reboots.', array('@vm' => $vmInfo->title)));
            } else {
              mercury_utilities_fmc_log_message('VM Agent', t('Uninstallation completed on VM @vm.', array('@vm' => $vmInfo->title)));
            }
            exit;
        }
      } else {
        print '<div class="res-message">Uninstallation failed because username & password are not set for the VM "' . $vmInfo->title . '"</div>';
        exit;
      }
    }
  } elseif ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_INSTALLED) {
    mercury_utilities_fmc_log_message('VM Agent', t('Uninstallation of VM agent failed for VM "@vm". ', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    print 'uninstall--' . $vmInfo->title;
    exit;
  } elseif ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_COMPATIBLE) {
    # This shouldn't happend; make it equivalent to the "not installed" case.
    mercury_utilities_fmc_log_message('VM Agent', t('Uninstallation of VM agent failed for VM "@vm". ', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    print 'uninstall--' . $vmInfo->title;
    exit;
  } else { // Any unknown or unhandled state is also a failure.
    mercury_utilities_fmc_log_message('VM Agent', t('Unable to communicate with VM @vm due to an invalid driver status. Uninstallation failed.', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    print 'error--' . $vmInfo->title;
    exit;
  }
}

/**
 *  Function to create the batch file for the installation
 */
function mercury_management_create_install_batch($filename, $port) {
  // TODO: This mkdir is redundant.
  // @see 787658
  $filestr = "mkdir %SystemDrive%\\temp\\FlashAccelInstallationFiles\r\n";
  // TODO: HGDEVLIST, HGGLOBALSTATE, and /l*v {logfile} are not used.
  // @see BURT 787651
  $filestr .= 'start /wait msiexec /i %SystemDrive%\\temp\\FlashAccelInstallationFiles\\' . $filename . ' HGSVCPORT=' . $port . ' HGDEVLIST=all HGGLOBALSTATE=1 /l*v %SystemDrive%\\temp\\FlashAccelInstallationFiles\\fa_install.log /q /norestart' . "\r\n";
  // Per BURT 787658, we need to create the result.log externally if the
  // installer would not be able to do it by itself.
  $filestr .= 'if "%ErrorLevel%" == "1625" echo [6] - Elevated administrative privileges required. > %SystemDrive%\\temp\\FlashAccelInstallationFiles\\result.log' . "\r\n";
  // And then propagate the error code.
  $filestr .= 'exit /b %ErrorLevel%' . "\r\n";

  $file_name = "sites/default/files/vmware_packages/install.bat";
  $fileHandler = fopen($file_name, 'w');
  fwrite($fileHandler, $filestr);
  fclose($fileHandler);
}

/**
 *  Function to create the batch file for the upgrade
 */
function mercury_management_create_upgrade_batch($filename, $port) {
  // TODO: This mkdir is redundant.
  // @see 787658
  $filestr = "mkdir %SystemDrive%\\temp\\FlashAccelInstallationFiles\r\n";
  // TODO: HGDEVLIST, HGGLOBALSTATE, and /l*v {logfile} are not used.
  // @see BURT 787651
  $filestr .= 'start /wait msiexec /i %SystemDrive%\\temp\\FlashAccelInstallationFiles\\' . $filename . ' HGSVCPORT=' . $port . ' /l*v %SystemDrive%\\temp\\FlashAccelInstallationFiles\\fa_upgrade.log /q /norestart' . "\r\n";
  // Per BURT 787658, we need to create the result.log externally if the
  // installer would not be able to do it by itself.
  $filestr .= 'if "%ErrorLevel%" == "1625" echo [6] - Elevated administrative privileges required. > %SystemDrive%\\temp\\FlashAccelInstallationFiles\\result.log' . "\r\n";
  // And then propagate the error code.
  $filestr .= 'exit /b %ErrorLevel%' . "\r\n";

  $file_name = "sites/default/files/vmware_packages/upgrade.bat";
  $fileHandler = fopen($file_name, 'w');
  fwrite($fileHandler, $filestr);
  fclose($fileHandler);
}

/**
 *  Function to install the mercury driver (Flash Accel)
 */
function mercury_management_installmercurydriveronvm($nid) {
  $vmInfo = node_load($nid);
  $credentials = mercury_management_get_credentials($vmInfo, $include_ancestors = true);

  // For the old code...
  $vc_hostname = $credentials['vcenter']['address'];
  $vc_username = $credentials['vcenter']['username'];
  $vc_password = $credentials['vcenter']['password'];
  $username = $credentials['username'];
  $password = $credentials['password'];

  $reboot = 0;
  if (isset($_REQUEST['reboot'])) {
    if ($_REQUEST['reboot'] == 'yes') {
      $reboot = 1;
    }
  }

  $req_type = '';
  if (isset($_REQUEST['step'])) {
    $req_type = $_REQUEST['step'];
  }
  if (!is_dir('sites/default/files/vmware_packages')) {
    mkdir('sites/default/files/vmware_packages/', '0777');
  }

  $api_error = mercury_management_has_vsphere_api_error($vmInfo);
  if ($api_error) {
    // Already logged.
    print 'error--' . trim($vmInfo->title);
    exit;
  }

  if ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_INSTALLED) {
    // This may need to change for BURT 756325. (checkbox override)
    if (mercury_management_guest_os_is_supported($vmInfo) === true) {
      if ($username != '' && $password != '') {

        switch ($req_type) {
          case 'start':
            print '<div class="res-message">Initializing Installation Process on the VM "' . $vmInfo->title . '".</div>';
            // Check and enable disk UUID
            $enableDiskUUID = execute_command("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' --vmname '" . trim($vmInfo->title) . "' --operation enableDiskUUID", $enable_disk_uuid_output_array);
            if (!empty($enableDiskUUID)) {
              mercury_utilities_fmc_log_message('VM Agent', $enableDiskUUID);
              if (stristr($enableDiskUUID, 'Error')) {
                print '<div class="res-message">Failed to set VM property disk.enableUUID. Aborted installation.</div>';
                mercury_utilities_fmc_log_message('VM Agent', t('Error: Failed to set VM property disk.enableUUID. Aborted installation.', WATCHDOG_ERROR));
                return "error";
              } else if (stristr($enableDiskUUID, 'Success')) {
                print '<div class="res-message">Enabled VM property disk.enableUUID. Please power cycle the VM before enabling caching.</div>';
                mercury_utilities_fmc_log_message('VM Agent', t('Enabled property disk.enableUUID on VM @vm. Please power cycle the VM before enabling caching.', array('@vm' => $vmInfo->title)));
              }
            }
            exit;
          case 'createBatch':
            if (!empty($vmInfo->field_install_port)) {
              $port = $vmInfo->field_install_port['und'][0]['value'];
            } else {
              $port = '7010';
            }
            mercury_management_create_install_batch(variable_get('vm_upgraded_MSI'), $port);
            // mercury_utilities_fmc_log_message('VM Agent', 'Installation Batch File Created.');

            print '<div class="res-message">Installation batch files created.</div>';

            exit;
          case 'copyMSI':
            exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' createDirectoryInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles"');
            $copyMSI = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromHostToGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "' . FLASH_AGENT_MSI_PATH . variable_get('vm_upgraded_MSI') . '" "C:\\temp\\FlashAccelInstallationFiles\\' . variable_get('vm_upgraded_MSI') . '"');
            if ($copyMSI == '') {
              // mercury_utilities_fmc_log_message('VM Agent', 'Copied the MSI Package to the Guest.');
              print '<div class="res-message">Copied MSI package to the guest.</div>';
            }
            exit;
          case 'copyBAT':
            exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' createDirectoryInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles"');
            $copyBAT = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromHostToGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "sites/default/files/vmware_packages/install.bat" "C:\temp\FlashAccelInstallationFiles\fa_install.bat"');
            if ($copyBAT == '') {
              // mercury_utilities_fmc_log_message('VM Agent', 'Install Batch File Copied to Guest.');
              print '<div class="res-message">Copied install batch file to guest.<br>Installing Agent on the Guest. Please Wait ...</div>';
            } else {
              mercury_utilities_fmc_log_message('VM Agent', t('VM @vm, Could not copy install batch file to C:\temp\FlashAccelInstallationFiles. @status', array('@vm' => $vmInfo->title, '@status' => $copyBAT)), WATCHDOG_ERROR);
              print 'copyFailed--' . $vmInfo->title;
            }
            exit;
          case 'install':
            // Record how long this takes for timeout debugging.
            $batch_start = time();
            $exeBat = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' runProgramInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "c:\temp\FlashAccelInstallationFiles\fa_install.bat"');
            $batch_time = time() - $batch_start;
            if (!empty($exeBat)) {
              // If this is not a Guest program failure we had errors with vmrun or batch file itself
              if (!stristr($exeBat,'Guest program exited with non-zero exit code')) {
                mercury_utilities_fmc_log_message('VM Agent', t('VM @vm, installation failed. @status', array('@vm' => $vmInfo->title, '@status' => $exeBat)), WATCHDOG_ERROR);
                
                // Losing the connection with the VM results in a terrible error
                // message.  Append something actionable.  @see BURT 788306
                // "Error: VMware Tools are not running in the guest"
                // Note that we don't know whether the batch file is done yet,
                // so it's unsafe to take any further action at this point.
                if (stristr($exeBat, 'VMware Tools are not running')) {
                  mercury_utilities_fmc_log_message(
                    'VM Agent',
                    t('Lost connection with VM @vm during installation after @sec seconds. Please refresh the VM using the wrench menu. If the install completed successfully a reboot may be needed before all functionality is available.',
                      array('@vm' => $vmInfo->title, '@sec' => $batch_time)),
                    WATCHDOG_ERROR
                  );
                }

                // Terminate the process.
                print 'vmrunBatError--' . $vmInfo->title;
                exit;
              }
            }

            // Checking for the mercury driver successful installation by downloading the result.log file
            // It is possible that there is no result.log file - in that case we return the vmrun error
            $isSuccess = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromGuestToHost '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles\result.log"  "sites/default/files/vmware_packages/installation_result.log"');
            $filename = 'sites/default/files/vmware_packages/installation_result.log';
            if ($isSuccess == '') {
              if (file_exists($filename)) {
                $content = file_get_contents($filename);
                if (!strstr($content, '[0]')) {
                  $isSuccess = $content;
                }
              } else {
                // If the result.log file is not there, check for download error, and msiexec error.
                // Report the appropriate error to the end user.
                if (empty($exeBat)) {
                  $isSuccess = 'did not find success indication in installer log file';
                } else {
                  $isSuccess  = $exeBat;
                }
              }
            } else { // Failed to get result.log
              // For permission errors, try to make the error message more
              // helpful.  An example message for non-admin users:
              // 'You do not have access rights to this file'
              if (
                stripos($isSuccess, 'access rights') !== false ||
                stripos($isSuccess, 'permission') !== false ||
                stripos($isSuccess, 'privileges') !== false
              ) {
                // Avoid redundant punctuation.
                $isSuccess = trim($isSuccess, ' .');
                $isSuccess .= '. Please see the Installation and Administration Guide for details on required Administrator privileges.';
              }

              // Also include result code if present for diagnostics.
              if (!empty($exeBat)) {
                $isSuccess .= " ($exeBat)";
              }
            }

            $reboot_completed = false;
            if ($isSuccess == '') {

	      // Annotate if the install was successful
	      mercury_management_access_annotations('vm', $vmInfo->title, 'set', variable_get('vm_package_agent_version', ''));

              if ($reboot == 1) {
                $reboot_status = exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' esxadmin --operation reboot --vmname '" . trim($vmInfo->title) . "'");
                print '<div class="res-message">Installation Process completed successfully on the VM "' . $vmInfo->title . '". Initiated VM reboot.<br /></div>';

                # We wait up to 900 seconds for the webservice to come up, and
                # up to 300 seconds after that for vCenter to re-populate some
                # key VM data (IP address and guest OS).
                $reboot_completed = mercury_utilities_poll_for_vm_webservice_online($vmInfo, 900, 300);
                if (false == $reboot_completed) {
                  mercury_utilities_fmc_log_message('VM Agent', t('Timed out waiting for VM @vm to finish rebooting.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                }
              } else {
                $reboot_status = exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' esxadmin --operation shutdown --vmname '" . trim($vmInfo->title) . "'");
                print '<div class="res-message">Installation Process completed successfully on the VM "' . $vmInfo->title . '". Initiated VM shutdown.<br /></div>';

                # We wait up to 900 seconds for vCenter to set the Power State field properly
                $shutdown_completed = mercury_utilities_poll_for_vm_shutdown_complete($vmInfo, 900);
                if (false == $shutdown_completed) {
                  mercury_utilities_fmc_log_message('VM Agent', t('Timed out waiting for VM @vm to finish shutting down.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                }
              }
              mercury_utilities_fmc_log_message('VM Agent', t('Installation successful on VM @vm', array('@vm' => $vmInfo->title)));
            } else {
              // Install failed.  Log and terminate.
              mercury_utilities_fmc_log_message('VM Agent', t('Installation process failed on VM @vm, result = @status', array('@vm' => $vmInfo->title, '@status' => $isSuccess)), WATCHDOG_ERROR);
              if (!empty($exeBat)) {
                if (!stristr($exeBat, '3010')) {
                  print 'installFailed--' . $vmInfo->title;
                } else {
                  print 'rebootRequired--' . $vmInfo->title;
                }
                exit;
              }
            }

            // Refresh the VM.
            $res = mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vmInfo->title, $credentials['host']['node']->nid);
            if ($reboot == 1) {
              if (stristr($res, "error")) {
                mercury_utilities_fmc_log_message('VM Agent', t('Unable to query cache state on VM @vm following install.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                $reboot_completed = false;
              }
              if (false == $reboot_completed) {
                mercury_utilities_fmc_log_message('VM Agent', t('Installation completed on VM @vm. Please refresh the VM after it reboots.', array('@vm' => $vmInfo->title)));
              } else {
                mercury_utilities_fmc_log_message('VM Agent', t('Installation completed on VM @vm.', array('@vm' => $vmInfo->title)));
              }
            } else {
              mercury_utilities_fmc_log_message('VM Agent', t('Installation completed on VM @vm. VM has been powered down.', array('@vm' => $vmInfo->title)));
            }
            exit;
        }
      } else {
        print '<div class="res-message">Installation failed because username & password are not set for the Virtual Machine "' . $vmInfo->title . '"</div>';
        exit;
      }
    } else {
      mercury_utilities_fmc_log_message('VM Agent', t('Installation failed because VM "@vm" is not running a supported guest OS.', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
      print 'Invalid OS--' . $vmInfo->title;
      exit;
    }
  } elseif ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_INSTALLED) {
    print 'install--' . $vmInfo->title;
    mercury_utilities_fmc_log_message('VM Agent', t('Installation failed because the VM Agent is already installed on "@vm"', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    exit;
  } elseif ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_COMPATIBLE) {
    print 'install--' . $vmInfo->title;
    mercury_utilities_fmc_log_message('VM Agent', t('Installation failed because the VM "@vm" is running an incompatible guest OS', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    exit;
  } else { // Any unknown or unhandled state is also a failure.
    mercury_utilities_fmc_log_message('VM Agent', t('Unable to communicate with VM @vm due to an invalid driver status. Installation failed.', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    print 'error--' . $vmInfo->title;
    exit;
  }
}

/**
 *  Function to install the mercury driver (Flash Accel)
 */
function mercury_management_upgrademercurydriveronvm($nid) {
  $vmInfo = node_load($nid);
  $credentials = mercury_management_get_credentials($vmInfo, $include_ancestors = true);

  // For the old code...
  $vc_hostname = $credentials['vcenter']['address'];
  $vc_username = $credentials['vcenter']['username'];
  $vc_password = $credentials['vcenter']['password'];
  $username = $credentials['username'];
  $password = $credentials['password'];

  $reboot = 0;
  if (isset($_REQUEST['reboot'])) {
    if ($_REQUEST['reboot'] == 'yes') {
      $reboot = 1;
    }
  }

  $req_type = '';
  if (isset($_REQUEST['step'])) {
    $req_type = $_REQUEST['step'];
  }
  if (!is_dir('sites/default/files/vmware_packages')) {
    mkdir('sites/default/files/vmware_packages/', '0777');
  }

  $api_error = mercury_management_has_vsphere_api_error($vmInfo);
  if ($api_error) {
    // Already logged.
    print 'error--' . trim($vmInfo->title);
    exit;
  }

  if ($vmInfo->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_INSTALLED) {
    // This may need to change for BURT 756325. (checkbox override)
    if (mercury_management_guest_os_is_supported($vmInfo) === true) {
      if ($username != '' && $password != '') {
        switch ($req_type) {
          case 'start':
            print '<div class="res-message">Initializing upgrade process on the VM "' . $vmInfo->title . '".</div>';
            exit;
          case 'versionCheck':
            $versionCheck = mercury_utilities_check_upgrade_vm($nid);
            if ($versionCheck == 'no-upgrade') {
              print 'version--' . trim($vmInfo->title);
            }

            // Synthetic upgrade: if the current VM agent is <= 1.2 and the new
            // agent is >= 1.3, we have to do a synthetic upgrade because of
            // the change from perUser to perMachine installs.
            $vm_package_agent_version = variable_get('vm_package_agent_version', '');
            $vm_current_agent_version = $vmInfo->field_fa_build_version['und'][0]['value'];
            if (
              mercury_utilities_version_compare($vm_current_agent_version, false, "1.2", $major_minor = true, $allow_equal = true) &&
              mercury_utilities_version_compare($vm_package_agent_version, "1.3", false, $major_minor = true, $allow_equal = true)
            ) {
              print 'synthetic--yes';
            }

            exit;
          case 'createBatch':
            if (!empty($vmInfo->field_install_port)) {
              $port = $vmInfo->field_install_port['und'][0]['value'];
            } else {
              $port = '7010';
            }
            // XXX: Why do we create these extra batch files?
            mercury_management_create_install_batch(variable_get('vm_upgraded_MSI'), $port);
            // mercury_utilities_fmc_log_message('VM Agent', 'Upgrade: Installation Batch File Created.');
            mercury_management_create_uninstall_batch($vmInfo);
            // mercury_utilities_fmc_log_message('VM Agent', 'Upgrade: Uninstall Batch File Created.');
            mercury_management_create_upgrade_batch(variable_get('vm_upgraded_MSI'), $port);
            // mercury_utilities_fmc_log_message('VM Agent', 'Upgrade: Batch File Created.');

            print '<div class="res-message">Upgrade Batch Files Created.</div>';

            exit;
          case 'copyMSI':
            exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' createDirectoryInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles"');
            $copyMSI = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromHostToGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "' . FLASH_AGENT_MSI_PATH . variable_get('vm_upgraded_MSI') . '" "C:\\temp\\FlashAccelInstallationFiles\\' . variable_get('vm_upgraded_MSI') . '"');
            if ($copyMSI == '') {
              //mercury_utilities_fmc_log_message('VM Agent', 'Upgrade: Copied the MSI Package to the Guest.');
              print '<div class="res-message">Copied the MSI Package to the Guest.</div>';
            }
            exit;
          case 'copyBAT':
            exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' createDirectoryInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles"');
            $copyBAT = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromHostToGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "sites/default/files/vmware_packages/upgrade.bat" "C:\temp\FlashAccelInstallationFiles\fa_upgrade.bat"');
            if ($copyBAT == '') {
              //mercury_utilities_fmc_log_message('VM Agent', 'Upgrade: Batch File Copied to Guest.');
              print '<div class="res-message">Upgrade Batch File Copied to Guest.<br>Upgrading Agent on the Guest. Please Wait ...</div>';
            } else {
              mercury_utilities_fmc_log_message('VM Agent', t('VM @vm, Could not copy upgrade batch file to C:\temp\FlashAccelInstallationFiles. @status', array('@vm' => $vmInfo->title, '@status' => $copyBAT)), WATCHDOG_ERROR);
              print 'copyFailed--' . $vmInfo->title;
            }
            exit;
          case 'install':
            $batch_start = time();
            $exeBat = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' runProgramInGuest '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "c:\temp\FlashAccelInstallationFiles\fa_upgrade.bat"');
            $batch_time = time() - $batch_start;
            if (!empty($exeBat)) {
              // If this is not a Guest program failure we had errors with vmrun or batch file itself
              if (!stristr($exeBat,'Guest program exited with non-zero exit code')) {
                mercury_utilities_fmc_log_message('VM Agent', t('VM @vm, upgrade failed. @status', array('@vm' => $vmInfo->title, '@status' => $exeBat)), WATCHDOG_ERROR);

                // Losing the connection with the VM results in a terrible error
                // message.  Append something actionable.  @see BURT 788306
                // "Error: VMware Tools are not running in the guest"
                // Note that we don't know whether the batch file is done yet,
                // so it's unsafe to take any further action at this point.
                if (stristr($exeBat, 'VMware Tools are not running')) {
                  mercury_utilities_fmc_log_message(
                    'VM Agent',
                    t('Lost connection with VM @vm during upgrade after @sec seconds. Please refresh the VM using the wrench menu. If the upgrade completed successfully a reboot may be needed before all functionality is available.',
                      array('@vm' => $vmInfo->title, '@sec' => $batch_time)),
                    WATCHDOG_ERROR
                  );
                }

                print 'vmrunBatError--' . $vmInfo->title;
                exit;
              }
            }

            // Checking for the mercury driver successful installation by downloading the result.log file
            // It is possible that there is no result.log file - in that case we return the vmrun error
            $isSuccess = exec("vmrun -T esx -h https://" . $vc_hostname . "/sdk -u '" . $vc_username . "' -p '" . $vc_password . "' -gu '" . $username . "' -gp '" . $password . "' copyFileFromGuestToHost '" . $vmInfo->field_vm_path_name['und'][0]['value'] . "'" . ' "C:\temp\FlashAccelInstallationFiles\result.log"  "sites/default/files/vmware_packages/installation_result.log"');
            $filename = 'sites/default/files/vmware_packages/installation_result.log';
            if ($isSuccess == '') {
              if (file_exists($filename)) {
                $content = file_get_contents($filename);
                if (!strstr($content, '[0]')) {
                  $isSuccess = $content;
                }
              } else {
                // If the result.log file is not there, check for download error, and msiexec error.
                // Report the appropriate error to the end user.
                if (empty($exeBat)) {
                  $isSuccess = 'did not find success indication in installer log file';
                } else {
                  $isSuccess  = $exeBat;
                }
              }
            } else { // Failed to get result.log
              // For permission errors, try to make the error message more
              // helpful.  An example message for non-admin users:
              // 'You do not have access rights to this file'
              if (
                stripos($isSuccess, 'access rights') !== false ||
                stripos($isSuccess, 'permission') !== false ||
                stripos($isSuccess, 'privileges') !== false
              ) {
                // Avoid redundant punctuation.
                $isSuccess = trim($isSuccess, ' .');
                $isSuccess .= '. Please see the Installation and Administration Guide for details on required Administrator privileges.';
              }

              // Also include result code if present for diagnostics.
              if (!empty($exeBat)) {
                $isSuccess .= " ($exeBat)";
              }
            }

            $reboot_completed = false;
            if ($isSuccess == '') {

	      // Annotate if the upgrade was successful
	      mercury_management_access_annotations('vm', $vmInfo->title, 'clear');
	      mercury_management_access_annotations('vm', $vmInfo->title, 'set', variable_get('vm_package_agent_version', ''));

              if ($reboot == 1) {
                print '<div class="res-message">Upgrade process completed successfully on the VM "' . $vmInfo->title . '". Initiated VM reboot.<br /></div>';
                $reboot_status = exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' esxadmin --operation reboot --vmname '" . trim($vmInfo->title) . "'");

                # We wait up to 900 seconds for the webservice to come up, and
                # up to 300 seconds after that for vCenter to re-populate some
                # key VM data (IP address and guest OS).
                $reboot_completed = mercury_utilities_poll_for_vm_webservice_online($vmInfo, 900, 300);
                if (false == $reboot_completed) {
                  mercury_utilities_fmc_log_message('VM Agent', t('Timed out waiting for VM @vm to finish rebooting.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                }
              } else {
                print '<div class="res-message">Upgrade process completed successfully on the VM "' . $vmInfo->title . '". Initiated VM shutdown.<br /></div>';
                $reboot_status = exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' esxadmin --operation shutdown --vmname '" . trim($vmInfo->title) . "'");

                # We wait up to 900 seconds for vCenter to set the Power State field properly
                $shutdown_completed = mercury_utilities_poll_for_vm_shutdown_complete($vmInfo, 900);
                if (false == $shutdown_completed) {
                  mercury_utilities_fmc_log_message('VM Agent', t('Timed out waiting for VM @vm to finish shutting down.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                }
              }
              mercury_utilities_fmc_log_message('VM Agent', t('Upgrade successful on VM @vm', array('@vm' => $vmInfo->title)));
            } else {
              // Upgrade failed.  Log and terminate.
              mercury_utilities_fmc_log_message('VM Agent', t('Upgrade failed on VM @vm. Result = @result', array('@vm' => $vmInfo->title, '@result' => $isSuccess)), WATCHDOG_ERROR);
              if (!empty($exeBat)) {
                if (!stristr($exeBat, '3010')) {
                  print 'installFailed--' . $vmInfo->title;
                } else {
                  print 'rebootRequired--' . $vmInfo->title;
                }
                exit;
              }
            }

            // Refresh the VM.
            $res = mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vmInfo->title, $credentials['host']['node']->nid);
            if ($reboot == 1) {
              if (stristr($res, "error")) {
                mercury_utilities_fmc_log_message('VM Agent', t('Unable to query cache state on VM @vm following upgrade.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
                $reboot_completed = false;
              }
              if (false == $reboot_completed) {
                mercury_utilities_fmc_log_message('VM Agent', t('Upgrade completed on VM @vm. Please refresh the VM after it reboots.', array('@vm' => $vmInfo->title)));
              } else {
                mercury_utilities_fmc_log_message('VM Agent', t('Upgrade completed on VM @vm.', array('@vm' => $vmInfo->title)));
              }
            } else {
              mercury_utilities_fmc_log_message('VM Agent', t('Upgrade completed on VM @vm. VM has been powered down.', array('@vm' => $vmInfo->title)));
            }
            exit;
        }
      } else {
        print '<div class="res-message">Upgrade failed because username & password are not set in the Virtual Machine "' . $vmInfo->title . '"</div>';
        print 'password--' . $vmInfo->title;
        exit;
      }
    } else {
      mercury_utilities_fmc_log_message('VM Agent', t('Unable to upgrade VM "@vm" as it is not running a supported guest OS.', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
      print 'Invalid OS--' . $vmInfo->title;
      exit;
    }
  } else { // Any unknown or unhandled state is also a failure.
    print 'install--' . $vmInfo->title;
    mercury_utilities_fmc_log_message('VM Agent', t('Upgrade failed because VM Agent is not installed on VM @vm or the VM is unreachable.', array('@vm' => $vmInfo->title)), WATCHDOG_ERROR);
    exit;
  }
}

/**
 * Performs a fake upgrade via store settings, uninstall, reinstall, and then
 * restoring the original settings.  This is needed for upgrades from <= 1.2R1
 * to >= 1.3 because of the perUser to perMachine installer change.
 */
function mercury_management_upgrademercurydriveronvm_synthetic($nid) {
  $vmInfo = node_load($nid);

  $req_type = '';
  if (!empty($_REQUEST['step'])) {
    $req_type = $_REQUEST['step'];
  }

  // On any failure, throw away the saved data.
  // The synthetic upgrade process is temporary so we are not going to
  // invest a lot of time in error recovery.  The user is able to fix it.
  // This needs to happen before the API check because it does not depend on it.
  if ($req_type == 'errorCleanup') {
    variable_del('synthetic-upgrade-' . intval($vmInfo->nid));
    print '<div class="res-message">FAMC temporary storage cleaned for VM "' . $vmInfo->title . '".</div>';
    exit;
  }

  $reboot = 0;
  if (isset($_REQUEST['reboot'])) {
    if ($_REQUEST['reboot'] == 'yes') {
      $reboot = 1;
    }
  }

  // For install/uninstall calls, we must reboot.  Otherwise we would not be
  // able to continue to the next step (install for uninstall, restore settings
  // after install).
  // If the user left the reboot box unchecked, we will then have to use $reboot
  // below after settings are restored and shut down the VM at the end.
  $_REQUEST['reboot'] = 'yes';

  // Add an easy way for other functions to detect a synthetic upgrade.
  $_REQUEST['synthetic-upgrade'] = true;

  // Note that if we have reached this point, it means we have passed the
  // standard upgrade preflight checks.  What is left is unique to the synthetic
  // branch, either because of install/uninstall or for the settings restore.

  // Unlike a real Windows upgrade, this requires the API
  // to fetch and restore the cache and metric settings.
  // Only check the API status during these two phases because the API will be
  // down between uninstall and install.
  // SaveSettings is the first synthetic step so this
  // gets hit before anything destructive happens.
  if (in_array($req_type, array('saveSettings', 'restoreSettings'))) {
    $api_error = mercury_management_has_fa_api_error($vmInfo);
    if ($api_error) {
      // Already logged.
      print 'error--' . trim($vmInfo->title);
      exit;
    }
  }

  // In every synthetic step, check for cache enabled.
  // It's cheap and results in failing earlier and non-destructively.
  // Otherwise we won't fail until we reach the uninstall steps.
  if ($vmInfo->field_fa_administration_state['und'][0]['value'] == "ADMIN_ENABLED") {
    print 'admin_enabled--' . $vmInfo->title;
    exit;
  }

  // Now perform the synthetic-specific actions or reroute to install/uninstall.
  // After uninstall and after install the called functions will wait for a
  // reboot.  If the reboot fails then the next step in the list will fail.
  // But we will either be in an installed or uninstalled state at that point
  // so it should be easy enough for the user to recover.
  // The one thing that will not be done is recovering the settings.
  switch($req_type) {
    case 'saveSettings':
      mercury_management_upgrademercurydriveronvm_synthetic_save_settings($vmInfo);
      break;
    case 'createUninstallBatch':
    case 'copyUninstallBat':
    case 'uninstall':
      // Replace "Uninstall" in the string with nothing (case sensitive).
      // This is because install and uninstall have overlapping actions.
      // The "case" must match the synthetic_upgrade_steps from the JS and then
      // be fixed to match mercury_management_uninstallmercurydriveronvm().
      $_REQUEST['step'] = str_replace('Uninstall', '', $_REQUEST['step']);
      mercury_management_uninstallmercurydriveronvm($nid);
      break;
    case 'createBatch':
    case 'copyMSI':
    case 'copyBAT':
    case 'install':
      mercury_management_installmercurydriveronvm($nid);
      break;
    case 'restoreSettings':
      // $reboot contains the customer selection (desired end state).
      // See above for additional comments.
      mercury_management_upgrademercurydriveronvm_synthetic_restore_settings($vmInfo, $reboot);
      break;
  }

  // Reaching this means a code bug.
  // All of the child actions above must exit.
  mercury_utilities_fmc_log_message(
    'VM Agent',
    t('Upgrade failed on VM @vm because @req_type unhandled.', array('@vm' => $vmInfo->title, '@req_type' => $req_type)),
    WATCHDOG_ERROR
  );
  print 'default--' . $vmInfo->title;
}

/**
 * Get cache settings from the VM and save them in the database temporarily.
 * @param stdClass $vmInfo The target VM
 */
function mercury_management_upgrademercurydriveronvm_synthetic_save_settings(stdClass $vmInfo) {
  // The API status and other pre-flight requirements are already checked in
  // the parent synthetic call function, so we can dive right in.

  // Pull both sets of settings out of the VM.
  $info = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgGetCacheInfo');
  if ($info == 'error_wsdl_call') {
    // Already logged.
    print 'error--' . trim($vmInfo->title);
    exit;
  }
  $options = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgGetCacheOptions');
  if ($options == 'error_wsdl_call') {
    // Already logged.
    print 'error--' . trim($vmInfo->title);
    exit;
  }

  // Store the data somewhere semi-temporary.
  // There is some chance that this will stick around because of an upgrade
  // failure, but there is really no significant impact because it is only used
  // immediately following a call to this function = data is fresh.
  // The cache tables would be an alternative that would definitely get cleaned
  // but it runs the risk of it being cleaned prematurely.
  variable_set('synthetic-upgrade-' . intval($vmInfo->nid), array(
    'HgGetCacheInfo' => $info,
    'HgGetCacheOptions' => $options,
  ));

  print '<div class="res-message">Cache configuration backed up for VM "' . $vmInfo->title . '".</div>';
  exit;
}

/**
 * Pull saved cache settings out of the database and apply them to the VM.
 * @param stdClass $vmInfo The target VM
 * @param int $leave_power_on if 0/false the VM will be shut down at the end.
 */
function mercury_management_upgrademercurydriveronvm_synthetic_restore_settings(stdClass $vmInfo, $leave_power_on = 1) {

  // Grab the VM's settings out of our temporary holding spot.
  $settings = variable_get('synthetic-upgrade-' . intval($vmInfo->nid), array());

  // Delete the variable now.
  // The synthetic upgrade is not intended to be a long-lived block of code,
  // so we are not spending a lot of time dealing with the usability of unlikely
  // error conditions.  The user is able to recover by themselves if this fails.
  variable_del('synthetic-upgrade-' . intval($vmInfo->nid));

  // Check for validity.
  if (empty($settings) || empty($settings['HgGetCacheInfo'])) {
    mercury_utilities_fmc_log_message(
      'VM Agent',
      t('Agent upgrade completed on @vm but cache settings could not be found to restore.', array('@vm' => $vmInfo->title)),
      WATCHDOG_ERROR
    );
    print 'noRestore--' . trim($vmInfo->title);
    exit;
  }

  // Set the cachespace_id first.  Some other actions require this first.
  if (!empty($settings['HgGetCacheInfo']['cachespace_id'])) {

    // Tell the agent about the old cache.
    // If this fails then the rest will likely fail, so error out.
    $setcache_parameter = array('request' => array('CacheIdentifier' => $settings['HgGetCacheInfo']['cachespace_id']));
    $result = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgSetCacheIdentifier', $setcache_parameter);
    if ($result == 'error_wsdl_call') {
      mercury_utilities_fmc_log_message(
        'VM Agent',
        t('Agent upgrade completed on @vm but the cache space could not be restored. Please reallocate cache for the VM.', array('@vm' => $vmInfo->title)),
        WATCHDOG_ERROR
      );
      print 'cacheAssign--' . trim($vmInfo->title);
      exit;
    }

    // Format the cache space to match the new agent configuration.
    /**
     * Commented out because the format fails if the host is Barium and this
     * should not strictly be required during an upgrade anyway.
     * @see BURT 785780
    $result = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgStartCacheFormat');
    if ($result == 'error_wsdl_call') {
      mercury_utilities_fmc_log_message(
        'VM Agent',
        t('Agent upgrade completed on @vm but the cache space could not be formatted. Please unallocate and reconfigure the cache.', array('@vm' => $vmInfo->title)),
        WATCHDOG_ERROR
      );
      print 'cacheFormat--' . trim($vmInfo->title);
      exit;
    }
     */

    // Sleep for a moment to ensure that the format completes.
    // It should be basically instant but this is what allocate also does.
    sleep(2);
  }

  // Perform both cache operations even if the first fails.  The failure may not
  // prevent the second from working, and since these are setting restores we
  // might as well try.
  $cache_operation_failure = false;

  // Global options.  Currently just Metrics Reset Interval.
  // Metrics reset interval potentially affects perf_state so do this before
  // the call to HgSetCacheState.
  $param = array('request' => array(
    'HgCacheOptions' => array(
      'MetricsResetInterval' => $settings['HgGetCacheOptions']['metrics_reset_interval'],
    )
  ));
  $result = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgSetCacheOptions', $param);
  if ($result == 'error_wsdl_call') {
    $cache_operation_failure = true;
  }

  // Global cache_mode, admin_state, perf_state
  $param = array('request' => array(
    'CacheMode' => $settings['HgGetCacheInfo']['cache_mode'],
    'CacheState' => $settings['HgGetCacheInfo']['admin_state'],
    'PerfState' => $settings['HgGetCacheInfo']['perf_state'],
  ));
  $result = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgSetCacheState', $param);
  if ($result == 'error_wsdl_call') {
    $cache_operation_failure = true;
  }

  if ($cache_operation_failure) {
    // Debug info is sufficient at this point.
    print 'cacheSettings--' . trim($vmInfo->title);
    exit;
  }

  // Continue on any error at this point.
  // Likely just one bad disk, fix the rest.
  $disk_errors = false;

  // Set the per-disk settings.
  $vdisk_nids = mercury_utilities_vm_vdisk_nid_fetch($vmInfo->nid);
  if (count($vdisk_nids)) {

    // Re-key the stored info based on vdisk_identifier so we can match it up.
    // The order could be different so don't rely on the integer keys.
    $stored_disks = array();
    foreach($settings['HgGetCacheInfo'] as $row) {
      // Some of these entries are not disks.
      if (!is_array($row) || empty($row['vdisk_identifier'])) {
        continue;
      }

      // It's a disk, add to the list and key by unique ID.
      $stored_disks[$row['vdisk_identifier']] = $row;
    }

    $params = new stdClass();
    $params->DeviceStateRequest = array();
    foreach ($vdisk_nids as $vdisk_id) {
      $vdisk_node = node_load($vdisk_id);
      $vdisk_identifier = field_get_items('node', $vdisk_node, 'field_vdisk_identifier');
      $vdisk_identifier_value = $vdisk_identifier[0]['value'];
      if (empty($stored_disks[$vdisk_identifier_value])) {
        continue; // No saved data.
      }

      $params->DeviceStateRequest['0'] = array(
        'CacheState' => $stored_disks[$vdisk_identifier_value]['admin_state'],
        'PerfState' => $stored_disks[$vdisk_identifier_value]['perf_state'],
        'CacheMode' => $stored_disks[$vdisk_identifier_value]['cache_mode'],
        'NaaId' => $vdisk_identifier_value
      );
      $disk_param = array('request' => $params);
      $result = mercury_management_cache_operation('nid', $vmInfo->nid, 'HgSetDeviceState', $disk_param);
      if (empty($result['response_status']) || $result['response_status'] != 'success') {
        // The command has already been logged, but make a note
        // regarding which disk had the issue.
        $disk_errors = true;
        mercury_utilities_fmc_log_message(
          'VM Agent',
          t('Cache reconfiguration failed for disk @disk on VM @vm. Please refresh VM and try again.',
            array('@vm' => $vmInfo->title, '@disk' => $vdisk_identifier_value)),
          WATCHDOG_ERROR
        );
      }
    }
  }

  // Log before refresh, so the log order isn't confusing.
  if (!$disk_errors) {
    mercury_utilities_fmc_log_message(
      'VM Agent',
      t('Cache configuration restored for VM @vm after upgrade.', array('@vm' => $vmInfo->title)),
      WATCHDOG_NOTICE
    );
  }

  // The VM has to be on to perform the cache actions, so we always reboot.
  // However we still need to honor the reboot checkbox state from the user.
  // If $leave_power_on == int(1) then it's already up, success.
  // But if int(0) we need to shut down when done.
  $power_message = '';
  if (!$leave_power_on) {
    $credentials = mercury_management_get_credentials($vmInfo, $include_ancestors = true);
    exec("perl /usr/lib/vmware-vcli/apps/vm/vmcontrol.pl --url https://" . $credentials['vcenter']['address'] . "/sdk/vimService  --username '" . $credentials['vcenter']['username'] . "' --password '" . $credentials['vcenter']['password'] . "' esxadmin --operation shutdown --vmname '" . trim($vmInfo->title) . "'");
    mercury_utilities_fmc_log_message(
      'VM Agent',
      t('Shutdown initiated for VM @vm after upgrade.', array('@vm' => $vmInfo->title)),
      WATCHDOG_NOTICE
    );

    // Final status message is confusing without this.
    $power_message = ' Shutdown initiated.';

    // We wait up to 900 seconds for vCenter to set the Power State field properly
    $shutdown_completed = mercury_utilities_poll_for_vm_shutdown_complete($vmInfo, 900);
    if (false == $shutdown_completed) {
      mercury_utilities_fmc_log_message('VM Agent', t('Timed out waiting for VM @vm to finish shutting down.', array('@vm' => $vmInfo->title)), WATCHDOG_WARNING);
    }
  }

  // Trigger a refresh to make sure we get the new cache state in the DB.
  // If powered on this gets the real cache info.
  // If powered off this correctly updates the VM status to red in FAMC.
  mercury_utilities_vm_refresh_action($vmInfo->nid, 'no-refresh');

  // Even though we continued past it anyway, still let the user know that
  // there were errors restoring disk settings.
  if ($disk_errors) {
    // Already logged.
    print 'diskErrors--' . trim($vmInfo->title);
    exit;
  }

  // Completed successfully.
  print '<div class="res-message">Cache configuration restored for VM "' . $vmInfo->title . '" after upgrade.' . $power_message . '</div>';
  exit;
}

/**
 *  Function to check whether the mercury driver installed in VM or not?
 * @deprecated This is never called.
 * @see mercury_management_wcmd_vmrun_chk_for_fa
 */
function mercury_management_checkformercuryinstalled($vc_hostname, $vc_username, $vc_password, $vm_nid) {
  $vm_info = node_load($vm_nid);
  $driver_status = "Unable to Find VM Tools";
  if (!empty($vm_info->field_vmware_tools)) {
    if ($vm_info->field_vmware_tools['und'][0]['value'] == 'VMware Tools is not running.' ||
      $vm_info->field_vmware_tools['und'][0]['value'] == 'VMware Tools has never been installed or has not run in the virtual machine.' ||
      $vm_info->field_vm_hostname['und'][0]['value'] == '') {
      $driver_status = FAMC_VM_DRIVER_STATUS_NOT_KNOWN;
    } else {
      $vm_username = mercury_management_get_vm_username($vm_info);
      $vm_password = mercury_management_get_vm_password($vm_info);
      $brand = mercury_utilities_get_fmc_brand_string();
      $status = exec("vmrun -T esx -h https://" . $vc_hostname .
        "/sdk -u '" . $vc_username .
        "' -p '" . $vc_password .
        "' -gu '" . $vm_username .
        "' -gp '" . $vm_password .
        "' fileExistsInGuest '" . $vm_info->field_vm_path_name['und'][0]['value'] .
        "'" . ' "C:\Program Files\\' . $brand . '\FlashAccel\FA-Service\FA-Service.dll"');
      if ($status == "The file exists.") {
        $driver_status = FAMC_VM_DRIVER_STATUS_INSTALLED;
      } else {
        $driver_status = FAMC_VM_DRIVER_STATUS_NOT_INSTALLED;
      }
    }
  }
  mercury_utilities_fmc_log_message('VM Agent', t('Check for existence of VM agent on "@vm" : @status', array('@vm' => $vm_info->title, '@status' => $driver_status)));
  return $driver_status;
}

function mercury_management_HgGetCacheInfo_popup($vm_id) {
  $output = '';
  //$result = array();
  //$param = array();
  //$result = mercury_management_cache_operation('nid', $vm_id, 'HgGetCacheInfo', $param);
  $vm_node = node_load($vm_id);
  $vm_name = $vm_node->title;
  //$driver_status = field_get_items('node', $vm_node, 'field_driver_status');
  $vm_status = mercury_management_get_vm_status($vm_node);
  //$memory_size = field_get_items('node', $vm_node, 'field_memory_size');
  $op_state = field_get_items('node', $vm_node, 'field_fa_operational_state');
  $build_version = field_get_items('node', $vm_node, 'field_fa_build_version');
  $cache_mode = field_get_items('node', $vm_node, 'field_fa_cache_mode');
  $admin_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  $cache_size = field_get_items('node', $vm_node, 'field_fa_cache_size');
  $sector_size = field_get_items('node', $vm_node, 'field_fa_sector_size');
  $perf_state = field_get_items('node', $vm_node, 'field_fa_performance_state');
  $metrics_interval = field_get_items('node', $vm_node, 'field_fa_metrics_reset_interval');
  $metrics_warning = ($perf_state[0]['value'] == "ADMIN_ENABLED" && $metrics_interval[0]['value'] == 0) ? " (<font color='red'>WARNING: Reset interval is set to 0 on VM</font>)" : "";
  //$cachespace_id = field_get_items('node', $vm_node, 'field_cachespace');
  $op_message = field_get_items('node', $vm_node, 'field_fa_operational_message');
  //$op_message_value = mercury_management_state2string($op_state[0]['value']);
  //$guest_os = field_get_items('node', $vm_node, 'field_guest_os');
  $output .= '<div class = "cache_info">';
  $output .= '<div class = "cache_info_header">';
  $output .= '<span class = "vm_name">' . $vm_name . '</span>';
  $output .= '<span class = "cache_size">Cache : ' . number_format(($cache_size[0]['value'] * $sector_size[0]['value']) / 1073741824, 2, '.', '') . 'GB</span>';

  $admin_state_display = explode('_', $admin_state[0]['value']);
  $op_state_display = explode('_', $op_state[0]['value']);
  $perf_state_display = explode('_', $perf_state[0]['value']);
  $cache_mode_display = explode('_', $cache_mode[0]['value']);

  $output .= '<span class = "flash_accel_status">' . $vm_status['icon'] . '</span>';
  $output .= '<span class = "flash_accel_message">' . $vm_status['status_message'] . '</span>';
  $output .= '</div>';
  $output .= '<div id="mcs3_container">';
  $output .= '<div class="customScrollBox">';
  $output .= '<div id="node-container">';

  $warning = mercury_management_vm_agent_compatibility_warning($vm_node);
  if (!empty($warning)) {
    $output .= '<div class="inline-messages warning">' . $warning . '</div>';
  }

  $output .= '<div class = "more_link"><span class = "mercury-button">' . l(t('More'), 'node/' . $vm_id) . '</span></div>';
  $output .= '<div class="node-inner-section first">';
  $output .= '<div class="node-inner-header">' . t('Flash Accel Overview') . '</div>';
  $output .= '<div class="node-inner-container">';
  $output .= '<div class="node-inner-left">';
  $output .= '<div class="info-odd"><div class = "vm_overview_details_fields" id = "build_version">VM Agent Version: ' . $build_version[0]['value'] . '</div></div>';
  $output .= '<div class="info-even"><div class = "vm_overview_details_fields" id = "admin_state">Cache State: ' . ucwords(strtolower($admin_state_display[1])) . '</div></div>';
  $output .= '<div class="info-odd"><div class = "vm_overview_details_fields" id = "operational_state">Operational State: ' . ucwords(strtolower($op_state_display[1])) . '</div></div>';
  $output .= '</div>';
  $output .= '<div class="node-inner-right">';
  $output .= '<div class="info-odd"><div class = "vm_overview_details_fields" id = "cache_mode">Cache Mode: ' . ucwords(strtolower($cache_mode_display[1])) . '</div></div>';
  //$output .= '<div class="info-even"><div class = "vm_overview_details_fields" id = "cache_space_id">Cache Space ID: ' . $cachespace_id[0]['value'] . '</div></div>';
  $output .= '<div class="info-odd"><div class = "vm_overview_details_fields" id = "operational_message">Operational Message: ' . $op_message[0]['value'] . '</div></div>';
  $output .= '<div class="info-even"><div class = "vm_overview_details_fields" id = "performance_state">Metrics: ' . ucwords(strtolower($perf_state_display[1])) . $metrics_warning . '</div></div>';
  $output .= '</div>';

  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '<div class="clr"></div>';
  $output .= '</div>';
  $node_info = node_load($vm_id);
  $node_content = node_view($node_info);
  $output .= '<div class="node-inner-section in-popup">';
  $output .= '<div class="node-inner-header">Key Metrics</div>';
  if ($perf_state[0]['value'] == "ADMIN_ENABLED" && $metrics_interval[0]['value'] > 0) {
    $output .= '<div class="node-inner-container" id="vm-metrics-content">';
    $output .= '<div class="view-content">';
    $output .= render($node_content['vm_metrics_entity_view_1']);
    $output .= '</div>';
    $output .= '<div class="clr"></div>';
    $output .= '</div>';
  } else {
    $output .= '<div class="node-inner-container">';
    $output .= '<div class="node-inner-left">';
    $output .= '<div class="view-content"><label>N/A</label></div>';
    $output .= '</div>';
    $output .= '<div class="clr"></div>';
    $output .= '</div>';
  }
  $output .= '</div>';
  $output .= '</div>';
  $output .= '</div>';
  $output .= '</div>';
  $output .= '<div class="dragger_container"><div class="dragger ui-draggable"></div></div>';
  $output .= '</div>';
  $output .= '<a href="#" class="scrollUpBtn"></a> <a href="#" class="scrollDownBtn"></a>';
  $output .= '</div>';
  //$output .= '<script>jQuery("#mcs3_container").mCustomScrollbar("vertical",300,"easeOutCirc",1.05,"auto","yes","yes",15);</script>';

  $output .= '</div>';
  return $output;
}

/*
 * Possible error outputs from rdmDisk.pl:
 * 1. Error: Backing datastore '<datastore name>' is not type VMFS
 * 2. Error: Cannot complete login due to an incorrect user name or password.
 * 3. Error: No host matching '<host name>' found.
 * 4. Error: Cannot find VM '<vm name>'
 * 5. Error: Cannot find datastore '<datastore name>'
 * 6. Error connecting to server at 'https://<vcenter name>/sdk/vimService': Bad hostname
 * 7. Error: Disk '$cachespace' is not assigned to any VM.
 * 8. Error: Cannot find disk ..
 * 9. Error: Could not Query RdmFilter Option
 * 10. Error: Could not set RdmFilter.HbaIsShared Option to false
 * 11. Error: cannot find Datacenter for pool
 */

/*
 * Function that Allocates a cachespace to a vm on Form Submit,
 *
 */

function mercury_management_allocateCacheToVM($vc_hostname, $host_nid, $vm_nid, $vmfs_datastore_name, $num_segments, $vc_username, $vc_password) {

  $vm = node_load($vm_nid);
  $host = node_load($host_nid);
  $vm_cache_identifier_set = false;
  $rdm_created = false;

  if (!empty($vm->field_cachespace['und'][0]['value'])) {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Cache is already allocated on @vm', array('@vm' => $vm->title)), WATCHDOG_ERROR, 'show');
    return "error";
  }

  // Make sure the cache state is disabled before taking any actions.
  // At this point there is no cache so it is safe to disable caching.
  $cache_state = field_get_items('node', $vm, 'field_fa_administration_state');
  if ($cache_state[0]['value'] == "ADMIN_ENABLED") {
    // Both disable and refresh regardless of results.  On failure the user will
    // have to refresh anyway so this saves them the trouble.
    $param = array('request' => array('CacheState' => 'ADMIN_DISABLED'));
    $disable_result = mercury_management_cache_operation('nid', $vm->nid, 'HgSetCacheState', $param);
    $refresh_result = mercury_utilities_vm_refresh_action($vm->nid, 'no-refresh');

    // Stop if either failed.
    if ($disable_result == 'error_wsdl_call') {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Cache is currently enabled. Please disable the cache before allocating the cache space on @vm', array('@vm' => $vm->title)), WATCHDOG_ERROR, 'show');
      return "error";
    }
    if (!$refresh_result) {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Pre-allocate refresh unsuccessful on @vm', array('@vm' => $vm->title)), WATCHDOG_ERROR, 'show');
      return "error";
    }
  }

  // Extract the datastore root name:
  $path = $vm->field_vm_path_name['und'][0]['value'];
  if (preg_match_all('/\[(.*?)\]/', $path, $match)) {
    $path_array = $match[1];
  }
  $datastore = $path_array[0];
  $cachespace_id = " ";

  // --------------- Do the basic VM checks first -----------------------
  // Check and enable disk UUID
  $enableDiskUUID = execute_command("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' --vmname '" . trim($vm->title) . "' --operation enableDiskUUID", $enableDiskUUIDArray);
  if (!empty($enableDiskUUID)) {
    if (stristr($enableDiskUUID, 'Error')) {
      mercury_utilities_fmc_log_message('VM Agent', 'Failed to set VM property disk.enableUUID. Aborted cache allocation.', WATCHDOG_ERROR);
      return "error";
    }
  }

  // Get RdmFilter.HbaIsShared value and make sure it is set correctly
  $result = array();
  $hbaIsShared = exec("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . $host->title . "' --vmname '" . trim($vm->title) . "' --cachespace '" . $cachespace_id . "' --datastore '" . $datastore . "' --operation testAndSetRdmFilterHbaIsShared", $result);
  foreach ($result as $output_line) {
    if (!empty($output_line) && stristr($output_line, 'Error')) {
      return "error";
    }

    if (!empty($output_line) && stristr($output_line, 'true')) {
      exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core adapter list | grep -i flashaccel", $adapterListOutput);

      // Replace all tabs and whitespaces with single spaces
      $adapterListOutput[0] = preg_replace('/[ ]{2,}|[\t]/', ' ', trim($adapterListOutput[0]));

      // Get just the first element 'vmhba'
      $adapterName = explode(' ', $adapterListOutput[0]);
      exec("esxcli -s '" . $vc_hostname . "' --username '" . $vc_username . "' --password '" . $vc_password . "' -h '" . $host->title . "' storage core adapter rescan --adapter " . $adapterName[0] . '', $adapterRescanOutput);
      break;
    }
  }

  // Allocate Cache Space on Device and get the NAA ID on the current host
  $cachespace_id = mercury_management_allocateCache($host->nid, $num_segments);
  if (stristr($cachespace_id, 'ERROR')) {
    return "error";
  }
  mercury_utilities_fmc_log_message('Cache Mgmt', t('Allocated cache on host @host for VM @vm', array('@vm' => $vm->title, '@host' => $host->title)));

  // Keep track of hosts where cache is allocated for rollback on error
  $allocatedHosts = array();
  $allocatedHosts[$host->nid] = $host->nid;

  // Check if the Virtual Machine is vMotionable
  $vm_vmotion_status = $vm->field_vm_vmotionable['und'][0]['value'];
  if ($vm_vmotion_status == 1) {
    $get_all_host = unserialize($vm->field_vm_vmotion_hostlist['und'][0]['value']);
    foreach ($get_all_host as $host_data) {
      $get_host_nid = $host_data;

      // skip current host - allocation has aleady been done
      if ($get_host_nid == $host->nid) {
        continue;
      }
      $get_host_data = node_load($host_data);

      // Allocate Cache Space on Device and input the NAA ID
      $cachespace_id_generated = mercury_management_allocateCache($get_host_nid, $num_segments, $cachespace_id);
      if (stristr($cachespace_id_generated, 'ERROR')) {
        goto cleanup_cache_space_on_error;
      }
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Allocated cache on host @host for VM @vm', array('@vm' => $vm->title, '@host' => $get_host_data->title)));
      $allocatedHosts[$get_host_nid] = $get_host_nid;
    }
  }

  // Add the cachespace as a RDM to the VM
  $result = array();
  $attachCacheSpace = exec_with_stderr("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . trim($host->title) . "' --vmname '" . trim($vm->title) . "' --cachespace '" . trim($cachespace_id) . "' --datastore '" . trim($vmfs_datastore_name) . "' --operation create --adapter lsisas", $result);

  if (empty($attachCacheSpace)) {
    $attachCacheSpace = 'Error: Unknown fault';
  }
  if (stripos($attachCacheSpace, 'Error:') !== false) {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', implode('<br>', $result), WATCHDOG_WARNING, NULL, NULL, 'no_footer');
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Cache allocation failed on VM @vm host @host: @error', array('@vm' => $vm->title, '@host' => $host->title, '@error' => $attachCacheSpace)), WATCHDOG_WARNING, NULL, NULL, 'no_footer');
    goto cleanup_cache_space_on_error;
  }
  $rdm_created = true;

  //If the RDM is successfully Added into the VM
  //Update the VM Node value for Cachespace value
  $vm->field_cachespace['und'][0]['value'] = $cachespace_id;
  node_save($vm);

  //Construct the NAA Identifier for assigning into the VM Cache Space
  $cachespace_id_value = explode('.', trim($cachespace_id));

  //Construct the Set Cache Indentifier Array Object
  $setcache_parameter = array('request' => array('CacheIdentifier' => $cachespace_id_value[1]));

  //Assign the Cache Identifier using Flash Accel WSDL
  $ws_setcache_result = mercury_management_cache_operation('nid', $vm_nid, 'HgSetCacheIdentifier', $setcache_parameter);
  if ($ws_setcache_result == 'error_wsdl_call') {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Error communicating cache space ID to VM @vm.', array('@vm' => $vm->title)), WATCHDOG_ERROR);
    goto cleanup_cache_space_on_error;
  }
  $vm_cache_identifier_set = true;

  // Construct the Format Cache Array Object
  $format_parameter = array('request' => array());

  // Format the Cache using Flash Accel WSDL
  $ws_format_result = mercury_management_cache_operation('nid', $vm_nid, 'HgStartCacheFormat', $format_parameter);
  if ($ws_format_result == 'error_wsdl_call') {
    mercury_utilities_fmc_log_message('Cache Mgmt', t('Failed to format cache following allocation on VM @vm.', array('@vm', $vm->title)), WATCHDOG_ERROR);
    goto cleanup_cache_space_on_error;
  }

  mercury_management_allocate_cache_done:
  return "attached";

  cleanup_cache_space_on_error:

  // TODO: Replace this mess with a call to unallocate?
  // Unallocate will need to handle partial allocations and not make a lot of
  // noise if certain unallocate steps weren't actually needed.

  // Make sure the cache ID gets cleaned on the VM if it was set.
  if ($vm_cache_identifier_set) {
    $setcache_parameter = array('request' => array('CacheIdentifier' => ''));
    $ws_setcache_result = mercury_management_cache_operation('nid', $vm_nid, 'HgSetCacheIdentifier', $setcache_parameter);
    if ($ws_setcache_result == 'error_wsdl_call') {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Error clearing cache space on VM @vm following allocation failure.', array('@vm' => $vm->title)), WATCHDOG_ERROR);
    }
  }

  // Undo rdmdisk.pl --operation create if called.
  if ($rdm_created) {
    $unattachCacheSpace = exec("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . trim($host->title) . "' --vmname '" . trim($vm->title) . "' --cachespace '" . trim($cachespace_id) . "' --datastore '" . trim($vmfs_datastore_name) . "' --operation remove");
    if (empty($unattachCacheSpace)) {
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Failed to remove cache RDM from VM @vm_name following allocation failure. @status', array('@vm_name' => $vm->title, '@status' => $unattachCacheSpace)), WATCHDOG_ERROR);
    }
  }

  foreach ($allocatedHosts as $host_nid) {
    //Free the Cache Sapce
    $free_cachespace_op = mercury_management_freeCache($host_nid, $cachespace_id);
    if (stristr($free_cachespace_op, 'ERROR')) {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Failed to unallocate cachespace "@cache_id" following allocation failure.', array('@cache_id' => $cachespace_id)), WATCHDOG_WARNING);
    }
  }
  return "error";
}

/*
 * Function that Unallocates a cachespace from a vm,
 * Webservice should be turned off
 *
 */

function mercury_management_unallocateCacheFromVM($vc_hostname, $host_nid, $vm_nid, $cachespace_id, $vc_username, $vc_password) {

  $vm = node_load($vm_nid);
  $host = node_load($host_nid);
  //extract the datastore root name:
  $path = $vm->field_vm_path_name['und'][0]['value'];
  if (preg_match_all('/\[(.*?)\]/', $path, $match)) {
    $path_array = $match[1];
  }
  $datastore = $path_array[0];

  // Tell VM to stop using the cache device
  // First change the admin state to disabled if it is not already disabled
  $cache_state = field_get_items('node', $vm, 'field_fa_administration_state');
  if ($cache_state[0]['value'] == "ADMIN_ENABLED") {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Cache is currently enabled. Please disable the cache before unallocating the cache space on @vm', array('@vm' => $vm->title)), WATCHDOG_ERROR, 'show');
    return false;
  }

  //Construct the Set Cache Indentifier Array Object
  $setcache_parameter = array('request' => array('CacheIdentifier' => ''));

  // Clear the Cache Identifier using Flash Accel WSDL
  // Skip and log if it can't possibly succeed (unallocate after uninstall).
  // Try anyway if status is unknown.
  if ($vm->field_driver_status['und'][0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_INSTALLED) {
    mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Skipping VM agent cache unallocate notification for @vm. VM agent is not installed.', array('@vm' => $vm->title)));
  } else {
    $ws_setcache_result = mercury_management_cache_operation('nid', $vm_nid, 'HgSetCacheIdentifier', $setcache_parameter);
    if ($ws_setcache_result == 'error_wsdl_call') {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Failed to clear cache space ID on VM @vm', array('@vm' => $vm->title)), WATCHDOG_ERROR);
    } else {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Cleared the VM @vm cache space ID', array('@vm' => $vm->title)));
    }
  }

  // remove the cachespace as a RDM to the VM
  // Note: if we were unable to notify the VM, we will get an operational state of 'cache was not found'.
  $unattachCacheSpace = exec("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . $host->title . "' --vmname '" . trim($vm->title) . "' --cachespace '" . $cachespace_id . "' --datastore '" . $datastore . "' --operation remove", $result);
  if (empty($unattachCacheSpace)) {
    watchdog('Cache Space Mgmt', t('Failed to unallocate cache on VM "@vm_name"', array('@vm_name' => $vm->title)));
    mercury_utilities_fmc_log_message('Cache Mgmt', t('Failed to unallocate cache from VM @vm_name.', array('@vm_name' => $vm->title)), WATCHDOG_ERROR);
    return false;
  }

  $host_nids = array();
  $hnid = $vm->field_host['und'][0]['target_id'];
  $host_nids[$hnid] = $hnid;

  // Check the vMotionable status of the VM
  $vm_vmotion_status = $vm->field_vm_vmotionable['und'][0]['value'];
  if ($vm_vmotion_status == 1) {
    $hostlist = unserialize($vm->field_vm_vmotion_hostlist['und'][0]['value']);
    if (count($hostlist) > 0) {
      foreach ($hostlist as $nid) {
        if ($nid == $hnid) {
          continue;
        }
        $host_nids[$nid] = $nid;
      }
    }
  }

  // Free the cache on all the hosts we allocated cache on
  // Note: hostlist contains all hosts including current host
  foreach ($host_nids as $host_nid) {
    $host_data = node_load($host_nid);
    //Remove the cachespace for all the Host
    $cachespace_id_cleared = mercury_management_freeCache($host_nid, $cachespace_id);
    if (!empty($cachespace_id_cleared)) {
      // cachespace was not removed
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Failed to unallocate cache on host @host', array('@host' => $host_data->title)), WATCHDOG_ERROR);
    } else {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Freed cache for VM @vm on host @host', array('@vm' => $vm->title, '@host' => $host_data->title)));
    }
  }

  // Even though some of the steps may not have completed, consider this a
  // success as far as the calling function is concerned.
  // Orphan cache spaces and such will be cleaned up later, or it's possible
  // that some of the hosts are gone away forever.
  return true;
}

/**
 * Checks a VM agent for compatibility with this FAMC install.
 * @param stdClass $vm The VM to check
 * @param boolean $check_package_version If true, checks the uploaded agent
 *   package as well.  This is mainly for agent upgrade and install.
 * @return string Warning text if a warning is needed, empty string otherwise
 */
function mercury_management_vm_agent_compatibility_warning(stdClass $vm, $check_package_version = false) {

  // Maximum supported is always the FAMC major/minor.
  // Not supporting future versions, only backward.
  $famc_version = mercury_utilities_get_fmc_version_string();

  // Check package version first if checking it at all.
  // Simplifies the logic and gives an actionable result.
  // Also show no warning if no agents have been uploaded.
  if ($check_package_version) {
    $vm_package_agent_version = variable_get('vm_package_agent_version', '');
    if (
      floatval($vm_package_agent_version) > 0.0 &&
      !mercury_utilities_version_compare($vm_package_agent_version, FLASH_AGENT_MINIMUM_SUPPORTED_VERSION, $famc_version, $major_minor = true, $allow_equal = true)
    ) {
      $warning = t(
        'Uploaded agent package version @version is not supported by FAMC version @famc.',
        array('@version' => $vm_package_agent_version, '@famc' => $famc_version)
      );
      mercury_utilities_fmc_log_message('VM Agent', $warning, WATCHDOG_WARNING);
      return $warning;
    }
  }

  // Pull agent version.
  // If agent version is not known, do not show the warning.
  // Calling code must provide a proper error message.
  if (empty($vm->field_fa_build_version['und'][0]['value']) || floatval($vm->field_fa_build_version['und'][0]['value']) < 1.0) {
    return '';
  }

  // Check the agent against the supported range.
  if (
    !mercury_utilities_version_compare($vm->field_fa_build_version['und'][0]['value'], FLASH_AGENT_MINIMUM_SUPPORTED_VERSION, $famc_version, $major_minor = true, $allow_equal = true)
  ) {
    $warning = t(
      'VM @vm agent version @version is not supported by FAMC version @famc.',
      array('@vm' => $vm->title, '@version' => $vm->field_fa_build_version['und'][0]['value'], '@famc' => $famc_version)
    );
    mercury_utilities_fmc_log_message('VM Agent', $warning, WATCHDOG_WARNING);
    return $warning;
  }

  // Success, return empty string for consistent type.
  return '';
}

/*
 * Function that checks if the host(s) are ready for cache allocation
 */
function mercury_management_vm_can_allocate_on_all_target_hosts($vm_nid) {

  $ready = true;
  $vm = node_load($vm_nid);
  $host_nids = array();
  $hnid = $vm->field_host['und'][0]['target_id'];
  $host_nids[$hnid] = $hnid;

  // Check the vMotionable status of the VM
  $vm_vmotion_status = $vm->field_vm_vmotionable['und'][0]['value'];
  if ($vm_vmotion_status == 1) {
    $hostlist = unserialize($vm->field_vm_vmotion_hostlist['und'][0]['value']);
    if (count($hostlist) > 0) {
      foreach ($hostlist as $nid) {
        if ($hnid != $nid) {
          $host_nids[$nid] = $nid;
        }
      }
    }
  }

  // If there is exactly one host in the target list (the current host),
  // then we don't care whether the customer has vMotion support.
  // More than one host in this context = must have vMotion enabled on all.
  // Non-vMotionable hosts should already have been filtered out by now.
  // @see mercury_management_vmotion_rebuild_target_host_list
  $vmotion = 0;
  if (count($host_nids) > 1) {
    $vmotion = 1;
  }

  // loop over the hosts
  foreach ($host_nids as $nid) {
    $host = node_load($nid);
    $ready = mercury_management_host_can_allocate($vm, $host, $vmotion);
    if ($ready !== true) {
      return false;
    }
  }
  return true;
}

/*
 * Function that Get Available Space on a Host (CIM) Operation 6
 * will occur when a Allocating VM space will return available segments
 */

function mercury_management_availableCache($vm_nid) {

  $vm = node_load($vm_nid);
  $host_nids = array();
  $hnid = $vm->field_host['und'][0]['target_id'];
  $host_nids[$hnid] = $hnid;

  // Check the vMotionable status of the VM
  $vm_vmotion_status = $vm->field_vm_vmotionable['und'][0]['value'];
  if ($vm_vmotion_status == 1) {
    $hostlist = unserialize($vm->field_vm_vmotion_hostlist['und'][0]['value']);
    if (count($hostlist) > 0) {
      foreach ($hostlist as $nid) {
        $host_nids[$nid] = $nid;
      }
    }
  }

  $num_segments = 0;
  foreach ($host_nids as $nid) {

    $host = node_load($nid);

    //get host username password else load the default username/password
    $username = mercury_management_get_host_username($host);
    $password = mercury_management_get_host_password($host);
    if ($username == '' || $password == '') {
      mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Failed to retrieve availiable cache on host @name. No username or password defined.', array('@name' => $host->title)), WATCHDOG_ERROR);
      return 0;
    }

    // get the cache segments available on the host
    $get_host_info2 = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $username . "' '" . $password . "' 7", $result);
    if (!empty($result)) {
      $errorCheck = explode(':', $result[0]);
      // fail if we cannot find out the available cachespace
      if ($errorCheck[0] == "ERROR CODE:") {
        mercury_utilities_fmc_log_message('Cache Space Mgmt', t('Error retrieving availiable cache space on host @name', array('@name' => $host->title, '@msg' => $result[1])), WATCHDOG_ERROR);
        return 0;
      }
      if ($get_host_info2 == 0) {
        return 0;
      }

      // record the smallest available cache space
      if (($get_host_info2 < $num_segments) || $num_segments == 0) {
        $num_segments = $get_host_info2;
      }
    }
  }
  return $num_segments;
}

/**
 * @deprecated Use mercury_management_get_credentials instead.
 */
function mercury_management_get_vm_username(stdClass $node) {
  if (!empty($node->field_vm_username) && $node->field_vm_username['und'][0]['value'] != '') {
    $username = fmc_cipher::decrypt($node->field_vm_username['und'][0]['value']);
  } else {
    $username = fmc_cipher::decrypt(variable_get('vm_username'));
  }
  return $username;
}

/**
 * @deprecated Use mercury_management_get_credentials instead.
 */
function mercury_management_get_vm_password(stdClass $node) {
  if (!empty($node->field_vm_password) && $node->field_vm_password['und'][0]['value'] != '') {
    $password = fmc_cipher::decrypt($node->field_vm_password['und'][0]['value']);
  } else {
    $password = fmc_cipher::decrypt(variable_get('vm_password'));
  }
  return $password;
}


/**
 * @deprecated Use mercury_management_get_credentials instead.
 */
function mercury_management_get_vm_port(stdClass $node) {
  if (!empty($node->field_install_port) && $node->field_install_port['und'][0]['value'] != '') {
    return intval($node->field_install_port['und'][0]['value']);
  }
  return FAMC_WEBSERVICE_DEFAULT_PORT;
}

/**
 * @see mercury_management_get_credentials
 */
function mercury_management_get_vm_credentials(stdClass $node) {
  return array(
    'address' => mercury_management_get_vm_address($node),
    'username' => mercury_management_get_vm_username($node),
    'password' => mercury_management_get_vm_password($node),
    'port' => mercury_management_get_vm_port($node),
  );
}

/**
 * @see mercury_management_get_ancestors
 */
function mercury_management_get_vm_ancestors(stdClass $node, &$info = array()) {
  $info['vm'] = $node;
  if (!empty($node->field_host)) {
    mercury_management_get_host_ancestors(node_load($node->field_host['und'][0]['target_id']), $info);
  }
  return $info;
}

/**
 * Non-recursive parent fetch similar to mercury_management_get_vm_ancestors.
 */
function mercury_management_get_vm_host(stdClass $node) {
  if (!empty($node->field_host['und'][0]['target_id'])) {
    return node_load($node->field_host['und'][0]['target_id']);
  }
  return NULL;
}

/**
 *  Obtain the VM hostname from the VM node data.
 */
function mercury_management_get_vm_hostname($vm_info)
{
  if ($vm_info->field_vm_hostname) {
    $vm_hostname = $vm_info->field_vm_hostname['und'][0]['value'];
  } else {
    $vm_hostname = "";
  }
  return $vm_hostname;
}


/**
 *  Obtain the VM address from the VM node data.
 */
function mercury_management_get_vm_address($vm_info)
{
  if (!empty($vm_info->field_vm_address_custom['und'][0]['value'])) {
    $vm_address = $vm_info->field_vm_address_custom['und'][0]['value'];
  } else {
    if (!empty($vm_info->field_vm_address['und'][0]['value'])) {
      $vm_address = $vm_info->field_vm_address['und'][0]['value'];
    } else {
      $vm_address = "";
    }
  }
  return $vm_address;
}


function mercury_management_get_vm_status(stdClass $node) {
  $result = array();
  $driver_status = field_get_items('node', $node, 'field_driver_status');
  $guest_os = field_get_items('node', $node, 'field_guest_os');
  $op_state = field_get_items('node', $node, 'field_fa_operational_state');
  $op_state_value = $op_state[0]['value'];
  $op_message_value = mercury_management_state2string($op_state_value);
  $guest_os_supported = mercury_management_guest_os_is_supported($node);

  $status_color = 'red';
  $status_message = 'Unknown Error';
  // For the LED display, discovery failures trump all others.
  // Make sure the user is aware of it!
  // In some contexts this is also the only place where the error text is shown.
  if (!empty($node->field_vm_discovery_error['und'][0]['value'])) {
    $status_message = 'Communication error with VM: ' .
      $node->field_vm_discovery_error['und'][0]['value'];
  } elseif ($driver_status[0]['value'] == FAMC_VM_DRIVER_STATUS_INSTALLED) {
    // Installed must override incompatible OS to be more forgiving.
    // TODO: Not a full solution, see BURT 756325. (checkbox override)
    $status_message = $op_message_value;
    if ($op_state_value == "OP_NORMAL") {
      $status_color = "green";
    } else {
      $status_color = "yellow";
      if (empty($node->field_cachespace['und'][0]['value'])) {
        $status_message = 'No Cache Allocated';
      }
    }
  } elseif ($guest_os[0]['value'] == FAMC_VM_GUEST_OS_NOT_KNOWN || $driver_status[0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_KNOWN) {
    // This should no longer be reached anymore because to the discovery error
    // field, but provide a unique error message to make it easier to debug.
    $status_message = 'Unable to Communicate with VM. Check for communication problems and try refreshing the VM.';
  } elseif (!$guest_os_supported || $driver_status[0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_COMPATIBLE) {
    $status_message = 'Guest OS is Not Compatible';
  } elseif ($driver_status[0]['value'] == FAMC_VM_DRIVER_STATUS_NOT_INSTALLED) {
    $status_message = 'Agent Not Installed';
    $status_color = "black";
  }

  $status_image = '<img src="' . base_path() . drupal_get_path('module', 'mercury_utilities') .
    '/images/' . $status_color . '.png" class="' . $status_color .
    '" alt="' . $status_message .'" title="' . $status_message .'"/>';
  $span_str = '<span class = "columns ' . $status_color . '" id = "column-4">' . $status_image . '</span>';

  $result['status_color'] = $status_color;
  $result['status_message'] = $status_message;
  $result['icon'] = $status_image;
  $result['wrapped_icon'] = $span_str;
  return $result;
}

function mercury_management_get_vm_count() {
  return db_query("SELECT count(*) from {node} where `type` = 'virtual_machine'")->fetchField();
}

/**
 * Returns a list of VMFS datastores available to $vm.
 * @param stdClass $vm
 * @return stdClass object containing datastore info or NULL
 */
function mercury_management_vm_get_valid_allocation_datastores(stdClass $vm) {

  $ancestors = mercury_management_get_vm_ancestors($vm);
  $response = famc_external_api_datastore_get_vmfs_datastores($ancestors['datacenter']);

  // This function needs to return a data object, not JSend format.
  if (famc_external_api_response_is_error($response)) {
    mercury_utilities_fmc_log_message(
      'Cache Mgmt',
      t('Could not get a list of datastores for @datacenter: @message',
        array('@datacenter' => $ancestors['datacenter']->title, '@message' => famc_external_api_response_get_error_message($response))
      ),
      WATCHDOG_ERROR
    );
    return NULL;
  }

  $data = famc_external_api_response_get_data($response);
  return mercury_management_vm_get_valid_allocation_datastores_inner($vm, $data);
}

// This is split off so it can be unit tested.
function mercury_management_vm_get_valid_allocation_datastores_inner(stdClass $vm, stdClass $data) {

  $data = clone $data;
  $host = mercury_management_get_vm_host($vm);
  $migration_hosts = mercury_management_vmotion_vm_migration_hosts($vm, $include_current = true);
  $migration_target_total_count = count($migration_hosts);

  // Filter out any datastores that are not accessible to the current host.
  // Non-VMFS was filtered out in perl, and whatever's left to check after this
  // is only for "recommendations" = later function.
  foreach($data as $datastore_name => $row) {
    if (!in_array($host->title, $row->hosts)) {
      unset($data->$datastore_name);
      continue;
    }

    // Now, add in extra data regarding sharing level and migration targets.
    $data->$datastore_name->migration_target_available_count = 0;
    $data->$datastore_name->migration_target_total_count = $migration_target_total_count;
    foreach($migration_hosts as $target) {
      if (in_array($target->title, $row->hosts)) {
        $data->$datastore_name->migration_target_available_count++;
      }
    }
  }

  return $data;
}

/**
 * Given a list of datastores, return a single datastore that is optimal.
 * @param stdClass $available_datastores
 * @return string A single datastore name or NULL.
 */
function mercury_management_vm_recommend_allocation_datastore(stdClass $available_datastores) {
  /*
   * Prefer most-shared even when migration is not enabled,
   * but REQUIRE fully-shared in all migration targets.
   * Limiting migration support is OK if the customer does it, but
   * must require manual intervention = cannot "recommend" it.
   *
   * If the VM is vMotionable, need:
   * - is DS shared on all target hosts? (must be true)
   *    -- Need count of migration hosts + count of ones that are on the DS
   * And then break ties by sorting:
   * - VMFS version (prefer 5 over 3)
   * - most-shared DS (tie-breaker) = count of hosts using DS
   */

  if (empty($available_datastores)) {
    return;
  }

  // Pull out the number of hosts that this VM can exist on.
  $available_datastores = clone($available_datastores);
  $first = reset($available_datastores);
  $migration_target_total_count = $first->migration_target_total_count;

  // If the VM can migrate, first eliminate all datastores that are not
  // available on all migration targets.
  if ($migration_target_total_count > 1) {
    foreach($available_datastores as $datastore_name => $row) {
      if ($row->migration_target_available_count < $migration_target_total_count) {
        unset($available_datastores->$datastore_name);
      }
    }
  }

  // Now every remaining datastore is viable (on all relevant hosts).
  // Sort for preference: VMFS-5 over VMFS-3, then by descending # of hosts.
  // array_multisort requires arrays of columns, so convert into that format.
  $sort_array = (array) $available_datastores;
  $counts = array();
  $vmfs = array();
  foreach($available_datastores as $datastore_name => $row) {
    $counts[$datastore_name] = count($row->hosts);
    $vmfs[$datastore_name] = $row->vmfs_version;
  }

  // VMFS desc, host count desc, result stored into $sort_array.
  array_multisort($vmfs, SORT_DESC, $counts, SORT_DESC, $sort_array);

  // Return the first datastore name in the result.
  reset($sort_array);
  return key($sort_array);
}
