<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that support vMotion features.
 */

/*
 * Function allocates cache on the specified hosts
 * Arguments:
 *  vm_nid = VM node id
 *  hosts = serialized list of hosts
 */
function mercury_management_addCacheHosts($vm_nid, $hostlist) {
  $vm = node_load($vm_nid);
  $hnid = $vm->field_host['und'][0]['target_id'];
  $current_host = node_load($hnid);
  $final_hostlist = array();

  // Get vcenter info
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_nid);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  // return if there are no hosts
  if (count($hostlist) == 0) {
    goto done_with_addition;
  }

  // Get the VM cache size
  if ($vm->field_cachespace['und'][0]['value'] == '') {
    goto done_with_addition;
  }

  $cachesize = ($vm->field_fa_cache_size['und'][0]['value'] * $vm->field_fa_sector_size['und'][0]['value']); // in bytes
  $segmentsize = $current_host->field_segment_size['und'][0]['value']; // in bytes
  $cachespace_id = $vm->field_cachespace['und'][0]['value'];

  // Get the datastore VM is using for the cache device
  $cache_datastore = execute_command("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --server " . $vc_hostname . " --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . $current_host->title . "' --vmname '" . trim($vm->title) . "' --operation getCacheDatastore", $cache_datastore_result);
  if (!empty($cache_datastore) && stristr($cache_datastore, "Error")) {
    watchdog('Cache Mgmt', t('Got error @err !!!', array('@err' => $cache_datastore)));
    mercury_utilities_fmc_set_message(t('Cannot determine datastore used by VM @vm for caching.', array('@vm' => $vm->title)), WATCHDOG_ERROR);
    goto done_with_addition;
  }
  foreach ($cache_datastore_result as $output_line) {
    if (!empty($output_line) && stristr($output_line, "Datastore:")) {
      $datastore_name_output = explode(":", $output_line);
      array_shift($datastore_name_output);
      $datastore_name = trim(implode(":", $datastore_name_output));
      break;
    }
  }
  if (empty($datastore_name)) {
    mercury_utilities_fmc_set_message(t('Cannot determine datastore used by VM @vm for caching.', array('@vm' => $vm->title)), WATCHDOG_ERROR);
    goto done_with_addition;
  }

  $vmotion = 0;
  if (count($hostlist) > 1 || $cachesize > 0) {
    $vmotion = 1;
  }

  // check if all hosts are ready
  foreach ($hostlist as $host_nid) {

    // Current host should already have the cache - so skip it
    if ($host_nid == $hnid) {
      continue;
    }

    $host = node_load($host_nid);

    // Do some basic sanity checks
    $host_result = mercury_management_host_can_allocate($vm, $host, $vmotion);
    if ($host_result !== true) {
      mercury_utilities_fmc_set_message(t('Cannot allocate cache on host @host. @details', array('@host' => $host->title, '@details' => $host_result)));
      continue;
    }

    // Check to see if this host has the currently used datastore
    $check_vmfs_out1 = execute_command("perl /usr/lib/vmware-vcli/apps/host/rdmDisk.pl --url https://" . $vc_hostname . "/sdk/vimService  --username '" . $vc_username . "' --password '" . $vc_password . "' --hostname '" . $host->title . "' --vmname '" . trim($vm->title) . "' --datastore '" . trim($datastore_name) . "' --operation checkForVMFSDatastore", $result1);
    if (!empty($check_vmfs_out1) && stristr($check_vmfs_out1, "Error")) {
      mercury_utilities_fmc_set_message(t('Datastore "@ds" missing. Cannot reserve cache on host @host.', array('@ds' => $datastore_name, '@host' => $host->title)), WATCHDOG_ERROR);
      continue;
    }
    foreach ($result1 as $output_line1) {
      if (!empty($output_line1) && stristr($output_line1, "VMFS Datastore:")) {
        $vmfs_ds_name_output = explode(":", $output_line1);
        array_shift($vmfs_ds_name_output);
        $vmfs_ds_name = trim(implode(":", $vmfs_ds_name_output));
        if ($datastore_name != $vmfs_ds_name) {
          mercury_utilities_fmc_set_message(t('Datastore @ds missing. Cannot reserve cache on host @host.', array('@ds' => $datastore_name, '@host' => $host->title)), WATCHDOG_WARNING);
          continue 2;
        }
      }
    }

    // Check if host has adequate cache space
    // get the cache segments available on the host
    $get_host_info2 = exec("/usr/lib/vmware-vcli/apps/host/host_control.py '" . trim($host->title) . "' '" . $username . "' '" . $password . "' 6", $result);
    if (!empty($result)) {
      $errorCheck = explode(':', $result[0]);
      // fail if we cannot find out the available cachespace
      if ($errorCheck[0] == "ERROR CODE:") {
        mercury_utilities_fmc_set_message(t('Error retrieving availiable cache space on host @name', array('@name' => $host->title, '@msg' => $result[1])), WATCHDOG_ERROR);
        continue;
      }
      $num_segments = $get_host_info2;
      $avail_cachespace = $num_segments * $segmentsize;
      if ($cachesize > $avail_cachespace) {
        mercury_utilities_fmc_set_message(t('Host @host does not have adequate cache space for reservation.', array('@host' => $host->title)));
        continue;
      } else {
        mercury_utilities_fmc_set_message(t('Failed to obtain available cache space on host @host.', array('@host' => $host->title)));
        continue;
      }
    }

    // All hosts in the list are compatible, have the matching datastore and adequate cache space
    // Go ahead and start allocating the cache
    $num_segments = $cachesize / $segmentsize;
    $cachespace_id_generated = mercury_management_allocateCache($host_nid, $num_segments, $cachespace_id);
    if (stristr($cachespace_id_generated, 'ERROR')) {
      mercury_utilities_fmc_set_message(t('Failed to allocate cache on host @host for VM @vm', array('@vm' => $vm->title, '@host' => $host->title)), 'error');
    }
    mercury_utilities_fmc_set_message(t('Allocated cache on host @host for VM @vm', array('@vm' => $vm->title, '@host' => $host->title)), 'info');
    $final_hostlist[] = $host_nid;
  }

  done_with_addition:
  // Return the list of hosts on this cache was allocated
  return $final_hostlist;
}

/*
 * Function unallocates cache on the specified hosts
 * Arguments:
 *  vm_nid = VM node id
 *  hosts = serialized list of hosts
 */

function mercury_management_delCacheHosts($vm_nid, $hostlist) {
  $vm = node_load($vm_nid);
  $hnid = $vm->field_host['und'][0]['target_id'];
  $current_host = node_load($hnid);

  // return if there are no hosts
  if (count($hostlist) == 0) {
    return;
  }

  // Check to see if cache is allocated to the VM
  $cachespace_id = $vm->field_cachespace['und'][0]['value'];
  if ($cachespace_id == '') {
    return;
  }

  // check if all hosts are ready
  $final_hostlist = array();
  foreach ($hostlist as $host_nid) {

    $host = node_load($host_nid);

    //Remove the cachespace for all the Host
    $cachespace_id_cleared = mercury_management_freeCache($host_nid, $cachespace_id);
    if (!empty($cachespace_id_cleared)) {
      //cachespace was not removed
      mercury_utilities_fmc_set_message(t('Failed to unallocate cache space on host @host for VM @vm', array('@vm' => $vm->title, '@host' => $host->title)), WATCHDOG_ERROR);
      continue;
    }
    mercury_utilities_fmc_set_message(t('Unallocated cache on host @host for VM @vm', array('@vm' => $vm->title, '@host' => $host->title)), 'info');
    $final_hostlist[] = $host_nid;
  }

  // Return the list of hosts on this cache was allocated
  return $final_hostlist;
}

/**
 * Update the list of target hosts based on the migration policy.
 * This removes hosts that are not ready for caching.
 * Data returned is "false" if skipped rebuild, true if rebuilt,
 * and potentially also errors and warnings.
 * @param stdClass $vm The VM to update. May be modified by reference.
 * @return stdClass response object data = [true, false], errors, warnings
 */
function mercury_management_vmotion_rebuild_target_host_list(stdClass $vm) {

  $credentials = mercury_management_get_credentials($vm, $include_ancestors = true);
  $current_host = $credentials['host']['node'];
  $warnings = array();
  $global_default_migration_enabled = variable_get('enable_migration', true);

  // Grab the policy, status, and current target list from the VM.
  $vm_hostlist = array();
  if (!empty($vm->field_vm_vmotion_hostlist['und'][0]['value'])) {
    $vm_hostlist = unserialize($vm->field_vm_vmotion_hostlist['und'][0]['value']);
  }
  if (!is_array($vm_hostlist)) {
    // Something went terribly wrong.
    return famc_external_api_format_as_error(t('Unable to unserialize host list. Check VM advanced settings.'));
  }
  // Policy is a little special.  Empty just indicates that this VM has never
  // been processed, and we need to retain that for later.
  $vm_policy = '';
  if (!empty($vm->field_vm_vmotion_policy['und'][0]['value'])) {
    $vm_policy = $vm->field_vm_vmotion_policy['und'][0]['value'];
  }
  $vm_vmotionable = 0;
  if (!empty($vm->field_vm_vmotionable['und'][0]['value'])) {
    $vm_vmotionable = intval($vm->field_vm_vmotionable['und'][0]['value']);
  }

  // If the VM has never been processed before, pull in the defaults.
  // The checkbox or host list being empty is not as meaningful in this case.
  if (empty($vm_policy)) {
    // If migration is globally disabled, need to fill in disabled.
    // Also fill in the other two migration fields to ensure compatibility.
    // Some other functions assume that it's populated correctly.
    // Use "default" so the system has a chance to auto-populate if the checkbox
    // is enabled at a later date.  Also indicates that it wasn't user-set.
    if (!$global_default_migration_enabled) {
      $vm->field_vm_vmotionable['und'][0]['value'] = 0;
      $vm->field_vm_vmotion_hostlist['und'][0]['value'] = serialize(array($current_host->nid));
      $vm->field_vm_vmotion_policy['und'][0]['value'] = "default";
      node_save($vm);

      // We still made node changes, so return true.
      return famc_external_api_format_as_success(true);
    }

    // Global default is enabled.
    // Pull in other defaults and apply to local scope.
    $vm_vmotionable = 1;

    // This will be used below to find the effective policy.
    $vm_policy = 'default';
  }

  // This should never happen, but will make any bugs obvious.
  if (empty($vm_policy) || !in_array($vm_policy, array('default', 'custom', 'cluster', 'datacenter'))) {
    return famc_external_api_format_as_error(
      t("VM policy still invalid after processing global default! '@policy'", array('@policy' => $vm_policy))
    );
  }

  // If the policy is "custom" then we cannot regenerate it.
  // TODO: In US1347 we also need to handle per-vm settings of "cluster"
  // and "datacenter" and refresh the host list based on that rule.
  // Right now that only leaves "custom" = do not change and
  // "default" = update based on the global settings.
  if ($vm_policy != 'default') { // TODO: implies "custom" for now, check only for "custom" in the future.
    // Need to check for invalid custom results.
    // TODO: move these checks to mercury_management_vm_can_allocate_on_all_target_hosts?
    // Not going to move them for now because error messages do not bubble up
    // to the form very well in that function.
    if (empty($vm_hostlist)) {
      return famc_external_api_format_as_error(
        t('No hosts have been selected for cache space reservation. Please select one or more hosts using the Advanced->Migration option in the wrench drop-down menu for the VM.')
      );
    }
    // Also invalid when the current host is not in the list.
    if (!in_array($current_host->nid, $vm_hostlist)) {
      return famc_external_api_format_as_error(
        t("The VM's current host is not in the list of selected migration targets. Please update the host list using the Advanced->Migration option in the wrench drop-down menu for the VM.")
      );
    }

    // No changes made.
    return famc_external_api_format_as_success(false);
  }

  // Now we've eliminated all other possibilities and we have to actually
  // refresh the target host list and potentially checkbox state.

  // Determine what policy we are going to apply, which might not be the same
  // as what we are saving to the node ($vm_policy).
  // If the policy is "default" that means use the global default.
  $effective_policy = $vm_policy;
  if ($vm_policy == 'default') { // Always true until US1347.
    // Default + globally disabled = VM is default + disabled.
    // Sync up the VM if needed.  Host list doesn't really matter.
    if (!$global_default_migration_enabled) {
      $vm_vmotionable = 0;
    } else {
      // Be sure to update the VM flag to match the global default.
      $vm_vmotionable = 1;

      // Apply global policy to VM.
      switch (variable_get('migration_policy', false)) {
        case 2:
          $effective_policy = 'cluster';
          break;
        case 3:
          $effective_policy = 'datacenter';
          break;
        case 4: // custom
          // Must determine host list automatically + cannot = fail.
          return famc_external_api_format_as_error(
            t('No hosts have been selected for cache space reservation. Please select one or more hosts using the Advanced->Migration option in the wrench drop-down menu for the VM.')
          );
        default: // bug
          return famc_external_api_format_as_error(
            t('Invalid migration policy defined.')
          );
      }
    }
  }

  // Vmotionable is not so much a property of the VM in vSphere, but rather
  // a property of the hosts.  In the VM node it's a actually just a way for
  // the user to disable cross-host allocation even though it's supported.
  // This MUST happen after the empty policy / global default logic!
  // Otherwise $vm_vmotionable may be empty or out of date.
  // TODO: In US1347 we may need to make vm_vmotionable into a triple:
  // [per-vm enabled, per-vm disabled, use global default]
  // For now default policy = keep in sync with global checkbox.
  if (!$vm_vmotionable) {
    // Make sure the VM checkbox state is correct.
    if (!empty($vm->field_vm_vmotionable['und'][0]['value'])) {
      // Not correct, update everything.
      // Mostly applies to blank fields.
      $vm->field_vm_vmotionable['und'][0]['value'] = 0;
      $vm->field_vm_vmotion_hostlist['und'][0]['value'] = serialize(array($current_host->nid));
      $vm->field_vm_vmotion_policy['und'][0]['value'] = $vm_policy;
      node_save($vm);
      return famc_external_api_format_as_success(true);
    }

    // All other migration-related code should ignore the host list, etc. when
    // the checkbox is unchecked.  So we don't need to update anything else.
    return famc_external_api_format_as_success(false);
  }

  // Pull a list of host names in the cluster or datacenter from vCenter.
  $host_names = mercury_management_allocationHostList($vm->nid, $effective_policy, $credentials['vcenter']['address'], $credentials['vcenter']['username'], $credentials['vcenter']['password']);

  // Ensure that the current host ends up in the list.
  // It seems like this was originally added as a workaround and is no longer
  // needed, but it doesn't hurt so leaving it in.
  if (!in_array($current_host->title, $host_names)) {
    $host_names[] = $current_host->title;
  }

  // Loop through the host targets, filter out invalid options.
  $hostlist_nids = array();
  foreach ($host_names as $hostname) {
    $hostname = trim($hostname);
    if (empty($hostname)) {
      continue; // Result may have blank lines.
    }

    // The external function returns them by name.  Load the node.
    $host_nid = db_query("SELECT nid FROM {node} WHERE type = :tp AND title = :ttl", array(':tp' => 'host', ':ttl' => $hostname))->fetchField();
    $host = node_load($host_nid);
    $host_result = true;
    if (!$host) {
      $host_result = t('Could not load data for host @host. Please rediscover and try again.', array('@host' => $hostname));
      mercury_utilities_fmc_log_message('Cache Space Mgmt', $host_result, WATCHDOG_ERROR);
    } else {
      // For non-current hosts, consider vMotion-enabled a requirement.
      // We will deal with the current host after the loop.
      $vmotion_required = false;
      if ($host->nid != $current_host->nid) {
        $vmotion_required = true;
      }

      // Check to see if we can allocate on this host.
      // TODO: In US1347 we will also need to check if there is an allocation
      // already, instead of calling can_allocate.  Otherwise we won't be able to
      // use this in a cron, etc.
      $host_result = mercury_management_host_can_allocate($vm, $host, $vmotion_required);
    }

    // Boolean true = OK, move on.
    if ($host_result === true) {
      $hostlist_nids[] = $host_nid;
      continue;
    }

    // $host_result contains an actual error message.
    // Current host failed = error.
    $full_error = t('Cannot allocate cache on @host. @reason', array('@host' => $host->title, '@reason' => $host_result));
    if ($host_nid == $current_host->nid) {
      // Escalate log entry here as an error.
      mercury_utilities_fmc_log_message('Cache Space Mgmt', $full_error, WATCHDOG_ERROR);
      return famc_external_api_format_as_error($full_error);
    }
    // Any other host, skip and return a warning.
    // We don't need to re-log here, it has already been logged.
    $warnings[] = $full_error;
  }

  // If there is only one host left (will be current host), whether or not the
  // vMotion feature is supported on the host is irrelevant.
  // But if we have multiple target hosts and cannot use the rest because
  // it's not possible to migrate off of the current host, need to
  // warn the user and remove the unusable hosts.
  $current_host_vmotion = 0;
  if (!empty($current_host->field_vmotion['und'][0]['value'])) {
    $current_host_vmotion = intval($current_host->field_vmotion['und'][0]['value']);
  }
  if (count($hostlist_nids) > 1 && $current_host_vmotion < 1) {
    $full_error = t('The current host @host does not support vMotion, so cache space will only be allocated on the current host.', array('@host' => $current_host->title));
    $warnings[] = $full_error;
    mercury_utilities_fmc_log_message('Cache Space Mgmt', $full_error, WATCHDOG_WARNING);
    $hostlist_nids = array($current_host->nid); // Only this can be used.
  }

  // Cannot reach this point without the checkbox being true (default or VM).
  // Make sure the database reflects that.
  $vm->field_vm_vmotionable['und'][0]['value'] = 1;

  // If we reach this point, at least the current host is in the list.
  // Update the node with the new targets.
  $vm->field_vm_vmotion_hostlist['und'][0]['value'] = serialize($hostlist_nids);

  // Right now this is always 'default' but in US1347 it could also be
  // "cluster" or "datacenter" at this point.
  $vm->field_vm_vmotion_policy['und'][0]['value'] = $vm_policy;

  // Update the node in the database.
  node_save($vm);

  // Prepare the response and return.
  $response = famc_external_api_format_as_success(true);
  if (!empty($warnings)) {
    $response->warnings = $warnings;
  }
  return $response;
}

/**
 * Returns an array of Hosts that a given VM can migrate to.
 * After allocation this is the same as the hosts that actually
 * have allocations but before that it may still be set.
 * So you may need to check field_cachespace depending on context!
 * XXX: It may even be better to not populate the host list until allocation
 * happens, which removes the ambiguity.  May require a new node field because
 * of the "custom" host target list must be set before allocate.
 * @param stdClass $vm
 * @param boolean $include_current If false, the current Host is excluded.
 * @return array of stdClass Host nodes
 */
function mercury_management_vmotion_vm_migration_hosts(stdClass $vm, $include_current = true) {

  // The nid of the host that currently contains the VM.
  $hnid = $vm->field_host['und'][0]['target_id'];

  // If asking to include the current Host, do that even if not vMotionable.
  $host_nids = array();
  if ($include_current) {
    $host_nids[$hnid] = $hnid;
  }

  // Ignore any other Hosts if not vMotionable.
  if (!empty( $vm->field_vm_vmotionable['und'][0]['value'])) {
    $hostlist = unserialize($vm->field_vm_vmotion_hostlist['und'][0]['value']);
    if (count($hostlist) > 0) {
      foreach ($hostlist as $nid) {
        if ($nid == $hnid) {
          continue;
        }
        $host_nids[$nid] = $nid;
      }
    }
  }

  if (empty($host_nids)) {
    return array();
  }

  // Load the nodes, and make sure that vMotion is actually enabled on each
  // host. If not enabled then it ended up in the VM's hostlist by mistake.
  $nodes = node_load_multiple($host_nids);
  foreach($nodes as $key => $host_node) {
    if (empty($host_node->field_vmotion['und'][0]['value'])) {
      unset($nodes[$key]);

      // Something likely went wrong here, so also log.
      $debug = t('Host @host did not have vMotion enabled, ignoring in migration target list for @vm', array('@vm' => $vm->title, '@host' => $host_node->title));
      mercury_utilities_fmc_log_message('Cache Mgmt', $debug, WATCHDOG_DEBUG, NULL, NULL, $skip_footer_log = true);
    }
  }

  return $nodes;
}
