<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions that wrap the web service.
 * Note that mm_wsdl.php is not included on all requests, and therefore
 * these functions need to be in a separate file.
 */

function mercury_management_webservices() {
  $output = '';
  $output .= '<div id = "webservices">';
  $output .= '<div class = "websdervices_list">';
  $output .= '<div class = "cache_info">' . l('HgGetCacheInfo', 'webservices/HgGetCacheInfo/') . '</div>';
  $output .= '<div class = "set_cache_info">' . l('HgSetCacheState', 'webservices/HgSetCacheState/') . '</div>';
  $output .= '</div>';
  $output .= '</div>';
  return $output;
}

/**
 * Function issue web service operation on the VM.
 * Note that this is not parallelized.
 * @see wsdl_cache_operation
 */
function mercury_management_cache_operation($type, $vm_id, $method, $param = array(), $identifier = "") {
  require_once('mercury_management.inc');

  $hgGetCacheInfo = array();
  $hgGetCacheOptions = array();
  $vmInfo = node_load($vm_id);

  if (empty($vmInfo->field_vm_address['und'][0]['value'])) {
    mercury_utilities_fmc_log_message('Cache Mgmt', t('Cannot obtain hostname/address for VM @vm in method @method', array('@vm' => $vmInfo->title, '@method' => $method)), WATCHDOG_ERROR);
    return 'error_wsdl_call';
  }
  $hostname = $vmInfo->field_vm_address['und'][0]['value'];

  $port = '7010';
  if (!empty($vmInfo->field_install_port['und'][0]['value'])) {
    $port = $vmInfo->field_install_port['und'][0]['value'];
  }

  //get host username password else load the default username/password
  $username = mercury_management_get_vm_username($vmInfo);
  $password = mercury_management_get_vm_password($vmInfo);

  // Construct the SOAP client request
  $client = mercury_management_wsdl_construct($hostname, $username, $password, $port, $return_error = true);
  if (!is_object($client)) {
    $message = 'Uknown Error';
    if (!empty($client)) {
      // $client may contain details.
      $message = $client;
    }
    mercury_utilities_fmc_log_message('Cache Mgmt',
      t('Error communicating with VM @vmname operation "@method" on port @port: @message',
        array('@vmname' => $vmInfo->title, '@method' => $method, '@port' => $port, '@message' => $message)),
      WATCHDOG_ERROR
    );
    return 'error_wsdl_call';
  }

  // Send the request to the VM
  $result = mercury_management_consume_webservice($client, $param, $method);
  if (array_key_exists('error', $result)) {
    mercury_utilities_fmc_log_message('Cache Mgmt',
      t('Cache operation "@method" on VM @vmname returned an error: @result',
        array('@vmname' => $vmInfo->title, '@method' => $method, '@result' => $result['message'])),
      WATCHDOG_ERROR
    );
    return 'error_wsdl_call';
  } else {
    // Not an error
    if ($method == "HgGetCacheInfo") {
      $diskCount = 0;
      foreach ($result['response']->HgGetCacheInfoResult->DiskCacheInfoList->DiskCacheInfo as $key => $val) {
        $hgGetCacheInfo[$diskCount]['cache_mode'] = $val->CacheMode;
        $hgGetCacheInfo[$diskCount]['admin_state'] = $val->AdminState;
        $hgGetCacheInfo[$diskCount]['op_state'] = $val->OperationalState;
        $hgGetCacheInfo[$diskCount]['op_message'] = $val->OpStateMsg;
        $hgGetCacheInfo[$diskCount]['perf_state'] = $val->PerfState;
        $hgGetCacheInfo[$diskCount]['vdisk_identifier'] = $val->Identifier;
        $hgGetCacheInfo[$diskCount]['path'] = $val->DriveLetters;
        $diskCount++;
      }
      $hgGetCacheInfo['disk_count'] = $diskCount;
      $hgGetCacheInfo['build_version'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->BuildVersion;
      $hgGetCacheInfo['cache_mode'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->CacheMode;
      $hgGetCacheInfo['admin_state'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->AdminState;
      $hgGetCacheInfo['cache_size'] = doubleval($result['response']->HgGetCacheInfoResult->GlobalCacheInfo->Sectors);
      $hgGetCacheInfo['sector_size'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->SectorSize;
      $hgGetCacheInfo['op_state'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->OperationalState;
      $hgGetCacheInfo['op_message'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->OpStateMsg;
      $hgGetCacheInfo['perf_state'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->PerfState;
      $hgGetCacheInfo['cachespace_id'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->Identifier;
      $hgGetCacheInfo['usable_blocks'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->FormatInfo->UsableBlocks;
      return $hgGetCacheInfo;
    } elseif ($method == "HgGetCacheOptions") {
      $hgGetCacheOptions['metrics_reset_interval'] = $result['response']->HgGetCacheOptionsResult->HgCacheOptions->MetricsResetInterval;
      return $hgGetCacheOptions;
    }
  }
  return $result;
}
