<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 * 
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *  
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *  
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *  
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 *  Create a worker process at the end of a pipe (vcenterdiscoverd.pl instance).
 */
function mercury_management_create_one_worker($params)
{
   $vc = $params['vc'];
   $user = $params['user'];
   $pswd = $params['pswd'];
   $err_file = $params['stderr_file'];
   $log_fh = $params['log_fh'];
   $log_tag = $params['log_tag'];
   
   $descspec = array(
      0 => array("pipe", "r"), # stdin is a pipe that the child will read from
      1 => array("pipe", "w"), # stdout is a pipe that the child will write to
      2 => array("file", $err_file, "a") # stderr is a file to write to
   );

   // XXX: This will probably be a pain when adding Hyper-V support.
   // Needs to be something more generic depending on the installed module.
   $cmd  = "/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscoverd.pl";
   $cmd .= " --url https://$vc/sdk/vimService";
   $cmd .= " --username '$user'";
   $cmd .= " --password '$pswd'";
 
   $process = proc_open($cmd, $descspec, $pipes, NULL, NULL);

   if (is_resource($process)) {

      # $pipes now looks like this:
      # 0 => writeable handle connected to child stdin
      # 1 => readable handle connected to child stdout
      # Any error output will be appended to $err_file

      stream_set_blocking($pipes[0], 0);
      stream_set_write_buffer($pipes[0], 0);
      stream_set_blocking($pipes[1], 0);
   
      $res = array(
         'wrkr_stdin'   => $pipes[0],
         'wrkr_stdout'  => $pipes[1],
         'wrkr_process' => $process,
         'wrkr_is_busy' => false,
         'wrkr_log_fh'  => $log_fh,
         'wrkr_log_tag' => $log_tag
      );

      # Now ping the worker to make sure it didn't die trying to connect to
      # vCenter Server.  We'll give it 30 seconds to get set up.
      if (mercury_management_ping_worker($res, 30)) {
         mercury_management_wrkr_log($res, "worker created\n");
      } else {
         mercury_management_destroy_one_worker($res);
         $res = false;
      }
   } else {
      $res = false;
   }

   return $res;
}


/**
 *  Check worker responsiveness
 */
function mercury_management_ping_worker(&$wrkr, $timeout)
{
   # Send the worker a no-op and make sure we get a
   # response; it may have died due to an inability to
   # connect to vCenter Server.
   fwrite($wrkr['wrkr_stdin'], "no_op\n");

   while (true) {

      # Set up for select
      $rd_arr = array($wrkr['wrkr_stdout']);
      $wr_arr = NULL;
      $ex_arr = NULL;
      $chgd = stream_select($rd_arr, $wr_arr, $ex_arr, $timeout);

      if (false === $chgd) {

         # Should never get here
         mercury_management_wrkr_log($wrkr, "stream_select failed!\n");
         $wrkr['wrkr_is_busy'] = true;
         return false;
      }
      elseif ($chgd > 0) {

         # Read returned
         $str = stream_get_contents($wrkr['wrkr_stdout']);
         if ($str != "") {

            # Check for known output terminator character
            if (stripos($str, "\x07") !== false) {
               return true; # This worker is healthy
            }

            # Wait for more output

         } else {

            # Empty read; worker has died
            mercury_management_wrkr_log($wrkr, "worker has died!\n");
            $wrkr['wrkr_is_busy'] = false;
            return false;
         }
      }
      else {

         # stream_select timed out; worker is hung
         mercury_management_wrkr_log($wrkr, "stream_select timeout!\n");
         $wrkr['wrkr_is_busy'] = true;
         return false;
      }
   }
}


/**
 *  Destroy a worker process.
 */
function mercury_management_destroy_one_worker(&$wrkr)
{
   # It is important to close any pipes before calling proc_close in order 
   # to avoid a deadlock.
   fclose($wrkr['wrkr_stdin']);
   fclose($wrkr['wrkr_stdout']);

   # If the worker is busy, assume it's hung, and kill it so we don't block.
   if (true == $wrkr['wrkr_is_busy']) {
      $status = proc_get_status($wrkr['wrkr_process']);
      exec('kill '.$status['pid'].' 2>/dev/null >&- >/dev/null');
      mercury_management_wrkr_log($wrkr, "worker killed\n");
      proc_close($wrkr['wrkr_process']);
   } else {
      $return_value = proc_close($wrkr['wrkr_process']);
      mercury_management_wrkr_log($wrkr, "worker destroyed\n");
   }
}


/**
 *  Open a log file for this module (truncating the previous one if it exists).
 */
function mercury_management_wa_open_log_fh()
{
   $file = mercury_management_get_log_dir() . '/mm_worker_pool_log.txt';
   $fh = fopen($file, "w");
   chmod($file, 0777);
   return $fh;
}


/**
 *  Close a log file.
 */
function mercury_management_wa_close_log_fh($log_fh)
{
   fclose($log_fh);
}


/**
 *  Write a string to a log file.
 */
function mercury_management_wa_log_to_fh($fh, $str)
{
   if ($fh != null) {
      fwrite($fh, $str);
      fflush($fh);
   }
}


/**
 *  Write a string to the log file associated with a worker array.
 */
function mercury_management_wa_log(&$wrkr_array, $str)
{
   mercury_management_wa_log_to_fh($wrkr_array['log_fh'], $str);
}

/**
 *  Write a string to the log file associated with a worker.
 */
function mercury_management_wrkr_log(&$wrkr, $str)
{
   $tag = $wrkr['wrkr_log_tag'];
   if ($tag == null) {
      $tag = "";
   }

   // These strings frequently contain passwords.
   $str = mercury_management_sanitize_command_for_log($str);

   mercury_management_wa_log_to_fh($wrkr['wrkr_log_fh'], "$tag: $str");
}


/**
 *  Convert a worker index into a worker log file "tag".
 */
function mercury_management_idx_to_log_tag($idx)
{
   return sprintf("w%02d", $idx);
}


/**
 *  Create an array of worker processes (vcenterdiscoverd.pl instances).
 */
function mercury_management_create_worker_array($num, $vc, $user, $pswd, $log_fh = null)
{
   $stderr_file = mercury_management_get_log_dir() . '/mm_worker_pool_stderr.txt';
   $stderr_hdl = fopen($stderr_file, "w");
   chmod($stderr_file, 0777);
   fclose($stderr_hdl);
   
   if (!isset($log_fh)) {
      $log_fh = mercury_management_wa_open_log_fh();
      $wrkr_array['my_log_fh'] = true;
   } else {
      $wrkr_array['my_log_fh'] = false;
   }

   $wrkr_array['vc'] = $vc;
   $wrkr_array['user'] = $user;
   $wrkr_array['pswd'] = $pswd;
   $wrkr_array['stderr_file'] = $stderr_file;
   $wrkr_array['log_fh'] = $log_fh;
   mercury_management_wa_log($wrkr_array, "create_worker_array($num, $vc, $user, '******')\n");
   $wrkr_array['wrkr_array'] = array();
   
   for ($i = 0; $i < $num; $i++) {
      $wrkr_array['log_tag'] = mercury_management_idx_to_log_tag($i);
      $wrkr = mercury_management_create_one_worker($wrkr_array);
      if ($wrkr === false) {
         # We may have attempted to set up too many connections to vCenter.
         # If some workers were created successfully, stop now and call it
         # a success.
         if ($i) {
            mercury_management_wa_log($wrkr_array, "stopping at $i workers due to error ($num requested)\n");
            break;
         }
         mercury_management_wa_log($wrkr_array, "could not create any workers! bad creds or vcenter down?\n");
         return false;
      } else {
         $wrkr_array['wrkr_array'][] = $wrkr;
      }
   }
   unset($wrkr_array['log_tag']);
   
   return $wrkr_array;
}


/**
 *  Destroy an array of worker processes.
 */
function mercury_management_destroy_worker_array(&$wrkr_array, $log_fh = null) 
{
   if ($wrkr_array === false) {
      return;
   }
   foreach ($wrkr_array['wrkr_array'] as &$wrkr) {
      mercury_management_destroy_one_worker($wrkr);
   }
   if ($wrkr_array['my_log_fh'] == true) {
      mercury_management_wa_close_log_fh($wrkr_array['log_fh']);
   }
}


/**
 *  Get a worker started on a unit of work (UoW).
 */
function mercury_management_start_worker_on_uow(&$wrkr, &$uow)
{
   mercury_management_wrkr_log($wrkr, "starting new uow\n");
   $wrkr['wrkr_is_busy'] = true;
   $wrkr['wrkr_uow'] = &$uow;
   $wrkr['wrkr_uow']['results'] = array();
   $wrkr['wrkr_uow']['curr_idx'] = -1;
   mercury_managment_invoke_uow_starting_func($wrkr);
   return mercury_management_give_worker_next_uow_item($wrkr);
}


/**
 *  Log multiple lines, prepending the given prefix to each line
 */
function mercury_management_wrkr_log_multiple(&$wrkr, $lines, $pfx)
{
   $cls = explode("\n", rtrim($lines));
   foreach ($cls as $line) {         
      mercury_management_wrkr_log($wrkr, "$pfx$line\n");
   }
}


/** 
 *  Call optional "starting" callback on uow if it exists
 */
function mercury_managment_invoke_uow_starting_func(&$wrkr)
{
   if (isset($wrkr['wrkr_uow']['starting_cbk'])) {
      $cbk = $wrkr['wrkr_uow']['starting_cbk'];
      call_user_func_array($cbk, array(&$wrkr['wrkr_uow']));
   }
}


/** 
 *  Call optional "finished" callback on uow if it exists
 */
function mercury_managment_invoke_uow_finished_func(&$wrkr)
{
   if (isset($wrkr['wrkr_uow']['finished_cbk'])) {
      $cbk = $wrkr['wrkr_uow']['finished_cbk'];
      call_user_func_array($cbk, array(&$wrkr['wrkr_uow']));
   }
}

/**
 * Retries the last worker command after a delay.
 * Default: Every 5 seconds for about 30 seconds (plus time for commands).
 * @param array $uow
 * @param string $retry_type Identifier for tracking the retry count
 * @param int $max_retries number of times to retry
 * @param int $retry_interval sleep seconds between tries
 * @return boolean true if retry scheduled, false if not eligible
 */
function mercury_management_uow_retry(&$uow, $retry_type, $max_retries = 6, $retry_interval = 5) {
  $idx = $uow['curr_idx'];
  $max_retries = intval($max_retries);
  $retry_interval = intval($retry_interval);

  if (empty($uow['retries'])) {
    $uow['retries'] = array();
  }
  if (empty($uow['retries'][$retry_type])) {
    $uow['retries'][$retry_type] = 0;
  }

  if ($max_retries <= 0 || $uow['retries'][$retry_type] >= $max_retries) {
    return false;
  }
  $uow['retries'][$retry_type]++;

  // Add a sleep to the queue.
  // This needs to happen in a different thread so the other workers can
  // continue.  Short of rewriting the worker thread system to use pthreads,
  // the simplest way to handle this is to use /bin/sleep in the queue.
  // Then just echo something so we can easily tell it finished.
  $uow['cmds'][$idx + 1] =
    "exec\n/bin/sleep $retry_interval; " .
    "echo sleep-done-{$uow['retries'][$retry_type]}-of-$max_retries\n";

  // No sleep callback needed.  This will continue on its way.
  // @see mercury_management_give_worker_next_uow_item()

  // Add the previous job back onto the queue.
  $uow['cmds'][$idx + 2] = $uow['cmds'][$idx];
  $uow['cbks'][$idx + 2] = $uow['cbks'][$idx];
  return true;
}

/**
 *  Advance to the next work item in a unit of work.
 */
function mercury_management_give_worker_next_uow_item(&$wrkr)
{
   for (;;) {
      $idx = $wrkr['wrkr_uow']['curr_idx'];
      if ($idx >= 0) {
         $cmd = $wrkr['wrkr_uow']['cmds'][$idx];
         mercury_management_wrkr_log_multiple($wrkr, $cmd, '--> ');
         if (isset($wrkr['wrkr_uow']['results'][$idx])) {
            $res = implode("\n", $wrkr['wrkr_uow']['results'][$idx]);
            mercury_management_wrkr_log_multiple($wrkr, $res, '<-- ');
         } else {
            mercury_management_wrkr_log($wrkr, "<-- (no results)\n");
         }
         if (isset($wrkr['wrkr_uow']['cbks'][$idx])) {
            $cbk = $wrkr['wrkr_uow']['cbks'][$idx];
            mercury_management_wrkr_log($wrkr, "invoking callback $cbk\n");
            $rc = call_user_func_array($cbk, array(&$wrkr['wrkr_uow'], $idx));
            if ($rc == false) {
               mercury_management_wrkr_log($wrkr, "uow terminated by callback $cbk\n");
               $wrkr['wrkr_is_busy'] = false;
               mercury_managment_invoke_uow_finished_func($wrkr);
               return $rc;
            }
         }
      }   
      $wrkr['wrkr_uow']['curr_idx']++;
      $idx = $wrkr['wrkr_uow']['curr_idx'];
      $cmd = $wrkr['wrkr_uow']['cmds'][$idx];
      if ($cmd == "no_op") {
         continue;
      }
      if ($cmd == "all_done") {
         mercury_management_wrkr_log($wrkr, "uow terminated by all_done command\n");
         $wrkr['wrkr_is_busy'] = false;
         mercury_managment_invoke_uow_finished_func($wrkr);
         return false;
      } else {
         mercury_management_wrkr_log($wrkr, "submitting command to worker\n");
         mercury_management_wrkr_log_multiple($wrkr, $cmd, ': ');
         fwrite($wrkr['wrkr_stdin'], $cmd);
         return true;
      }
   }
}


/**
 *  Record one line of the result output from a work item.
 */
function mercury_management_record_uow_item_result_line(&$wrkr, $line)
{
   $idx = $wrkr['wrkr_uow']['curr_idx'];
   $wrkr['wrkr_uow']['results'][$idx][] = rtrim($line);
}


/**
 *  Trim a trailing empty line from the result output if there is one.  This is
 *  to account for an artifact that may be introduced by vcenterdiscoverd.pl.
 */
function mercury_management_uow_item_result_trim(&$wrkr)
{
   $idx = $wrkr['wrkr_uow']['curr_idx'];
   $lastline = end($wrkr['wrkr_uow']['results'][$idx]);
   if ($lastline == '') {
      array_pop($wrkr['wrkr_uow']['results'][$idx]);
   }
   reset($wrkr['wrkr_uow']['results'][$idx]);
}


/**
 *  Give all idle workers something to work on, if there's enough work.
 */
function mercury_management_start_idle_workers(&$uow_queue, &$wrkr_array, $num_uows, &$uow_idx, &$busy_wrkrs)
{
   foreach ($wrkr_array['wrkr_array'] as &$wrkr) {
      if (false == $wrkr['wrkr_is_busy']) {
         if ($uow_idx < $num_uows) {
            mercury_management_start_worker_on_uow($wrkr, $uow_queue[$uow_idx]);
            if (true == $wrkr['wrkr_is_busy']) {
               $busy_wrkrs++;
            }
            $uow_idx++;
            if ($uow_idx == $num_uows) {
               break;
            }
         }
      } else {
         $busy_wrkrs++;
      }
   }
}
   

/**
 *  Attempt to recover from a select timeout.
 */
function mercury_management_recovered_from_timeout(&$wrkr_array, $restart_workers)
{
   /* On entry, all workers in the array are assumed to be either idle or stuck. */
   foreach ($wrkr_array['wrkr_array'] as $wrkr_idx => &$wrkr) {
      if (true == $wrkr['wrkr_is_busy']) {
         mercury_management_wrkr_log($wrkr, "select timeout detected! stuck command follows:\n");
         $idx = $wrkr['wrkr_uow']['curr_idx'];
         $cmd = $wrkr['wrkr_uow']['cmds'][$idx];
         mercury_management_wrkr_log_multiple($wrkr, $cmd, '! ');
         $continue = false;
         if (isset($wrkr['wrkr_uow']['timeout_cbk'])) {
            $cbk = $wrkr['wrkr_uow']['timeout_cbk'];
            $continue = call_user_func_array($cbk, array(&$wrkr['wrkr_uow'], $idx));
         } else {
            mercury_management_wrkr_log($wrkr, "wrkr has no select timeout handler\n");
         }
         if (false == $continue) {
            mercury_management_wrkr_log($wrkr, "aborting attempted select recovery\n");
            return false;
         }
         mercury_management_wrkr_log($wrkr, "restarting worker...\n");
         mercury_management_destroy_one_worker($wrkr);
         $wrkr_array['log_tag'] = mercury_management_idx_to_log_tag($wrkr_idx);
         $wrkr = mercury_management_create_one_worker($wrkr_array);
         unset($wrkr_array['log_tag']);
         mercury_management_wrkr_log($wrkr, "worker restarted successfully\n");
      }
   }
   
   mercury_management_wa_log($wrkr_array, "successful select timeout recovery\n");
   return true;
}


/**
 *  Process a queue of UoWs until complete or aborted.
 */
function mercury_management_process_uow_queue(&$uow_queue, &$wrkr_array, $continue_cb = null)
{
   $num_uows = count($uow_queue);
   $busy_wrkrs = 0;
   $uow_idx = 0;

   mercury_management_start_idle_workers($uow_queue, $wrkr_array, $num_uows, $uow_idx, $busy_wrkrs);
   
   while ($busy_wrkrs > 0) {
      $rd_arr = array();
      foreach ($wrkr_array['wrkr_array'] as &$wrkr) {
         if (true == $wrkr['wrkr_is_busy']) {
            $rd_arr[] = $wrkr['wrkr_stdout'];
         }
      }
      $wr_arr = NULL;
      $ex_arr = NULL;
      if (isset($wrkr_array['select_timeout'])) {
         $sel_to = $wrkr_array['select_timeout'];
      } else {
         $sel_to = 120;
      }
      $chgd = stream_select($rd_arr, $wr_arr, $ex_arr, $sel_to);
      if ($continue_cb != null) {
         if (call_user_func($continue_cb) == false) {
            mercury_management_wa_log($wrkr_array, "process_uow_queue killed by $continue_cb!\n");
            return false;
         }
      }
      if (false === $chgd) {
         mercury_management_wa_log($wrkr_array, "stream_select failed!\n");
         return false;
      } elseif ($chgd > 0) {
         foreach ($rd_arr as $rd) {
            $idx = 0; # This is inelegant
            foreach ($wrkr_array['wrkr_array'] as $idx => &$wrkr) {
               if ($wrkr['wrkr_stdout'] == $rd) {
                  break;
               }
            }
            $wrkr = &$wrkr_array['wrkr_array'][$idx];
            $str = stream_get_contents($wrkr['wrkr_stdout']);
            if ($str != "") {
               $str = rtrim($str, "\n");
               $lines = explode("\n", $str);
               $ascii_bel_or_slash_a = "\x07";
               foreach ($lines as $line) {
                  if ($line == $ascii_bel_or_slash_a) {
                     # Account for blank line possibly added by vcenterdiscoverd.pl
                     mercury_management_uow_item_result_trim($wrkr);
                     if (false === mercury_management_give_worker_next_uow_item($wrkr)) {
                        if ($uow_idx < $num_uows) {
                           mercury_management_start_worker_on_uow($wrkr, $uow_queue[$uow_idx]);
                           $uow_idx++;
                        } else {
                           $busy_wrkrs--;
                           mercury_management_wrkr_log($wrkr, "out of work; worker stays idle\n");
                        }
                     }
                  } else {
                     mercury_management_record_uow_item_result_line($wrkr, $line);
                  }
               }
            } else {
               mercury_management_wrkr_log($wrkr, "empty read!\n");
               return false;
            }
         }
      } else {
         /* Timeout */
         mercury_management_wa_log($wrkr_array, "stream_select timeout!\n");

         /* Attempt timeout recovery */
         if ($uow_idx < $num_uows) {
            $restart_workers = true;
         } else {
            $restart_workers = false;
         }
         if (false == mercury_management_recovered_from_timeout($wrkr_array, $restart_workers)) {
            return false;
         }
         
         /* 
          * Assumption at occurrence of timeout is that all workers are either idle or stuck.
          * Stuck workers have been killed and restarted, and their UOWs failed.  Therefore
          * all workers are now idle.
          */
         $busy_wrkrs = 0;
         if (true == $restart_workers) {
            mercury_management_start_idle_workers($uow_queue, $wrkr_array, $num_uows, $uow_idx, $busy_wrkrs);
         }
      }
   }
   
   return true;
}


/**
 *  Obtain a list of datacenters.
 * @deprecated not used
 */
function mercury_management_wa_get_dclist(&$wrkr_array)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = "dclist\n";
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   return $uow['results'][0];
}


/**
 *  Obtain a list of VMs.
 */
function mercury_management_wa_get_vmlist(&$wrkr_array)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = "vmlist\n";
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   return $uow['results'][0];
}


/**
 *  Obtain hostinfo for specified host.
 */
function mercury_management_wa_get_hostinfo(&$wrkr_array, $hostname)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = "hostinfo\n$hostname\n";
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   return $uow['results'][0];
}


/**
 *  Execute a command line.
 */
function mercury_management_wa_exec(&$wrkr_array, $cmd_line)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = "exec\n$cmd_line\n";
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   $rv = mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   if ($rv === false) {
      return false;
   } else {
      return $uow['results'][0];
   }
}


/**
 *  Get the FA VIB.
 * @deprecated has no references and needs to be merged into famc_external_api
 */
function mercury_management_wa_host_software_vib_get(&$wrkr_array, $host_name, $host_username, $host_password)
{
  $cmd = mercury_management_wcmd_host_fa_vib_status($host_name, $host_username, $host_password);
   return mercury_management_wa_exec($wrkr_array, $cmd);
}


/**
 *  Determine FA VIB status.
 * @deprecated not currently used.
 */
function mercury_management_wa_host_fa_vib_status(&$wrkr_array, $host_name, $host_username, $host_password)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = mercury_management_wcmd_host_fa_vib_status($host_name, $host_username, $host_password);
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   return $uow['results'][0];
}


/**
 *  Obtain the vCenter version.
 */
function mercury_management_wa_get_vcversion(&$wrkr_array)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = mercury_management_wcmd_vcenter_version();
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   return $uow['results'][0][0];
}


/**
 *  Obtain a list of hosts and vms in a datacenter.
 */
function mercury_management_wa_get_vmlist_dc(&$wrkr_array, $dc)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = mercury_management_wcmd_vmlist_dc($dc);
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   return $uow['results'][0];
}


/**
 *  Execute a webservice call.
 */
function mercury_management_wa_wsdl(&$wrkr_array, $hostname, $username, $password, $method, $param = NULL, $port = NULL, $suppress_error_responses = false)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = mercury_management_wcmd_mm_wsdl($hostname, $username, $password, $method, $param, $port);
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   $rv = mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   if ($rv === false) {
      return false;
   } else {
      $result = implode("\n", $uow['results'][0]);
      return mercury_management_parse_mm_wsdl($result, $hostname, $suppress_error_responses);
   }
}


/**
 *  Check a VM's credentials.
 */
function mercury_management_wa_vmrun_chk_cred(&$wrkr_array, $username, $password, $path)
{
   $vc_hostname = $wrkr_array['vc'];
   $vc_username = $wrkr_array['user'];
   $vc_password = $wrkr_array['pswd'];
   $uow['cmds'] = array();
   $uow['cmds'][] = mercury_management_wcmd_vmrun_chk_cred($vc_hostname, $vc_username, $vc_password, $username, $password, $path);
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   $rv = mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   if ($rv === false) {
      return false;
   } else {
      return $uow['results'][0];
   }
}


/**
 *  Obtain vminfo for a VM.
 */
function mercury_management_wa_vminfo(&$wrkr_array, $vm_name)
{
   $uow['cmds'] = array();
   $uow['cmds'][] = mercury_management_wcmd_vminfo($vm_name);
   $uow['cmds'][] = "all_done";
   $uow_queue = array();
   $uow_queue[] = &$uow;
   $rv = mercury_management_process_uow_queue($uow_queue, $wrkr_array);
   if ($rv === false) {
      return false;
   } else {
      return $uow['results'][0];
   }
}
