<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * NOTE: This file is used in contexts where Drupal is not bootstrapped,
 * so be careful what you call!
 *
 * Specifically this is only used in parallel requests.
 * @see mercury_management_cache_operation
 */

function wsdl_cache_operation($hostname, $username, $password, $method, $param = NULL, $port = NULL) {
  require_once('mercury_management.inc');

  if (empty($hostname) || $hostname == 'Not Known') {
    return "error_wsdl_call|Unable to call '$method' because VM hostname is not known.";
  }

  if (!isset($port)) {
    $port = '7010';
  }

  // Construct the SOAP client request
  $client = mercury_management_wsdl_construct($hostname, $username, $password, $port, $return_error = true);
  if (!is_object($client)) {
    $message = 'Uknown Error';
    if (!empty($client)) {
      // $client may contain details.
      $message = $client;
    }
    return "error_wsdl_call|Error communicating with VM $hostname on cache operation '$method' on port $port ($message)";
  }

  // Send the request to the VM
  $result = mercury_management_consume_webservice($client, $param, $method);
  if (array_key_exists('error', $result)) {
    return "error_wsdl_call|Cache operation '$method' on VM $hostname returned an error: {$result['message']}";
  }

  // Not an error
  if ($method == "HgGetCacheInfo") {
    $hgGetCacheInfo = array();
    $diskCount = 0;
    foreach ($result['response']->HgGetCacheInfoResult->DiskCacheInfoList->DiskCacheInfo as $key => $val) {
      $hgGetCacheInfo[$diskCount]['cache_mode'] = $val->CacheMode;
      $hgGetCacheInfo[$diskCount]['admin_state'] = $val->AdminState;
      $hgGetCacheInfo[$diskCount]['op_state'] = $val->OperationalState;
      $hgGetCacheInfo[$diskCount]['op_message'] = $val->OpStateMsg;
      $hgGetCacheInfo[$diskCount]['perf_state'] = $val->PerfState;
      $hgGetCacheInfo[$diskCount]['vdisk_identifier'] = $val->Identifier;
      $hgGetCacheInfo[$diskCount]['path'] = $val->DriveLetters;
      $diskCount++;
    }
    $hgGetCacheInfo['disk_count'] = $diskCount;
    $hgGetCacheInfo['build_version'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->BuildVersion;
    $hgGetCacheInfo['cache_mode'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->CacheMode;
    $hgGetCacheInfo['admin_state'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->AdminState;
    $hgGetCacheInfo['cache_size'] = doubleval($result['response']->HgGetCacheInfoResult->GlobalCacheInfo->Sectors);
    $hgGetCacheInfo['sector_size'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->SectorSize;
    $hgGetCacheInfo['op_state'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->OperationalState;
    $hgGetCacheInfo['op_message'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->OpStateMsg;
    $hgGetCacheInfo['perf_state'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->PerfState;
    $hgGetCacheInfo['cachespace_id'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->Identifier;
    $hgGetCacheInfo['usable_blocks'] = $result['response']->HgGetCacheInfoResult->GlobalCacheInfo->FormatInfo->UsableBlocks;
    $result = $hgGetCacheInfo;
  } elseif ($method == "HgGetCacheOptions") {
    $hgGetCacheOptions = array();
    $hgGetCacheOptions['metrics_reset_interval'] = $result['response']->HgGetCacheOptionsResult->HgCacheOptions->MetricsResetInterval;
    $result = $hgGetCacheOptions;
  }

  return $result;
}

$longopts = array(
   "hostname:",   # hostname: value required
   "username:",   # username: value required
   "password:",   # password: value required
   "method:",     # method: value required
   "port:",       # port:  value required
   "param:"       # param: value required
);

$options = getopt("", $longopts);
if (!isset($options['hostname']) ||
    !isset($options['username']) ||
    !isset($options['password']) ||
    !isset($options['method'])) {
   print("usage: mm_wsdl.php --hostname <hostname> --username <username> --password <password> --method <method> (--param <param>) (--port <port>)\n");
} else {
   if (!isset($options['param'])) {
      $options['param'] = "";
   }
   if (!isset($options['port'])) {
      $options['port'] = '7010';
   }
   $result = wsdl_cache_operation(
      $options['hostname'],
      $options['username'],
      $options['password'],
      $options['method'],
      $options['param'],
      $options['port']
   );
   var_export($result);
}
