<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for the Agent tab.
 */

/**
 * Function to check the VM Agent package
 * Returning false is an error condition only on upload.  OK on removal.
 * @return boolean true if a valid package, false otherwise.
 */
function mercury_utilities_set_vm_agent_package() {
  // Check for MSI packages
  variable_set('windows_package', FLASH_AGENT_MSI_PATH);

  $get_msi_packages_path = variable_get('windows_package');
  $latest_msi_file = mercury_utilities_get_latestFile($get_msi_packages_path);

  //PATH INFO RETURNS: DIRNAME, BASENAME, EXTENSION, FILENAME FROM A PATH
  $details_msi = pathinfo($get_msi_packages_path . $latest_msi_file);

  //EXTRACT EXTENSION FROM THE DETAILS ARRAY CREATED BY PATH INFO
  $extension_msi = '';
  if (!empty($details_msi['extension'])) {
    $extension_msi = strtolower($details_msi['extension']);
  }

  $vm_cur_MSI = '';
  $vm_upgraded_MSI = '';
  $vm_package_agent_version = '';
  if (($extension_msi != 'msi')) {
  } else if ($latest_msi_file == 'not_found' || empty($latest_msi_file)) {
  } else if (($extension_msi != 'msi')) {
  } else {
    // Should be valid, try to get the version.
    // If no version then we will fail later, so fail early too.
    // "R" etc. should be uppercase, so ensure case consistency.
    $file_name = strtoupper($details_msi['filename']);
    $get_package_version = explode('WIN_', $file_name);
    if (!empty($get_package_version[1])) {
      $vm_package_agent_version = $get_package_version[1];
      $vm_cur_MSI = $latest_msi_file;
      $vm_upgraded_MSI = $latest_msi_file;
    }
  }

  variable_set('vm_cur_MSI', $vm_cur_MSI);
  variable_set('vm_upgraded_MSI', $vm_upgraded_MSI);
  variable_set('vm_package_agent_version', $vm_package_agent_version);

  if (empty($vm_upgraded_MSI)) {
    return false;
  }

  return true;
}

/**
 * Function to check the host agent package
 * Returning false is an error condition only on upload.  OK on removal.
 * @return boolean true if a valid package, false otherwise.
 */
function mercury_utilities_set_host_agent_package() {
  variable_set('vib_package', FLASH_AGENT_VIB_PATH);
  $get_vib_packages_path = variable_get('vib_package');
  $lastest_vib_file = mercury_utilities_get_latestFile($get_vib_packages_path);

  $details_vib = pathinfo($get_vib_packages_path . $lastest_vib_file);
  $extension_vib = '';

  if (!empty($details_vib['extension'])) {
    $extension_vib = strtolower($details_vib['extension']);
  }

  $host_cur_zip = '';
  $host_upgraded_zip = '';
  $host_package_agent_version = '';
  if ($extension_vib == '') {
  } else if ($lastest_vib_file == 'not_found' || empty($lastest_vib_file)) {
  } else if (($extension_vib != 'zip' && $extension_vib != 'vib')) {
  } else {
    // Should be valid, try to get the version.
    // If no version then we will fail later, so fail early too.
    // "R" etc. should be uppercase, so ensure case consistency.
    $file_name = strtoupper($details_vib['filename']);
    $get_package_version = explode('ESX_', $file_name);
    if (!empty($get_package_version[1])) {
      $host_package_agent_version = $get_package_version[1];
      $host_cur_zip = $lastest_vib_file;
      $host_upgraded_zip = $lastest_vib_file;
    }
  }

  variable_set('host_cur_zip', $host_cur_zip);
  variable_set('host_upgraded_zip', $host_upgraded_zip);
  variable_set('host_package_agent_version', $host_package_agent_version);

  if (empty($host_upgraded_zip)) {
    return false;
  }

  return true;
}

/**
 * Function to upload VM and Host Agent package files
 */
function mercury_utilities_agents_settings_form() {
  $var_file_host_vm_agents = variable_get('fmc_host_vm_agents');

  // Get the human-readable version info file from the agent package.
  $fmc_version_info = '';
  if (!empty($var_file_host_vm_agents)) {
    if (file_exists(FMC_AGENTS_INSTALL_PATH . 'version.txt')) {
      $file_handle = fopen(FMC_AGENTS_INSTALL_PATH . 'version.txt', 'r');
      while (!feof($file_handle)) {
        $fmc_version_info = $fmc_version_info . '<pre>' . trim(fgetss($file_handle)) . '</pre>';
      }
      fclose($file_handle);
    }
  }

  $form = array();

  $form['agents_description_title'] = array(
    '#type' => 'markup',
    '#markup' => t('<h2>Agent Settings</h2>'),
  );

  $form['fmc_agents'] = array(
    '#type' => 'fieldset',
    '#title' => 'Agent Settings',
    '#collapsible' => false,
    '#collapsed' => false
  );

  if (!empty($var_file_host_vm_agents) && $fmc_version_info != '') {

    $form['fmc_agents']['agents_description'] = array(
      '#type' => 'markup',
      '#markup' => t('<b>The following versions of the Flash Accel Host and Guest OS Agents are currently available for installation or upgrade. Click Delete Agents if you want to delete the current agent packages and import newer or older versions of the agents.</b><br/><br/>'),
    );

    $vm_package_agent_version = variable_get('vm_package_agent_version', '');
    $host_package_agent_version = variable_get('host_package_agent_version', '');

    // If this is reached and the package variables are not set, try to rebuild
    // the strings. This should only happen during development (database reset).
    if (empty($vm_package_agent_version) || empty($host_package_agent_version)) {
      mercury_utilities_set_vm_agent_package();
      mercury_utilities_set_host_agent_package();
      $vm_package_agent_version = variable_get('vm_package_agent_version', '');
      $host_package_agent_version = variable_get('host_package_agent_version', '');
    }

    // If the major or minor version of the agents does not match FAMC,
    // display a warning.
    // This may be intentional, so don't consider it an error.
    // TODO: more severe language for unsupported version vs simple mismatch?
    // It's not possible in Copper, revisit in Dubnium.
    $famc_version = mercury_utilities_get_fmc_version_string();
    if (
      !mercury_utilities_version_compare($vm_package_agent_version, $famc_version, $famc_version, $major_minor = true, $allow_equal = true) ||
      !mercury_utilities_version_compare($host_package_agent_version, $famc_version, $famc_version, $major_minor = true, $allow_equal = true)
    ) {
      $mismatch_text = t('For best results, please use an agent package that matches the management console version.');
      mercury_utilities_fmc_log_message('Agent', $mismatch_text, WATCHDOG_DEBUG, NULL, NULL, $skip_footer_log = true);
      $form['fmc_agents']['version_mismatch'] = array(
        '#type' => 'markup',
        '#markup' => '<div class="inline-messages warning">' . $mismatch_text . "</div>",
      );
    }

    $form['fmc_agents']['host_vm_agents'] = array(
      '#type' => 'markup',
      '#markup' => $fmc_version_info,
    );

    $form['fmc_agents']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Delete Agents'),
      '#submit' => array('mercury_utilities_agents_remove'),
      '#validate' => array(),
      '#prefix' => t('<br/>')
    );
  } else {

    $form['fmc_agents']['agents_description'] = array(
      '#type' => 'markup',
      '#markup' => t('<b>Click Import Agents to import Flash Accel Host and Guest OS Agent installation packages. You are prompted to select the Debian file for the agent installation packages (which you should already have downloaded from the !brand Support Site).</b><br/><br/>', array('!brand' => mercury_utilities_get_fmc_brand_string())),
    );

    $form['fmc_agents']['no_fmc_agents'] = array(
      '#type' => 'markup',
      '#markup' => t('<b>No Agents have been imported</b>'),
      '#suffix' => t('<br/>'),
      '#prefix' => t('<br/>')
    );

    $form['fmc_agents']['host_vm_agents'] = array(
      '#type' => 'file',
      '#title' => t('Select a FMC Agents package'),
      '#description' => t('Upload the FMC Agents package file, allowed extensions: .zip, .gz'),
    );

    $form['fmc_agents']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Import Agents'),
    );
  }


  return $form;
}

/**
 * Function to validate Agent uploaded files
 */
function mercury_utilities_agents_settings_form_validate($form, &$form_state) {
  $unzipped_folder = 'public://fmc_agents/';
  file_prepare_directory($unzipped_folder, FILE_CREATE_DIRECTORY);

  $values = $form_state['values'];
  $file_host_vm_agents = file_save_upload('host_vm_agents', array(
    'file_validate_extensions' => array('zip gz'), $unzipped_folder, FILE_EXISTS_REPLACE
  ));

  if (!$file_host_vm_agents) {
    form_set_error('host_vm_agents', t('Please select a valid Agent package file.'));
  } else {
    if ($file_host_vm_agents = file_move($file_host_vm_agents, $unzipped_folder, FILE_EXISTS_REPLACE)) {
      $form_state['values']['file_host_vm_agents'] = $file_host_vm_agents;
    } else {
      form_set_error('host_vm_agents', t('Failed to upload the Agent Package.'));
    }
  }
}

/**
 * Function to upload the Agents pacakge files
 */
function mercury_utilities_agents_settings_form_submit($form, &$form_state) {

  $file_host_vm_agents = $form_state['values']['file_host_vm_agents'];
  unset($form_state['values']['file_host_vm_agents']);

  $file_host_vm_agents->status = FILE_STATUS_PERMANENT;
  file_save($file_host_vm_agents);

  $agent_file_info = pathinfo(FMC_AGENTS_IMPORT_PATH . $file_host_vm_agents->filename);

  if (strtolower($agent_file_info['extension']) == 'zip') {
    exec('unzip -o ' . FMC_AGENTS_IMPORT_PATH . $file_host_vm_agents->filename . ' -d ' . FMC_AGENTS_IMPORT_PATH, $output);
    $content_grabbed = implode("\n", $output);
  } elseif (strtolower($agent_file_info['extension']) == 'gz') {
    exec('gunzip -N ' . FMC_AGENTS_IMPORT_PATH . $file_host_vm_agents->filename, $output);
    $content_grabbed = implode("\n", $output);
  }
  $get_deb_file = mercury_utilities_get_latestFile(FMC_AGENTS_IMPORT_PATH, 'fmc_agent');

  if ($get_deb_file == 'not_found') {
    mercury_utilities_fmc_log_message('Agent', t('The specified package could not be imported. The file may be corrupt.'), WATCHDOG_ERROR, 'show');
    if (file_delete($file_host_vm_agents, TRUE)) {
      exec('rm -r ' . FMC_AGENTS_IMPORT_PATH . '*');
      variable_del('fmc_host_vm_agents');
      drupal_goto('config/fmc/settings/agents');
    }
  } else {
    //Install the Debian package
    $package_ok = 0;
    $package_name = '';
    $package_version = 0;
    //Get the Debian package info and check for package name
    $check_fmc_agent_package = exec('sudo /usr/bin/dpkg --info ' . FMC_AGENTS_IMPORT_PATH . $get_deb_file, $result_package_info);
    if (is_array($result_package_info)) {
      foreach ($result_package_info as $info_detail) {
        if (stristr($info_detail, 'flashaccel-base-mc')) {
          $package_ok = 1;
          $package_name = $info_detail;
          break;
        } else {
          $package_ok = 0;
          $package_name = '';
        }
      }
    }
    if ($package_ok == 1) {
      $check_fmc_agent_package_contents = exec('sudo /usr/bin/dpkg -c ' . FMC_AGENTS_IMPORT_PATH . $get_deb_file, $result_package_contents);
      //drupal_set_message(print_r($result_package_contents, 1));
      if (is_array($result_package_contents)) {
        foreach ($result_package_contents as $content_detail) {
          if (stristr($content_detail, 'version.txt')) {
            $package_version = 1;
            break;
          } else {
            $package_version = 0;
          }
        }
      }
    }

    //If both package name and version file validates
    $open_success = false;
    if ($package_ok == 1 && $package_version == 1) {
      $install_fmc_package = exec('sudo /usr/bin/dpkg -i --log=' . FMC_AGENTS_IMPORT_PATH . 'install_log.txt ' . FMC_AGENTS_IMPORT_PATH . $get_deb_file, $result_package_install);
      variable_set('fmc_host_vm_agents', $file_host_vm_agents);

      // Update other variables.
      if (
        mercury_utilities_set_host_agent_package() &&
        mercury_utilities_set_vm_agent_package()
      ) {
        $open_success = true;
      }
    }

    if ($open_success) {
      //Hacking the Drupal message system for any other message not to come along with the current message
      $message = 'Agents are uploaded successfully into FMC.';
      mercury_utilities_fmc_log_message('Agent', $message, WATCHDOG_NOTICE, 'show');
      $keys = array_keys($_SESSION['messages']['status']);
      $last_key = end($keys);

      //Substitute default message by custom message
      if (isset($_SESSION['messages']['status'][($last_key - 1)])) {
        $_SESSION['messages']['status'][($last_key - 1)] = $message;
        unset($_SESSION['messages']['status'][$last_key]);
      } else {
        $_SESSION['messages']['status'][$last_key] = $message;
      }
    } else {
      file_delete($file_host_vm_agents, TRUE);
      exec('rm -r ' . FMC_AGENTS_IMPORT_PATH . '*');
      variable_del('fmc_host_vm_agents');
      mercury_utilities_fmc_log_message('Agent', t('The specified package could not be imported. The file may be corrupt.'), WATCHDOG_ERROR, 'show');
      drupal_goto('config/fmc/settings/agents');
    }
  }
}

/**
 * Form to remove and delete Agents File and Database Records
 */
function mercury_utilities_agents_remove($form, &$form_state) {
  $file_host_vm_agents = variable_get('fmc_host_vm_agents', '');
  if (empty($file_host_vm_agents)) {
    return;
  }

  $agent_file_info = pathinfo(FMC_AGENTS_IMPORT_PATH . $file_host_vm_agents->filename);

  if (file_delete($file_host_vm_agents, TRUE)) {
    exec('rm -r ' . FMC_AGENTS_IMPORT_PATH . '*');
    variable_del('fmc_host_vm_agents');
    $uninstall_fmc_package = exec('sudo /usr/bin/dpkg -r flashaccel-base-mc');
    //Update the Host ESX Package info after FMC Agents uninstallation
    mercury_utilities_set_host_agent_package();
    //Update the VM MSI Package info after FMC Agents uninstallation
    mercury_utilities_set_vm_agent_package();
    mercury_utilities_fmc_log_message('Agent', t('Agent deletion was successful.'), WATCHDOG_NOTICE, 'show');
  }
}

/**
 * Function to set Flash Accel Packages folder path
 * XXX: This is never used!
 */
function mercury_utilities_packages_settings_form() {
  $form = array();
  $form['windows_package'] = array(
    '#type' => 'textfield',
    '#title' => t('Folder Path for the location of the Flash Accel Windows MSI Package'),
    '#description' => t('Please enter the absolute folder path. e.g. "' . FLASH_AGENT_MSI_PATH . '"'),
    '#default_value' => variable_get('windows_package', ''),
    '#required' => TRUE
  );

  $form['vib_package'] = array(
    '#type' => 'textfield',
    '#title' => t('Folder Path for the location of the Flash Accel Host VIB Package'),
    '#description' => t('Please enter the absolute folder path. e.g. "' . FLASH_AGENT_VIB_PATH . '"'),
    '#default_value' => variable_get('vib_package', ''),
    '#required' => TRUE
  );

  return system_settings_form($form);
}

/**
 * Function for validating if the folder exist and also contains file
 * XXX: This is never used!
 */
function mercury_utilities_packages_settings_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  $get_msi_packages_path = $values['windows_package'];
  $get_vib_packages_path = $values['vib_package'];

  $latest_msi_file = mercury_utilities_get_latestFile($get_msi_packages_path);
  $lastest_vib_file = mercury_utilities_get_latestFile($get_vib_packages_path);

  //PATH INFO RETURNS: DIRNAME, BASENAME, EXTENSION, FILENAME FROM A PATH
  $details_msi = pathinfo($get_msi_packages_path . $latest_msi_file);
  $details_vib = pathinfo($get_vib_packages_path . $lastest_vib_file);

  //EXTRACT EXTENSION FROM THE DETAILS ARRAY CREATED BY PATH INFO
  $extension_msi = '';
  $extension_vib = '';
  if (!empty($details_msi['extension'])) {
    $extension_msi = strtolower($details_msi['extension']);
  }
  if (!empty($details_vib['extension'])) {
    $extension_vib = strtolower($details_vib['extension']);
  }

  if ($latest_msi_file == 'not_found' || empty($latest_msi_file)) {
    variable_set('vm_cur_MSI', '');
    variable_set('vm_upgraded_MSI', '');
    form_set_error('windows_package', 'Windows MSI Package could be found at the entered Folder location. Please enter a valid Folder location.');
  } else if ($extension_msi != 'msi') {
    variable_set('vm_cur_MSI', '');
    variable_set('vm_upgraded_MSI', '');
    form_set_error('windows_package', 'Windows MSI Package could be found at the entered Folder location. Please make sure that the Folder contains file with only ".msi" extension.');
  }
  if ($lastest_vib_file == 'not_found' || empty($lastest_vib_file)) {
    variable_set('host_cur_zip', '');
    variable_set('host_upgraded_zip', '');
    form_set_error('vib_package', 'Host VIB Package could be found at the entered Folder location. Please enter a valid Folder location.');
  } else if ($extension_vib != 'zip' && $extension_vib != 'vib') {
    variable_set('host_cur_zip', '');
    variable_set('host_upgraded_zip', '');
    form_set_error('vib_package', 'Host VIB Package could be found at the entered Folder location. Please make sure that the Folder contains file with only ".zip" or "vib" extension.');
  }
}
