<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for Drupal's block content system.
 */

/**
 * Implements hook_block_info().
 */
function mercury_utilities_block_info() {
  //Block for Selectot Tools for all VMs
  $blocks['select_vm_block'] = array(
    'info' => t('Mercury VM Selector Tool'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );

  //Block for Selectot Tools for all Host
  $blocks['select_host_block'] = array(
    'info' => t('Mercury Host Selector Tool'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );

  //Block for Selectot Tools for VM Disks
  $blocks['select_disk_block'] = array(
    'info' => t('Mercury Disk Selector Tool'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );

  //Block for Bulk Actions on VMs
  $blocks['bulk_actions_vm_block'] = array(
    'info' => t('Mercury VM Bulk Actions'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );

  //Block for Bulk Actions on VMs
  $blocks['bulk_actions_vdisk_block'] = array(
    'info' => t('Mercury VM Disk Bulk Actions'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );

  //Block for Bulk Actions on Host
  $blocks['bulk_actions_host_block'] = array(
    'info' => t('Mercury Host Bulk Actions'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );

  //Block for Bulk Actions on Inventory
  $blocks['ongoing_processes_block'] = array(
    'info' => t('Ongoing Processes'),
    'cache' => DRUPAL_NO_CACHE,
    'properties' => array('administrative' => TRUE)
  );
  return $blocks;
}

/**
 * Implements hook_block_view().
 */
function mercury_utilities_block_view($delta = '') {
  $block = array();
  switch ($delta) {

    case 'select_vm_block':
      $block['subject'] = t('Selector Tool');
      $block['content'] = mercury_utilities_selector_tool_vm();
      break;

    case 'select_host_block':
      $block['subject'] = t('Selector Tool');
      $block['content'] = mercury_utilities_selector_tool_host();
      break;

    case 'select_disk_block':
      $block['subject'] = t('Selector Tool');
      $block['content'] = mercury_utilities_selector_tool_vm_disks();
      break;

    case 'bulk_actions_vm_block':
      $block['subject'] = t('Bulk Actions');
      $block['content'] = mercury_utilities_display_bulk_actions_vm();
      break;

    case 'bulk_actions_vdisk_block':
      $block['subject'] = t('Bulk Actions');
      $block['content'] = mercury_utilities_display_bulk_actions_vm_disk();
      break;

    case 'bulk_actions_host_block':
      $block['subject'] = t('Bulk Actions');
      $block['content'] = mercury_utilities_display_bulk_actions_host();
      break;

    case 'ongoing_processes_block':
      $block['subject'] = t('Ongoing Processes');
      $block['content'] = mercury_utilities_display_ongoing_processes();
  }
  return $block;
}

/**
 * Function to dislapy the Selector Tools for VM
 * @return $block
 */
function mercury_utilities_selector_tool_vm() {
  $output = '';
  if (mercury_management_get_vm_count() > 0) {
    $output .= '<ul class="menu">';
    $output .= '<li class="leaf vm-selector"><a class="selector-tool-item selector-tool-item-vm" href="/vm-inventory" title="Select All VMs">Select All VMs</a></li>';
    $output .= '<li class="leaf vm-unselector"><a class="selector-tool-item unselector-tool-item-vm" href="/vm-inventory" title="Unselect All VMs">Unselect All VMs</a></li>';
    $output .= '<ul id="block-vm-refine" class="menu vm_refine">';
    $output .= '<li class="vmrefine_submenu">Refine</li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item vm-flash-accel-installed" title="Select VMs with Flash Accel Installed" href="/vm-inventory">Flash Accel Installed</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item vm-flash-accel-uninstalled" title="Select VMs with Flash Accel not Installed" href="/vm-inventory">Flash Accel Not Installed</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item vm-caching-enabled" title="Select Caching Enabled VMs" href="/vm-inventory">Caching Enabled</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item vm-caching-disabled" title="Select Caching Disabled VMs" href="/vm-inventory">Caching Disabled</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item vm-cache-metrics-enabled" title="Select Cache Metrics Enabled VMs" href="/vm-inventory">Cache Metrics Enabled</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item vm-cache-metrics-disabled" title="Select Cache Metrics Disabled VMs" href="/vm-inventory">Cache Metrics Disabled</a></li>';
    $output .= '</ul></ul>';
  }
  return $output;
}

/**
 * Function to dislapy the Selector Tools for Host
 * @return $block
 */
function mercury_utilities_selector_tool_host() {
  $output = '';
  if (mercury_management_get_host_count() > 0) {
    $output .= '<ul class="menu">';
    $output .= '<li class="leaf host-selector"><a class="selector-tool-item selector-tool-item-host" href="/host-inventory">Select All Hosts</a></li>';
    $output .= '<li class="leaf host-unselector"><a class="selector-tool-item unselector-tool-item-host" href="/host-inventory">Unselect All Hosts</a></li>';
    // TODO: The inner UL has to be wrapped in an LI.  Add one and fix the CSS.
    $output .= '<ul id="block-host-refine" class="menu host_refine">';
    $output .= '<li class="hostrefine_submenu">Refine</li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item host-flash-accel-installed" href="/host-inventory">Flash Accel Installed</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item host-flash-accel-uninstalled" href="/host-inventory">Flash Accel Not Installed</a></li>';
    $output .= '</ul></ul>';
  }
  return $output;
}

/**
 * Function to dislapy the Selector Tools for disks
 * @return $block
 */
function mercury_utilities_selector_tool_vm_disks() {
  $get_page_path = filter_xss($_GET['q']);
  $vdisk_ids = mercury_utilities_vm_vdisk_nid_fetch(arg(1));
  $row_count = count($vdisk_ids);

  $output = '';
  if ($row_count > 0) {
    $output .= '<ul class="menu">';
    $output .= '<li class="leaf disk-selector"><a class="selector-tool-item selector-tool-item-disk" href="#">Select All Disks</a></li>';
    $output .= '<li class="leaf disk-unselector"><a class="selector-tool-item unselector-tool-item-disk" href="#">Unselect All Disks</a></li>';
    $output .= '<ul id="block-disk-refine" class="menu disk_refine">';
    $output .= '<li class="diskrefine_submenu">Refine</li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item disk-caching-enabled" title="Select Caching Enabled Disks" href="#">Caching Enabled</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item disk-caching-disabled" title="Select Caching Disabled Disks" href="#">Caching Disabled</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item disk-cache-metrics-enabled" title="Select Cache Metrics Enable Disks" href="#">Cache Metrics Enabled</a></li>';
    $output .= '<li class="refine-leaf"><a class="selector-tool-item disk-cache-metrics-disabled" title="Select Cache Metrics Disabled Disks" href="#">Cache Metrics Disabled</a></li>';
    $output .= '</ul></ul>';
  }
  return $output;
}

/**
 * Function for displaying Bulk Actions for VM in a Block
 */
function mercury_utilities_display_bulk_actions_vm() {

  $get_page_path = filter_xss($_GET['q']);

  $output = '';
  $output .= '<ul id="bulk_actions_block_menus" class="bulk_actions_menus">';

  //Display Bulk actions related to VMs
  if ($get_page_path == 'home' || $get_page_path == 'vm-inventory') {
    $output .= '<li id="buld_refresh_vm" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('VM Refresh', '', $class = 'bulk_actions_link', $alt = 'Execute a Refresh operation') . '</li>';
    $output .= '<li id="install_flash_msi" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('VM Agent Install', '', $class = 'bulk_actions_link', $alt = 'Flash Accel Guest OS Agent') . '</li>';
    $output .= '<li id="upgrade_flash_msi" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('VM Agent Upgrade', '', $class = 'bulk_actions_link', $alt = 'Flash Accel Guest OS Upgrade') . '</li>';
    $output .= '<li id="uninstall_flash_msi" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('VM Agent Uninstall', '', $class = 'bulk_actions_link', $alt = 'Flash Accel Guest OS Agent') . '</li>';
    //$output .= '<li id="allocate_cache_space" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('Allocate Cache Space', '', $class = 'bulk_actions_link', $alt = 'Allocate Cache space') . '</li>';
    $output .= '<li id="unallocate_cache_space" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('Unallocate Cache Space', '', $class = 'bulk_actions_link', $alt = 'Unallocate Cache space') . '</li>';
    $output .= '<li id="change_cache_mode" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('Cache Mode', '', $class = 'bulk_actions_link', $alt = 'Change') . '</li>';
    $output .= '<li id="change_cache_state" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('Cache State', '', $class = 'bulk_actions_link', $alt = 'Change') . '</li>';
    $output .= '<li id="change_cache_metrics" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('Cache Metrics', '', $class = 'bulk_actions_link', $alt = 'Change') . '</li>';
    $output .= '<li id="vm_advance" class="bulk_actions">' . ctools_popups_bulk_action_vm_link('Advanced', '', $class = 'bulk_actions_link', $alt = 'Advanced') . '</li>';
  }

  $output .= '</ul>';

  return $output;
}

/**
 * Function for displaying Bulk Actions for Host in a Block
 */
function mercury_utilities_display_bulk_actions_host() {

  $get_page_path = filter_xss($_GET['q']);

  $output = '';
  $output .= '<ul id="bulk_actions_block_menus" class="bulk_actions_menus">';

  //Display Bulk actions related to Host
  if ($get_page_path == 'home' || $get_page_path == 'host-inventory') {

    $output .= '<li id="bulk_refresh_host" class="bulk_actions">' . ctools_popups_bulk_action_host_link('Host Refresh', '', $class = 'bulk_actions_link', $alt = 'Execute a Refresh operation') . '</li>';
    $output .= '<li id="install_bundle" class="bulk_actions">' . ctools_popups_bulk_action_host_link('Host Agent Install', '', $class = 'bulk_actions_link', $alt = 'Install the Flash Accel software') . '</li>';
    $output .= '<li id="upgrade_bundle" class="bulk_actions">' . ctools_popups_bulk_action_host_link('Host Agent Upgrade', '', $class = 'bulk_actions_link', $alt = 'Upgrade the Flash Accel software') . '</li>';
    $output .= '<li id="uninstall_bundle" class="bulk_actions">' . ctools_popups_bulk_action_host_link('Host Agent Uninstall', '', $class = 'bulk_actions_link', $alt = 'Uninstall the Flash Accel software') . '</li>';
  }

  $output .= '</ul>';

  return $output;
}

/**
 * Function for displaying Bulk Actions for VM Disks in a Block
 */
function mercury_utilities_display_bulk_actions_vm_disk() {

  $get_page_path = filter_xss($_GET['q']);

  $output = '';
  $output .= '<ul id="bulk_actions_block_menus" class="bulk_actions_menus">';

  //Display Bulk actions related to vDisk of VMs
  //if ($get_page_path == '/virtual_machine/%/vm_disks') {
  $output .= '<li id="disk_change_refresh" class="bulk_actions">' . l(t('VM Disk Refresh'), 'home', array('attributes' => array('class' => 'vm-disk-bulk-refrsh'))) . '</li>';
  $output .= '<li id="disk_change_cache_mode" class="bulk_actions">' . ctools_popups_bulk_action_vdisk_link('Cache Mode', '', $class = 'bulk_actions_link', $alt = 'Change') . '</li>';
  $output .= '<li id="disk_change_cache_state" class="bulk_actions">' . ctools_popups_bulk_action_vdisk_link('Cache State', '', $class = 'bulk_actions_link', $alt = 'Change') . '</li>';
  $output .= '<li id="disk_change_cache_metrics" class="bulk_actions">' . ctools_popups_bulk_action_vdisk_link('Cache Metrics', '', $class = 'bulk_actions_link', $alt = 'Change') . '</li>';
  $output .= '<li id="disk_change_advance" class="bulk_actions">' . ctools_popups_bulk_action_vdisk_link('Advanced', '', $class = 'bulk_actions_link', $alt = 'Advanced') . '</li>';

  //}

  $output .= '</ul>';

  return $output;
}

/**
 *  Function to display the ongoing processes
 */
function mercury_utilities_display_ongoing_processes() {
  $output = '';
  $vm_query = db_query("SELECT nid FROM {node} WHERE type = :tp AND status = :st", array(':tp' => 'virtual_machine', ':st' => 1));
  if (!empty($vm_query)) {
    $class = 'odd';
    foreach ($vm_query as $vm) {
      $param = array();
      $attrInfo = mercury_management_cache_operation('nid', $vm->nid, 'HgGetCacheInfo', $param);
      if ($attrInfo != 'error_wsdl_call') {
        $vm_info = node_load($vm->nid);
        $op_message = $attrInfo['op_message'];
        $op_message_arr = explode('Cache format:', $op_message);
        if (count($op_message_arr) > 1) {
          $vm_link = l($vm_info->title, drupal_get_path_alias('node/' . $vm->nid));
          $output .= '<div class="process-item ' . $class . '"><div class="left">' . $vm_link . '</div><div class="right"><span class="process_name">Cache format</span><span class="progress">' . trim($op_message_arr[1]) . '</span></div><div class="clr"></div></div>';
          $class = ($class == 'odd') ? 'even' : 'odd';
        }
      }
      //$vm_link = l($vm_info->title, drupal_get_path_alias('node/' . $vm->nid));
      //$output .= '<div class="process-item ' . $class . '"><div class="left">' . $vm_link . '</div><div class="right"><span class="process_name">Cache format</span><span class="progress">Testing</span></div><div class="clr"></div></div>';
      //$class = ($class == 'odd') ? 'even' : 'odd';
    }
  }
  if ($output == '') {
    $output .= '<div class="no-process">There is no ongoing process currently.</div>';
  }
  return $output;
}
