<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for the cron tab (crontab tab).
 */

/**
 * Function to set CRON JOB time schedule for Rediscovery functonality and the hosts/vms to rediscover
 */
function mercury_utilities_cron_settings_form() {

  $form = array();
  $vm_was_checked = FALSE;

  $redesc_time = variable_get('redesc_cron_hour', '0');
  if ($redesc_time == 0) {
    $msg = "Currently auto rediscovery is disabled.";
  } else {
    $msg = "Currently auto rediscovery will run at " . ($redesc_time - 1) . ":00 every day.";
  }

  $form['cron_description'] = array(
    '#type' => 'markup',
    '#markup' => t('<b>Select the hour when auto rediscovery of the vCenter should be done. <br/>@msg</b>', array("@msg" => $msg)),
    '#prefix' => '<h2>Auto-Rediscover Settings</h2><div id="cron-hour-block" class="mercury-host-block"><div style="padding:10px">',
  );

  $discovery_times = array('Disabled', 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23);

  $form['redesc_cron_hour'] = array(
    '#type' => 'select',
    '#options' => $discovery_times,
    '#title' => t('Hour to execute (24 Hour Format)'),
    '#default_value' => variable_get('redesc_cron_hour', '0'),
    '#description' => t('Select the hour when vCenter rediscovery should be done'),
    '#prefix' => '<table><tr><td>',
    '#suffix' => '</td>',
  );

  $form['cron_submit'] = array(
    '#type' => 'button',
    '#value' => t('Save Configuration'),
    //'#submit' => array('mercury_utilities_cron_settings_save'),
    '#ajax' => array(
      'callback' => 'mercury_utilities_cron_settings_save',
      'wrapper' => 'box',
    //'path' => 'ctools-popups/nojs/mmc/rediscover'
    ),
    '#prefix' => '<td>',
    '#suffix' => '</td></tr></table>',
  );

  $vcenters = db_query("SELECT nid FROM {node} WHERE type = :tp AND status = :st", array(':tp' => 'vcenter', ':st' => 1));

  if (!empty($vcenters)) {

    foreach ($vcenters as $vc) {

      $vcInfo = node_load($vc->nid);
      $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
      $username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
      $password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

      // TODO: Right now we don't actually use the host and VM list, just the DC list.
      // However, when we change the DC list to show indications of support level
      // (red text + error symbol if > 5000, yellow with warning for > 1000)
      // we will then need to know the counts here.
      $vCenterArr = mercury_management_gethosts_virtualmachines($vcInfo, $vc_hostname, $username, $password);

      $form['discover']['disc_text'] = array(
        '#prefix' => '</div></div><h2>vCenter Inventory Details</h2>',
        // TODO: Div is not allowed inside <h5>. Also <h5> is bad.
        '#suffix' => '<h5><div class="select-links"><a href="#" class="s-all">Select All</a>  |  <a href="#" class="s-r-all">Unselect All</a>  |  <a href="#" class="enable-takeover">Enable Takeover</a>  |  <a href="#" class="disable-takeover">Disable Takeover</a></div></h5><br/><table><tr><td width=530px>',
        '#markup' => t('<b>Below are the datacenters currently managed by the vCenter. <br/><br/>Select the datacenters you want to use with Flash Accel. Click Discover All to discover all the hosts and VMs in the selected datacenters.  Click Discover New to discover only the previously undiscovered hosts and VMs in the selected datacenters -- hosts and VMs that don\'t currently appear in the Host Inventory or VM Inventory pages.</b><br/><br/>
<b>If the check box for a datacenter is disabled, the datacenter is being managed
by an instance of the FMC running on another VM. If you are sure you want to
manage the datacenter with your instance of the FMC (for example, because you
know that the VM for the other instance has been deleted), click Enable
Takeover. </b><br/><br/>'),
      );

      if (!empty($vCenterArr)) {

        //Sort Alphabetically case-insensitive (asc)
        uksort($vCenterArr, "strnatcasecmp");

        $dc_count = 0;
        $loaded_array = variable_get('rediscover_item_settings', array());
        foreach ($vCenterArr as $dc_name => $datacenter) {
          $attrArr = array();
          if (!empty($datacenter)) {
            if (!empty($loaded_array)) {
              foreach ($loaded_array as $item) {
                if ($dc_name == $item) {
                  $attrArr = array('checked' => 'checked');
                }
              }
            }

            $datacenter_status = mercury_utilities_check_datacenter_state($vc->nid, $dc_name);
            $prefix_class = 'enable-datacenter selectable';
            if ($datacenter_status != 'enable') {
              $attrArr = array('disabled' => 'disabled');
              $prefix_class = 'disable-datacenter';
            }

            $form['discover']['dc_' . $dc_count] = array(
              '#type' => 'checkbox',
              '#title' => '<span class = "' . $datacenter_status . '-dc"><b>' . $dc_name . '</b></span>',
              '#return_value' => $dc_name,
              '#attributes' => $attrArr,
              '#prefix' => '<span class = "' . $prefix_class . '">',
              '#suffix' => '</span>',
            );

            $dc_count++;
          }
        }
      }
    }
  }

  $form['disc_all'] = array(
    '#type' => 'submit',
    '#value' => t('Discover All'),
    '#ajax' => array(
      'callback' => 'mercury_utilities_discover_all_submit',
      'wrapper' => 'box',
    ),
    '#prefix' => '</td><td>',
    '#suffix' => '<br><br>',
  );

  $form['disc_new'] = array(
    '#type' => 'submit',
    '#value' => t('Discover New'),
    '#ajax' => array(
      'callback' => 'mercury_utilities_discover_new_submit',
      'wrapper' => 'box',
    ),
    '#suffix' => '</td></tr></table>',
  );

  return $form;
}

function mercury_utilities_cron_settings_save($form, &$form_state) {
  $values = $form_state['values'];
  variable_set('redesc_cron_hour', $values['redesc_cron_hour']);
  mercury_utilities_fmc_log_message('Discovery', t('Auto-Rediscovery Settings have been saved successfully.'), WATCHDOG_NOTICE, 'show');
  ctools_include('ajax');
  ctools_add_js('ajax-responder');
  $out = array();
  $out[] = ctools_ajax_command_redirect($_SERVER['HTTP_REFERER']);
  print ajax_render($out);
  exit;
}

// XXX: This is probably redundant code.
function mercury_utilities_check_datacenter_state($vcenter_id, $datacenter) {
  $vc_info = node_load($vcenter_id);
  $uuid = $vc_info->field_fmc_uuid['und'][0]['value'];
  $vcenterInfo = exec("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server " . $vc_info->field_hostname['und'][0]['value'] . " --username " . "'" . fmc_cipher::decrypt($vc_info->field_username['und'][0]['value']) . "'" . " --password " . "'" . fmc_cipher::decrypt($vc_info->field_password['und'][0]['value']) . "'" . " --operation getDataCenterAnnotation --datacenter " . "'" . $datacenter . "'" . " --annotation " . $uuid, $result);
  if (!empty($result) && ($result[0] == "Status: Not Annotated" || $result[0] == "Status: Annotated")) {
    return "enable";
  } else {
    return "disable";
  }
}
