<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for the discovery tab.
 */

/**
 * This gets called when you hit the discover all or new buttons on the
 * discovery settings tab. All it does is record your DC selections for later
 * and then open the pop-up to begin the process.
 */
function mercury_utilities_discover_all_submit($form, &$form_state) {
  $values = $form_state['values'];

  if (!empty($values)) {

    $selected_dcs = array();
    $count = 0;
    foreach ($values as $key => $dc_name) {
      if (stristr($key, 'dc_')) {
        if (!empty($dc_name)) {
          mercury_utilities_set_datacenter_annotation($dc_name);
          $selected_dcs[] = $dc_name;
          $count++;
        }
      }
    }
  }

  variable_set('rediscover_item_settings', $selected_dcs);

  // This will set the discovery level to "all" unless called by
  // mercury_utilities_discover_new_submit() below.
  // @see mercury_management_rediscover_confirm
  global $discovery_scope_level;
  if (empty($discovery_scope_level)) {
    $discovery_scope_level = 'all';
  }
  ctools_popups_mmc_rediscover('nojs');
}

function mercury_utilities_discover_new_submit($form, &$form_state) {
  // New and All do exactly the same thing in this context except for setting
  // this one variable, which gets carried through and results in only
  // creating new nodes instead of modifying existing nodes.
  global $discovery_scope_level;
  $discovery_scope_level = 'new';
  return mercury_utilities_discover_all_submit($form, $form_state);
}

/**
 * Generates the popup for discovery.
 * Used by both the header link and the discovery settings tab.
 * $js is bool(true) for the header link, 'nojs' otherwise.
 */
function mercury_management_rediscover_popup($js = 'nojs') {
  // If there is no form submit here = header discover link, just discover
  // "all" using the most recent datacenter targets.
  global $discovery_scope_level;
  if ($js === true) {
    $discovery_scope_level = 'all';
  }

  $output = '';
  $output .= '<div class = "rediscover-popup">' . drupal_render(drupal_get_form('mercury_management_rediscover_confirm')) . '</div>';
  $output .= '<div class = "processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div>';
  $output .= '<div class="fmc_discovery_meter fmc_discovery_blue"><span id="fmc_discovery_bar"></span></div><div id="fmc_discovery_percent"></div><div id="fmc_discovery_status"></div>';
  return $output;
}

/**
 * Function to set datacenter annotation with vcenter UUID
 */
function mercury_utilities_set_datacenter_annotation($datacenter) {
  $vcenters = db_query("SELECT nid FROM {node} WHERE type = :tp AND status = :st", array(':tp' => 'vcenter', ':st' => 1));
  if (!empty($vcenters)) {
    foreach ($vcenters as $vc) {
      $vc_info = node_load($vc->nid);
    }
  }
  $uuid = $vc_info->field_fmc_uuid['und'][0]['value'];
  $vcenterInfo = exec("/usr/bin/perl /usr/lib/vmware-vcli/apps/general/vcenterdiscover.pl --server " . $vc_info->field_hostname['und'][0]['value'] . " --username " . "'" . fmc_cipher::decrypt($vc_info->field_username['und'][0]['value']) . "'" . " --password " . "'" . fmc_cipher::decrypt($vc_info->field_password['und'][0]['value']) . "'" . " --operation setDataCenterAnnotation --datacenter " . "'" . $datacenter . "'" . " --annotation " . $uuid, $result);
}

/**
 * Set the number of ticks that will occur during this discovery
 */
function mercury_utilities_discovery_progress_init_ticks($ticks) {
  variable_set('rediscover_total_ticks', $ticks);
  variable_set('rediscover_current_ticks', 0);
}

/**
 * On completion, reset to 100%.
 * This is slightly safer than zero, can't have divide by zero errors.
 */
function mercury_utilities_discovery_progress_complete_ticks() {
  variable_set('rediscover_total_ticks', 100);
  variable_set('rediscover_current_ticks', 100);
}

/**
 * Indicate that $ticks ticks have in fact occurred during this discovery
 */
function mercury_utilities_discovery_progress_register_ticks($ticks) {
  $tick_sum = variable_get('rediscover_current_ticks', 0) + $ticks;
  variable_set('rediscover_current_ticks', $tick_sum);
}

/**
 * Function to retrieve the current discovery percentage
 */
function mercury_utilities_get_discovery_percentage() {
  $vcInfo = mercury_management_vcenter_details();
  $rediscover_status = $vcInfo->field_rediscover_status['und'][0]['value'];

  // "start" (normal) and "stop" (starting to cancel) are the two in-progress
  // states.  Empty string and terminated are completed states.
  if (empty($rediscover_status) || $rediscover_status == 'rediscover_terminated') {
    $percentage = 100;
  } else {
    $total_ticks = variable_get('rediscover_total_ticks', 0);
    $current_ticks = variable_get('rediscover_current_ticks', 0);
    if (!$total_ticks || !$current_ticks) {
      $percentage = 0;
    } else {
      if ($current_ticks >= $total_ticks) {
        // Only really complete should return 100%, which redirects.
        // Normally this should never be reached, but just in case...
        $percentage = 99;
      } else {
        $percentage = ($current_ticks * 100) / $total_ticks;
      }
    }
  }
  print 'percentage_discovered---' . intval($percentage);
  exit;
}

/**
 * Function to get async details for Discovery progress from /tmp/discovery.txt file
 */
function mercury_utilities_get_discovery_status() {
  $line_content = '';
  $read_file = mercury_management_get_log_dir() . "/discovery.txt";

  if (file_exists($read_file)) {
    $get_contents = exec('tail -8 ' . $read_file, $file_output);

    foreach ($file_output as $file_data) {
      $line_content = $line_content . $file_data . '<br/>';
    }
  }

  print 'discovery_text---' . $line_content;
  exit;
}