<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for error handling.
 */
function mercury_utilities_set_custom_error_handlers() {

  // For all PHP errors and uncaught exceptions, log a stack trace.
  // TODO: It's possible to detect suppressed error messages in the error
  // handler, but not in error_get_last.
  // Try doing that to only send a 500 when it's not suppressed, in copper.
  // @see mercury_management_init()
  set_error_handler(function($error_level, $message, $filename, $line, $context) {
    /*
     * Only alter behavior for the types we care about.
     * Some errors immediately stop execution and never reach this.
     *
     * Important: the $error_types parameter to set_error_handler doesn't
     * work here because it would avoid the Drupal error handler for NOTICE,
     * etc. by setting anything not in the list to use the PHP default.
     * That prints the NOTICE and breaks some popup responses and such.
     */
    if (!in_array($error_level, array(E_ERROR, E_USER_ERROR, E_RECOVERABLE_ERROR))) {
      return mercury_utilities_drupal_error_handler_real($error_level, $message, $filename, $line, $context);
    }

    // Do not log this in the footer.  It is usually large.
    // TODO: Export all views to code so we can easily modify them.
    // The console log and export hide 'php', hence 'PHP Trace'.
    mercury_utilities_fmc_log_message(
      'PHP Trace', // TODO: change to 'php' in copper.  See above.
      'Error trace for ' .
      check_plain($message) . ":<br/>\n" .
      mercury_utilities_format_backtrace(debug_backtrace()), WATCHDOG_ERROR, NULL, NULL, TRUE
    );

    // Save a separate footer log entry to say there are details elsewhere.
    mercury_utilities_fmc_set_message(
      t("An error occurred, and data that may be useful to !brand support has been logged.<br/>\nUse Export Logs from Console Settings > Logs to extract this data.", array('!brand' => mercury_utilities_get_fmc_brand_string())), 'error'
    );

    // Always continue with the existing error handlers.
    return mercury_utilities_drupal_error_handler_real($error_level, $message, $filename, $line, $context);
  });

  // @see set_error_handler above.
  set_exception_handler(function(Exception $e) {
    // Log the details.
    mercury_utilities_log_exception($e);

    // Since this was uncaught, no other code added a status message.
    // Add one so the user knows to look for the issue.
    mercury_utilities_fmc_set_message(
      t("An error occurred, and data that may be useful to !brand support has been logged.<br/>\nUse Export Logs from Console Settings > Logs to extract this data.", array('!brand' => mercury_utilities_get_fmc_brand_string())), 'error'
    );

    return _drupal_exception_handler($e);
  });

  // This is a workaround that can be deleted in PHP5.4
  // Without it, php fatal errors come back as 200 instead of 500.
  // @see http://bugs.xdebug.org/view.php?id=587
  // XXX: Someone added an ini_set to 1 in index.php,
  // better to just delete that and fix php.ini instead.
  ini_set('display_errors', '0');

  // The error *handler* cannot catch runtime fatal errors,
  // so log fatal errors this way instead.
  // WARNING: This still gets triggered for suppressed errors!
  // Only the error *handler* can tell if suppressed.
  // Compile errors can only be found in the Apache error logs.
  drupal_register_shutdown_function(function() {
    $error = error_get_last();
    if (!empty($error) && in_array($error['type'], array(E_ERROR, E_USER_ERROR, E_RECOVERABLE_ERROR))) {
      mercury_utilities_fmc_log_message(
        'PHP Trace', // TODO: change to 'php' in copper.  See above.
        var_export($error, true),
        WATCHDOG_ERROR, NULL, NULL, TRUE
      );
    }
  });
}

function mercury_utilities_log_exception(Exception $e) {
  mercury_utilities_fmc_log_message(
    'PHP Trace', // TODO: change to 'php' in copper.  See above.
    'Exception trace for ' .
    check_plain($e->getMessage()) . ":<br/>\n" .
    mercury_utilities_format_backtrace($e->getTrace()), WATCHDOG_ERROR, NULL, NULL, TRUE
  );
}

// TODO: Change this to use format_backtrace instead or remove it entirely
// after https://drupal.org/node/1158322 merges.
// Most of this code will be in Drupal core, hopefully soon!
function mercury_utilities_format_backtrace(array $backtrace) {
  $return = '';
  foreach ($backtrace as $trace) {
    $call = array('function' => '', 'args' => array());
    if (isset($trace['class'])) {
      $call['function'] = $trace['class'] . $trace['type'] . $trace['function'];
    } elseif (isset($trace['function'])) {
      $call['function'] = $trace['function'];
    } else {
      $call['function'] = 'main';
    }
    if (isset($trace['args'])) {
      foreach ($trace['args'] as $arg) {
        if (is_scalar($arg)) {
          $call['args'][] = is_string($arg) ? '\'' . filter_xss($arg) . '\'' : $arg;
        } else {
          $call['args'][] = ucfirst(gettype($arg));
        }
      }
    }

    // This part is currently custom but hopefully will be similar to the core
    // version eventually.
    $call['location'] = '';
    if (!empty($trace['file']) && !empty($trace['line'])) {
      $call['location'] = ' at ' . check_plain($trace['file']) . ':' . intval($trace['line']);
    }

    $return .= $call['function'] . '(' . implode(', ', $call['args']) . "){$call['location']}<br/>\n";
  }
  return $return;
}

/**
 * We have to override this function because the default
 * always says the source is the closure above.
 * Exception handling uses the Exception's trace, so it's not needed there.
 * @see _drupal_error_handler_real
 */
function mercury_utilities_drupal_error_handler_real($error_level, $message, $filename, $line, $context) {
  if ($error_level & error_reporting()) {
    $types = drupal_error_levels();
    list($severity_msg, $severity_level) = $types[$error_level];

    // BEGIN MODIFIED SECTION
    $trace = debug_backtrace();
    // Remove one layer of function.
    // _drupal_get_last_caller will remove one more,
    // and then we'll be at the real source instead of the closure above.
    array_shift($trace);
    $caller = _drupal_get_last_caller($trace);
    // END MODIFIED SECTION

    if (!function_exists('filter_xss_admin')) {
      require_once DRUPAL_ROOT . '/includes/common.inc';
    }

    // We treat recoverable errors as fatal.
    _drupal_log_error(array(
      '%type' => isset($types[$error_level]) ? $severity_msg : 'Unknown error',
      // The standard PHP error handler considers that the error messages
      // are HTML. We mimick this behavior here.
      '!message' => filter_xss_admin($message),
      '%function' => $caller['function'],
      '%file' => $caller['file'],
      '%line' => $caller['line'],
      'severity_level' => $severity_level,
      ), $error_level == E_RECOVERABLE_ERROR);
  }
}

/**
 * For linking to online help articles within error messages and warnings.
 * @param int $article_id
 * @param string $text optional link text, default 'Online Help'
 * @return string HTML, pre-sanitized
 */
function mercury_utilities_error_get_help_link($article_id, $text = 'Online Help') {
  // "context" is the identifier for the entire help app and is a constant.
  return t(
    '<a href="/fa_help/wwhelp/wwhimpl/api.htm?context=GUID-965EF899-19AC-4D1A-9057-78660229ACC0&topic=@id" target="_blank">@text</a>',
    array('@id' => $article_id, '@text' => $text)
  );
}
