<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for general host actions.
 */

/**
 * Function to check the VIB Package for Host Upgrade
 */
function mercury_utilities_check_upgrade_host($host_nid) {

  $get_node_info = node_load($host_nid);
  $node_vib_version = $get_node_info->field_vib_version['und'][0]['value'];
  if (empty($node_vib_version)) {
    mercury_utilities_fmc_log_message('Host Agent', t('Host agent could not be upgraded on @host, as the host does not report any version. Please check on the host status.', array('@host' => $get_node_info->title)), WATCHDOG_ERROR);
    return 'no-upgrade';
  }

  $host_package_agent_version = variable_get('host_package_agent_version', '');
  if (empty($host_package_agent_version)) {
    mercury_utilities_fmc_log_message('Host Agent', t('Host agent could not be upgraded on @host, as the agent package version could not be determined. Please check on the uploaded agent package.', array('@host' => $get_node_info->title)), WATCHDOG_ERROR);
    return 'no-upgrade';
  }

  // XXX: This is inconsistent with the VM version, BURT 750383
  // @see mercury_utilities_check_upgrade_vm
  if (!mercury_utilities_version_compare(
    $host_package_agent_version, $node_vib_version,
    $max = false, $major_minor = false, $allow_equal = false
  )) {
    mercury_utilities_fmc_log_message('Host Agent', t('Host agent could not be upgraded on @host, as it is newer than the version currently available to FAMC.', array('@host' => $get_node_info->title)), WATCHDOG_ERROR);
    return 'no-upgrade';
  }

  return 'upgrade_vib';
}

/**
 * Function for calling the refresh function for individual host
 */
function mercury_utilities_host_refresh_action($host_id, $norefresh = "") {
  $host = node_load($host_id);
  $dcInfo = node_load($host->field_datacenter['und'][0]['target_id']);
  $vcInfo = node_load($dcInfo->field_related_vcenter['und'][0]['target_id']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $username = mercury_management_get_host_username($host);
  $password = mercury_management_get_host_password($host);

  if ($username != '' && $password != '') {
    $host_name = $host->title;
    $result = mercury_management_updatehost($vc_hostname, $vc_username, $vc_password, trim($host_name), $dcInfo->nid, FALSE);
    $error_check = explode('--', $result);
    if (count($error_check) == 1) {
      mercury_utilities_fmc_log_message('Host Refresh', t('Host @host_name refreshed.', array('@host_name' => $host_name)));
    } else {
      mercury_utilities_fmc_log_message('Host Refresh', t('Host @host_name refresh unsuccessful.', array('@host_name' => $host_name)), WATCHDOG_ERROR);
    }
  } else {
    mercury_utilities_fmc_log_message('Host Refresh', t('Refresh failed on host "@host" as no username or password is defined.', array('@host' => $host->title)), WATCHDOG_ERROR);
  }
  if ($norefresh == '') {
    drupal_goto($_SERVER['HTTP_REFERER']);
  }
}

function mercury_utilities_host_settings() {
  $output = '';
  $output .= '<div id= "host-settings">' . drupal_render(drupal_get_form('mercury_utilities_host_settings_form')) . '</div>';
  return $output;
}

function mercury_utilities_host_settings_form() {
  //mercury_utilities_set_host_agent_package();
  $form = array();
  $form['host_username'] = array(
    '#type' => 'textfield',
    '#title' => t('User Name'),
    '#description' => ('Please enter the username.'),
    '#default_value' => fmc_cipher::decrypt(variable_get('host_username', '')),
    '#required' => true,
    '#prefix' => '<div>Please enter the default username and password for the hosts managed by this vCenter. These credentials are used to install the Flash Accel host agent. Following discovery, if there are hosts in your datacenter that use non-default credentials, you will need to override these default values for those hosts using the "edit" option.</div>',
  );
  $form['host_password'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#description' => ('Please enter the password.'),
    '#attributes' => array('value' => fmc_cipher::decrypt(variable_get('host_password', ''))),
    '#required' => true
  );

  /*
    $form['host_upgraded_zip'] = array(
    '#type' => 'textfield',
    '#title' => t('Host Agent Package'),
    '#attributes' => array('value' => variable_get('host_cur_zip', ''), 'readonly' => 'readonly'),
    '#prefix' => '<table><tr><td width=450px>',
    '#suffix' => '</td>',
    );
    $form['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#submit' => array('mercury_utilites_update_host_package'),
    '#prefix' => '<td align=left style="padding-top:13px">',
    '#suffix' => '</td></tr></table>',
    );
   */
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
    '#submit' => array('host_settings_log_settings'),
  );

  return $form;
}

function host_settings_log_settings($form, $form_state) {
  $values = $form_state['values'];
  variable_set('host_username', fmc_cipher::encrypt(trim($values['host_username'])));
  // DO NOT trim the password; leading and trailing spaces are supported
  variable_set('host_password', fmc_cipher::encrypt($values['host_password']));
  mercury_utilities_fmc_log_message('System Settings', t('Host settings saved successfully'), WATCHDOG_INFO, 'show');
}

// XXX: This is no longer used (button is commented out)
function mercury_utilites_update_host_package($form, $form_state) {
  $updateHost = exec("/var/www/updateFromIso.sh esx", $result);
  mercury_utilities_fmc_log_message('Host Agent', t($updateHost));
  if (!empty($result)) {
    foreach ($result as $output) {
      if (stristr($output, 'Error')) {
        $error = explode(':', $output);
        mercury_utilities_fmc_log_message('Host Agent', t(' @msg', array('@msg' => $error[1])), WATCHDOG_WARNING, 'show');
      } else {
        mercury_utilities_fmc_log_message('Host Agent', t('Updated to @msg', array('@msg' => $output)), WATCHDOG_NOTICE, 'show');
      }
    }
  }
}
