<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains a "library" of general use functions.
 */

function mercury_utilities_get_fmc_asup_uuid($reset = false) {
  $cache = cache_get('fmc_uuid');
  if (empty($cache->data) || $reset) {
    $config = parse_ini_file('/etc/flashaccel/asup.conf');
    if (empty($config['systemID'])) {
      return '';
    }
    $my_uuid = trim($config['systemID']);
    cache_set('fmc_uuid', $my_uuid);
  } else {
    $my_uuid = $cache->data;
  }
  return $my_uuid;
}

function mercury_utilities_get_fmc_brand_string($reset = false) {
  $cache = cache_get('fmc_brand');
  if (empty($cache->data) || $reset) {
    preg_match('/VENDOR_NAME="([^\"]*)"/', file_get_contents(FMC_BRAND_FILE), $matches);
    $brand = $matches[1];
    cache_set('fmc_brand', $brand);
  } else {
    $brand = $cache->data;
  }
  return $brand;
}

function mercury_utilities_get_fmc_copyright_string($reset = false) {
  $cache = cache_get('fmc_copyright');
  if (empty($cache->data) || $reset) {
    preg_match('/COPYRIGHT="([^\"]*)"/', file_get_contents(FMC_BRAND_FILE), $matches);
    $copyright = $matches[1];
    cache_set('fmc_copyright', $copyright);
  } else {
    $copyright = $cache->data;
  }
  return $copyright;
}

function mercury_utilities_get_fmc_version_string($reset = false) {
  $cache = cache_get('fmc_version');
  if (empty($cache->data) || $reset) {
    $version = trim(file_get_contents(FMC_FULL_VERSION_FILE));
    cache_set('fmc_version', $version);
  } else {
    $version = $cache->data;
  }
  return $version;
}

/**
 * Sometimes we have to prevent certain behaviors when the unit tests are
 * running, typically because they have a database dependency.  Use this.
 * @return boolean
 */
function mercury_utilities_unit_tests_running() {
  global $drupal_test_info;
  if (!empty($drupal_test_info)) {
    return true;
  }
  return false;
}

/**
 * Compare two version strings. This is a wrapper for PHP's version_compare
 * that is designed to simplify our specific use cases.
 * @param string $version The version to check
 * @param string $min Version must be >= this if provided
 * @param string $max Version must be <= this if provided
 * @param boolean $major_minor if true compare only major and minor versions
 * @param boolean $allow_equal if false this will fail if identical versions
 * @return boolean true if the $version passes the checks
 */
function mercury_utilities_version_compare($version, $min = false, $max = false, $major_minor = true, $allow_equal = true) {
  if (empty($version) || (empty($min) && empty($max))) {
    // This is a misuse of the function and undefined behavior.
    throw new \Exception('Version compare requires $version and either $min or $max!');
  }

  // Patch and build numbers typically do not affect compatibility.
  // NOTE: Some versions have hyphens instead of periods, but version_compare
  // converts to periods automatically.  In our code the hyphen is only for
  // the build number so it doesn't apply to $major_minor's logic.
  if ($major_minor) {
    // This pattern assumes at least two sections, which ours always have.
    $version = (string) floatval($version);
    if (!empty($min)) {
      $min = (string) floatval($min);
    }
    if (!empty($max)) {
      $max = (string) floatval($max);
    }
  } else {
    // Not major_minor, so we have to deal with R-releases.
    // 1.2.0R1 is "higher" than 1.2.0, but that's backward of the typical use:
    // 1.2.0RC4 < 1.2.0 final.  So the built-in version compare doesn't help.
    // Solution: R-releases never have build numbers in the string, so treat
    // the R-release as a large build number.
    // Caveat: We only have experimental evidence of R releases never using the
    // build number for the win agent, and there's no guarantee the rules won't
    // change in the future regardless.  IMO the most reliable thing to do is
    // also increase the patch number.
    $version = str_replace('R', '.9999999999', $version);
    if (!empty($min)) {
      $min = str_replace('R', '.9999999999', $min);
    }
    if (!empty($max)) {
      $max = str_replace('R', '.9999999999', $max);
    }
  }

  if (!empty($min)) {
    $op = '>=';
    if (!$allow_equal) {
      $op = '>';
    }
    // Returns true or false if $op specified.
    $comp = version_compare($version, $min, $op);
    if (!$comp) {
      return false;
    }
  }

  if (!empty($max)) {
    $op = '<=';
    if (!$allow_equal) {
      $op = '<';
    }
    // Returns true or false if $op specified.
    $comp = version_compare($version, $max, $op);
    if (!$comp) {
      return false;
    }
  }

  return true;
}

/**
 * XXX: NOT USED
 */
function mercury_utilities_total_node_counter() {
  $node_counter = array();
  $host_query = db_select('node', 'node');
  $host_query->condition('node.status', 1)
    ->condition('node.type', 'host');
  $host_query->fields('node', array('nid'));
  $temp_host_result = $host_query;
  $host_result = $host_query->execute();
  $host_count = $temp_host_result->countQuery()->execute()->fetchField();
  $node_counter['total_host'] = $host_count;
  $flash_counter = 0;
  while ($host_row = $host_result->fetch()) {
    $host_node = mercury_utilities_node_load_uncached($host_row->nid);
    $bundle_install = field_get_items('node', $host_node, 'field_bundle_status');
    if ($bundle_install[0]['value'] == "Agent Installed") {
      $flash_counter++;
    }
  }
  $node_counter['total_flash_device'] = $flash_counter;

  $vm_query = db_select('node', 'node');
  $vm_query->condition('node.status', 1)
    ->condition('node.type', 'virtual_machine');
  $vm_query->fields('node', array('nid'));
  $temp_vm_result = $vm_query;
  $vm_count = $temp_vm_result->countQuery()->execute()->fetchField();
  $vm_result = $vm_query->execute();
  $vdisk_count = 0;
  $total_flash_vm = 0;
  while ($vm_row = $vm_result->fetch()) {
    $vm_id = $vm_row->nid;
    $vm_node = mercury_utilities_node_load_uncached($vm_id);
    $driver_status = field_get_items('node', $vm_node, 'field_driver_status');
    if ($driver_status[0]['value'] == "Agent Installed") {
      $total_flash_vm++;
    }
    $vdisk_query = db_select('node', 'node');
    $vdisk_query->leftJoin('field_data_field_virtual_machine', 'field_data_field_virtual_machine', 'node.nid = field_data_field_virtual_machine.entity_id AND field_data_field_virtual_machine.entity_type = :node AND field_data_field_virtual_machine.deleted = :deleted', array(':node' => 'node', ':deleted' => 0));
    $vdisk_query->fields('node', array('nid', 'title'));
    $vdisk_query->condition('node.status', 1)
      ->condition('field_data_field_virtual_machine.field_virtual_machine_target_id', $vm_id)
      ->condition('node.type', 'vdisk');
    $vdisk_counter = $vdisk_query->countQuery()->execute()->fetchField();
    $vdisk_count += $vdisk_counter;
  }
  $node_counter['total_vm'] = $vm_count;
  $node_counter['total_vdisk'] = $vdisk_count;
  $node_counter['total_flash_vm'] = $total_flash_vm;
  return $node_counter;
}

// XXX: NOT USED
function mercury_utilities_total_flash_on_host($host_id) {
  $result = array();
  $total_flash_vm = 0;
  $vm_query = db_select('node', 'node');
  $vm_query->leftJoin('field_data_field_host', 'field_data_field_host', 'node.nid = field_data_field_host.entity_id');
  $vm_query->fields('node', array('nid', 'title'));
  $vm_query->condition('node.status', 1)
    ->condition('field_data_field_host.field_host_target_id', $host_id)
    ->condition('node.type', 'virtual_machine');
  $count_query = $vm_query;
  $vm_result = $vm_query->orderBy('node.title', 'DESC')
    ->execute();
  $total_vm = $count_query->countQuery()->execute()->fetchField();
  while ($vm_row = $vm_result->fetch()) {
    $vm_node = mercury_utilities_node_load_uncached($vm_row->nid);
    $driver_status = field_get_items('node', $vm_node, 'field_driver_status');
    if ($driver_status[0]['value'] == "Agent Installed") {
      $total_flash_vm++;
    }
  }
  $result['total_flash_vm'] = $total_flash_vm;
  $result['total_vm'] = $total_vm;
  return $result;
}

/**
 * Function to get the latest file from a directory
 * @dir = Absolute directory path
 */
function mercury_utilities_get_latestFile($dir, $fmc_agent = NULL) {
  if (empty($dir)) {
    return;
  } else if (!file_exists($dir)) {
    $lastModFile = 'not_found';
    return $lastModFile;
  }

  $lastMod = 0;
  $lastModFile = '';
  if (!empty($fmc_agent)) {
    foreach (scandir($dir) as $entry) {
      if (is_file($dir . $entry)) {
        $agent_file_info = pathinfo($dir . $entry);
        if (strtolower($agent_file_info['extension']) == 'deb') {
          return $entry;
        } else {
          $lastModFile = 'not_found';
        }
      } else {
        $lastModFile = 'not_found';
      }
    }
  } else {
    foreach (scandir($dir) as $entry) {
      if (is_file($dir . $entry) && filectime($dir . $entry) > $lastMod) {
        $lastMod = filectime($dir . $entry);
        $lastModFile = $entry;
      }
    }
  }

  return $lastModFile;
}

/**
 * Loads a single node and imediately cleans out the entity cache
 * to avoid spiking PHP memory usage when we're iterating through all
 * nodes in a large datacenter.
 * $nid = Node ID.
 */
function mercury_utilities_node_load_uncached($nid) {

  return node_load($nid, NULL, TRUE);

  // The following code seems preferable, but does not achieve the
  // required memory savings even when every single node loaded is
  // immediately removed from the cache.  I suspect this is because
  // using unset on an entry in the element cache does not free
  // all of the memory associated with that entry.
  //
   // $node_controller = entity_get_controller('node');
  // $nids = array($nid);
  // $conditions = array();
  // $nodes = $node_controller->load($nids, $conditions);
  // $node_controller->resetCache($nids);
  // return $nodes ? reset($nodes) : FALSE;
}

/**
 * Loads all published nodes of a given type efficiently.
 * @param string $type 'host' etc.
 * @return array of Node objects or NULL.
 */
function mercury_utilities_node_load_all($type) {
  $query = new EntityFieldQuery();

  $query->entityCondition('entity_type', 'node')
    ->entityCondition('bundle', $type)
    ->propertyCondition('status', 1);

  $result = $query->execute();

  if (isset($result['node'])) {
    // There are potentially many results here, do not cache.
    return node_load_multiple(array_keys($result['node']), array(), true);
  }
}

/**
 * Because host/vm/etc. do not share fields directly, their node object store
 * the data in different places.  This function abstracts getting them.
 * @param stdClass $node Node object
 * @param string $field 'username' etc.
 * @return mixed The value of the field in the node.
 * @todo add stdClass type check when we have soak time
 */
function mercury_utilities_get_node_field($node, $field) {

  // Host uses 'field_host_username' etc.
  $infix = '';
  switch ($node->type) {
    case 'host': $infix = 'host_';
      break;
    case 'virtual_machine': $infix = 'vm_';
      break;
  }

  // Check for a field using the infix first.
  $full_field = "field_{$infix}{$field}";
  if (!isset($node->$full_field)) {
    // Otherwise it's in the default location.
    $full_field = "field_$field";
  }
  if (!isset($node->$full_field)) {
    return;
  }

  $var_field = $node->$full_field;
  return $var_field['und'][0]['value'];
}

/**
 * Because host/vm/etc. do not share fields directly, their node object store
 * the data in different places.  This function abstracts setting them.
 * @param stdClass $node Node object
 * @param string $field 'username' etc.
 * @param mixed $value the new value for the field
 */
function mercury_utilities_set_node_field($node, $field, $value) {

  // Host uses 'field_host_username' etc.
  $infix = '';
  switch ($node->type) {
    case 'host': $infix = 'host_';
      break;
    case 'virtual_machine': $infix = 'vm_';
      break;
  }

  // Check for a field using the infix first.
  $full_field = "field_{$infix}{$field}";
  if (!isset($node->$full_field)) {
    // Otherwise it's in the default location.
    $full_field = "field_$field";
  }

  $node->{$full_field}['und'][0]['value'] = $value;
  $node->{$full_field}['und'][0]['safe_value'] = check_plain($value);
}

/**
 * Function to get application environment based on the version info
 * Will return one of 'production', 'debug' or 'internal'
 */
function mercury_utilities_application_env($reset = false) {

  // The env does not normally change except via redeploy.
  if (!$reset) {
    $cache = cache_get('fmc_application_env');
    if (!empty($cache->data)) {
      return $cache->data;
    }
  }

  // Assume environment is production
  $env = 'production';

  $version = mercury_utilities_get_fmc_version_string();

  // Split into four part array
  $version_array = preg_split('/\./', $version);

  // The fourth part must be longer than one char
  if (strlen($version_array[3]) > 1) {

    // Get the last digit
    $last_digit = substr($version_array[3], -1);

    /**
     * As defined in .../nightly_build/lib/Mercury/Build/DriverVersion.pm
     * -------------------------------------------------------------------------------
     *       - Release build            (no promote id)-    '0'
     *       - Emergency release build  PromoteID 'R' -     '1'
     *       - Patch                    PromoteID 'P' -     '2'
     *       - Debug                    PromoteID 'D' -     '3'
     *       - BlackDuck                PromoteID 'B' -     '6'
     *       - Release Candidate        PromoteID 'RC'-     '7'
     *       - Non-release build        PromoteID 'X'-      '8'
     *       - Dev Build and all others PromoteID NA -      '9'
     * ------------------------------------------------------------------------------
     */
    // See if we have a debug-build
    if ($last_digit == 3) {
      $env = 'debug';

      // None of these should go outside
    } elseif ($last_digit >= 6) {
      $env = 'internal';
    }
  }

  cache_set('fmc_application_env', $env);

  return $env;
}
