<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for the logging tab and logging messages.
 */


/**
 * Similar to drupal_set_message() function
 * Used for custom messege logs in Common Footer
 * @param $message
 * @param $type
 * @param $repeat
 */
function mercury_utilities_fmc_set_message($message = NULL, $type = 'status', $show_message = NULL) {
  $repeat = TRUE;
//Dispaly the message
  if ($show_message == 'show') {
    drupal_set_message($message, $type);
  }

  $get_mesg = variable_get('fmc-messages');
  $_SESSION['fmc-messages'] = $get_mesg;
  $set_messages = array();
  $log_level = variable_get('log_level', '0');
  $log_message = 0;
  if ($log_level == 0) {
    $log_message = 1;
  }
  if ($log_level == 1) {
    if ($type != "status") {
      $log_message = 1;
    }
  }
  if ($log_level == 2) {
    if ($type != "status" && $type != "warning") {
      $log_message = 1;
    }
  }
  if ($message && $log_message == 1) {
    if (!isset($_SESSION['fmc-messages'])) {
      $_SESSION['fmc-messages'] = array();
    }

    if ($repeat || !in_array($message, $_SESSION['fmc-messages'])) {
      $_SESSION['fmc-messages'][] = '<span class="' . $type . '">' . date('m/d/Y H:i') . ': ' . $message . '</span>';
    }

    if (isset($_SESSION['fmc-messages'])) {
      if (count($_SESSION['fmc-messages']) > 100) {
        array_shift($_SESSION['fmc-messages']);
        variable_set('fmc-messages', $_SESSION['fmc-messages']);
      } else {
        variable_set('fmc-messages', $_SESSION['fmc-messages']);
      }
    } else {
      if (count($get_mesg) > 100) {
        array_shift($get_mesg);
        variable_set('fmc-messages', $get_mesg);
      } else {
        variable_set('fmc-messages', $get_mesg);
      }
    }
  }

  $get_mesg_final = variable_get('fmc-messages');

  return $get_mesg_final;
}

/*
 * Function for logging the watchdog log apporpiate to the log-level
 */

function mercury_utilities_fmc_log_message($type, $message, $severity = WATCHDOG_NOTICE, $show_message = NULL, $link = NULL, $skip_footer_log = NULL) {
  $variables = array();
  $log_level = variable_get('log_level', '0');
  if ($log_level == 0) {
    watchdog($type, $message, $variables, $severity, $link);
  }
  if ($log_level == 1) {
    if ($severity != WATCHDOG_NOTICE && $severity != WATCHDOG_INFO) {
      watchdog($type, $message, $variables, $severity, $link);
    }
  }
  if ($log_level == 2) {
    if ($severity != WATCHDOG_NOTICE && $severity != WATCHDOG_INFO && $severity != WATCHDOG_WARNING) {
      watchdog($type, $message, $variables, $severity, $link);
    }
  }

  if ($severity == WATCHDOG_WARNING) {
    $type = 'warning';
  } elseif ($severity == WATCHDOG_ERROR) {
    $type = 'error';
  } else {
    $type = 'status';
  }

  if (empty($skip_footer_log)) {
    mercury_utilities_fmc_set_message($message, $type, $show_message);
  }
}


/**
 * Function to provide Log Settings interface
 */
function mercury_utilities_log_settings_form() {
  $form = array();
  $form['log_description_title'] = array(
    '#type' => 'markup',
    '#markup' => t('<h2>Log Settings</h2>'),
  );
  $form['fmc_log_settings'] = array(
    '#type' => 'fieldset',
    '#title' => 'Log Level',
    '#collapsible' => false,
    '#collapsed' => false
  );
  $form['fmc_log_settings']['log_description'] = array(
    '#type' => 'markup',
    '#markup' => t('<b>Choose which type of messages should be saved in the system message log. Note: your selection will apply to all future messages, but will not cause messages already in the log to be removed.</b>'),
  );

  $form['fmc_log_settings']['log_level'] = array(
    '#type' => 'select',
    '#options' => array('0' => 'Log all messages', '1' => 'Log errors and warnings only', '2' => 'Log errors only'),
    '#default_value' => variable_get('log_level', '0'),
  );

  $form['fmc_log_settings']['log_level_save'] = array(
    '#type' => 'submit',
    '#value' => 'Save Configuration',
    '#submit' => array('mercury_utilities_save_log_level'),
    '#prefix' => '<div class="log_level_save">',
    '#suffix' => '</div><script>
             	jQuery("#edit-log-level-save").live("click", function() {
  							jQuery("#edit-log-level-save").attr("disabled", "disabled");
             	});
  					</script>'
  );

  $form['fmc_log_export'] = array(
    '#type' => 'fieldset',
    '#title' => 'Log Export',
    '#collapsible' => false,
    '#collapsed' => false
  );
  $form['fmc_log_export']['export_description'] = array(
    '#type' => 'markup',
    '#markup' => t('<b>Click the Export Logs button to save the system message log to a file.</b>'),
  );

  // Note: This floats right, so buttons render in opposite order.
  $form['fmc_log_export']['export_all_logs'] = array(
    '#type' => 'submit',
    '#value' => 'Export Full Debug Log',
    '#title' => 'Exports all log files known to FMC to assist with debugging.',
    '#submit' => array('famc_asup_download'),
    '#prefix' => '<div class="export_log_button">',
    '#suffix' => '</div>',
  );
  $form['fmc_log_export']['export_log'] = array(
    '#type' => 'submit',
    '#value' => 'Export Logs',
    '#submit' => array('mercury_utilities_export_log_file'),
    '#prefix' => '<div class="export_log_button">',
    '#suffix' => '</div>',
  );
  return $form;
}

/**
 * Function to save the log level on the settings page
 */
function mercury_utilities_save_log_level($form, &$form_state) {
  $values = $form_state['values'];
  variable_set('log_level', $values['log_level']);
  mercury_utilities_fmc_log_message('System Settings', t('Log-level Settings have been saved successfully.'), WATCHDOG_NOTICE, 'show');
}

/**
 * Function to export watchdog log file.
 */
function mercury_utilities_export_log_file() {
  // This is a different display of the "Console Logs Report" view
  // and its contents generally match the normal page view.
  drupal_goto("export-console-log");
}
