<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for users, permissions, and authentication.
 */

/**
 *  hook_permission for the mercury management console
 */
function mercury_utilities_permission() {
  return array(
    'administer mercury_utilities settings' => array(
      'title' => t('Administer Mercury Utilities'),
      'description' => t('Perform maintenance tasks for Mercury Utilities'),
    ),
    'access mercury_utilities' => array(
      'title' => t('Access Mercury Utilities'),
      'description' => t('Access Mercury Utilities'),
    ),
  );
}

/**
 *  hook_menu-alter for the fmc console
 */
function mercury_utilities_menu_alter(&$items) {
  global $user;

  $items['user']['access callback'] = 'mercury_utilities_user_access_check';
}

/**
 *  function to check whether a user has access to the user page, only the super-admin sees this, otherwise it will be access denied
 */
function mercury_utilities_user_access_check() {
  global $user;
  if ($user->uid == 1 || !$user->uid) {
    return TRUE;
  }
  return FALSE;
}

/**
 *  hook_form_alter for the User Edit Page and User Login Form
 */
function mercury_utilities_form_alter(&$form, &$form_state, $form_id) {
  if ($form_id == 'user_profile_form') {
    $form['account']['current_pass']['#description'] = 'Enter your current password to change the E-mail address or Password.';
  }
  //Refactor the invalid user login message
  if ($form_id == 'user_login') {
    $form['#validate'][] = 'mercury_utilities_user_login_check';
  }

  //Username/Password blank validation for Host/VM Settings Form
  if ($form_id == 'mercury_utilities_host_settings_form' || $form_id == 'mercury_utilities_vm_settings_form') {
    $form['#validate'][] = 'mercury_utilities_console_settings_validation';
  }
}

/**
 * Function to validate the username/password for Host/VM settings form
 */
function mercury_utilities_console_settings_validation(&$form, &$form_state) {
  global $_SESSION;
  if (isset($_SESSION['messages']['error'])) {
    $message = 'Sorry, username or password can\'t be blank.';
    unset($_SESSION['messages']['error']);
    $_SESSION['messages']['error'][] = $message;
  }
}

/**
 * Function to Refactor the Invalid user login message
 */
function mercury_utilities_user_login_check(&$form, &$form_state) {
  global $_SESSION;

  if (isset($_SESSION['messages']['error'])) {
    $message = 'Sorry, username or password is invalid.';
    unset($_SESSION['messages']['error']);
    $_SESSION['messages']['error'][] = $message;
  }
}

/**
 * Function for calling the user settings for normal user
 */
function mercury_utilities_user_settings_form($form, &$form_state, $account, $category = 'account') {
  global $user;
  // During initial form build, add the entity to the form state for use during
  // form building and processing. During a rebuild, use what is in the form
  // state.
  if (!isset($form_state['user'])) {
    $form_state['user'] = $account;
  } else {
    $account = $form_state['user'];
  }

  // @todo Legacy support. Modules are encouraged to access the entity using
  //   $form_state. Remove in Drupal 8.
  $form['#user'] = $account;
  $form['#user_category'] = $category;
  $form['account']['current_pass'] = array(
    '#title' => 'Current password',
    '#type' => 'password',
    '#size' => 25,
    '#required' => true,
    '#description' => t('Please enter the Current password to change User settings.'),
  );
  $form['account']['pass'] = array(
    '#type' => 'password_confirm',
    '#size' => 25,
    '#description' => t('To change the current user password, enter the new password in both fields.'),
  );

  // Add the timezone fields.
  system_user_timezone($form, $form_state);

  $form['submit'] = array(
    '#type' => 'submit',
    '#weight' => '999', // Timezone is set to 6.
    '#value' => t('Save'),
    '#prefix' => '<script>
      		jQuery(".form-type-password.form-item-pass-pass1 label").html("New password");
      		</script>'
  );

  return $form;
}

/**
 * Function for calling the user settings for normal user
 */
function mercury_utilities_user_settings_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  require_once DRUPAL_ROOT . '/' . variable_get('password_inc', 'includes/password.inc');
  if (user_check_password($form_state['values']['current_pass'], $form_state['user']) == TRUE) {
    module_load_include('pages.inc', 'user', 'user');
    user_profile_form_submit($form, $form_state);

    // We also need to change the *default* timezone, which is used in cron.
    if (!empty($form_state['values']['timezone'])) {
      variable_set('date_default_timezone', $form_state['values']['timezone']);
    }

    $keys = array_keys($_SESSION['messages']['status']);
    $last_key = end($keys);
    $message = 'Settings updated successfully.';

    //Substitute default message by custom message
    $_SESSION['messages']['status'][$last_key] = $message;
    mercury_utilities_fmc_log_message('System Settings', $message, WATCHDOG_NOTICE);
  } else {
    mercury_utilities_fmc_log_message('System Settings', 'Settings have not been saved because current password is incorrect', WATCHDOG_WARNING, 'show');
  }
}
