<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for common or site-wide validation.
 */

/**
 * Function to display the Rediscover PopUp
 * when Inventory discovery process is on Progress *
 */
//Load the Rediscover PopUp when Rediscover process is going on
function mercury_utilities_rediscover_popup() {
  $vcInfo = mercury_management_vcenter_details();
  if (!$vcInfo) {
    // vCenter node not configured correctly yet. Nothing more to do.
    return;
  }

  $check_rediscover = $vcInfo->field_rediscover_status['und'][0]['value'];

  // Not doing discovery and state is already clean.
  // Do not modify the node on empty.  It creates a race condition.
  if (empty($check_rediscover)) {
    return;
  }

  // If discovery is not in progress or cancelling, nothing to do here.
  if ($check_rediscover != 'rediscover_start' && $check_rediscover != 'rediscover_stop') {
    return;
  }

  // Discovery is in progress.  May need to show the popup.

  //Clear Expand all Cookie
  $_COOKIE['expand_all'] = 0;
  $_COOKIE['expand_host'] = 0;

  //Do not load the PopUp dialog box in these URLs
  $ignore_paths = array(
    'console-logs-report',
    'user',
    'admin',
    'fa_help',
    'confirm',
    'view',
    'rediscover',
    'discovery',
  );

  $path_start = explode('/', current_path());
  if (count($path_start) > 0) {
    $path_get = $path_start[0];
  } else {
    $path_get = current_path();
  }

  if (in_array($path_get, $ignore_paths)) {
    return;
  }

  // This may be triggered during an AJAX request.
  // If so, do an AJAX redirect.  The target page will show the popup.
  if (
    (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && $_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest') || $_GET['q'] == 'system/ajax' // Some forms do not post via jQuery.
  ) {
    return ctools_popups_ajax_redirect();
  }

  //Create the PopUp for Rediscover process going on
  $output = '<div id="popup_rediscover" style="display: none;"><div style="z-index: 1000; position: absolute; left: 0px; margin: 0px; background: none repeat scroll 0% 0% rgb(255, 255, 255); opacity: 0.55; top: 0px;" id="modalBackdrop"></div>';
  $output .= '<div style="z-index: 1001; position: absolute; display: block; height: 0px;" id="modalContent"> <div class="ctools-modal-content" style="width: 1079.2px; height"> <div class="modal-header"><a href="#" class="close"><img title="Close popup" alt="Close popup" src="' . base_path() . drupal_get_path('module', 'ctools') . '/images/icon-close-window.png" typeof="foaf:Image"></a><span class="modal-title" id="modal-title">Loading...</span></div><div class="modal-content" id="modal-content" style="width: 1054.2px;"><div id="modal-throbber"><div class="modal-throbber-wrapper"><div id="redisc_mesg"></div><div class="messages warning">vCenter Discovery is in progress. No other operation can be executed until rediscovery completes.  Please refresh this page when the progress indicator reaches 100%</div>';
  $output .= '<div class="fmc_discovery_meter fmc_discovery_blue"><span id="fmc_discovery_bar"></span></div><div id="fmc_discovery_percent"></div><div id="fmc_discovery_status"></div>';
  $output .='<div style="float: right; font-size: 20px">' . l('Logout', 'user/logout') . '</div><div class="rediscover-load-popup" style="float: right; font-size: 20px"><a href="#">Cancel</a> &nbsp;&nbsp;&nbsp;</div></div></div></div></div></div></div>';
  print $output;

  $colorbox_js = 'jQuery("#popup_rediscover").show(); jQuery("#modalContent").css({top:"50%",left:"50%", margin:"-"+(jQuery("#modalContent").height() / 2)+"px 0 0 -"+(jQuery("#modalContent").width() / 2)+"px"}); jQuery("#modalBackdrop").css({height: (jQuery(document).height())+500+"px", width: (jQuery(document).width())+"px"}); SetCookie("expand_all",0); SetCookie("expanded_host",""); showDiscoveryPerentage();';
  drupal_add_js($colorbox_js, 'inline');
}

/**
 * Check if settings are set, and redirect if not set
 */
function mercury_utilities_check_settings() {
  if (user_is_anonymous()) {
    return;
  }

  $ignore_paths = array(
    'user/logout',
    'node/add/vcenter',
    'config/fmc/settings',
    'config/fmc/settings/host',
    'config/fmc/settings/vm',
    'copyright',
    'discovery/get_percent_status',
  );

  if (in_array($_GET['q'], $ignore_paths)) {
    return;
  }

  // Skip all admin pages.
  if (substr($_GET['q'], 0, 6) === 'admin/') {
    return;
  }

  $message = '';
  $redirect_path = '';
  $link = '';

  // Check if vCenter is setup
  $query = db_select('node', 'node');
  $query->condition('node.status', 1)
    ->condition('node.type', 'vcenter');
  $result = $query->fields('node', array('nid', 'title'))
    ->orderBy('node.title', 'DESC');
  $vcenter_row_count = $result->countQuery()->execute()->fetchField();
  //mercury_utilities_fmc_log_message('VCenter count', $vcenter_row_count);
  //return;
  if ($vcenter_row_count == 0) {
    $redirect_path = 'node/add/vcenter';
    //$message = t('A vCenter has not been setup. Add vCenter <a href="@url">here</a>.', array('@url' => url($redirect_path)));
    $message = t('A vCenter has not been configured. Please add vCenter information and credentials.');
  }

  if (empty($redirect_path)) {

    $vcenter_nid = db_query('SELECT nid FROM {node} WHERE type = :tp AND status = :st', array(':tp' => 'vcenter', ':st' => 1))->fetchField();
    $vcInfo = node_load($vcenter_nid);
    $vc_hostname = (!empty($vcInfo->field_hostname)) ? $vcInfo->field_hostname['und'][0]['value'] : '';
    $vc_username = (!empty($vcInfo->field_hostname)) ? fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']) : '';
    $vc_password = (!empty($vcInfo->field_hostname)) ? fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']) : '';
    if ($vc_hostname == '' || $vc_username == '' || $vc_password == '') {
      $redirect_path = 'config/fmc/settings';
      //$message = t('The default vCenter credentials are not set. Please enter the vCenter credentials <a href="@url">here</a>.', array('@url' => url($redirect_path)));
      $message = t('The default vCenter credentials are not set. Please enter the vCenter credentials.');
    }
  }

  // Import only requires the vCenter credentials.
  // Export doesn't really "require" anything.
  if (empty($redirect_path) && $_GET['q'] == 'config/fmc/settings/export_import') {
    return;
  }

  if (empty($redirect_path)) {
    // Check default host settings
    $host_username = fmc_cipher::decrypt(variable_get('host_username'));
    $host_password = fmc_cipher::decrypt(variable_get('host_password'));
    if (empty($host_username) || empty($host_password)) {
      $redirect_path = 'config/fmc/settings/host';
      //$message = t('The default host credentials are not set. Please set the default host credentials <a href="@url">here</a>.', array('@url' => url($redirect_path)));
      $message = t('The default host credentials are not set. Please set the default host credentials.');
    }
  }

  if (empty($redirect_path)) {

    // Check default VM settings
    $vm_username = fmc_cipher::decrypt(variable_get('vm_username'));
    $vm_password = fmc_cipher::decrypt(variable_get('vm_password'));
    if (empty($vm_username) || empty($vm_password)) {
      $redirect_path = 'config/fmc/settings/vm';
      //$message = t('The default VM credentials are not set. Please set the default VM credentials <a href="@url">here</a>.', array('@url' => url($redirect_path)));
      $message = t('The default VM credentials are not set. Please set the default VM credentials.');
    }
  }

  if (!empty($redirect_path)) {
    drupal_set_message($message, 'warning');

    // This may be triggered during an AJAX request. If so, do an AJAX redirect.
    if (
      (isset($_SERVER['HTTP_X_REQUESTED_WITH']) && $_SERVER['HTTP_X_REQUESTED_WITH'] == 'XMLHttpRequest') || $_GET['q'] == 'system/ajax' // Some forms do not post via jQuery.
    ) {
      return ctools_popups_ajax_redirect($redirect_path);
    }

    // Otherwise just do a standard redirect.
    drupal_goto($redirect_path);
  }
}
