<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for general vcenter actions.
 */

function mercury_utilities_vcenter_settings_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  mercury_utilities_vcenter_settings_form_trim($values);
  if (is_numeric($values['vcenter_nid'])) {
    $vc_info = node_load($values['vcenter_nid']);
    $vc_info->title = $values['vcenter_name'];
    $vc_info->field_hostname['und'][0]['value'] = $values['vcenter_hostname'];
    $vc_info->field_username['und'][0]['value'] = fmc_cipher::encrypt($values['vcenter_username']);
    $vc_info->field_password['und'][0]['value'] = fmc_cipher::encrypt($values['vcenter_password']);

    /* If UUID is not set, generate a new one and set it */
    if (empty($vc_info->field_fmc_uuid) || $vc_info->field_fmc_uuid['und'][0]['value'] == '') {
      $uuid = mercury_utilities_get_fmc_asup_uuid();
      if ($uuid == '') {
        mercury_utilities_fmc_log_message('System Settings', t("Unable to determine ASUP System ID for this FMC!"), WATCHDOG_ERROR, 'show');
      } else {
        $vc_info->field_fmc_uuid['und'][0]['value'] = $uuid;
      }
    }

    node_save($vc_info);
    // Record the time
    $time = strtotime('now');
    variable_set('time_from_last_discovery', $time);

    //Saving the Default values for Migration Settings if its not saved earlier
    $get_migration_values = variable_get('enable_migration');
    if (empty($get_migration_values)) {
      variable_set('enable_migration', 1);
      variable_set('migration_policy', 2);
    }

    mercury_utilities_fmc_log_message('System Settings', t('vCenter @vcenter settings have been saved successfully.', array('@vcenter' => $values['vcenter_name'])), WATCHDOG_NOTICE, 'show');
  }
}

/**
 * Function to validate vCenter version
 */
function mercury_utilities_vcenter_version_is_supported($version) {
  # For now, we support everything starting with a 5.
  if (preg_match('/^5\./', $version)) {
    return TRUE;
  } else {
    return FALSE;
  }
}

function mercury_utilities_vcenter_settings() {
  $output = '';
  $output .= '<div id= "vcenter-settings">' . drupal_render(drupal_get_form('mercury_utilities_vcenter_settings_form')) . '</div>';
  return $output;
}

function mercury_utilities_vcenter_settings_form() {
  $form = array();
  $vCenter_nid = db_query("SELECT * FROM {node} WHERE type = :tp AND status = :st", array(':tp' => 'vcenter', ':st' => 1));
  if (!empty($vCenter_nid)) {
    $i = 0;
    foreach ($vCenter_nid as $res) {
      $vcInfo = node_load($res->nid);
      $vc_name = $vcInfo->title;
      $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
      $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
      $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
      $form[$i]['vcenter'] = array(
        '#type' => 'fieldset',
        '#title' => $res->title,
        '#collapsible' => true,
        '#collapsed' => false
      );
      $form[$i]['vcenter']['vcenter_name'] = array(
        '#type' => 'textfield',
        '#title' => t('vCenter Alias'),
        '#description' => ('Please enter an alias for the vCenter.'),
        '#default_value' => $vc_name
      );
      $form[$i]['vcenter']['vcenter_hostname'] = array(
        '#type' => 'textfield',
        '#title' => t('Hostname'),
        '#description' => ('Please enter the vCenter hostname.'),
        '#default_value' => $vc_hostname
      );
      $form[$i]['vcenter']['vcenter_username'] = array(
        '#type' => 'textfield',
        '#title' => t('Username'),
        '#description' => ('Please enter the vCenter username.'),
        '#default_value' => $vc_username
      );
      $form[$i]['vcenter']['vcenter_password'] = array(
        '#type' => 'password',
        '#title' => t('Password'),
        '#description' => ('Please enter the vCenter password.'),
        '#default_value' => $vc_password,
        '#attributes' => array('value' => $vc_password),
      );
      $form[$i]['vcenter']['vcenter_nid'] = array(
        '#type' => 'hidden',
        '#value' => $res->nid
      );
    }
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Configuration')
  );
  return $form;
}

function mercury_utilities_vcenter_settings_form_trim(&$values) {
  if (!empty($values['vcenter_name'])) {
    $values['vcenter_name'] = trim($values['vcenter_name']);
  }
  if (!empty($values['vcenter_hostname'])) {
    $values['vcenter_hostname'] = trim($values['vcenter_hostname']);
  }
  if (!empty($values['vcenter_username'])) {
    $values['vcenter_username'] = trim($values['vcenter_username']);
  }
  // DO NOT trim the password; leading and trailing spaces are legal
}

function mercury_utilities_vcenter_settings_form_validate($form, &$form_state) {
  $values = $form_state['values'];
  mercury_utilities_vcenter_settings_form_trim($values);
  if (is_numeric($values['vcenter_nid'])) {
    $vc_info = node_load($values['vcenter_nid']);

    if (!empty($values['vcenter_hostname']) && !empty($values['vcenter_username']) && !empty($values['vcenter_password'])) {

      // check if the vcenter is reachable
      $reachable = exec("ping -w 3 " . $values['vcenter_hostname'] . " 2>&1", $result);
      if (!empty($result)) {
        foreach ($result as $output) {
          if (stristr($output, '100% packet loss') || stristr($output, 'unknown host')) {
            form_set_error('vcenter_hostname', 'vCenter hostname is invalid or not reachable.');
            //drupal_set_message(t('vCenter hostname is invalid or not reachable.', array('@vcenter' => $values['vcenter_hostname'])), 'error');
            return;
          }
        }
      }

      // get the vcenter version (also validates credentials)
      $valid = famc_external_api_manager_validate_credentials(
        NULL,
        $values['vcenter_hostname'],
        $values['vcenter_username'],
        $values['vcenter_password']
      );
      if ($valid !== true) {
        form_set_error('vcenter_username', $valid); // Will contain details.
      }
    } else if (empty($values['vcenter_username'])) {
      form_set_error('vcenter_username', 'Please enter vCenter Username.');
    } else if (empty($values['vcenter_password'])) {
      form_set_error('vcenter_password', 'Please enter vCenter Password.');
    }
  }
}
