<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin K\ufffdfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for general vm actions.
 */

/**
 * Function for calling the refresh function for individual vm
 * WARNING: If you load a VM node and then call this, be sure to refresh the
 * node after!
 * @todo: Make a version of this that takes a node and auto-refreshes like
 * @see mercury_management_refresh_host
 * @return true or false (or redirects)
 */
function mercury_utilities_vm_refresh_action($vm_id, $norefresh = "") {
  global $base_url;
  $vm_node = node_load($vm_id);
  $vm_name = $vm_node->title;
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_id);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  $host_nid = $vm_data['host'];
  $result = mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vm_name, $host_nid);
  $result_split = explode('--', $result);
  $return = false;
  if (count($result_split) > 1) {
    mercury_utilities_fmc_log_message('VM Refresh', t('Refresh failed, unable connect to VM @vm_name.', array('@vm_name' => $vm_name)), WATCHDOG_WARNING);
  } else {
    $return = true;
    famc_analytics_fetch_vm_metric($vm_id);
    mercury_utilities_fmc_log_message('VM Refresh', t('VM @vm_name refreshed', array('@vm_name' => $vm_name)));
  }

  if ($norefresh == '') {
    drupal_goto($_SERVER['HTTP_REFERER']);
  }
  return $return;
}

/**
 * Function to perform a VM refresh and verify communication with the VM
 * @todo If this is going to get used again, it needs to have the node_load and
 * pass by reference added in mercury_management_refresh_host
 * @deprecated This is never used, everywhere calls mercury_management_updatevm
 * or mercury_utilities_vm_refresh_action
 */
function mercury_utilities_refresh_vm($vm_nid) {
  $vm_node = node_load($vm_nid);
  $vm_name = $vm_node->title;
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_nid);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $vm_host = $vm_data['host'];
  $res = mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vm_name, $vm_host);
  return $res;
}

/**
 * Function to get all the VM Disks Nids under a VM
 * @param $vm_id
 * @return vDisk Nid values
 */
function mercury_utilities_vm_vdisk_nid_fetch($vm_id) {
  $vdisk_nids = array();
  $vdisk_query = db_select('node', 'node');
  $vdisk_query->leftJoin('field_data_field_virtual_machine', 'field_data_field_virtual_machine', 'node.nid = field_data_field_virtual_machine.entity_id AND field_data_field_virtual_machine.entity_type = :node AND field_data_field_virtual_machine.deleted = :deleted', array(':node' => 'node', ':deleted' => 0));
  $vdisk_query->fields('node', array('nid', 'title'));
  $vdisk_query->condition('node.status', 1)
    ->condition('field_data_field_virtual_machine.field_virtual_machine_target_id', $vm_id)
    ->condition('node.type', 'vdisk');
  $vdisk_result = $vdisk_query->orderBy('node.title', 'DESC')
    ->execute();
  while ($vdisk_row = $vdisk_result->fetch()) {
    $vdisk_nids[] = $vdisk_row->nid;
  }
  return $vdisk_nids;
}

/**
 * Function to check the VM Agent Package for VM Upgrade
 */
function mercury_utilities_check_upgrade_vm($vm_id) {

  $get_node_info = node_load($vm_id);
  $node_msi_version = $get_node_info->field_fa_build_version['und'][0]['value'];
  if (empty($node_msi_version)) {
    mercury_utilities_fmc_log_message('VM Agent', t('VM agent could not be upgraded on @vm, as the VM does not report any version. Please check on the VM status.', array('@vm' => $get_node_info->title)), WATCHDOG_ERROR);
    return 'no-upgrade';
  }

  $vm_package_agent_version = variable_get('vm_package_agent_version', '');
  if (empty($vm_package_agent_version)) {
    mercury_utilities_fmc_log_message('VM Agent', t('VM agent could not be upgraded on @vm, as the agent package version could not be determined. Please check on the uploaded agent package.', array('@vm' => $get_node_info->title)), WATCHDOG_ERROR);
    return 'no-upgrade';
  }

  if (!mercury_utilities_version_compare($vm_package_agent_version, $node_msi_version)) {
    mercury_utilities_fmc_log_message('VM Agent', t('The VM agent software on @vm could not be upgraded, as it is newer than the version currently available to FAMC.', array('@vm' => $get_node_info->title)), WATCHDOG_ERROR);
    return 'no-upgrade';
  }

  return 'upgrade_msi';
}

/*
 * Poll a VM believed to have FA installed for up to X seconds waiting for
 * the webservice to come online.  Expected use is to detect the end of
 * reboots.
 * @see BURT 721912 for details on the selected maximums.
 */
function mercury_utilities_poll_for_vm_webservice_online($vmInfo, $max_wait_seconds_vm, $max_wait_seconds_vcenter) {
  # Obtain vcenter credentials
  $vcInfo = mercury_management_vcenter_details();
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  # Create a worker array
  $wa = mercury_management_create_worker_array(1, $vc_hostname, $vc_username, $vc_password);
  if ($wa === false) {
    return false;
  }

  # Obtain VM credentials and port
  $hostname = mercury_management_get_vm_address($vmInfo);
  $username = mercury_management_get_vm_username($vmInfo);
  $password = mercury_management_get_vm_password($vmInfo);
  $port = mercury_management_get_vm_port($vmInfo);

  # Record start time
  $start_seconds = time();

  # Commence polling
  while (true) {
    // Suppress error messages since we expect it to be down at least once.
    $rv = mercury_management_wa_wsdl(
      $wa, $hostname, $username, $password, 'HgGetCacheOptions',
      NULL, $port, $suppress_error_responses = true
    );
    $el = time() - $start_seconds;

    if (!empty($rv) && ($rv != 'error_wsdl_call')) {
      $result = true;
      break;
    }
    if ($el > $max_wait_seconds_vm) {
      $result = false;
      break;
    }
    sleep(5);
  }

  # If we saw the webservice come up, optionally poll vCenter looking for
  # the VM-specific data to populate.
  if (($result == true) && ($max_wait_seconds_vcenter > 0)) {
    mercury_utilities_wait_for_valid_vminfo($wa, $vmInfo->title, $max_wait_seconds_vcenter, 'reboot');
  }

  # Free the worker array
  mercury_management_destroy_worker_array($wa);

  # Return the result
  return $result;
}

/*
 * Poll a VM not believed to have FA installed for up to X seconds waiting
 * for a "fileExistsInGuest" that does not produce CannotConnect.  Expected
 * use is to detect the end of reboots after FA uninstall.
 * @see BURT 721912 for details on the selected maximums.
 */
function mercury_utilities_poll_for_vm_reboot_complete($vmInfo, $max_wait_seconds_vm, $max_wait_seconds_vcenter) {
  # Obtain vcenter credentials
  $vcInfo = mercury_management_vcenter_details();
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  # Create a worker array
  $wa = mercury_management_create_worker_array(1, $vc_hostname, $vc_username, $vc_password);
  if ($wa === false) {
    return false;
  }

  # Obtain VM credentials and path
  $username = mercury_management_get_vm_username($vmInfo);
  $password = mercury_management_get_vm_password($vmInfo);
  $path = $vmInfo->field_vm_path_name['und'][0]['value'];

  # Record start time
  $start_seconds = time();

  # Commence polling
  while (true) {
    $rv = mercury_management_wa_vmrun_chk_cred($wa, $username, $password, $path);
    $rv = empty($rv) ? 'CannotConnect' : mercury_management_parse_vmrun_chk_cred($rv);
    $el = time() - $start_seconds;

    if ($rv != 'CannotConnect') {
      $result = true;
      break;
    }
    if ($el > $max_wait_seconds_vm) {
      $result = false;
      break;
    }
    sleep(5);
  }

  # If we saw the VM come up, optionally poll vCenter looking for
  # the VM-specific data to populate.
  if (($result == true) && ($max_wait_seconds_vcenter > 0)) {
    mercury_utilities_wait_for_valid_vminfo($wa, $vmInfo->title, $max_wait_seconds_vcenter, 'reboot');
  }

  # Free the worker array
  mercury_management_destroy_worker_array($wa);

  # Return the result
  return $result;
}

/*
 * Poll a VM up to X seconds waiting for vCenter to recognize that it has
 * been powered down.  Expected use is to detect the end of shutdowns after
 * FA install/upgrade.
 * @see BURT 721912 for details on the selected maximums.
 */
function mercury_utilities_poll_for_vm_shutdown_complete($vmInfo, $max_wait_seconds) {
  # Obtain vcenter credentials
  $vcInfo = mercury_management_vcenter_details();
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);

  # Create a worker array
  $wa = mercury_management_create_worker_array(1, $vc_hostname, $vc_username, $vc_password);
  if ($wa === false) {
    return false;
  }

  # Poll vCenter looking for Power State to change from poweredOn
  $result = mercury_utilities_wait_for_valid_vminfo($wa, $vmInfo->title, $max_wait_seconds, 'shut_down');

  # Free the worker array
  mercury_management_destroy_worker_array($wa);

  # Return the result
  return $result;
}

/*
 * Poll vCenter for VM info and try to detect when it becomes valid.  There
 * is a short time after reboot where certain values are 'Not Known', which
 * confuses VM refresh (red LEDs, that sort of thing).  There is a short time
 * after VM shutdown where Power State is still poweredOn.
 * @see BURT 721912 for details on the selected maximums.
 */
function mercury_utilities_wait_for_valid_vminfo(&$wa, $vm_name, $max_wait_seconds, $shut_down_or_reboot) {
  # Record start time
  $start_seconds = time();

  # Commence polling
  while (true) {
    $rv = mercury_management_wa_vminfo($wa, $vm_name);
    $vm_info_with_key = mercury_management_parse_vminfo($rv);
    $el = time() - $start_seconds;

    # If we can't get it, something's wrong with vCenter; stop waiting.
    # Other code will report the condition soon enough.
    if (empty($rv) || empty($vm_info_with_key)) {
      return false;
    }
    if ($shut_down_or_reboot != 'shut_down') {
      // Make sure VMware Tools is returning valid data = fully loaded.
      // We also need to check the VMware Tools status because vCenter caches
      // some things when the VM is shut down, so it's possible to get results
      // without VMware Tools actually running.
      // This can be strict because we know that the DNS/IP config is valid if
      // we get this far.
      if (
        mercury_management_discover_vm_vmware_tools_running($vm_info_with_key) &&
        mercury_management_discover_vm_vmware_tools_valid($vm_info_with_key, $strict = true)
      ) {
        return true;
      }
    } else {
      // As soon as it stops being considered "powered on" then a shutdown
      // poll should be considered a success.
      if ($vm_info_with_key['Power State'] != 'poweredOn') {
        return true;
      }
    }
    if ($el > $max_wait_seconds) {
      return false;
    }
    sleep(5);
  }
}

function mercury_utilities_vm_settings_form() {
  //mercury_utilities_set_vm_agent_package();
  $form = array();
  $form['vm_username'] = array(
    '#type' => 'textfield',
    '#title' => t('User Name'),
    '#description' => ('Please enter the username.'),
    '#default_value' => fmc_cipher::decrypt(variable_get('vm_username', '')),
    '#required' => true,
    '#prefix' => '<div>Please enter the default username and password for the VMs managed by this vCenter. These credentials are used to install the Flash Accel VM agent. Following discovery, if there are VMs in your datacenter that use non-default credentials, you will need to override these default values for those VMs using the "edit" option.</div>',
  );
  $form['vm_password'] = array(
    '#type' => 'password',
    '#title' => t('Password'),
    '#description' => ('Please enter the password.'),
    '#attributes' => array('value' => fmc_cipher::decrypt(variable_get('vm_password', ''))),
    '#required' => true
  );
  /*
    $form['vm_upgraded_MSI'] = array(
    '#type' => 'textfield',
    '#title' => t('VM Agent Package'),
    '#attributes' => array('value' => variable_get('vm_cur_MSI', ''), 'readonly' => 'readonly'),
    '#prefix' => '<table><tr><td width=450px>',
    '#suffix' => '</td>',
    );
    $form['update'] = array(
    '#type' => 'submit',
    '#value' => t('Update'),
    '#submit' => array('mercury_utilites_update_vm_package'),
    '#prefix' => '<td align=left style="padding-top:13px">',
    '#suffix' => '</td></tr></table>',
    );
    $form['minimum_cache_space_vm'] = array(
    '#type' => 'textfield',
    '#title' => t('Minimum Cache Space Size (MB)'),
    '#default_value' => variable_get('minimum_cache_space_vm',1024),
    '#description' => t('Input the minimum Cache Space allocation to all VMs in MB'),
    '#size' => 6,
    '#required' => TRUE
    );
   */
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration'),
    '#submit' => array('vm_settings_log_settings'),
  );

  return $form;
}

function vm_settings_log_settings($form, $form_state) {
  $values = $form_state['values'];
  variable_set('vm_username', fmc_cipher::encrypt(trim($values['vm_username'])));
  // DO NOT trim the password; leading and trailing spaces are legal
  variable_set('vm_password', fmc_cipher::encrypt($values['vm_password']));
  mercury_utilities_fmc_log_message('System Settings', t('VM settings saved successfully'), WATCHDOG_INFO, 'show');
}

// XXX: This is no longer used (button is commented out)
function mercury_utilites_update_vm_package($form, $form_state) {
  $updateVM = exec("/var/www/updateFromIso.sh windows", $result);
  mercury_utilities_fmc_log_message('VM Agent', t($updateVM));
  if (!empty($result)) {
    foreach ($result as $output) {
      if (stristr($output, 'Error')) {
        $error = explode(':', $output);
        mercury_utilities_fmc_log_message('VM Agent', t(' @msg', array('@msg' => $error[1])), WATCHDOG_WARNING, 'show');
      } else {
        mercury_utilities_fmc_log_message('VM Agent', t('Updated to @msg', array('@msg' => $output)), WATCHDOG_NOTICE, 'show');
      }
    }
  }
}

function mercury_utilities_update_vm_vdisks_from_cacheinfo($vm_nid, $attrInfo) {

  # Fetch the vdisk nodes
  $vdisk_nids = mercury_management_find_dependent_nodes(array($vm_nid), 'vdisk', 'field_virtual_machine');
  if (empty($vdisk_nids)) {
    return;
  }
  $vdisk_nodes = node_load_multiple($vdisk_nids);

  # Build a table of vdisk nodes indexed by id
  $vdisks_by_id = array();
  foreach ($vdisk_nodes as $vn) {
    $identifier = strtolower(trim($vn->field_vdisk_identifier['und'][0]['value']));
    $vdisks_by_id[$identifier] = $vn;
  }

  # Now go through the cache info and update the appropriate vdisk nodes
  $diskCount = $attrInfo['disk_count'];
  for ($index = 0; $index < $diskCount; $index++) {
    $identifier = strtolower(trim($attrInfo[$index]['vdisk_identifier']));
    if (!empty($vdisks_by_id[$identifier])) {
      $vdisk_node = $vdisks_by_id[$identifier];
      $vdisk_node->field_fa_cache_mode['und'][0]['value'] = $attrInfo[$index]['cache_mode'];
      $vdisk_node->field_fa_administration_state['und'][0]['value'] = $attrInfo[$index]['admin_state'];
      $vdisk_node->field_fa_performance_state['und'][0]['value'] = $attrInfo[$index]['perf_state'];
      $vdisk_node->field_fa_operational_state['und'][0]['value'] = $attrInfo[$index]['op_state'];
      $vdisk_node->field_fa_operational_message['und'][0]['value'] = $attrInfo[$index]['op_message'];
      node_save($vdisk_node);
      unset($vdisks_by_id[$identifier]);
    }
  }
}
