<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for host wrench actions.
 */

/**
 * Function to display the Wrench Menu for a Host
 * Enter description here ...
 * @param Numeric $host_nid
 * @param Array $host_node
 */
function mercury_utilities_host_wrench_menu($host_nid, $host_node = NULL) {
  $output = '';
  if (!isset($host_node)) {
    $host_node = node_load($host_nid);
  }
  if (isset($host_node->title)) {
    $host_name = $host_node->title;
    $host_name = str_replace(" ", "__", $host_name);
  }

  //Check the Avialablity of VIB Package to install/upgrade for Host
  $check_current_vib = trim(variable_get('host_cur_zip'));
  $check_upgrade_vib = trim(variable_get('host_upgraded_zip'));
  if ($check_current_vib != '' && $check_upgrade_vib != '') {
    $vib_package_install_link = ctools_popups_host_settings_link('Install', $host_nid, 'settings', 'Settings');
    $vib_package_install = '<li class = "second-level-li">' . $vib_package_install_link . '</li>';
  } else {
    $vib_package_install = '<li class = "second-level-li inactive">Install</li>';
  }

  if ($check_current_vib != '' && $check_upgrade_vib != '') {
    $vib_package_upgrade_link = ctools_popups_host_settings_link('Upgrade', $host_nid, 'settings', 'Settings');
    $vib_package_upgrade = '<li class = "second-level-li">' . $vib_package_upgrade_link . '</li>';
  } else {
    $vib_package_upgrade = '<li class = "second-level-li inactive">Upgrade</li>';
  }

  // TODO: element type div is not allowed directly under <ul>.
  $output .= '<div class = "wrench-menu-host wrench-menu ' . $host_name . '" id = "wrench-menu-host-' . $host_nid . '">';
  $output .= '<ul class = "first-level-ul">';
  $output .= '<li class = "first-level-li expandable">Software';
  $output .= '<ul class = "second-level-ul">';
  $output .= $vib_package_install;
  $output .= $vib_package_upgrade;
  $output .= '<li class = "second-level-li last">' . ctools_popups_host_settings_link('Uninstall', $host_nid, 'settings', 'Settings') . '</li>';
  $output .= '</ul>';
  $output .= '</li>';
  $output .= '<li class = "first-level-li expandable">Flash Device';
  $output .= '<ul class = "second-level-ul">';
  $output .= '<li class = "second-level-li">' . ctools_popups_host_settings_link('Assign', $host_nid, 'settings', 'Settings') . '</li>';
  $output .= '<li class = "second-level-li">' . ctools_popups_host_settings_link('Unassign', $host_nid, 'settings', 'Settings') . '</li>';
  $output .= '<li class = "second-level-li last">' . ctools_popups_host_settings_link('Initialize', $host_nid, 'settings', 'Settings') . '</li>';
  $output .= '</ul>';
  $output .= '</li>';
  $output .= '<li class = "first-level-li host-refresh-link">' . l('Refresh', 'refresh/host/' . $host_nid, array('attributes' => array('title' => 'Refresh the Host Agent properties for the Host'))) . '</li>';
  $output .= '<li class = "first-level-li last">' . ctools_popups_host_settings_link('Edit', $host_nid, 'settings', 'edit') . '</li>';
  $output .= '</ul>';
  $output .= '</div>';
  return $output;
}

/**
 * Function for calling the refresh function of Host and redirect to Home from Ctool PopUp
 */
function mercury_utilities_host_refresh_action_home($host_id, $norefresh = "") {
  global $base_url;
  ctools_include('ajax');
  ctools_include('modal');
  $host = node_load($host_id);
  $host_name = $host->title;
  $dcInfo = node_load($host->field_datacenter['und'][0]['target_id']);
  $vcInfo = node_load($dcInfo->field_related_vcenter['und'][0]['target_id']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $result = mercury_management_updatehost($vc_hostname, $vc_username, $vc_password, $host_name, $dcInfo->nid);
  if ($result) {
    mercury_utilities_fmc_log_message('Host Refresh', t('Host @host_name refreshed successfully.', array('@host_name' => $host_name)));
  } else {
    mercury_utilities_fmc_log_message('Host Refresh', t('Host @host_name refresh unsuccessful.', array('@host_name' => $host_name)), WATCHDOG_ERROR);
  }
  $out = array();
  $out[] = ctools_ajax_command_redirect($base_url);
  print ajax_render($out);
  exit;
}

/**
 * popup for the host refresh action confirmation
 * XXX: This does not appear to be used anymore (reloads without confirm).
 */
function mercury_utilities_host_refresh_popup($host_id) {
  $output = '';
  $host_data = node_load($host_id);
  $host_name = $host_data->title;
  $driver_status = field_get_items('node', $host_data, 'field_bundle_status');
  $message = "This will refresh the host configuration and cache information. If you are sure click 'Refresh'.";
  $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Refresh', 'refresh/host/' . $host_id) . '</span></div>';
  $output = '<div class = "poopup-setting-page">';
  $output .= '<div class = "vm-name"><span>Host Name: ' . $host_name . '</span></div>';
  $output .= '<div class = "driver-status">' . $driver_status . '</div>';
  $output .= '<div class = "notify-message">' . $message . '</div>';
  $output .= $button_code;
  $output .= '</div>';
  return $output;
}

function mercury_utilities_host_settings_uninstall_popup($host_id) {
  $output = '';
  mercury_utilities_host_refresh_action($host_id, "no_refresh");
  $host_node = node_load($host_id);
  $bundle_status = field_get_items('node', $host_node, 'field_bundle_status');
  $host_name = $host_node->title;
  $message = '';
  $button_code = '';
  $bundle_status_value = $bundle_status[0]['value'];
  if (empty($bundle_status_value) || $bundle_status_value == FAMC_HOST_BUNDLE_STATUS_NOT_KNOWN) {
    $message = "Cannot uninstall. Unable to communicate with Host.";
  } elseif ($bundle_status_value == FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED) {
    $message = "Uninstall can't be done as the Flash Accel host agent is not installed.";
  } elseif (!empty($host_node->field_naa_identifier['und'][0]['value']) || !empty($host_node->field_hbtl['und'][0]['value'])) {
    $message = "Uninstall can't be done as a flash device is assigned to the host. Please unassign the flash device and retry.";
  } else {
    $vib_version = field_get_items('node', $host_node, 'field_vib_version');
    $host_package_agent_version = variable_get('host_package_agent_version', '');
    $title = 'Uninstalling host agent version ' . $vib_version[0]['value'];
    $message = "Do you want to continue?";
    $type = "uninstall";
    $link_text = "Uninstall";
    $button_code = '<div class = "confirm-action"><div class ="host-' . $type . ' mercury-button" id="host-' . $host_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';
  }

  $output .= '<div class = "popup-host-setting-uninstall">';
  if (!empty($title)) {
    $output .= '<div class = "row-title"><span><strong>' . $title . '</strong></span></div><br>';
  }
  $output .= '<div class = "host-name"><span>Host Name: ' . $host_name . '</span></div>';
  $output .= '<div class = "host-driver-details"><span class = "driver-details">Flash Accel Host Agent Status: ' . $bundle_status_value . '</span></div>';
  $output .= '<div class = "confirm-message">' . $message . '</div>';

  if (!empty($button_code)) {
    // Uninstall doesn't need to check the package version, won't be used.
    $warning = mercury_management_host_agent_compatibility_warning($host_node, $check_package_version = false);

    if (!empty($warning)) {
      $output .= '<div class="inline-messages warning">' . $warning . '</div>';
    }
    $output .= $button_code;
  }

  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

function mercury_utilities_host_settings_install_popup($host_id) {
  $output = '';
  mercury_utilities_host_refresh_action($host_id, "no_refresh");
  $host_node = node_load($host_id);
  $bundle_status = field_get_items('node', $host_node, 'field_bundle_status');
  $host_name = $host_node->title;
  $message = '';
  $button_code = '';
  $bundle_status_value = $bundle_status[0]['value'];
  if ($bundle_status_value == FAMC_HOST_BUNDLE_STATUS_NOT_INSTALLED) {
    $host_package_agent_version = variable_get('host_package_agent_version', '');
    $title = "Installing host agent version $host_package_agent_version";
    $message = "This will install the Flash Accel host agent on the host. Do you wish to continue?";
    $type = "install";
    $link_text = "Install";
    $button_code = '<div class = "confirm-action"><div class ="host-' . $type . ' mercury-button" id="host-' . $host_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';
  } else if ($bundle_status_value == FAMC_HOST_BUNDLE_STATUS_INSTALLED) {
    $message = "Host agent is already installed on the host.";
  } else if ($bundle_status_value == FAMC_HOST_BUNDLE_STATUS_PARTIAL_INSTALL) {
    $message = "Host agent is partially installed on the host. Please complete the uninstall to continue.";
  } else {
    $message = "Cannot install host agent as host is unreachable or not compatible.";
  }
  $output .= '<div class = "popup-host-setting-install">';
  if (!empty($title)) {
    $output .= '<div class = "row-title"><span><strong>' . $title . '</strong></span></div><br>';
  }
  $output .= '<div class = "host-name"><span>Host Name: ' . $host_name . '</span></div>';
  //$output .= '<div class = "host-driver-details"><span class = "driver-details">' . $bundle_status_value . '</span></div>';
  $output .= '<div class = "confirm-message">' . $message . '</div>';

  if (!empty($button_code)) {
     // Install and upgrade will use the agent package so check it.
    $warning = mercury_management_host_agent_compatibility_warning($host_node, $check_package_version = true);

    if (!empty($warning)) {
      $output .= '<div class="inline-messages warning">' . $warning . '</div>';
    }
    $output .= $button_code;
  }

  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

/**
 * Function to Assign SSD Device to a Host
 * @see ctools_popups_host_setting_assign
 * @deprecated no longer called, form gets called directly instead.
 */
function mercury_utilities_host_settings_assign_popup($host_id) {
  $output = '';
  $host_node = node_load($host_id);
  $bundle_status = field_get_items('node', $host_node, 'field_bundle_status');
  $host_name = $host_node->title;
  $bundle_status_value = $bundle_status[0]['value'];

  $generate_form = drupal_render(drupal_get_form('mercury_utilities_host_settings_assign_devices', $host_id));

  $output .= '<div class = "popup-host-setting-install">';
  $output .= '<div class = "host-name"><span>Host Name: ' . $host_name . '</span></div>';
  $output .= '<div class = "host-driver-details"><span class = "driver-details">Flash Accel host agent status: ' . $bundle_status_value . '</span></div>';
  $output .= '<div class = "confirm-message">' . $generate_form . '</div>';
  $output .= '</div>';

  return $output;
}

/**
 * Function to list all the SSD Device for a particual host of Assigning it into the Host
 */
function mercury_utilities_host_settings_assign_devices($form, &$form_state) {
  $form = array();

  $host_id = filter_xss(arg(5));
  $host_node = node_load($host_id);

  $agent_warning = mercury_management_host_agent_compatibility_warning($host_node);
  if (!empty($agent_warning)) {
    $form['agent_warning'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="inline-messages warning">' . $agent_warning . "</div>",
    );
  }

  $api_error = mercury_management_has_fa_api_error($host_node);
  if (!empty($api_error)) {
    $form['api_error'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="inline-messages error">' . $api_error . '</div>',
    );
    return $form;
  }

  //Query the Devices that are not yet Assigned into the Host
  if ($host_id != '') {
    $options = array();

    $query = "SELECT node.nid
        FROM
        {node} node
        INNER JOIN {field_data_field_is_ssd} field_data_field_is_ssd ON node.nid = field_data_field_is_ssd.entity_id AND (field_data_field_is_ssd.entity_type = 'node' AND field_data_field_is_ssd.deleted = '0')
        LEFT JOIN {field_data_field_related_host} field_data_field_related_host ON node.nid = field_data_field_related_host.entity_id AND (field_data_field_related_host.entity_type = 'node' AND field_data_field_related_host.deleted = '0')
        WHERE (( (field_data_field_related_host.field_related_host_target_id = :host_id ) )AND(( (node.status = '1') AND (node.type IN  ('device')) )))
        ORDER BY node.title ASC";

    $result = db_query($query, array(':host_id' => $host_id))->fetchCol();

    foreach ($result as $nid) {
      $device_node = node_load($nid);

      if ($device_node->field_serial_no['und'][0]['value'] != $host_node->field_naa_identifier['und'][0]['value']) {
        $device_naa_id = trim($device_node->field_serial_no['und'][0]['value']);

        // Prefer a custom device name when possible (user set in vSphere).
        // Node titles cannot be empty.
        $short_device_name = $device_node->title;

        // If the device id is not present in the title, add it.
        // Default device names contain it already and some custom will.
        if (strpos($short_device_name, $device_naa_id) === false) {
          $short_device_name .= " ($device_naa_id)";
        }

        // Space is limited.
        if (strlen($short_device_name) > 38) {
          $short_device_name = substr($short_device_name, 0, 35) . "...";
        }

        //Do not display if the Model is Flash Accel
        if ($device_node->field_model['und'][0]['value'] != 'Flash Accel') {
          $options[$nid] = t('<div class="device_list"><div class="naa_name" title="' . $device_node->title . '">' .
            $short_device_name . '</div><div class="device_size">' .
            number_format($device_node->field_size['und'][0]['value'] / 1024, 2, '.', '') .
            ' (GB)</div><div class="device_size">' . $device_node->field_vendor['und'][0]['value'] . ' - ' .
            $device_node->field_model['und'][0]['value'] . '</div></div>');
        }
      }
    }
  }


  $naa_identity = '';
  $host_info = node_load($host_id);
  //Displaying the attached Flash Device into the Host.
  if (!empty($host_info->field_naa_identifier['und'][0]['value']) && !empty($host_info->field_hbtl['und'][0]['value'])) {
    $index = $host_info->field_hbtl['und'][0]['value'];
    $naa_identity = $host_info->field_naa_identifier['und'][0]['value'];
    $attached_device = t('<div class="device_attached_list"><div class="naa_name" title="' . $host_info->field_naa_identifier['und'][0]['value'] . '">' . substr(trim($host_info->field_naa_identifier['und'][0]['value']), 0, 35) . '...' . '</div><div class="device_size">' . number_format($host_info->field_total_size['und'][0]['value'] / 1073741824, 2, '.', '') . ' (GB)</div></div>');

    $form['item_exist_title'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="device_popup_title">The currently assigned flash device is listed below. Please unassign this flash device prior to assigning another device.</div>'
    );

    $form['item_exist'] = array(
      '#type' => 'markup',
      '#markup' => $attached_device,
    );
  } else if ($host_info->field_bundle_status['und'][0]['value'] == "Agent Not Installed") {
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages status">The host agent is not installed.  Please install the host agent before assigning a cache device</div>',
    );
  } else if (count($options) > 0 && ($host_info->field_naa_identifier['und'][0]['value'] == '' && $host_info->field_hbtl['und'][0]['value'] == '')) {
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div id="device_popup_title">Please select the flash device to use for caching.</div>',
    );

    $form['node_device'] = array(
      '#type' => 'radios',
      '#options' => $options,
    );

    $form['pager'] = array('#value' => theme('pager'));


    $form['host_id'] = array(
      '#type' => 'hidden',
      '#value' => $host_id
    );

    $form['assigned_device_id'] = array(
      '#type' => 'hidden',
      '#value' => $naa_identity
    );

    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Assign to Host'),
      '#suffix' => '<script>
          jQuery("#edit-submit").click(function(){
            jQuery("#edit-submit").hide();
            jQuery("a.close").hide();
		        jQuery(".processing").show();
            jQuery("div.res-message").remove();
            jQuery("#warning-message").append( \'<div class="res-message red">Please do not close the browser window until the process completes.</div>\');
            jQuery("#warning-message").append(\'<div class="res-message">Assigning device to Host ...</div>\');

          });
        </script>'
    );

    $form['item_2'] = array(
      '#type' => 'markup',
      '#markup' => '<div id="warning-message"></div>',
    );

    $form['processing'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div>',
    );
  } else {
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages status">No flash device is currently available to use as cache.</div>',
    );
  }

  return $form;
}

/**
 * Adding the SSD into the Host using hook_submit()
 */
function mercury_utilities_host_settings_assign_devices_submit($form, &$form_state) {
  ctools_include('ajax');
  ctools_include('modal');
  $values = $form_state['values'];

  //Host Node Id
  $host_nid = $values['host_id'];

  //Device Node
  $device_nid = $values['node_device'];

  //Existed Device Node
  $assigned_device_nid = $values['assigned_device_id'];
  $host = node_load($host_nid);

  //Unassign the previous Device from the Host
  if (empty($device_nid)) {
    mercury_utilities_fmc_log_message('Flash Device', t('Please select a device to assign on host @host.', array('@host' => $host->title)), WATCHDOG_ERROR, 'show');
    $form_state['message'] = theme('status_messages');
    $form_state['executed'] = TRUE;
  } else if (!empty($assigned_device_nid)) {
    $device_data = node_load($device_nid);
    $device_title = $device_data->field_serial_no['und'][0]['value'];

    $unassign_device = mercury_management_unattachDeviceFromHost($host_nid);

    if ($unassign_device == 'unattached') {
      //Assign Device to Host
      if (!empty($device_nid)) {
        $assign_device = mercury_management_attachDeviceToHost($host_nid, $device_nid);
        if ($assign_device == 'attached') {
          //mercury_utilities_host_refresh_action_home($host_nid, "norefresh");
          mercury_utilities_fmc_log_message('Flash Device', t('Flash device "@device_name" assigned successfully on host "@host_name". Please refresh host.', array('@device_name' => $device_title, '@host_name' => $host->title)));
        } elseif ($assign_device == 'attached_refresh') {
          mercury_utilities_fmc_log_message('Flash Device', t('Flash device "@device_name" assigned successfully on host "@host_name".', array('@device_name' => $device_title, '@host_name' => $host->title)));
          //drupal_set_message(t('Host @host_name refresh Successful.', array('@host_name' => $host->title)), 'status');
          //mercury_utilities_fmc_log_message('Host Refresh', t('Host @host_name Refreshed Successful.', array('@host_name' => $host->title)));
        } else if ($assign_device == "maintenance") {
          mercury_utilities_fmc_log_message('Flash Device', t('Host @host_name is currently in maintenance mode. Please exit maintenance mode to assign device for caching.', array('@host_name' => $host->title)), WATCHDOG_WARNING, 'show');
        } else {
          mercury_utilities_fmc_log_message('Flash Device', t('Failed to assign device on host. Please check the logs for more information.'), WATCHDOG_ERROR);
        }
      } else {
        //drupal_set_message(t('Cache Space is already allocated to the VMs. Please remove the Cache Space from the VMs before Assign or Change.'), WATCHDOG_WARNING);
        mercury_utilities_fmc_log_message('Flash Device', $unassign_device, WATCHDOG_WARNING, 'show');
      }
    } else if ($unassign_device == "maintenance") {
      mercury_utilities_fmc_log_message('Flash Device', t('Host @host_name is currently in maintenance mode. Please exit maintenance mode to assign device for caching.', array('@host_name' => $host->title)), WATCHDOG_WARNING, 'show');
    } else {
      // XXX - we should not end up here -- as we should never be assigning to a host with a cache device..
      //drupal_set_message(t('Cache Space is already allocated to the VMs. Please remove the Cache Space from the VMs before Assign or Change.'), 'error');
      mercury_utilities_fmc_log_message('Flash Device', $unassign_device, WATCHDOG_WARNING);
    }
  } else {
    $device_data = node_load($device_nid);
    $device_title = $device_data->field_serial_no['und'][0]['value'];

    $assign_device = mercury_management_attachDeviceToHost($host_nid, $device_nid);
    if ($assign_device == 'attached') {
      //mercury_utilities_host_refresh_action_home($host_nid, "norefresh");
      mercury_utilities_fmc_log_message('Flash Device', t('Flash device @device_name assigned successfully on host @host_name. Please refresh host.', array('@device_name' => $device_title, '@host_name' => $host->title)));
    } elseif ($assign_device == 'attached_refresh') {
      mercury_utilities_fmc_log_message('Flash Device', t('Flash device @device_name assigned successfully on host @host_name.', array('@device_name' => $device_title, '@host_name' => $host->title)));
      mercury_utilities_host_refresh_action($host_nid, "norefresh"); //Host Refresh and Redirect to current page
      //drupal_set_message(t('Host @host_name refresh successful.', array('@host_name' => $host->title)), 'status');
      //mercury_utilities_fmc_log_message('Host Refreshed', t('Host @host_name Refreshed Successful.', array('@host_name' => $host->title)));

      $out = array();
      $out[] = ctools_ajax_command_redirect($_SERVER['HTTP_REFERER']);
      print ajax_render($out);
      exit;
    } elseif ($assign_device == 'error') {
      mercury_utilities_fmc_log_message('Flash Device', t('Device not assigned on host. Please check the logs for more information.'), WATCHDOG_ERROR, 'show');
      $form_state['message'] = theme('status_messages');
      $form_state['executed'] = TRUE;
    } elseif ($assign_device == "maintenance") {
      mercury_utilities_fmc_log_message('Flash Device', t('Host @host_name is currently in maintenance mode. Please exit maintenance mode to assign device for caching.', array('@host_name' => $host->title)), WATCHDOG_WARNING, 'show');
      $form_state['message'] = theme('status_messages');
      $form_state['executed'] = TRUE;
    } else {
      mercury_utilities_fmc_log_message('Flash Device', t('Please select a device to assign on the host @host.', array('@host' => $host->title)), WATCHDOG_WARNING, 'show');
    }
  }
  /* $form_state['executed'] = TRUE;
    $out = array();
    $out[] = ctools_ajax_command_redirect($_SERVER['HTTP_REFERER']);
    print ajax_render($out);
    exit;
   */
}

function mercury_utilities_host_settings_upgrade_popup($host_id) {
  $output = '';
  mercury_utilities_host_refresh_action($host_id, 'no-refresh');
  $host_node = node_load($host_id);
  $bundle_status = field_get_items('node', $host_node, 'field_bundle_status');
  $host_name = $host_node->title;
  $message = '';
  $title = '';
  $button_code = '';
  $bundle_status_value = $bundle_status[0]['value'];
  $type = "upgrade";
  $link_text = "Upgrade";
  if ($bundle_status_value == "Agent Installed") {
    $vib_version = field_get_items('node', $host_node, 'field_vib_version');
    $host_package_agent_version = variable_get('host_package_agent_version', '');
    $title = 'Upgrading host agent ' . $vib_version[0]['value'] .  " to version $host_package_agent_version";
    $message = "This will upgrade the Flash Accel host agent in the host. If you are sure click Upgrade.";
    $button_code = '<div class = "confirm-action"><div class ="host-' . $type . ' mercury-button" id="host-' . $host_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';
  } else {
    $message = "Cannot upgrade host agent.  Unable to communicate with host, or Flash Accel is not installed on host.";
  }
  $output .= '<div class = "popup-host-setting-install">';
  if (!empty($title)) {
    $output .= '<div class = "row-title"><span><strong>' . $title . '</strong></span></div><br>';
  }
  $output .= '<div class = "host-name"><span>Host Name: ' . $host_name . '</span></div>';
  $output .= '<div class = "host-driver-details"><span class = "driver-details">Host Agent Status: ' . $bundle_status_value . '</span></div>';
  $output .= '<div class = "confirm-message">' . $message . '</div>';

  if (!empty($button_code)) {
     // Install and upgrade will use the agent package so check it.
    $warning = mercury_management_host_agent_compatibility_warning($host_node, $check_package_version = true);

    if (!empty($warning)) {
      $output .= '<div class="inline-messages warning">' . $warning . '</div>';
    }
    $output .= $button_code;
  }

  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

/**
 * @see ctools_popups_host_setting_unassign
 * @deprecated no longer called, form gets called directly instead.
 */
function mercury_utilities_host_settings_unassign_popup($host_id) {
  $output = '';
  $host_node = node_load($host_id);
  $bundle_status = field_get_items('node', $host_node, 'field_bundle_status');
  $host_name = $host_node->title;
  $bundle_status_value = $bundle_status[0]['value'];
  $type = "unassaign";
  $link_text = "Unassign";

  $generate_form = drupal_render(drupal_get_form('mercury_utilities_host_settings_unassign_devices', $host_id));

  $output .= '<div class = "popup-host-setting-unassign">';
  $output .= '<div class = "host-name"><span>Host Name: ' . $host_name . '</span></div>';
  $output .= '<div class = "host-driver-details"><span class = "driver-details">' . $bundle_status_value . '</span></div>';
  $output .= '<div class = "confirm-message">' . $generate_form . '</div>';
  //$output .=  $button_code = '<div class = "confirm-action"><div class ="host-' . $type . ' mercury-button" id="host-' . $host_id . '"><span class = "mercury-button-model">' . $link_text. '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';
  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

/**
 * Function to list all the SSD Device for a particual host for UnAssigning from a Host
 */
function mercury_utilities_host_settings_unassign_devices($form, &$form_state) {
  $form = array();

  $host_id = filter_xss(arg(5));

  $options = array();

  if ($host_id != '') {
    $host_info = node_load($host_id);

    $agent_warning = mercury_management_host_agent_compatibility_warning($host_info);
    if (!empty($agent_warning)) {
      $form['agent_warning'] = array(
        '#type' => 'markup',
        '#markup' => '<div class="inline-messages warning">' . $agent_warning . "</div>",
      );
    }

    $api_error = mercury_management_has_fa_api_error($host_info);
    if (!empty($api_error)) {
      $form['api_error'] = array(
        '#type' => 'markup',
        '#markup' => '<div class="inline-messages error">' . $api_error . '</div>',
      );
      return $form;
    }

    $allocated_vms = $host_info->field_allocated_virtual_machines['und'][0]['value'];
    if ($allocated_vms == 0) {
      //Displaying the attached Flash Device into the Host.
      if (!empty($host_info->field_naa_identifier) && !empty($host_info->field_hbtl)) {
        if (!empty($host_info->field_naa_identifier['und'][0]['value']) && !empty($host_info->field_hbtl['und'][0]['value'])) {
          $index = $host_info->field_hbtl['und'][0]['value'];
          $cache_devid = trim($host_info->field_naa_identifier['und'][0]['value']);
          if (strlen($cache_devid) > 38) {
            $cache_devid = substr(trim($host_info->field_naa_identifier['und'][0]['value']), 0, 35) . '...';
          }
          $options[$index] = t('<div class="device_list"><div class="naa_name" title="' . $host_info->field_naa_identifier['und'][0]['value'] . '">' . $cache_devid . '</div><div class="device_size">' . number_format($host_info->field_total_size['und'][0]['value'] / 1073741824, 2, '.', '') . ' (GB)</div></div>');
        }
      }

      if (count($options) > 0) {
        $form['item'] = array('#type' => 'markup',
          '#markup' => '<div class="device_popup_title">Please select the device to unassign from the host.</div>',
        );

        $form['node_device'] = array('#type' => 'radios',
          '#options' => $options,
        );

        $form['pager'] = array('#value' => theme('pager'));

        $form['host_id'] = array('#type' => 'hidden',
          '#value' => $host_id
        );

        $form['submit'] = array(
          '#type' => 'submit',
          '#value' => t('Unassign from Host'),
          '#suffix' => '<script>
			            jQuery("#edit-submit").click(function(){
			            jQuery("#edit-submit").hide();
			            jQuery("a.close").hide();
		        			jQuery(".processing").show();
		        			jQuery("div.res-message").remove();
			            jQuery("#warning-message").append( \'<div class="res-message red">Please do not close the browser window until the process completes.</div>\');
			            jQuery("#warning-message").append(\'<div class="res-message">Unassigning device from the Host ...</div>\');
						});
            </script>'
        );
        $form['item_2'] = array(
          '#type' => 'markup',
          '#markup' => '<div id="warning-message"></div>',
        );

        $form['processing'] = array(
          '#type' => 'markup',
          '#markup' => '<div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div>',
        );
      } else {
        $form['item'] = array('#type' => 'markup',
          '#markup' => '<div class="messages status">No Flash Device is currently assigned for caching on this host.</div>',
        );
      }
    } else {
      // XXX: Seems like the vm count field doesn't update on host refresh,
      // which causes this to be hit until discovery is run again.
      $form['item'] = array('#type' => 'markup',
        '#markup' => '<div class="messages status">Cache Space is currently allocated to VMs. Please deallocate the cache space before unassigning the device.</div>',
      );
    }
  }

  return $form;
}

/**
 * Removing SSD devices into the Host using hook_submit()
 */
function mercury_utilities_host_settings_unassign_devices_submit($form, &$form_state) {
  ctools_include('ajax');
  ctools_include('modal');
  $values = $form_state['values'];

  //Host Node nid
  $host_nid = $values['host_id'];
  $host = node_load($host_nid);
  //Device Node nid
  $device_nid = $values['node_device'];

  //Unassign Device from Host
  if (!empty($device_nid)) {

    //$device_data = node_load($device_nid);
    //$device_title = $device_data->field_serial_no['und'][0]['value'];

    $unassign_device = mercury_management_unattachDeviceFromHost($host_nid);
    if ($unassign_device == 'unattached') {
      mercury_utilities_fmc_log_message('Flash Device', t('Flash device unassigned successfully from the host @host_name.', array('@host_name' => $host->title)));
      mercury_utilities_host_refresh_action($host_nid, "norefresh"); //Host Refresh and Redirect to current page
    } else if ($unassign_device == "maintenance") {
      mercury_utilities_fmc_log_message('Flash Device', t('Host @host_name is currently in maintenance mode. Please turn off maintenance mode to unassign device for caching.', array('@host_name' => $host->title)), WATCHDOG_WARNING, 'show');
    } else if ($unassign_device == "cachespace") {
      mercury_utilities_fmc_log_message('Flash Device', t('Cache space is currently allocated to the VMs. Please deallocate the cache space before unassigning the device.'), WATCHDOG_WARNING, 'show');
    } else {
      mercury_utilities_fmc_log_message('Flash Device', t('Failed to unassign device on host @host_name.', array('@host_name' => $host->title)), WATCHDOG_WARNING, 'show');
    }
  } else {
    /* XXX Since we are doing only one device for now, should this be selected by default */
    mercury_utilities_fmc_log_message('Flash Device', t('Please select a device to unassign from host @host.', array('@host' => $host->title)), WATCHDOG_WARNING, 'show');
  }
  //$form_state['executed'] = TRUE;
  $out = array();
  $out[] = ctools_ajax_command_redirect($_SERVER['HTTP_REFERER']);
  print ajax_render($out);
  exit;
}

function mercury_utilities_host_settings_initialize_popup($host_id) {
  $output = '';
  $host_node = node_load($host_id);
  $bundle_status = field_get_items('node', $host_node, 'field_bundle_status');
  $host_name = $host_node->title;
  $bundle_status_value = $bundle_status[0]['value'];
  $cachespace_status = field_get_items('node', $host_node, 'field_allocated_size');
  $host_status_details = mercury_management_get_host_status($host_node);

  $type = "initialize";
  $link_text = "Initialize";
  $output .= '<div class = "popup-host-setting-initialize">';
  $output .= '<div class = "host-name"><span>Host Name: ' . $host_name . '</span></div>';
  $output .= '<div class = "host-driver-details"><span class = "driver-details">Host Agent Status: ' . $bundle_status_value . '</span></div>';

  if ($host_status_details['status_color'] == 'red') {
    $output .= '<div class = "res-message red">Cannot initialize the cache. Check the host status for communication errors.</div>';
  } elseif ($bundle_status_value == 'Agent Installed') {
    if ($cachespace_status[0]['value'] != 0) {
      $output .= '<div class = "confirm-message">' . t('This will initialize the cache on host. If you are sure click Initialize.') . '</div>';

      $warning = mercury_management_host_agent_compatibility_warning($host_node);
      if (!empty($warning)) {
        $output .= '<div class="inline-messages warning">' . $warning . '</div>';
      }

      $output .= $button_code = '<div class = "confirm-action"><div class ="host-' . $type . ' mercury-button" id="host-' . $host_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';
    } else {
      $output .= '<div class = "res-message red">Cannot initialize the cache. Please confirm that a Flash Device has been assigned.</div>';
    }
  } else {
    $output .= '<div class = "res-message red">Flash Accel Agent not Installed or the Host: ' . $host_node->title . ' is Unavailable.';
  }
  $output .= '</div>';
  return $output;
}

function mercury_utilities_hostinfo_popup($host_id) {
  $output = '';
  $host_node = node_load($host_id);
  $host_name = $host_node->title;
  //$cpu_model = $host_node->field_cpu_model['und'][0]['value'];
  //$cpu_usage = $host_node->field_cpu_usage['und'][0]['value'];
  //$boot_time = $host_node->field_boottime['und'][0]['value'];
  //$host_status = $host_node->field_host_status['und'][0]['value'];
  $host_status_details = mercury_management_get_host_status($host_node);
  $physical_memory = $host_node->field_physical_memory['und'][0]['value'];
  $physical_memory_value = explode(' ', $physical_memory);
  $physical_memory_value = number_format($physical_memory_value[0] / 1073741824, 2, '.', '') . 'GB';
  //$bundle_status = $host_node->field_bundle_status['und'][0]['value'];
  $sofware_on_host = $host_node->field_software_on_host['und'][0]['value'];
  //$cache_state = $host_node->field_cache_state['und'][0]['value'];
  $segment_size = field_get_items('node', $host_node, 'field_segment_size');
  $allocated_segment = field_get_items('node', $host_node, 'field_allocated_segments');
  $total_segment = field_get_items('node', $host_node, 'field_total_segments');
  //$cache_state = field_get_items('node', $host_node, 'field_cache_state');
  //$device_state = field_get_items('node', $host_node, 'field_device_state');
  $first_part = ceil(($segment_size[0]['value'] * $allocated_segment[0]['value']) / 1073741824);
  $second_part = ceil(($segment_size[0]['value'] * $total_segment[0]['value']) / 1073741824);
  $vendor = field_get_items('node', $host_node, 'field_host_vendor');
  $model = field_get_items('node', $host_node, 'field_host_model');
  $platform_name = $vendor[0]['value'] . "-" . $model[0]['value'];
  $vib_version = field_get_items('node', $host_node, 'field_vib_version');
  $naa_id = field_get_items('node', $host_node, 'field_naa_identifier');
  $total_size = field_get_items('node', $host_node, 'field_total_size');

  $str = '<span class = "host-status">' . $host_status_details['icon'] . '</span>';

  $output .= '<div class = "host-info">';
  $output .= '<div class = "host-info-header">';
  $output .= '<span class = "host-name">' . $host_name . '</span>';
  $output .= '<span class = "host-cache">Cache: ' . $first_part . 'GB / ' . $second_part . 'GB</span>';
  $output .= $str . '<span class = "bundle-status">' . $host_status_details['status_message'] . '</span>';
  $output .= '</div>';
  $output .= '<div class = "host-info-overview">';

  $warning = mercury_management_host_agent_compatibility_warning($host_node);
  if (!empty($warning)) {
    $output .= '<div class="inline-messages warning">' . $warning . '</div>';
  }

  $output .= '<div class = "more-info-link"><span class = "mercury-button">' . l('More', 'node/' . $host_id) . '</span></div>';
  $output .= '<div class = "wraper-div clearfix">';
  $output .= '<div class = "host-info-left">';
  $output .= '<div class = "overview-heading"><strong>Host Overview</strong></div>';
  $output .= '<div class = "overview-details">';
  // $output .= '<div class = "cpu-model">CPU Model: ' . $cpu_model . '</div>';
  //$output .= '<div class = "cpu-usage">CPU Usage: ' . $cpu_usage . '</div>';
  $output .= '<div class = "platform-name">Platform Name: ' . $platform_name . '</div>';
  //$cur_time = strtotime('now');
  //$last_boot_time = explode('T', $boot_time);
  //$last_boot = strtotime($last_boot_time[0]);
  // $output .= '<div class = "boot-time">Up Time: ' . date_calc_diff($last_boot, $cur_time, false, false, true, false, false, false) . '</div>';
  //$output .= '<div class = "host-status">Host Status: ' . $host_status . '</div>';
  $output .= '<div class = "physical-memory">Physical Memory: ' . $physical_memory_value . '</div>';
  $output .= '<div class = "software-on-host">Software on Host: ' . $sofware_on_host . '</div>';
  $output .= '</div>';
  $output .= '</div>';

  $output .= '<div class = "host-info-right">';

  $output .= '<div class = "overview-heading"><strong>Flash Accel Overview</strong></div>';
  $output .= '<div class = "overview-details">';
  $output .= '<div class = "flash-accel-status">Flash Accel Host Agent Installed: ' . $host_status_details['installed'] . '</div>';
  $output .= '<div class = "package-version">Host Agent Version: ' . $vib_version[0]['value'] . '</div>';
  $output .= '</div>';
  $output .= '</div>';

  $output .= '</div>';

  $output .= '<div class = "overview-heading"><strong>Device Overview</strong></div>';
  $output .= '<div class = "overview-details">';
  $output .= '<div class = "flash-device-name">Device Name: ' . $naa_id[0]['value'] . '</div>';
  $output .= '<div class = "device-size">Device Size: ' . number_format($total_size[0]['value'] / 1073741824, 2, '.', '') . 'GB</div>';
  $output .= '</div>';
  $output .= '</div>';
  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

