<?php
/**
 *  Flash Accel Management Console (FMC)
 *  Copyright (C) 2013, NetApp, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *  Javascript
 *
 *    Farbtastic - Copyright (c) 2010 Matt Farina
 *
 *    jQuery - Copyright (c) 2010 John Resig
 *
 *    jQuery BBQ - Copyright (c) 2010 "Cowboy" Ben Alman
 *
 *    jQuery Cookie - Copyright (c) 2006 Klaus Hartl
 *
 *    jQuery Form - Copyright (c) 2010 Mike Alsup
 *
 *    jQuery Once - Copyright (c) 2009 Konstantin Käfer
 *
 *    jQuery UI - Copyright (c) 2010 by the original authors
 *      (http://jqueryui.com/about)
 *
 *    Sizzle.js - Copyright (c) 2010 The Dojo Foundation (http://sizzlejs.com/)
 *
 *  PHP
 *
 *    ArchiveTar - Copyright (c) 1997 - 2008 Vincent Blavet
 */

/**
 * This file contains functions for VM wrench actions.
 */

/**
 * Function to display the Wrench Menu for a VM
 * @param Numeric $vm_id
 * @param Array $vm_node
 */
function mercury_utilities_vm_wrench_menu($vm_id, $vm_node = NULL) {
  $str = '';
  $msi_package_status = '';
  if (!isset($vm_node)) {
    $vm_node = node_load($vm_id);
  }
  if (isset($vm_node->title)) {
    $vm_name = $vm_node->title;
    $vm_name = str_replace(" ", "__", $vm_name);
  }

  //Check the Avialablity of MSI Package to install/upgrade for VM
  // XXX: Usability issue. When the agent is not installed yet,
  // it's hard to figure out why only "uninstall" is highlighted.
  $check_current_msi = trim(variable_get('vm_cur_MSI'));
  $check_upgrade_msi = trim(variable_get('vm_upgraded_MSI'));
  if ($check_current_msi != '' && $check_upgrade_msi != '') {
    $msi_package_install_link = ctools_popups_vm_settings_link('Install', $vm_id, 'settings', 'Settings');
    $msi_package_install = '<li class = "second-level-li">' . $msi_package_install_link . '</li>';
  } else {
    $msi_package_install = '<li class = "second-level-li inactive">Install</li>';
  }

  if ($check_current_msi != '' && $check_upgrade_msi != '') {
    $msi_package_upgrade_link = ctools_popups_vm_settings_link('Upgrade', $vm_id, 'settings', 'Settings');
    $msi_package_upgrade = '<li class = "second-level-li">' . $msi_package_upgrade_link . '</li>';
  } else {
    $msi_package_upgrade = '<li class = "second-level-li inactive">Upgrade</li>';
  }

  // TODO: element type div is not allowed directly  <ul>.
  $str .= '<div class = "wrench-menu-vm wrench-menu ' . $vm_name . '" id = "wrench-menu-vm-' . $vm_id . '">';
  $str .= '<ul class = "first-level-ul">';
  $str .= '<li class = "first-level-li expandable">Software';
  $str .= '<ul class = "second-level-ul">';
  $str .= $msi_package_install;
  $str .= $msi_package_upgrade;
  // XXX: We don't check to see if it's installed and show as inactive if not.
  $str .= '<li class = "second-level-li last">' . ctools_popups_vm_settings_link('Uninstall', $vm_id, 'settings', 'Settings') . '</li>';
  $str .= '</ul>';
  $str .= '</li>';
  $str .= '<li class = "first-level-li expandable">Cache Space';
  $str .= '<ul class = "second-level-ul">';
  $str .= '<li class = "second-level-li">' . ctools_popups_vm_settings_link('Allocate', $vm_id, 'settings', 'cache-space') . '</li>';
  $str .= '<li class = "second-level-li last">' . ctools_popups_vm_settings_link('Unallocate', $vm_id, 'settings', 'cache-space') . '</li>';
  $str .= '</ul>';
  $str .= '</li>';
  $enable_check = 0;
  $vm_cache_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  if ($vm_cache_state[0]['value'] == "ADMIN_ENABLED") {
    // XXX: $message is unused.
    $message = "Cache is currently enabled on the VM. Please disable the cache prior to initiating a format.";
    $enable_check = 1;
  } else {
    $vdisk_query = db_select('node', 'node');
    $vdisk_query->leftJoin('field_data_field_virtual_machine', 'field_data_field_virtual_machine', 'node.nid = field_data_field_virtual_machine.entity_id AND field_data_field_virtual_machine.entity_type = :node AND field_data_field_virtual_machine.deleted = :deleted', array(':node' => 'node', ':deleted' => 0));
    $vdisk_query->fields('node', array('nid', 'title'));
    $vdisk_query->condition('node.status', 1)
      ->condition('field_data_field_virtual_machine.field_virtual_machine_target_id', $vm_id)
      ->condition('node.type', 'vdisk');
    $vdisk_result = $vdisk_query->orderBy('node.title', 'DESC')
      ->execute();
    $vdisk_counter = 1;
    $enable_vdisk = array();
    while ($vdisk_row = $vdisk_result->fetch()) {
      $vdisk_id = $vdisk_row->nid;
      $vdisk_node = mercury_utilities_node_load_uncached($vdisk_id);
      $cache_state = field_get_items('node', $vdisk_node, 'field_fa_administration_state');
      if ($cache_state[0]['value'] == "ADMIN_ENABLED") {
        $enable_vdisk[$vdisk_counter] = $vdisk_node->title;
        $enable_check = 1;
      }
      $vdisk_counter++;
    }
  }

  $str .= '<li class = "first-level-li expandable">Cache State';
  $str .= '<ul class = "second-level-ul">';
  $str .= '<li class = "second-level-li vm-cache-enable-link">' . l('Enable', 'vm/settings/HgSetCacheState/CacheState/enable/' . $vm_id, array('attributes' => array('title' => 'Enable Cache'))) . '</li>';
  $str .= '<li class = "second-level-li last vm-cache-disable-link">' . l('Disable', 'vm/settings/HgSetCacheState/CacheState/disable/' . $vm_id, array('attributes' => array('title' => 'Disable Cache'))) . '</li>';
  $str .= '</ul>';
  $str .= '</li>';
  $str .= '<li class = "first-level-li expandable">Cache Mode';
  $str .= '<ul class = "second-level-ul">';
  $str .= '<li class = "second-level-li vm-wrtiethru-link">' . l('Write Thru', 'vm/settings/HgSetCacheState/CacheMode/writethru/' . $vm_id, array('attributes' => array('title' => 'Change Cache Mode to Write Thru'))) . '</li>';
  $str .= '<li class = "second-level-li vm-wrtiearound-link">' . l('Write Around', 'vm/settings/HgSetCacheState/CacheMode/writearound/' . $vm_id, array('attributes' => array('title' => 'Change Cache Mode to Write Around'))) . '</li>';
  $str .= '<li class = "second-level-li last vm-passthru-link">' . l('Pass Thru', 'vm/settings/HgSetCacheState/CacheMode/passthru/' . $vm_id, array('attributes' => array('title' => 'Change Cache Mode to Pass Thru'))) . '</li>';
  $str .= '</ul>';
  $str .= '</li>';
  $str .= '<li class = "first-level-li expandable">Cache Metrics';
  $str .= '<ul class = "second-level-ul">';
  $str .= '<li class = "second-level-li vm-metric-enable-link">' . l('Enable', 'vm/settings/HgSetCacheState/PerfState/enable/' . $vm_id, array('attributes' => array('title' => 'Enable Cache Metrics'))) . '</li>';
  $str .= '<li class = "second-level-li vm-metric-disable-link">' . l('Disable', 'vm/settings/HgSetCacheState/PerfState/disable/' . $vm_id, array('attributes' => array('title' => 'Disable Cache Metrics'))) . '</li>';
  $str .= '<li class = "second-level-li last vm-metric-reset-link">' . l('Reset', 'vm/settings/HgClearPerfMetrics/PerfState/reset/' . $vm_id, array('attributes' => array('title' => 'Reset Cache Metrics'))) . '</li>';
  $str .= '</ul>';
  $str .= '</li>';
  $str .= '<li class = "first-level-li">' . ctools_popups_vm_settings_link('Advanced', $vm_id, 'settings', 'advance') . '</li>';
  $str .= '<li class = "first-level-li">' . l('VM Disk Settings', 'virtual_machine/' . $vm_id . '/vm_disks', array('attributes' => array('title' => 'Go to VDisk Inventory Listings for the VM'))) . '</li>';
  $str .= '<li class = "first-level-li vm-refresh-link">' . l('Refresh', 'refresh/vm/' . $vm_id, array('attributes' => array('title' => 'Refresh the Flash Accel properties for the VM'))) . '</li>';
  $str .= '<li class = "first-level-li last">' . ctools_popups_vm_settings_link('Edit', $vm_id, 'settings', 'edit') . '</li>';
  $str .= '</ul>';
  $str .= '</div>';

  return $str;
}

/**
 * Function to display all the disks for a given VM.
 * @param $vm_id
 */
function mercury_utilities_vm_disks_display($vm_id) {
  $output = '';
  $error_message = '';
  //mercury_utilities_vm_refresh_action($vm_id, "no_refresh");
  $vm_node = node_load($vm_id);
  $driver_status = field_get_items('node', $vm_node, 'field_driver_status');
  $agent_build_version = field_get_items('node', $vm_node, 'field_fa_build_version');

  if ($driver_status[0]['value'] == "Agent Installed" && $agent_build_version[0]['value'] != 'Not Known') {
    $even_odd_counter = 1;
    $counter = 1;
    $vdisk_ids = mercury_utilities_vm_vdisk_nid_fetch($vm_id);
    $row_count = count($vdisk_ids);
    $wrench_image = '<img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/wrench.png" class = "wrench vdisk"/>';
    $wrench_image_vm = '<img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/wrench.png" class = "wrench vm"/>';
    if (count($vdisk_ids)) {
      $output .= '<div class = "vm-disk-view-title"><span class = "vm-name"><strong>VM: ' . $vm_node->title . '</strong></span><span class = "vm-wrench">' . $wrench_image_vm;
      $output .= mercury_utilities_vm_wrench_menu($vm_id, $vm_node) . '</span></div>';

      // Rather than add this to all of the vDisk operations which are past this
      // page, just show the incompatibility warning on the listing.
      $warning = mercury_management_vm_agent_compatibility_warning($vm_node);
      if (!empty($warning)) {
        // HTML is a little screwy on this page, should clean it up eventually.
        $output .= '<br /><br /><div class="inline-messages warning">' . $warning . '</div><br />';
      }

      $output .= '<div id="vm-block-' . $vm_id . '" class="mercury-vm-block">';
      $output .= '<div class="vm-row mercury-row" id="vm-' . $vm_id . '">';
      $output .= '<div class="vm-vdisk-list"><span id = "column-1"><span class = "row-title">Disks</span></span></div>';
      $output .= '</div>';
      $table_for_vdisks = '<table class="vm-allvdisk-table views-table">';
      $table_for_vdisks .= '<thead><tr><th class="header-checkbox"><input type="checkbox" name="' . $vm_node->title . '" value="' . $vm_id . '" class = "checkbox"></th><th class="header header-first">Path</th><th class="header">Type</th><th class="header">Cache State</th><th class="header">Status</th><th>Read/Write IOPS</th><th>Read/Write Latency</th><th class="info_column"></th></tr></thead>';

      foreach ($vdisk_ids as $key => $val) {
        $vdisk_node = node_load($val);
        $disk_path = field_get_items('node', $vdisk_node, 'field_path');
        $disk_type = field_get_items('node', $vdisk_node, 'field_vdisk_type');
        $disk_cache_state = field_get_items('node', $vdisk_node, 'field_fa_administration_state');
        $disk_op_state = field_get_items('node', $vdisk_node, 'field_fa_operational_state');
        $disk_metric_state = field_get_items('node', $vdisk_node, 'field_fa_performance_state');
        $disk_type = field_get_items('node', $vdisk_node, 'field_vdisk_type');
        $vdisk_metric_id = mercury_utilities_vm_vdisk_metric_nid_fetch($val);
        $metric_read_ops_value = '';
        $metric_write_ops_value = '';
        $metric_read_latency_value = '';
        $metric_write_latency_value = '';
        if ($vdisk_metric_id != '' && $disk_metric_state[0]['value'] == "ADMIN_ENABLED") {
          $metric_node = node_load($vdisk_metric_id);
          $metric_read_ops = field_get_items('node', $metric_node, 'field_vdisk_read_ops');
          $metric_read_ops_value = $metric_read_ops[0]['value'];
          $metric_write_ops = field_get_items('node', $metric_node, 'field_vdisk_write_ops');
          $metric_write_ops_value = $metric_write_ops[0]['value'];
          $metric_read_latency = field_get_items('node', $metric_node, 'field_vdisk_read_latency');
          $metric_read_latency_value = $metric_read_latency[0]['value'];
          $metric_write_latency = field_get_items('node', $metric_node, 'field_vdisk_write_latency');
          $metric_write_latency_value = $metric_write_latency[0]['value'];
        } else {
          $metric_read_ops_value = 'N';
          $metric_write_ops_value = 'A';
          $metric_read_latency_value = 'N';
          $metric_write_latency_value = 'A';
        }
        $vdisk_status = mercury_utilities_vm_vdisk_status_icon_value($vm_id, $val);
        $vdisk_op_message_value = mercury_management_state2string($disk_op_state[0]['value']);
        if ($vdisk_op_message_value == "Not Known") {
          $vdisk_op_message_value = "Unable to communicate with VM agent";
        }
        if ($vdisk_status == "red") {
          $status_image = '<img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/red.png" class = "red" alt = "Unable to Connect" title = "Unable to Communicate with VM"/>';
          $status_string = '<span class = "columns red" id = "column-4">' . $status_image . '</span>';
        } elseif ($vdisk_status == "black") {
          $status_image = '<img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/black.png" class = "black" alt = "Not Installed on VM" title = "Not Installed on VM"/>';
          $status_string = '<span class = "columns black" id = "column-4">' . $status_image . '</span>';
        } elseif ($vdisk_status == "yellow") {
          $status_image = '<img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/yellow.png" class = "yellow" alt = "Cache Disabled" title = "' . $vdisk_op_message_value . '"/>';
          $status_string = '<span class = "columns yellow" id = "column-4">' . $status_image . '</span>';
        } elseif ($vdisk_status == "green") {
          $status_image = '<img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/green.png" class = "green" alt = "Normal" title = "' . $vdisk_op_message_value . '"/>';
          $status_string = '<span class = "columns green" id = "column-4">' . $status_image . '</span>';
        }
        $disk_cache_state_value = get_human_readable_name($disk_cache_state[0]['value']);
        $disk_op_state_value = get_human_readable_name($disk_op_state[0]['value']);
        $disk_metric_state_value = get_human_readable_name($disk_metric_state[0]['value']);
        $refine_cache_class = '';
        $refine_metric_class = '';
        if ($disk_cache_state_value == "Enabled") {
          $refine_cache_class = 'cache-enable';
        } elseif ($disk_cache_state_value == "Disabled") {
          $refine_cache_class = 'cache-disable';
        }
        if ($disk_metric_state_value == "Enabled") {
          $refine_metric_class = 'metric-enable';
        } elseif ($disk_metric_state_value == "Disabled") {
          $refine_metric_class = 'metric-disable';
        }

        if ($even_odd_counter % 2 == 0) {
          if ($counter == $row_count) {
            $table_for_vdisks .= '<tr class = "even last mercury-row ' . $refine_cache_class . ' ' . $refine_metric_class . '"  id = "disk-' . $val . '">';
          } elseif ($counter == 1) {
            $table_for_vdisks .= '<tr class = "even first mercury-row ' . $refine_cache_class . ' ' . $refine_metric_class . '" id = "disk-' . $val . '">';
          } else {
            $table_for_vdisks .= '<tr class = "even mercury-row ' . $refine_cache_class . ' ' . $refine_metric_class . '" id = "disk-' . $val . '">';
          }
        } else {
          if ($counter == 1) {
            $table_for_vdisks .= '<tr class = "odd first mercury-row ' . $refine_cache_class . ' ' . $refine_metric_class . '" id = "disk-' . $val . '">';
          } elseif ($counter == $row_count) {
            $table_for_vdisks .= '<tr class = "odd last mercury-row ' . $refine_cache_class . ' ' . $refine_metric_class . '" id = "disk-' . $val . '">';
          } else {
            $table_for_vdisks .= '<tr class = "odd mercury-row ' . $refine_cache_class . ' ' . $refine_metric_class . '" id = "disk-' . $val . '">';
          }
        }
        $table_for_vdisks .= '<td class = "columns columns-checkbox"><input type="checkbox" name="' . $vdisk_node->title . '" value="' . $val . '" class = "checkbox disks"> </td>';
        $table_for_vdisks .= '<td><span>' . $disk_path[0]['value'] . '</span><span class = "vdisk-wrench">' . $wrench_image . mercury_utilities_vdisk_wrench_menu($val, $vdisk_node) . '</span></td>';
        $table_for_vdisks .= '<td>' . $disk_type[0]['value'] . '</td>';
        $table_for_vdisks .= '<td>' . $disk_cache_state_value . '</td>';
        $table_for_vdisks .= '<td>' . $disk_op_state_value . '</td>';
        $table_for_vdisks .= '<td>' . $metric_read_ops_value . '/' . $metric_write_ops_value . '</td>';
        $table_for_vdisks .= '<td>' . $metric_read_latency_value . '/' . $metric_write_latency_value . '</td>';
        $table_for_vdisks .= '<td><span class = "columns mercury-button" id = "column-3">' . ctools_popups_vdiskinfo_link('Info', $val, 'info', '') . '</span><span>' . $status_string . '</span></td>';
        $table_for_vdisks .= '</tr>';
        $counter++;
        $even_odd_counter++;
      }
      $table_for_vdisks .= '</table>';
      $output .= '<div class="vm-disks-section">' . $table_for_vdisks . '</div>';
      $output .= '</div>';
    } else {
      $error_message = "The Flash Accel VM Agent on this VM could not be reached, so VM disk configuration information is not available. Possible causes of this error include: VM powered down, networking issues (DNS resolution failure for VM host name, for example), or Flash Accel VM Agent not installed on VM. Please ensure that this VM is powered on, refresh it (using Refresh in the wrench menu), and try again. If the problem persists, troubleshoot a possible networking issue.";
    }
  } elseif ($driver_status[0]['value'] == "Agent Not Installed") {
    $error_message = "The Flash Accel VM Agent is not installed on this VM, so VM disk configuation information is not available. If you believe the VM Agent is installed, please refresh this VM and try again.";
  } else {
    $error_message = "The Flash Accel VM Agent on this VM could not be reached, so VM disk configuration information is not available. Possible causes of this error include: VM powered down, networking issues (DNS resolution failure for VM host name, for example), or Flash Accel VM Agent not installed on VM. Please ensure that this VM is powered on, refresh it (using Refresh in the wrench menu), and try again. If the problem persists, troubleshoot a possible networking issue.";
  }
  if ($error_message != '') {
    $output = '<div class = "disk-page-error">' . $error_message . '</div>';
  }
  return $output;
}

/**
 * Popup builder for VM install, uninstall, and upgrade.
 */
function mercury_utilities_vm_settings_popup($vm_id, $type) {
  $output = '';
  $title = '';
  $warning = '';
  $button_code = '';
  mercury_utilities_vm_refresh_action($vm_id, 'no-refresh');
  $vm_node = node_load($vm_id);
  $driver_status = field_get_items('node', $vm_node, 'field_driver_status');
  $cache_space = field_get_items('node', $vm_node, 'field_cachespace');
  $admin_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  $vm_name = $vm_node->title;
  $driver_status_value = $driver_status[0]['value'];
  $message = mercury_management_has_vsphere_api_error($vm_node);
  $vm_package_agent_version = variable_get('vm_package_agent_version', '');

  if ($message) {
    // Communication error.
    $button_code = '<div class = "vm-driver-details messages error"><div class = "driver-details">' . $message . '</div></div>';
  } elseif ($type == "install") {
    if ($driver_status_value == "Agent Installed") {
      $message = "Flash Accel VM agent is already installed.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    } elseif ($driver_status_value == "Agent Not Installed") {
      $title = "Installing VM agent version $vm_package_agent_version";
      $message = "This will install the Flash Accel VM agent. If you are sure click Install.";
      $link = 'vm/fmc/install/' . $vm_id;
      $link_text = "Install";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div><div class="res-message red">Note: the VM will be shut down automatically when the install completes. To reboot it instead, check the checkbox below.</div><br><div class = "reboot-check"> Reboot: <input type = "checkbox" name = "vm-reboot-check" value = "vm-reboot-check" class = "vm-reboot-check check"></div><div class ="' . $type . ' mercury-button" id="vm-' . $vm_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';

      // Install and upgrade will use the agent package so check it.
      $warning = mercury_management_vm_agent_compatibility_warning($vm_node, $check_package_version = true);

      // This may need to change for BURT 756325. (checkbox override)
    } elseif (mercury_management_guest_os_is_supported($vm_node) !== true) {
      $message = "Flash Accel VM agent cannot be installed because the VM is not running a supported guest OS.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    } else {
      $message = "Flash Accel VM agent cannot be installed as FMC is unable to communicate with the VM.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    }
  } elseif ($type == "uninstall") {
    if ($driver_status_value == "Agent Installed") {
      if ($cache_space[0]['value'] != '') {
        $message = "Unallocate the cache space from the VM before uninstalling the VM agent.";
        $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
      } elseif ($admin_state[0]['value'] == "ADMIN_ENABLED") {
        $message = "Disable the cache on the VM before uninstalling the VM agent.";
        $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
      } else {
        $agent_build_version = field_get_items('node', $vm_node, 'field_fa_build_version');
        $agent_build_version_value = $agent_build_version[0]['value'];
        $title = "Uninstalling VM agent version $agent_build_version_value";
        $message = "This will uninstall the Flash Accel VM agent. If you are sure click Uninstall.";
        $link = 'vm/fmc/uninstall/' . $vm_id;
        $link_text = "Uninstall";
        $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div><div class="res-message red">Note: the VM will reboot automatically when the uninstall completes.</div><br><div class = "reboot-check"><input type = "checkbox" name = "vm-reboot-check" value = "vm-reboot-check" class = "vm-reboot-check check" checked = "checked" style="display:none;"></div><div class ="' . $type . ' mercury-button" id="vm-' . $vm_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';

        // Uninstall doesn't need to check the package version, won't be used.
        $warning = mercury_management_vm_agent_compatibility_warning($vm_node, $check_package_version = false);
      }
    } elseif ($driver_status_value == "Agent Not Installed") {
      $message = "No Flash Accel VM agent is installed.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    } else {
      $message = "Cannot uninstall the VM agent due to an error in the VM.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    }
  } else { // upgrade
    /*  if ($cache_space[0]['value'] != '') {
      $message = "Unallocate the Cache Space from the VM first to Uninstall the Flash Accel Guest OS Agent.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
      } elseif ($admin_state[0]['value'] == "ADMIN_ENABLED") {
      $message = "Disable the Cache State first to Uninstall the Flash Accel Guest OS Agent.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
      } else { */
    // This may need to change for BURT 756325. (checkbox override)
    if (mercury_management_guest_os_is_supported($vm_node) !== true) {
      $message = "Flash Accel VM agent cannot be upgraded because the VM is not running a supported guest OS.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    } elseif ($driver_status_value == "Agent Installed") {
      $agent_build_version = field_get_items('node', $vm_node, 'field_fa_build_version');
      $agent_build_version_value = $agent_build_version[0]['value'];
      $title = "Upgrading VM agent $agent_build_version_value to version $vm_package_agent_version";
      $message = "This will upgrade the Flash Accel VM agent. If you are sure click Upgrade.";
      $link_text = "Upgrade";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div><div class="res-message red">Note: the VM will be shut down automatically when the upgrade completes. To reboot it instead, check the checkbox below.</div><br><div class = "reboot-check"> Reboot: <input type = "checkbox" name = "upgrade-vm-reboot-check" value = "upgrade-vm-reboot-check" class = "upgrade-vm-reboot-check check"></div><div class ="' . $type . ' mercury-button" id="vm-' . $vm_id . '"><span class = "mercury-button-model">' . $link_text . '</span></div><div id="vm-status-message" class="operation-message"></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';

      // Install and upgrade will use the agent package so check it.
      $warning = mercury_management_vm_agent_compatibility_warning($vm_node, $check_package_version = true);
    } else {
      $message = "Cannot run upgrade process as Flash Accel Agent is not installed or unreachable.";
      $button_code = '<div class = "vm-driver-details"><div class = "driver-details">' . $message . '</div></div>';
    }

  }
  $output .= '<div class = "popup-setting-page">';
  if (!empty($title)) {
    $output .= '<div class = "row-title"><span><strong>' . $title . '</strong></span></div><br>';
  }
  $output .= '<div class = "vm-name"><span>VM Name: ' . $vm_name . '</span></div>';
  if (!empty($warning)) {
    $output .= '<div class="inline-messages warning">' . $warning . '</div>';
  }
  $output .= $button_code;
  //$output .= '<div class="close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

/**
 * Function for calling the refresh function for individual vm
 * @deprecated no more references, single VM refresh no longer uses popup.
 */
function mercury_utilities_vm_refresh_action_popup($vm_id) {
  global $base_url;
  ctools_include('ajax');
  ctools_include('modal');
  $vm_node = node_load($vm_id);
  $vm_name = $vm_node->title;
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_id);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  //$driver_status = field_get_items('node', $vm_node, 'field_driver_status');
  $driver_status = mercury_management_checkformercuryinstalled($vc_hostname, $vc_username, $vc_password, $vm_node->nid);
  if ($driver_status == "Unable to Communicate with VM") {
    mercury_utilities_fmc_log_message('VM Refresh', t('Refresh failed, unable connect to VM @vm_name.', array('@vm_name' => $vm_name)), WATCHDOG_WARNING);
  } else {
    $host_nid = $vm_data['host'];
    $result = mercury_management_updatevm($vc_hostname, $vc_username, $vc_password, $vm_name, $host_nid);
    $result_split = explode('--', $result);
    if (count($result_split) > 1) {
      mercury_utilities_fmc_log_message('VM Refresh', t('VM @vm_name refresh failed. Unable to connect to VM to retrieve cache information.', array('@vm_name' => $vm_name)), WATCHDOG_WARNING);
    } else {
      famc_analytics_fetch_vm_metric($vm_id);
      mercury_utilities_fmc_log_message('VM Refresh', t('VM @vm_name refreshed successfully', array('@vm_name' => $vm_name)));
    }
  }
  $out = array();
  $out[] = ctools_ajax_command_redirect($base_url);
  print ajax_render($out);
  exit;
}

/**
 * popup for the vm refresh action confirmation
 * @deprecated no references
 */
function mercury_utilities_vm_refresh_popup($vm_id) {
  $output = '';
  $vm_data = node_load($vm_id);
  $vm_name = $vm_data->title;
  $driver_status = field_get_items('node', $vm_data, 'field_driver_status');
  $message = "This will refresh the VM configuration and cache information. If you are sure click Refresh.";
  $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Refresh', 'refresh/vm/' . $vm_id) . '</span></div>';
  $output = '<div class = "poopup-setting-page">';
  $output .= '<div class = "vm-name"><span>VM Name: ' . $vm_name . '</span></div>';
  $output .= '<div class = "driver-status">' . $driver_status_value . '</div>';
  $output .= '<div class = "notify-message">' . $message . '</div>';
  $output .= $button_code;
  $output .= '</div>';
  return $output;
}

/**
 * popup for the advance options for VMs
 */
function mercury_utilities_vm_settings_advance_options($vm_id) {
  $output = '';
  $message = '';
  $warning = '';
  $button_code = '';
  $vm_node = node_load($vm_id);
  $driver_status = field_get_items('node', $vm_node, 'field_driver_status');

  // If the API is unavailable then all tabs are unavailable.
  $api_error = mercury_management_has_fa_api_error($vm_node);
  if ($api_error) {
    return '<div class="messages error">' . $api_error . '</div>';
  }

  $output .= '<div class = "advance-tabs-form">';
  $output .= '<div class = "advance-tabs-wrapper clearfix"><div class = "advance-tab active" id = "set-cache-tab">Set Cache State</div><div class = "advance-tab" id = "cache-format-tab">Cache Format</div><div class = "advance-tab" id = "migration-tab">Migration</div></div>';
  $output .= '<div class = "tab-menu" id = "set-cache">';
  $output .= '<div class = "set-cache-form">';
  if ($driver_status[0]['value'] == 'Agent Installed') {
    $output .= drupal_render(drupal_get_form('mercury_utilities_vm_settings_advance_options_form', $vm_id));
  } else {
    $output .= 'Flash Accel Agent not Installed or the VM: ' . $vm_node->title . ' is Unavailable.';
  }
  $output .= '</div>';
  $output .= '</div>';
  $output .= '<div class = "cache-format-form">';
  $vm_cache_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  $cache_space = field_get_items('node', $vm_node, 'field_cachespace');
  if ($cache_space[0]['value'] == '') {
    $message = "No cache is allocated to the VM. Please allocate cache to the VM before initiating a format.";
  } else {
    if ($vm_cache_state[0]['value'] == "ADMIN_ENABLED") {
      $message = "The cache is enabled in the VM. Please disable the cache before initiating a format.";
    } else {
      $message = "This will start a cache format. If you are sure click 'Start'.";
      $button_code = '<div class = "cache-format-action"><span class = "mercury-button">' . l('Start', 'vm/settings/HgStartCacheFormat/format/start/' . $vm_id) . '</span><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div></div>';
      $warning = mercury_management_vm_agent_compatibility_warning($vm_node);
    }
  }
  // For this popup, put the warning at the top.
  // Otherwise it's jarring when you switch tabs.
  if (!empty($warning)) {
    $output .= '<div class="inline-messages warning">' . $warning . '</div>';
  }
  $output .= '<div class = "notify-message">' . $message . '</div>';
  $output .= $button_code;
  $output .= '</div>';
  $output .= '<div class = "tab-menu" id = "migration-policy">';
  $output .= '<div class = "migration-form">';
  $output .= drupal_render(drupal_get_form('mercury_utilities_vm_settings_advance_migration_form', $vm_id));
  $output .= '</div>';
  $output .= '</div>';
  $output .= '</div>';
  return $output;
}

function mercury_utilities_vm_settings_advance_options_form($form, &$form_state, $vm_id) {
  //if (empty($vm_id)) {
  //  $vm_id = $form_state['vm_id'];
  //}
  $vm_node = node_load($vm_id);
  $cache_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  $cache_mode = field_get_items('node', $vm_node, 'field_fa_cache_mode');
  $perf_state = field_get_items('node', $vm_node, 'field_fa_performance_state');
  if ($cache_state[0]['value'] == "ADMIN_ENABLED") {
    $cache_state_default = 0;
  } else {
    $cache_state_default = 1;
  }
  if ($cache_mode[0]['value'] == "CM_WRITETHRU") {
    $cache_mode_default = 0;
  } elseif ($cache_mode[0]['value'] == "CM_PASSTHRU") {
    $cache_mode_default = 1;
  } else {
    $cache_mode_default = 2;
  }
  if ($perf_state[0]['value'] == "ADMIN_ENABLED") {
    $perf_state_default = 0;
  } else {
    $perf_state_default = 1;
  }
  $cache_states = array(0 => t('Enable'), 1 => t('Disable'));
  $cache_modes = array(0 => t('Write Thru'), 1 => t('Pass Thru'), 2 => t('Write Around'));
  $perf_states = array(0 => t('Enable'), 1 => t('Disable'));

  $agent_warning = mercury_management_vm_agent_compatibility_warning($vm_node);
  if (!empty($agent_warning)) {
    $form['agent_warning'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="inline-messages warning">' . $agent_warning . "</div>",
    );
  }

  $form['cachestate'] = array(
    '#type' => 'radios',
    '#title' => t('Cache State'),
    '#default_value' => $cache_state_default,
    '#options' => $cache_states,
    '#description' => t('Select the cache state.'),
    /* '#access' => $admin, */
  );
  $form['cachemode'] = array(
    '#type' => 'radios',
    '#title' => t('Cache Mode'),
    '#default_value' => $cache_mode_default,
    '#options' => $cache_modes,
    '#description' => t('Select the cache mode.'),
    /* '#access' => $admin, */
  );
  $form['perfstate'] = array(
    '#type' => 'radios',
    '#title' => t('Metrics'),
    '#default_value' => $perf_state_default,
    '#options' => $perf_states,
    '#description' => t('Select the cache metric state.'),
    /* '#access' => $admin, */
  );
  $form['vm_id'] = array(
    '#type' => 'hidden',
    '#value' => $vm_id
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
    '#suffix' => '<script>jQuery("#edit-submit").click(function () { jQuery("#edit-submit").hide()});</script>'
  );
  return $form;
}

function mercury_utilities_vm_settings_advance_options_form_submit($form, &$form_state) {
  ctools_include('ajax');
  ctools_include('modal');
  $values = $form_state['values'];
  if ($values['vm_id'] != '') {
    $selected_cache_state = $values['cachestate'];
    $selected_cache_mode = $values['cachemode'];
    $selected_perf_state = $values['perfstate'];
    $vm_id = $values['vm_id'];
    if ($selected_cache_state == 0) {
      $cache_state = "ADMIN_ENABLED";
    } elseif ($selected_cache_state == 1) {
      $cache_state = "ADMIN_DISABLED";
    }
    if ($selected_cache_mode == 0) {
      $cache_mode = "CM_WRITETHRU";
    } elseif ($selected_cache_mode == 1) {
      $cache_mode = "CM_PASSTHRU";
    } elseif ($selected_cache_mode == 2) {
      $cache_mode = "CM_WRITEAROUND";
    }
    if ($selected_perf_state == 0) {
      $perf_state = "ADMIN_ENABLED";
    } elseif ($selected_perf_state == 1) {
      $perf_state = "ADMIN_DISABLED";
    }
    $param = array('request' => array('CacheMode' => $cache_mode, 'CacheState' => $cache_state, 'PerfState' => $perf_state));
    $ws_result = mercury_management_cache_operation('nid', $vm_id, 'HgSetCacheState', $param);
    if ($ws_result == 'error_wsdl_call') {
      $vm = node_load($vm_id);
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Error reconfiguring cache on VM "@vm". VM agent communication error.', array('@vm' => $vm->title)), WATCHDOG_ERROR);
      if ($cache_state == "ADMIN_ENABLED") {
        mercury_utilities_fmc_log_message('Cache Mgmt', t("If the cache operation continues to fail, please check the Installation and Administration guide to confirm that all memory and other requirements have been met."), WATCHDOG_ERROR);
        mercury_utilities_fmc_log_message('Cache Mgmt', t("You can also try rebooting the VM, formatting the cache in the VM's advanced settings, and then enabling the cache again."), WATCHDOG_ERROR);
      }
    } elseif ($ws_result['response_status'] == 'success') {
      $param = array();
      $vm_info = node_load($vm_id);
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache reconfigured successfully on VM @vm', array('@vm' => $vm_info->title)));

      // If cache or perf was disabled insert an empty perf node
      if ($cache_state == "ADMIN_DISABLED" || $perf_state == "ADMIN_DISABLED") {
	famc_save_empty_vm_metric($vm_id);
      }

      $attrInfo = mercury_management_cache_operation('nid', $vm_info->nid, 'HgGetCacheInfo', $param);
      //mercury_utilities_fmc_log_message('attrInfo', $attrInfo);
      if (!empty($attrInfo)) {
        $vm_info->field_fa_build_version['und'][0]['value'] = $attrInfo['build_version'];
        $vm_info->field_fa_cache_mode['und'][0]['value'] = $attrInfo['cache_mode'];
        $vm_info->field_fa_administration_state['und'][0]['value'] = $attrInfo['admin_state'];
        $vm_info->field_fa_cache_size['und'][0]['value'] = $attrInfo['cache_size'];
        $vm_info->field_fa_sector_size['und'][0]['value'] = $attrInfo['sector_size'];
        $vm_info->field_fa_operational_state['und'][0]['value'] = $attrInfo['op_state'];
        $vm_info->field_fa_operational_message['und'][0]['value'] = $attrInfo['op_message'];
        $vm_info->field_fa_performance_state['und'][0]['value'] = $attrInfo['perf_state'];
        if ($attrInfo['cachespace_id'] != '') {
          $vm_info->field_cachespace['und'][0]['value'] = 'naa.' . $attrInfo['cachespace_id'];
        }
        node_save($vm_info);
      }
    } else {
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache reconfiguration failed on VM @vm. Please refresh VM and try again.', array('@vm' => $vm_info->title)), WATCHDOG_ERROR);
      if ($cache_state == "ADMIN_ENABLED") {
        mercury_utilities_fmc_log_message('Cache Mgmt', t("If the cache operation continues to fail, please check the Installation and Administration guide to confirm that all memory and other requirements have been met."), WATCHDOG_ERROR);
        mercury_utilities_fmc_log_message('Cache Mgmt', t("You can also try rebooting the VM, formatting the cache in the VM's advanced settings, and then enabling the cache again."), WATCHDOG_ERROR);
      }
    }
  }
  $out = array();
  $out[] = ctools_ajax_command_redirect($_SERVER['HTTP_REFERER']);
  print ajax_render($out);
  exit;
}

/**
 * Function to set the Individual VM Migration settings
 * @param Object $form
 * @param Object $form_state
 * @param String $vm_id
 */
function mercury_utilities_vm_settings_advance_migration_form($form, &$form_state, $vm_nid) {
  $form = array();
  $custom_host_list = array();

  // Get the vcenter, datacenter and host information
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_nid);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $datacenter_nid = $vm_data['datacenter'];
  $vm_host_nid = $vm_data['host'];
  $vm_host = node_load($vm_host_nid);
  $vm_info = node_load($vm_nid);

  $agent_warning = mercury_management_vm_agent_compatibility_warning($vm_info);
  if (!empty($agent_warning)) {
    $form['agent_warning'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="inline-messages warning">' . $agent_warning . "</div>",
    );
  }

  // Form heading
  $form['vm_migration_settings'] = array(
    '#type' => 'fieldset',
    '#title' => t('<b>VM Migration Policy Settings for VM: ' . $vm_info->title . '</b>'),
    '#collapsible' => false,
    '#collapsed' => false
  );

  $default_policy = variable_get('migration_policy');
  $default_policies = array(2 => 'Cluster', 3 => 'Data Center', 4 => 'Host');

  $vm_hostlist_value = field_get_items('node', $vm_info, 'field_vm_vmotion_hostlist');
  $vm_hostlist = $vm_hostlist_value[0]['value'];
  if (!empty($vm_hostlist) && count(unserialize($vm_hostlist)) > 1) {
    $migration_enabled = '1';
    //Get the list of Host for migration
    $custom_host_list = unserialize($vm_hostlist);
  } else {
    // TODO: In US1347 we may need to refine this.  Only one host doesn't
    // automatically mean we have to uncheck the box.  It matters when
    // we try to automatically add more hosts in the future but not now.
    $migration_enabled = '0';
    $custom_host_list = array();
    $custom_host_list[] = $vm_host_nid;
  }

  if ($default_policy == 2) {
    $policy_current[2] = '(Global)';
    $policy_current[3] = '';
  } elseif ($default_policy == 3) {
    $policy_current[2] = '';
    $policy_current[3] = '(Global)';
  }

  $form['vm_migration_settings']['enable_vm_migration'] = array(
    '#type' => 'checkbox',
    '#title' => 'Enable Migration',
    '#default_value' => $migration_enabled
  );

  $form['vm_migration_settings']['host_list'] = array(
    '#type' => 'fieldset',
    '#title' => t('Select Host'),
    '#collapsible' => false,
    '#collapsed' => false
  );

  $all_host_ids = array();
  $all_checked = array();
  $all_host_class = array();
  $disable_host_current = array();
  $migrate_to_host = array();
  $current_host = '';

  // Get the list of hosts in the datacenter
  $datacenter_hostlist = array();
  $datacenter_hostnames = mercury_management_allocationHostList($vm_nid, "datacenter", $vc_hostname, $vc_username, $vc_password);
  foreach ($datacenter_hostnames as $hostname) {
    $datacenter_hostlist[] = db_query("SELECT nid FROM {node} WHERE type = :tp AND title = :ttl", array(':tp' => 'host', ':ttl' => trim($hostname)))->fetchField();
  }

  // Get the list of hosts in the cluster
  $cluster_hostlist = array();
  $cluster_hostnames = mercury_management_allocationHostList($vm_nid, "cluster", $vc_hostname, $vc_username, $vc_password);
  foreach ($cluster_hostnames as $hostname) {
    $cluster_hostlist[] = db_query("SELECT nid FROM {node} WHERE type = :tp AND title = :ttl", array(':tp' => 'host', ':ttl' => trim($hostname)))->fetchField();
  }

  /**
   * Loop over the hosts in the datacenter
   */
  foreach ($datacenter_hostlist as $host_nid) {

    $host_data = mercury_utilities_node_load_uncached($host_nid);

    // skip the host if there is no agent or SSD is not initialized
    $bundle_install = field_get_items('node', $host_data, 'field_bundle_status');
    $cache_state = field_get_items('node', $host_data, 'field_cache_state');
    if ($bundle_install[0]['value'] != "Agent Installed" || $cache_state[0]['value'] != 3) {
      continue;
    }

    // TODO: This should check discovery phase, but because this form handles
    // transient problems rather poorly right now I'm going to leave them in the
    // list and let the error handling deal with it.
    // Ideally the user should be able to select hosts with transient
    // problems (like rebooting) and FAMC will eventually correct itself.
    // Revisit this in Dubnium along with the related resize and lazy allocation
    // changes, which are going to have similar problems.
    //$api_error = mercury_management_has_fa_api_error($host_data);

    // add host to the list
    $all_host_ids[$host_nid] = $host_data->title;

    // check if the host is in the cluster list
    if (in_array(trim($host_data->title), $cluster_hostnames)) {
      $host_class = 'cluster';
    } else {
      $host_class = 'datacenter';
    }

    // check if the VM is currently on this host
    if ($vm_host_nid == $host_nid) {
      $all_host_class['class'] = array('current_host');
      $disable_host_current = array('disabled' => 'disabled', 'checked' => 'checked');
      $current_host = $host_nid;
    } else {
      $all_host_class['class'] = array($host_class);
      $disable_host_current = array();
      $migrate_to_host[] = 'mgn_hid_' . $host_nid;

      // now check if this host was previously selected
      if (in_array($host_nid, $custom_host_list)) {
        $disable_host_current = array('checked' => 'checked');
      } else {
        if ($migration_enabled == 0) {
          $disable_host_current = array('disabled' => 'disabled');
        }
      }
    }

    $form['vm_migration_settings']['host_list']['mgn_hid_' . $host_nid] = array(
      '#type' => 'checkbox',
      '#title' => $host_data->title,
      '#attributes' => array_merge($disable_host_current, $all_host_class)
    );
  }

  $form['previous_host_list'] = array(
    '#type' => 'hidden',
    '#value' => serialize($custom_host_list)
  );

  $form['migration_host_list'] = array(
    '#type' => 'hidden',
    '#value' => implode('|', $migrate_to_host)
  );

  $form['vm_current_host'] = array(
    '#type' => 'hidden',
    '#value' => $current_host
  );

  $form['vm_migration_settings']['vm_migration_policy'] = array(
    '#type' => 'markup',
    '#markup' => '<div id="block-migration-selector">
		  <h2 class="block-title">Refine Policy</h2>
		  <div class="content">
		  <ul class="menu">
		  <li class="leaf cluster-selector"><a title="Cluster" href="#" class="selector-tool-item-cluster">Cluster</a> ' . $policy_current[2] . '</li>
		  <li class="leaf datacenter-selector"><a title="Datacenter" href="#" class="selector-tool-item-datacenter">Datacenter</a> ' . $policy_current[3] . '</li></ul>
		  </div>
		  </div>',
  );

  $form['vm_migration_settings']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Configurations'),
    '#prefix' => '<div id="save-migration-selector">
                  <script>
                  var is_selected = 0;
                  jQuery("#edit-enable-vm-migration").click(function() {
			is_selected = jQuery(this).attr("checked");
			if (is_selected) {
			   jQuery("#edit-host-list .form-checkbox.cluster").removeAttr("disabled", "disabled");
			   jQuery("#edit-host-list .form-checkbox.datacenter").removeAttr("disabled", "disabled");
			} else {
			   jQuery("#edit-host-list .form-checkbox.cluster").attr("disabled", "disabled");
			   jQuery("#edit-host-list .form-checkbox.datacenter").attr("disabled", "disabled");
			   jQuery("#edit-host-list .form-checkbox.cluster").removeAttr("checked");
			   jQuery("#edit-host-list .form-checkbox.datacenter").removeAttr("checked");
			}
		  });
		  jQuery("#block-migration-selector .leaf.cluster-selector").click(function(e){
			if (is_selected) {
			    jQuery("#edit-host-list .form-checkbox.datacenter").removeAttr("checked");
      			    jQuery("#edit-host-list .form-checkbox.cluster").attr("checked", "checked");
      			}
		  });
  	    	  jQuery("#block-migration-selector .leaf.datacenter-selector").click(function(e){
      			if (is_selected) {
      			    jQuery("#edit-host-list .form-checkbox").attr("checked", "checked");
      			}
		  });
		</script>',
    '#suffix' => '</div>'
  );

  $form['vm_id'] = array(
    '#type' => 'hidden',
    '#value' => $vm_nid
  );

  return $form;
}

/**
 * Function to save the Individual VM Migration settings
 */
function mercury_utilities_vm_settings_advance_migration_form_submit($form, &$form_state) {

  $all_host_save = array();
  $values = $form_state['values'];
  $all_host_list = explode('|', $values['migration_host_list']);
  $previous_host_list = unserialize($values['previous_host_list']);
  $current_host = $values['vm_current_host'];

  // Note: all_host_list includes all hosts except the
  // current host
  $all_host_save[] = $current_host;
  foreach ($all_host_list as $host_nid) {
    $get_host_nid_val = $values[$host_nid];
    if ($get_host_nid_val > 0) {
      $get_host_nid = explode('mgn_hid_', $host_nid);
      $all_host_save[] = $get_host_nid[1];
    }
  }

  $vm_data = mercury_utilities_node_load_uncached($values['vm_id']);
  $cache_size_val = field_get_items('node', $vm_data, 'field_fa_cache_size');
  $cache_size = $cache_size_val[0]['value'];

  // If cache space is already allocateted to the VM
  if ($cache_size > 0) {
    $host_list_deleted = array_diff($previous_host_list, $all_host_save);
    $host_list_added = array_diff($all_host_save, $previous_host_list);

    // Add cache to newly added hosts
    if (count($host_list_added) > 0) {
      $addhosts = mercury_management_addCacheHosts($vm_data->nid, $host_list_added);
    }

    // Delete cache to hosts that were deleted
    if (count($host_list_deleted) > 0) {
      $delhosts = mercury_management_delCacheHosts($vm_data->nid, $host_list_deleted);
    }
    // reconstruct the new set of hosts
    $rem_hosts = array_diff($previous_host_list, (array) $delhosts);
    $all_host_save = array_merge((array) $rem_hosts, (array) $addhosts);
    $new_host_list = serialize($all_host_save);
  } else {
    $new_host_list = serialize($all_host_save);
  }

  /**
   * If the only selected host is the current host disable vmotion
   * If vmotion was disabled save current host in hostlist (this is so that we do not use global policy)
   */
  $vm_data->field_vm_vmotion_policy['und'][0]['value'] = "custom";
  if (count($all_host_save) > 1) {
    $vm_data->field_vm_vmotionable['und'][0]['value'] = 1;
    $vm_data->field_vm_vmotion_hostlist['und'][0]['value'] = $new_host_list;
  } else {
    // TODO: In US1347 we may need to refine this.  Only one host doesn't
    // automatically mean we have to uncheck the box.  It matters when
    // we try to automatically add more hosts in the future but not now.
    $vm_data->field_vm_vmotionable['und'][0]['value'] = 0;
    if (count($all_host_save) == 1) {
      $vm_data->field_vm_vmotion_hostlist['und'][0]['value'] = $new_host_list;
    } else {
      $vm_data->field_vm_vmotion_hostlist['und'][0]['value'] = '';
    }
  }

  node_save($vm_data);
}

/**
 * This handles "other" VM popups, but currently that's only unallocate.
 * The remainder have been moved to "Advanced."
 */
function mercury_utilities_vm_settings_others_popup($type, $action, $vm_id) {
  $output = '';
  $vm_node = node_load($vm_id);

  $api_error = mercury_management_has_fa_api_error($vm_node);
  if ($api_error) {
    return '<div class="messages error">' . $api_error . '</div>';
  }

  $driver_status = field_get_items('node', $vm_node, 'field_driver_status');
  $cache_space = field_get_items('node', $vm_node, 'field_cachespace');
  $vm_name = $vm_node->title;
  $driver_status_value = $driver_status[0]['value'];
  $param = array();
  $button_code = '';
  $admin_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  $cache_mode = field_get_items('node', $vm_node, 'field_fa_cache_mode');
  $perf_state = field_get_items('node', $vm_node, 'field_fa_performance_state');
  //$result = mercury_management_cache_operation('nid', $vm_id, 'HgGetCacheInfo', $param);
  /**
   * Most of these no longer have individual confirmation popups,
   * but rather are grouped together under Advanced.
  if ($type == "cache-state") {
    $cache_state = $admin_state[0]['value'];
    if ($action == "Enable" && $cache_state == "ADMIN_ENABLED") {
      $message = "Cache is already enabled.";
    } elseif ($action == "Enable" && ($cache_state == "ADMIN_DISABLED" || $cache_state == "ADMIN_UNKNOWN")) {
      if ($cache_space[0]['value'] == '') {
        $message = "Please allocate cache to the VM before enabling it.";
      } else {
        $message = "This will enable the cache. If you are sure click Enable";
        $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Enable', 'vm/settings/HgSetCacheState/CacheState/enable/' . $vm_id) . '</span></div>';
      }
    } elseif ($action == "Disable" && ($cache_state == "ADMIN_ENABLED" || $cache_state == "ADMIN_UNKNOWN")) {
      $message = "This will disable the cache. If you are sure click Disable";
      $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Disable', 'vm/settings/HgSetCacheState/CacheState/disable/' . $vm_id) . '</span></div>';
    } elseif ($action == "Disable" && $cache_state == "ADMIN_DISABLED") {
      $message = "Cache is already disabled.";
    }
  } elseif ($type == "cache-mode") {
    $cache_mode = $cache_mode[0]['value'];
    if ($action == "Write Thru") {
      if ($cache_mode == "CM_PASSTHRU" || $cache_mode == "CM_WRITEAROUND") {
        $message = "This will change the cache mode to writethru. If you are sure click Write Thru.";
        $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Write Thru', 'vm/settings/HgSetCacheState/CacheMode/writethru/' . $vm_id) . '</span></div>';
      } else {
        $message = "Cache is already in writethru mode.";
      }
    } elseif ($action == "Write Around") {
      if ($cache_mode == "CM_PASSTHRU" || $cache_mode == "CM_WRITETHRU") {
        $message = "This will change the cache mode to writearound. If you are sure click Write Around.";
        $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Write Around', 'vm/settings/HgSetCacheState/CacheMode/writearound/' . $vm_id) . '</span></div>';
      } else {
        $message = "Cache is already in writearound mode.";
      }
    } elseif ($action == "Pass Thru") {
      if ($cache_mode == "CM_WRITEAROUND" || $cache_mode == "CM_WRITETHRU") {
        $message = "This will change the cache mode to passthru. If you are sure click Pass Thru.";
        $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Pass Thru', 'vm/settings/HgSetCacheState/CacheMode/passthru/' . $vm_id) . '</span></div>';
      } else {
        $message = "Cache is already in passthru mode.";
      }
    }
  } elseif ($type == "cache-metrics") {
    $perf_state = $perf_state[0]['value'];
    if ($action == "Enable") {
      if ($perf_state == "ADMIN_ENABLED") {
        $message = "Cache metrics are already enabled.";
      } else {
        $message = "This will enable cache metrics. If you are sure click Enable.";
        $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Enable', 'vm/settings/HgSetCacheState/PerfState/enable/' . $vm_id) . '</span></div>';
      }
    } elseif ($action == "Disable") {
      if ($perf_state == "ADMIN_DISABLED") {
        $message = "Cache metrics are already disabled.";
      } else {
        $message = "This will disable cache metrics. If you are sure click Disable.";
        $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Disable', 'vm/settings/HgSetCacheState/PerfState/disable/' . $vm_id) . '</span></div>';
      }
    } elseif ($action == "Reset") {
      $message = "This will reset the cache metrics. If you are sure click Reset.";
      $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Reset', 'vm/settings/HgClearPerfMetrics/PerfState/reset/' . $vm_id) . '</span></div>';
    }
  } elseif ($type == "cache-format") {
    if ($action == "Start") {
      $enable_check = 0;
      if ($cache_space[0]['value'] == '') {
        $message = "Please allocate cache to the VM before initiating a cache format.";
        $enable_check = 1;
      } else {
        $vm_node = node_load($vm_id);
        $vm_cache_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
        if ($vm_cache_state[0]['value'] == "ADMIN_ENABLED") {
          $message = "Cache is currently enabled on the VM. Please disable the cache before initiating a format.";
          $enable_check = 1;
        } else {
          $vdisk_query = db_select('node', 'node');
          $vdisk_query->leftJoin('field_data_field_virtual_machine', 'field_data_field_virtual_machine', 'node.nid = field_data_field_virtual_machine.entity_id AND field_data_field_virtual_machine.entity_type = :node AND field_data_field_virtual_machine.deleted = :deleted', array(':node' => 'node', ':deleted' => 0));
          $vdisk_query->fields('node', array('nid', 'title'));
          $vdisk_query->condition('node.status', 1)
            ->condition('field_data_field_virtual_machine.field_virtual_machine_target_id', $vm_id)
            ->condition('node.type', 'vdisk');
          $vdisk_result = $vdisk_query->orderBy('node.title', 'DESC')
            ->execute();
          $vdisk_counter = 1;
          $enable_vdisk = array();
          while ($vdisk_row = $vdisk_result->fetch()) {
            $vdisk_id = $vdisk_row->nid;
            $vdisk_node = node_load($vdisk_id);
            $cache_state = field_get_items('node', $vdisk_node, 'field_fa_administration_state');
            if ($cache_state[0]['value'] == "ADMIN_ENABLED") {
              $enable_vdisk[$vdisk_counter] = $vdisk_node->title;
              $enable_check = 1;
            }
            $vdisk_counter++;
          }
        }
        if ($enable_check == 0) {
          $vm_op_message = field_get_items('node', $vm_node, 'field_fa_operational_message');
          if (stristr($vm_op_message[0]['value'], 'format')) {
            if (stristr($vm_op_message[0]['value'], '100')) {
              $message = "This will start the cache format. If you are sure click 'Start'.";
              $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Start', 'vm/settings/HgStartCacheFormat/format/start/' . $vm_id) . '</span></div>';
            } else {
              $message = "A format operation is already in progress. Please 'Refresh' the VM '" . $vm_node->title . "' to query format Progress";
            }
          } else {
            $message = "This will start the cache format. If you are sure click 'Start'.";
            $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Start', 'vm/settings/HgStartCacheFormat/format/start/' . $vm_id) . '</span></div>';
          }
        } else {
          foreach ($enable_vdisk as $val) {
            $button_code .= '<div class = "enable-vdisk-details"><span class = "enable-vdisk-name">' . $val . '</span><span class = "columns mercury-button" id = "column-3">' . ctools_popups_vdiskinfo_link('Info', $vdisk_id, 'Info', '') . '</span></div>';
          }
        }
      }
    } elseif ($action == "Stop") {
      $message = "This will stop the cache format. If you are sure click 'Stop'.";
      $button_code = '<div class = "action-button"><span class = "mercury-button">' . l('Stop', 'vm/settings/HgStopCacheFormat/format/stop/' . $vm_id) . '</span></div>';
    }
  } elseif ($type == "refresh") {
    $message = "This will refresh the VM configuration and cache state. If you are sure click Refresh.";
  } elseif ($type == "edit") {
    $message = "This will provide the ability to edit VM configuration and credentials. If you are sure click Edit.";
  } elseif ($type == "advance") {
    $message = "This will provide the ability to do advanced VM configuration.";
  }
   */

  if ($type == 'cache-space' && $action == 'Unallocate') {
    $cache_space = field_get_items('node', $vm_node, 'field_cachespace');
    $admin_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
    if ($cache_space[0]['value'] == '') {
      $message = "<div class = 'messages warning'>No cache has been allocated to the VM.</div>";
    } else {
      if ($admin_state[0]['value'] == "ADMIN_ENABLED") {
        $message = "<div class = 'messages warning'>Cache is currently enabled. Please disable the cache before unallocating the cache space.</div>";
      } else {
        $message = "<div class = 'unallocate-vm'><div>This will unallocate the cache from the VM. If you are sure click Unallocate.</div></div>";
        $button_code = '<div class = "action-button unallocatevm"><span class = "mercury-button">' . l('Unallocate', 'vm/settings/unallocatecache/' . $vm_id) . '</span></div><div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div>';
      }
    }
  } else {
    return '<div class="messages error">Unknown request!</div>';
  }

  $output .= '<div class = "popup-setting-page">';
  $output .= '<div class = "vm-name"><span>VM Name: ' . $vm_name . '</span></div>';
  //$output .= '<div class = "driver-status">' . $driver_status_value . '</div>';
  $output .= '<div class = "notify-message">' . $message . '</div>';
  if (!empty($button_code)) {
    // If we got past all of the fatal errors, do an agent warning check.
    $warning = mercury_management_vm_agent_compatibility_warning($vm_node);
    if (!empty($warning)) {
      $output .= '<div class="inline-messages warning">' . $warning . '</div>';
    }

    // Output the button HTML.
    $output .= $button_code;
  }
  //$output .= '<div class = "close-popup"><span class = "mercury-button">' . l('Close', '') . '</span></div>';
  $output .= '</div>';
  return $output;
}

/**
 * Function to Allocate Cachespace to a VM
 */
function mercury_utilities_vm_settings_allocate_popup($vm_id) {
  $output = '';
  $vm_node = node_load($vm_id);
  $driver_status = field_get_items('node', $vm_node, 'field_driver_status');
  $vm_name = $vm_node->title;
  $driver_status_value = $driver_status[0]['value'];

  //$output_cachespace = '';

  $generate_form = drupal_render(drupal_get_form('mercury_utilities_vm_settings_allocate_cachespace', $vm_id));

  $output .= '<div class = "popup-host-setting-install">';
  $output .= '<div class = "host-name"><span>VM Name: ' . $vm_name . '</span></div>';
  $output .= '<div class = "host-driver-details"><span class = "driver-details">Flash Accel VM agent status: ' . $driver_status_value . '.</span></div>';
  $output .= '<div class = "confirm-message">' . $generate_form . '</div>';
  $output .= '</div>';

  return $output;
}

/**
 * Function to allocate cachspace form
 */
function mercury_utilities_vm_settings_allocate_cachespace($form, &$form_state) {
  $form = array();
  $vm_id = filter_xss(arg(5));

  // Before doing anything else, trigger a refresh.
  // Must happen before node_load.
  mercury_utilities_vm_refresh_action($vm_id, "no_refresh");

  $vm_load_data = node_load($vm_id);
  $vm_name = $vm_load_data->title;
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_id);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $host_load_data = node_load($vm_data['host']);

  $api_error = mercury_management_has_fa_api_error($vm_load_data);
  if ($api_error) {
    $form['api_error'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages error">' .$api_error . '</div>',
    );
    return $form;
  }

  // Return back if a cache space already exists
  if (!empty($vm_load_data->field_cachespace['und'][0]['value'])) {
    $form['item'] = array('#type' => 'markup',
      '#markup' => '<div class="messages warning">' . t('Cache Space is already allocated to the VM') . '</div>',
    );
    return $form;
  }

  // Before going any further, make sure that the current host is ready for
  // allocation.  One of the next few host operations will fail regardless,
  // but doing this separate check is fast and allows us to print a more helpful
  // error message for customers that are using the product for the first time.
  $current_host_status = mercury_management_get_host_status($host_load_data);
  if (empty($current_host_status['status_color']) || $current_host_status['status_color'] != 'green') {
    $form['item'] = array('#type' => 'markup',
      '#markup' => '<div class="messages error">' . t(
        'The current host @host is not ready for allocation. Please see the host info for details.',
        array('@host' => $host_load_data->title)) . '</div>',
    );
    return $form;
  }

  // Need to make sure the host target list is accurate for the VM
  // and that the targets can actually be used for caching.
  // This will have no effect if the migration policy is 'custom'.
  $target_response = mercury_management_vmotion_rebuild_target_host_list($vm_load_data);
  if (famc_external_api_response_is_error($target_response)) {
    $form['target_host_list_result'] = array('#type' => 'markup',
      '#markup' => '<div class="messages error">' .
        t('Unable to build host target list. @reason',
          array('@reason' => famc_external_api_response_get_error_message($target_response))) .
        '</div>',
    );
    return $form;
  }

  // There might also be non-fatal issues.  We still need to notify the user.
  if (!empty($target_response->warnings)) {
    $form['target_host_list_result'] = array('#type' => 'markup',
      '#markup' => '<div class="messages warning">' .
        t('Unable to allocate on some target hosts. Check logs for details.') .
        '</div>',
    );
  }

  // Check if the hosts are ready - correct agent installed, ssd
  // assigned/initialized, etc.  This is partly redundant with the checks in
  // mercury_management_vmotion_rebuild_target_host_list but that is unavoidable
  // because the earlier ones don't run if using a custom target list, etc.
  $hostsReady = mercury_management_vm_can_allocate_on_all_target_hosts($vm_id);
  if ($hostsReady == false) {
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages warning">' . t('Cannot allocate/reserve cache space across hosts. One or more hosts do not meet Flash Accel migration group requirements. See log for more details.') . '</div>',
    );
    return $form;
  }

  // Check if there is a shared VMFS datastore
  $available_datastores = mercury_management_vm_get_valid_allocation_datastores($vm_load_data);
  if (empty($available_datastores)) {
    // Link error message to the "Allocating cache" page.  Low available width.
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages error">' . t(
        'No VMFS datastores found on @host. !HelpLink', array(
          '@host' => $host_load_data->title,
          '!HelpLink' => mercury_utilities_error_get_help_link('GUID-53FD7A94-E11E-42DE-9917-41C3F84B9305'))) .
      '</div>',
    );
    return $form;
  }

  $recommended_datastore = mercury_management_vm_recommend_allocation_datastore($available_datastores);

  // TODO: We should allow the user to choose a datastore.
  // Needs sensible default, can use the logic we currently use for auto-select.
  // After we let the user choose, no recommended datastore becomes a warning.
  if (empty($recommended_datastore)) {
    // Link error message to the "Allocating cache" page.  Low available width.
    $form['item'] = array(
      '#type' => 'markup',
      // Note that this is only reached in the vMotion case.
      // If there are datastores and vMotion is not enabled, we will always
      // find one to recommend.
      '#markup' => '<div class="messages error">' . t(
        'No shared VMFS datastores found on migration targets. !HelpLink', array(
          '!HelpLink' => mercury_utilities_error_get_help_link('GUID-53FD7A94-E11E-42DE-9917-41C3F84B9305'))) .
      '</div>',
    );
    return $form;
  }
  $form['vmfs_datastore_name'] = array(
    '#type' => 'value',
    '#value' => $recommended_datastore,
  );

  // Cache segment size
  $segment_size = $host_load_data->field_segment_size['und'][0]['value'] / (1024 * 1024);
  if ($segment_size == 0) {
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages warning">' . t('Error obtaining host cache configuration information. Please refresh host and try again.') . '</div>',
    );
    return $form;
  }

  // Get available cache across all hosts
  $available_segments = mercury_management_availableCache($vm_id);
  // XXX: This will not work if some hosts have a different segment size.
  $available_mb = $available_segments * $segment_size; // MB, not bytes.

  // Get the Minimum Cache Space setting value for a VM (in MB)
  $cache_space_settings_value = variable_get('minimum_cache_space_vm', 1024);
  if ($available_mb - ($cache_space_settings_value * 1024) < 0) {
    $form['item'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages warning">' . t('There is not enough space available on the host(s) to allocate a cache space.  No flash device(s) has been assigned, or assigned flash device(s) do not contain the required cache space.') . '</div>',
    );
    return $form;
  }

  // Allocate the cache
  $form['item_1'] = array(
    '#type' => 'markup',
    '#markup' => '<div id="cache_error_mesg"></div>',
    '#suffix' => '<script>
        	jQuery("#edit-submit").live("click", function() {
		        jQuery("#edit-submit").hide();
		        jQuery(".form-item-num-segments-manually").hide();
		        jQuery(".form-item-manually-allocate-segments").hide();
		        jQuery("a.close").hide();
		        jQuery("#attribute-message").hide();
		        jQuery(".processing").show();
		        jQuery(".transfer-slider").hide();
		        jQuery("#migration_host_list").hide();
		        jQuery("div.res-message").remove();
            jQuery(".form-item-enable-vm-cache").hide();
            jQuery(".form-item-enable-vm-metrics").hide();
            jQuery("#info-message").hide();
            jQuery("#warning-message").append( \'<div class="res-message red">Please do not close the browser window until the process completes.</div>\');
            jQuery("#warning-message").append(\'<div class="res-message">Allocating Cache Space...</div>\');
		    });

	        if (jQuery("#modal-content .error").length > 0) {
			    var error_mesg = jQuery("#modal-content .error").html();

			    if (error_mesg.length > 1 ){
			      jQuery(".modal-header").append(\'<img class="error_popup_close" title="Close_refresh" style="cursor: pointer; float: right; padding-top: 5px;" alt="Close_refresh" src=" ' . base_path() . drupal_get_path('module', 'ctools') . '/images/icon-close-window.png" typeof="foaf:Image">\');
			    	jQuery("a.close").hide();
			    	jQuery("div.res-message").remove();
			    	//Refresh the page on error while Allocation Cache
			  		jQuery(".modal-header img.error_popup_close").live("click", function(e) {
			  		  e.preventDefault();
       				location.reload();
			  		});
			    }
			    jQuery("#edit-submit").show();
			    jQuery("#edit-num-segments-manually").removeAttr("checked");
		    }
		    </script>',
  );

  $agent_warning = mercury_management_vm_agent_compatibility_warning($vm_load_data);
  if ($agent_warning) {
    $form['agent_warning'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages warning">' . $agent_warning . '</div>',
    );
  }

  $form['item_2'] = array(
    '#type' => 'markup',
    '#markup' => '<div id = "attribute-message">' . t('The following options enable caching and cache metrics collection for the VM and its disks automatically, after cache allocation completes. If necessary, these features can be individually disabled later on a subset of disks such as the boot disk. Remember that you must enable caching and cache metrics collection on the VM and at least one of its disks for caching and metrics collection to be performed.</span>') . '</div>',
  );

  $form['enable_vm_cache'] = array(
    '#type' => 'checkbox',
    '#default_value' => 1,
    '#title' => t('Enable caching for the VM and its disks'),
  );
  $form['enable_vm_metrics'] = array(
    '#type' => 'checkbox',
    '#default_value' => 1,
    '#title' => t('Enable cache metrics collection for the VM and its disks'),
  );


  // If the selected (recommended) datastore is VMFS-3, need to detect that and
  // also limit to 2TB + display warning.
  // TODO: Need to find a more general way when the user gets to pick. on-click?
  $vmfs_max_allocation_mb = mercury_management_host_get_largest_supported_allocation_size_mb(
    $host_load_data,
    $available_datastores->$recommended_datastore->vmfs_version
  );
  if ($available_mb > $vmfs_max_allocation_mb) {
    // Cut back the max allocation to the VMFS limit.
    $available_mb = $vmfs_max_allocation_mb;

    // Link error message to the "Allocating cache" page.  Low available width.
    $form['vmfs_limit'] = array(
      '#type' => 'markup',
      '#markup' => '<div class="messages warning">' . t(
        'The maximum allocation for a VMFS-3 datastore is 2 TB minus 256 MB. !HelpLink', array(
          '!HelpLink' => mercury_utilities_error_get_help_link('GUID-53FD7A94-E11E-42DE-9917-41C3F84B9305'))) .
      '</div>',
    );
  }

  $form['item_3'] = array(
    '#type' => 'markup',
    '#markup' => '<div id = "info-message">' . t('<br/><hr/><br/><strong>Please use the slider to allocate cache space or enter a value by selecting \'Enter Manually\'.<br/> You can allocate minimum of ' . $cache_space_settings_value * 1024 . ' MB and maximum of ' . $available_mb . ' MB. </strong><br/>') . '</div>',
  );

  // Check to see if the selected host(s) belongs to DRS/HA enabled cluster
  $drs_ha_enabled = 0;
  $host_nodes = mercury_management_vmotion_vm_migration_hosts($vm_load_data, $include_current = true);
  if (!empty($host_nodes)) {
    foreach ($host_nodes as $host) {
      if ($host->field_drs['und'][0]['value'] == 1 || $host->field_ha['und'][0]['value'] == 1) {
        $drs_ha_enabled = 1;
        break;
      }
    }
  }
  if ($drs_ha_enabled) {
    $form['item_4'] = array(
      '#type' => 'markup',
      '#markup' => '<div id = "drs-message">' . t('<br/><strong>NOTE:</strong> The VM\'s host(s) are part of a cluster with DRS/HA enabled. It is recommended that VM-Host affinity rules be added to the cluster\'s DRS configuration, so that the VM is only provisioned on Flash Accel enabled host(s).<br/>') . '</div>',
    );
  }

  $form['item_5'] = array(
    '#type' => 'markup',
    '#markup' => '<div id = "warning-message">' . t('') . '</div>',
  );
  $form['processing'] = array(
    '#type' => 'markup',
    '#markup' => '<div class="processing"><img src = " ' . base_path() . drupal_get_path('module', 'mercury_utilities') . '/images/processing.gif"/></div>',
  );

  $form['num_segments_manually'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enter Manually'),
    '#attributes' => array('id' => 'edit-num-segments-manually')
  );

  $form['num_segments'] = array(
    '#type' => 'transfer_slider',
    '#title' => t('Size'),
    '#left_value' => ($cache_space_settings_value * 1024),
    '#right_value' => $available_mb - ($cache_space_settings_value * 1024), // Slider uses max: ($left + $right)
    '#left' => t('MB Allocated'),
    '#right' => t('MB Left'),
    '#suffix' => '<script>
        jQuery(".form-item-manually-allocate-segments").hide();
        jQuery("#edit-num-segments-left").attr("readonly","readonly");
        jQuery("#edit-num-segments-right").attr("readonly","readonly");
        jQuery("#edit-num-segments-manually").click(function () {
	        if (jQuery("#edit-num-segments-manually").is(":checked")) {
	        		jQuery(".form-item-manually-allocate-segments").show();
					    jQuery(".transfer-slider.container-inline").hide();
					} else {
					    jQuery(".form-item-manually-allocate-segments").hide();
					    jQuery(".transfer-slider.container-inline").show();
					}
					jQuery("#edit-manually-allocate-segments").val("");
				});
        </script>'
  );
  $form['manually_allocate_segments'] = array(
    '#type' => 'textfield',
    '#title' => t('Enter Cache Size'),
    '#description' => t('Value specified must be a multiple of 256 MB'),
    '#size' => 10,
    '#attributes' => array('id' => 'edit-manually-allocate-segments')
  );
  $form['vm_id'] = array(
    '#type' => 'hidden',
    '#value' => $vm_id,
  );

  /*
   * TODO: We should really start displaying this list again.  Revisit during
   * lazy allocation implementation (may need to indicate which hosts are lazy).
   * Would also be good to show a list of hosts and allocation status on
   * some or all VM status screens.
    // Display the list of hosts on which cache will be allocated
    $custom_host_list = array();
    $vmotion_enabled = $vm_load_data->field_vm_vmotionable['und'][0]['value'];
    if ($vmotion_enabled == 1) {
    $vm_hostlist_value = field_get_items('node', $vm_load_data, 'field_vm_vmotion_hostlist');
    $vm_hostlist = $vm_hostlist_value[0]['value'];
    if (!empty($vm_hostlist)) {
    $vm_hostnids = unserialize($vm_hostlist);
    foreach ($vm_hostnids as $host_nid) {
    $host_data = mercury_utilities_node_load_uncached($host_nid);
    $custom_host_list[] = $host_data->title;
    }
    } else {
    $form['item'] = array(
    '#type' => 'markup',
    '#markup' => '<div class="messages warning">' . t('No hosts have been selected for the VM @vm. Please select hosts prior to cache allocation', array('@vm' => $vm_load_data->title)) . '</div>',
    );
    return $form;
    }

    if (count($custom_host_list) > 0){
    $allocation_host_names = '<li>'. implode('</li><li>', $custom_host_list) .'</li>';
    }

    $form['policy_settings'] = array(
    '#type' => 'markup',
    '#prefix' => '<br><br>',
    '#markup' => t('<div id="migration_host_list" style="float: right; margin-right: 50px"><div>Cache space will be reserved on the following hosts:</div><div><ul>' . $allocation_host_names . '</ul></div></div>')
    );
    }
   */
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Allocate'),
    '#prefix' => '<br><br>',
    '#attributes' => array('id' => 'edit-submit'),
  );

  return $form;
}

/**
 * Validation the cachespace value entered using hook_validate()
 */
function mercury_utilities_vm_settings_allocate_cachespace_validate($form, &$form_state) {
  $values = $form_state['values'];

  $vm_nid = $values['vm_id'];
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_nid);
  $host_nid = $vm_data['host'];
  $host_data = node_load($host_nid);

  /**
   * Only for manual allocation we need to check for validity.
   * Non manual case the slider will not allow for invalid allocations
   */
  $manually_allocate = $values['num_segments_manually'];
  if ($manually_allocate == 1) {
    // Check if the manually specified value is valid number
    if (!is_numeric($values['manually_allocate_segments']) || $values['manually_allocate_segments'] <= 0) {
      form_set_error('manually_allocate_segments', 'Cache size value specified is not valid.');
      return;
    }

    // Compute number of segments requested
    $segment_size = $host_data->field_segment_size['und'][0]['value'] / (1024 * 1024);
    if (($values['manually_allocate_segments'] % $segment_size) != 0) {
      form_set_error('manually_allocate_segments', 'Cache size value is not a multiple of 256 MB');
      return;
    }

    // Compute number of segments requested
    $num_segments = (ceil($values['manually_allocate_segments'] / $segment_size));

    // Get available cache across all hosts
    $available_segments = mercury_management_availableCache($vm_nid);

    // Get the minimun Cache Space setting value for a VM (in GB)
    $cache_space_settings_value = variable_get('minimum_cache_space_vm', 1024);
    $allocated_vm_cache_space_settings = ($cache_space_settings_value * 1024) / $segment_size;

    if ($num_segments < $allocated_vm_cache_space_settings) {
      form_set_error('num_segments', 'Cache size specified is smaller than minimal cache size value.');
    } elseif ($num_segments > $available_segments) {
      form_set_error('num_segments', 'Cache size specified is larger than available size.');
    }
  }
}

/**
 * Allocating the cachespace to a vm using hook_submit()
 */
function mercury_utilities_vm_settings_allocate_cachespace_submit($form, &$form_state) {
  $values = $form_state['values'];
  $vm_nid = $values['vm_id'];
  $vmfs_datastore_name = $values['vmfs_datastore_name'];
  $vm_get_data = node_load($vm_nid);
  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_nid);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $host_nid = $vm_data['host'];
  $host_data = node_load($host_nid);

  // Cache segment size
  $segment_size = $host_data->field_segment_size['und'][0]['value'] / (1024 * 1024);

  $manually_allocate = $values['num_segments_manually'];
  if ($manually_allocate == 1) {
    $num_segments = (ceil($values['manually_allocate_segments'] / $segment_size));
  } else {
    $num_segments = ($values['num_segments']['left'] / $segment_size);
  }

  // Allocate the cache
  $allocated_status = mercury_management_allocateCacheToVM($vc_hostname, $host_nid, $vm_nid, $vmfs_datastore_name, $num_segments, $vc_username, $vc_password);
  if ($allocated_status != "attached") {

    mercury_utilities_fmc_log_message('Cache Mgmt', t('Failed to allocate cache space to the VM @vm. Please check log for details', array('@vm' => $vm_get_data->title)), WATCHDOG_ERROR, 'show');
    // Update VM node and remove cache
    $vm_get_data->field_cachespace['und'][0]['value'] = '';
    $vm_get_data->field_fa_cache_size['und'][0]['value'] = 0;
    $vm_get_data->field_fa_sector_size['und'][0]['value'] = 0;

    // If the host list was auto selected - forget it now
    if ($vm_get_data->field_vm_vmotion_policy['und'][0]['value'] == "default") {
      $vm_get_data->field_vm_vmotionable['und'][0]['value'] = 0;
      $vm_get_data->field_vm_vmotion_hostlist['und'][0]['value'] = '';
      $vm_get_data->field_vm_vmotion_policy['und'][0]['value'] = '';
    }
    node_save($vm_get_data);
    goto form_submit_done;
  }

  $no_vm_cache_operation = 0;
  if ($values['enable_vm_cache'] == 1 && $values['enable_vm_metrics'] == 1) {
    $param = array('request' => array('CacheState' => 'ADMIN_ENABLED', 'PerfState' => 'ADMIN_ENABLED'));
    $cache_state = 'ADMIN_ENABLED';
    $perf_state = 'ADMIN_ENABLED';
    $log_message = 'For best caching performance, disable caching and cache metrics collection for the boot disk and other disks that should not be cached.';
    $log_message2 = '';
  } elseif ($values['enable_vm_cache'] == 1 && $values['enable_vm_metrics'] == 0) {
    $cache_state = 'ADMIN_ENABLED';
    $perf_state = 'ADMIN_DISABLED';
    $param = array('request' => array('CacheState' => 'ADMIN_ENABLED'));
    $log_message = 'For best caching performance, disable caching for the boot disk and other disks that should not be cached.';
    $log_message2 = 'You must enable cache metrics collection on the VM and at least one of its disks for performance statistics to be collected.';
  } elseif ($values['enable_vm_cache'] == 0 && $values['enable_vm_metrics'] == 1) {
    $param = array('request' => array('PerfState' => 'ADMIN_ENABLED'));
    $cache_state = 'ADMIN_DISABLED';
    $perf_state = 'ADMIN_ENABLED';
    $log_message = 'You must enable caching on the VM and at least one of its disks for caching to be performed.';
    $log_message2 = 'For best caching performance, disable cache metrics collection for disks on which you do not need to measure caching performance.';
  } else {
    $no_vm_cache_operation = 1;
    $cache_state = 'ADMIN_DISABLED';
    $perf_state = 'ADMIN_DISABLED';
    $log_message = 'You must enable caching and cache metrics collection on the VM and at least one of its disks for caching and metrics collection to be performed. For best caching performance, do not enable caching and metrics collection for the boot disk and other disks that should not be cached.';
    $log_message2 = '';
  }
  if ($no_vm_cache_operation == 0) {

    /**
     * Wait a few secs for format to complete
     * There is no synchronous format and format progress is broken
     */
    sleep(2);

    // Set the cache and performance state
    $result = mercury_management_cache_operation('nid', $vm_nid, 'HgSetCacheState', $param);
    if ($result == 'error_wsdl_call') {
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache reconfiguration failed on VM "@vm". Please refresh VM and try again.', array('@vm' => $vm_get_data->title)), WATCHDOG_WARNING);
    } elseif ($result['response_status'] == 'success') {
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache reconfiguration operation successful on VM @vm.', array('@vm' => $vm_get_data->title)), WATCHDOG_NOTICE, 'show');
    }
    $vdisk_nids = mercury_utilities_vm_vdisk_nid_fetch($vm_nid);
    if (count($vdisk_nids)) {
      //$vdisk_ids = '';
      $params = new stdClass();
      $params->DeviceStateRequest = array();
      foreach ($vdisk_nids as $val) {
        //$vdisk_ids .= $val . ',';
        $vdisk_id = $val;
        $vdisk_node = node_load($vdisk_id);
        $vdisk_identifier = field_get_items('node', $vdisk_node, 'field_vdisk_identifier');
        $vdisk_identifier_value = $vdisk_identifier[0]['value'];
        $params->DeviceStateRequest['0'] = array('CacheState' => $cache_state, 'PerfState' => $perf_state, 'NaaId' => $vdisk_identifier_value);
        $disk_param = array('request' => $params);
        $vdisk_result = mercury_management_cache_operation('nid', $vm_nid, 'HgSetDeviceState', $disk_param);
        if (!empty($vdisk_result['response_status']) && $vdisk_result['response_status'] == 'success') {
          mercury_utilities_fmc_log_message(
            'Cache Mgmt',
            t('Cache reconfiguration successful for disk @disk on VM @vm.',
                array('@vm' => $vm_get_data->title, '@disk' => $vdisk_identifier_value)),
            WATCHDOG_NOTICE
          );
        } else {
          mercury_utilities_fmc_log_message(
            'Cache Mgmt',
            t('Cache reconfiguration failed for disk @disk on VM @vm. Please refresh VM and try again.',
                array('@vm' => $vm_get_data->title, '@disk' => $vdisk_identifier_value)),
            WATCHDOG_ERROR
          );
        }
      }
      //mercury_utilities_vdisk_settings_bulkaction_advance($vdisk_param, $vdisk_ids);
    }
    mercury_utilities_fmc_log_message('Cache Mgmt', $log_message, WATCHDOG_NOTICE);
    if ($log_message2 != '') {
      mercury_utilities_fmc_log_message('Cache Mgmt', $log_message2, WATCHDOG_NOTICE);
    }
  }

  mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache space was successfully allocated to the VM "@vm_title".', array('@vm_title' => $vm_get_data->title)));

  form_submit_done:

  // Refresh all the hosts
  $host_nids = array();
  if (!empty($vm_get_data->field_vm_vmotion_hostlist['und'][0]['value'])) {
    $host_nids = unserialize($vm_get_data->field_vm_vmotion_hostlist['und'][0]['value']);
  } else {
    $host_nids[] = $vm_get_data->field_host['und'][0]['target_id'];
  }
  foreach ($host_nids as $host_id) {
    mercury_utilities_host_refresh_action($host_id, "no_refresh");
  }

  // Refresh VM state
  mercury_utilities_vm_refresh_action($vm_nid, "no_refresh");

  $form_state['executed'] = TRUE;
}

function mercury_utilities_vm_settings_action($method, $type, $action, $vm_id) {
  // Get the current state / mode
  $vm_node = node_load($vm_id);
  $cache_state = field_get_items('node', $vm_node, 'field_fa_administration_state');
  $cache_mode = field_get_items('node', $vm_node, 'field_fa_cache_mode');
  $perf_state = field_get_items('node', $vm_node, 'field_fa_performance_state');
  $cache_space = field_get_items('node', $vm_node, 'field_cachespace');

  // There is no confirm dialog for these actions so there is no place to
  // display a warning.  Call this anyway to log in footer.
  mercury_management_vm_agent_compatibility_warning($vm_node);

  if ($method == "HgSetCacheState") {
    if ($type == "CacheState") {
      if ($action == "enable") {
        if ($cache_state[0]['value'] == "ADMIN_ENABLED") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache is already enabled on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING, 'show');
          drupal_goto($_SERVER['HTTP_REFERER']);
        } elseif ($cache_space[0]['value'] == '') {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache space must be allocated on VM @vm before the cache can be enabled.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('CacheState' => 'ADMIN_ENABLED'));
        }
      } elseif ($action == "disable") {
        if ($cache_state[0]['value'] == "ADMIN_DISABLED") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache is already disabled on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('CacheState' => 'ADMIN_DISABLED'));
        }
      }
    } elseif ($type == "CacheMode") {
      if ($action == "passthru") {
        if ($cache_mode[0]['value'] == "CM_PASSTHRU") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache mode is already set to passthru on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('CacheMode' => 'CM_PASSTHRU'));
        }
      } elseif ($action == "writethru") {
        if ($cache_mode[0]['value'] == "CM_WRITETHRU") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache mode is already set to writethru on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('CacheMode' => 'CM_WRITETHRU'));
        }
      } elseif ($action == "writearound") {
        if ($cache_mode[0]['value'] == "CM_WRITEAROUND") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache mode is already set to writearound on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('CacheMode' => 'CM_WRITEAROUND'));
        }
      }
    } elseif ($type == "PerfState") {
      if ($action == "enable") {
        if ($perf_state[0]['value'] == "ADMIN_ENABLED") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache metrics are already enabled on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('PerfState' => 'ADMIN_ENABLED'));
        }
      } elseif ($action == "disable") {
        if ($perf_state[0]['value'] == "ADMIN_DISABLED") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache metrics are already disabled on VM @vm.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
          drupal_goto($_SERVER['HTTP_REFERER']);
        } else {
          $param = array('request' => array('PerfState' => 'ADMIN_DISABLED'));
        }
      }
    }
  } elseif ($method == "HgClearPerfMetrics") {
    $param = array();
  } elseif ($method == "HgStartCacheFormat") {
    // XXXX - blocksize and blocks per chunk should not be set
    $param = array('request' => array());
  } elseif ($method == "HgStopCacheFormat") {
    $param = array();
  }

  // Do the operation on the VM
  $result = mercury_management_cache_operation('nid', $vm_id, $method, $param);
  if ($result == 'error_wsdl_call') {
    mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache reconfiguration failed on VM @vm. Please refresh VM and try again.', array('@vm' => $vm_node->title)), WATCHDOG_WARNING);
    if ($type == "CacheState" && $action == "enable") {
      mercury_utilities_fmc_log_message('Cache Mgmt', t("If the cache operation continues to fail, please check the Installation and Administration guide to confirm that all memory and other requirements have been met."), WATCHDOG_ERROR);
      mercury_utilities_fmc_log_message('Cache Mgmt', t("You can also try rebooting the VM, formatting the cache in the VM's advanced settings, and then enabling the cache again."), WATCHDOG_ERROR);
    }
  } elseif ($result['response_status'] == 'success') {
    $vm_info = node_load($vm_id);
    if ($method == 'HgSetCacheState') {
      if ($type == 'CacheState') {
        if ($action == 'enable') {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache enabled successfully on VM @vm.', array('@vm' => $vm_info->title)));
        } elseif ($action == 'disable') {
          // XXX: This still gets hit on some other types of errors.
          // Or maybe it was a success that didn't update the node correctly?
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache disabled successfully VM @vm.', array('@vm' => $vm_info->title)));
        }
      } elseif ($type == 'CacheMode') {
        if ($action == 'passthru') {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache mode changed to passthru on VM @vm.', array('@vm' => $vm_info->title)));
        } elseif ($action == 'writethru') {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache mode changed to writethru on VM @vm.', array('@vm' => $vm_info->title)));
        } elseif ($action == 'writearound') {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache mode changed to writearound on VM @vm.', array('@vm' => $vm_info->title)));
        }
      } elseif ($type == "PerfState") {
        if ($action == "enable") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache metrics enabled successfully on VM @vm.', array('@vm' => $vm_info->title)));
        } elseif ($action == "disable") {
          mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache metrics disabled successfully on VM @vm.', array('@vm' => $vm_info->title)));
        }
      }
    } elseif ($method == "HgClearPerfMetrics") {
      $entity_ids = array($vm_id);
      $vids = array();
      famc_analytics_delete_metrics_by_time(FAMC_ANALYTICS_VM_METRIC_TYPE, 0, $entity_ids);
      $vdisk_ids = famc_analytics_get_vdisk_nodes($vm_id, $filters = array());
      foreach ($vdisk_ids as $key => $val) {
        $vids[] = $key;
      }
      famc_analytics_delete_metrics_by_time(FAMC_ANALYTICS_VDISK_METRIC_TYPE, 0, $vids);
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache metrics were reset on VM @vm.', array('@vm' => $vm_info->title)));
    } elseif ($method == "HgStartCacheFormat") {
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache format started. Please refresh VM @vm to track format progress.', array('@vm' => $vm_info->title)));
    } elseif ($method == "HgStopCacheFormat") {
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache Format stopped. Please refresh VM @vm to query format status.', array('@vm' => $vm_info->title)));
    }

    // If cache or perf was disabled insert an empty perf node
    if (($type == "CacheState" || $type == "PerfState") && $action == "disable") {
      famc_save_empty_vm_metric($vm_id);
    }

  } else {
    mercury_utilities_fmc_log_message('Cache Mgmt', t('Cache reconfiguration operation failed. Please refresh VM and try again.'), WATCHDOG_ERROR);
    if ($type == "CacheState" && $action == "enable") {
      mercury_utilities_fmc_log_message('Cache Mgmt', t("If the cache operation continues to fail, please check the Installation and Administration guide to confirm that all memory and other requirements have been met."), WATCHDOG_ERROR);
      mercury_utilities_fmc_log_message('Cache Mgmt', t("You can also try rebooting the VM, formatting the cache in the VM's advanced settings, and then enabling the cache again."), WATCHDOG_ERROR);
    }
  }
  $param = array();
  $vm_info = node_load($vm_id);
  $attrInfo = mercury_management_cache_operation('nid', $vm_info->nid, 'HgGetCacheInfo', $param);
  if ($attrInfo != 'error_wsdl_call') {
    if (!empty($attrInfo)) {
      $vm_info->field_fa_build_version['und'][0]['value'] = $attrInfo['build_version'];
      $vm_info->field_fa_cache_mode['und'][0]['value'] = $attrInfo['cache_mode'];
      $vm_info->field_fa_administration_state['und'][0]['value'] = $attrInfo['admin_state'];
      $vm_info->field_fa_cache_size['und'][0]['value'] = $attrInfo['cache_size'];
      $vm_info->field_fa_sector_size['und'][0]['value'] = $attrInfo['sector_size'];
      $vm_info->field_fa_operational_state['und'][0]['value'] = $attrInfo['op_state'];
      $vm_info->field_fa_operational_message['und'][0]['value'] = $attrInfo['op_message'];
      $vm_info->field_fa_performance_state['und'][0]['value'] = $attrInfo['perf_state'];
      if ($attrInfo['cachespace_id'] != '') {
        $vm_info->field_cachespace['und'][0]['value'] = 'naa.' . $attrInfo['cachespace_id'];
      }
      node_save($vm_info);
      mercury_utilities_update_vm_vdisks_from_cacheinfo($vm_id, $attrInfo);
    } else {
      //drupal_set_message(t('VM update operation failed. Please refresh the VM @vm to get updated values.', array('@vm' => $vm_node->title)), 'error');
      mercury_utilities_fmc_log_message('Cache Mgmt', t('Unable to refresh VM cache state after reconfiguration on VM @vm', array('@vm' => $vm_node->title)), WATCHDOG_ERROR);
    }
  } else {
    //drupal_set_message(t('VM update operation failed. Please refresh the VM @vm to get updated values.', array('@vm' => $vm_node->title)), 'error');
    mercury_utilities_fmc_log_message('Cache Mgmt', t('Unable to connect to VM agent on @vm after reconfiguration', array('@vm' => $vm_node->title)), WATCHDOG_ERROR);
  }
  drupal_goto($_SERVER['HTTP_REFERER']);
}

function mercury_utilities_vm_settings_cachespace($type, $vm_id, $refresh = true) {
  // Make sure that we have the correct state info before proceeding.
  // Unallocate is potentially disruptive if caching is actually enabled
  // and we remove the cache device.
  // The only time we don't want to do this is if
  // we're already in the middle of a refresh.
  if ($refresh) {
    mercury_utilities_vm_refresh_action($vm_id, 'no_refresh');
  }

  $vm_data = mercury_management_get_host_dc_vc_of_vm($vm_id);
  $vcInfo = node_load($vm_data['vcenter']);
  $vc_hostname = $vcInfo->field_hostname['und'][0]['value'];
  $vc_username = fmc_cipher::decrypt($vcInfo->field_username['und'][0]['value']);
  $vc_password = fmc_cipher::decrypt($vcInfo->field_password['und'][0]['value']);
  $host_id = $vm_data['host'];
  $vm_node = node_load($vm_id);
  $cache_space = field_get_items('node', $vm_node, 'field_cachespace');
  $hostlist = $vm_node->field_vm_vmotion_hostlist['und'][0]['value'];

  $unallocate_result = mercury_management_unallocateCacheFromVM($vc_hostname, $host_id, $vm_id, $cache_space[0]['value'], $vc_username, $vc_password);

  // Update VM node and remove cache.
  // Only do this on success, not if the preconditions were not met.
  if ($unallocate_result) {
    $vm_node->field_cachespace['und'][0]['value'] = '';
    $vm_node->field_fa_cache_size['und'][0]['value'] = 0;
    $vm_node->field_fa_sector_size['und'][0]['value'] = 0;

    // If the host list was auto selected - forget it now
    if ($vm_node->field_vm_vmotion_policy['und'][0]['value'] == "default") {
      $vm_node->field_vm_vmotionable['und'][0]['value'] = 0;
      $vm_node->field_vm_vmotion_hostlist['und'][0]['value'] = '';
      $vm_node->field_vm_vmotion_policy['und'][0]['value'] = '';
    }
    node_save($vm_node);
  }

  // Refresh all the hosts
  $host_nids = array();
  if (!empty($hostlist)) {
    $host_nids = unserialize($hostlist);
  } else {
    $host_nids[] = $vm_node->field_host['und'][0]['target_id'];
  }
  foreach ($host_nids as $host_id) {
    mercury_utilities_host_refresh_action($host_id, "no_refresh");
  }

  // Refresh VM state.
  // Skip if in the middle of a refresh AND we failed.
  // Need to start the refresh over anyway if the original data is now stale.
  if ($unallocate_result || $refresh) {
    mercury_utilities_vm_refresh_action($vm_id, 'no_refresh');
  }
}
